#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

# Passed as "group": Accept Grade, Class, or All for group to 
# find for attendance.
# Passed as "mode": Weekly or Biweekly Mode. (or Date Range?)
# Passed as "class": The value of the group: ie. 8c, 10.

my %lex = ( 'Multi-Day Attendance Entry' => 'Multi-Day Attendance Entry',
	    'Attendance' => 'Attendance',
	    'Name' => 'Name',
	    'Continue' => 'Continue',
	    'Error' => 'Error',
	    'Weekly' => 'Weekly',

	    );

use DBI;
use CGI;
use Time::JulianDay;

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;

# Setup MaxPPD.(Maximum Periods per Day)
my $maxppd;  # Maximum periods per day
if ( not %g_ppd ) { 
    print "Periods per Day not set in configuration file!";
    die;
}
foreach my $key ( keys %g_ppd){
    if ($g_ppd{$key} > $maxppd){ $maxppd = $g_ppd{$key};}
} 

# Set # of weeks to do.
if ( $arr{mode} eq $lex{Weekly} ){ $weekcount = 1;}
else {$weekcount = 2;}


# Get Date
if ($arr{date}) {
    # Generate a list from those dates. Assume year-mon-day
    ($year,$month,$day) = split /-/,$arr{date};
    $jd = julian_day($year,$month,$day);
    $dow = day_of_week($jd);
    $mondayjd = $jd - ($dow-1);
    if (length($month) == 1){ $month = "0".$month;}
    if (length($day) == 1){ $day = "0".$day;}
    $currsdate = "$year$month$day"; 
    $currdate = "$year-$month-$day";
} else { # get current date
    @tim = localtime(time);
    $year = @tim[5] + 1900;
    $month = @tim[4] + 1;
    $day = @tim[3];
    $jd = julian_day($year,$month,$day);
    $mondayjd = $jd - ($tim[6]-1); #This will now give Monday of that week.
    if (length($month) == 1){ $month = "0".$month;}
    if (length($day) == 1){ $day = "0".$day;}
    $currdate = "$year-$month-$day";
    $currsdate = "$year$month$day"; 
    $currdate1 = "@month[$month] $day, $year";
}

# Setup Array and Hash to hold Dates and DOW Dates
mkDateData($mondayjd,$weekcount);

$dsn = "DBI:$dbtype:dbname=$dbase";
$dbh = DBI->connect($dsn,$user,$password);

# Setup the select from passed values: group, class.
if ($arr{group} eq 'Class' and $arr{class}){
    $select = " where homeroom = '$arr{class}' ";
} elsif ($arr{group} eq 'Grade' and $arr{class}){
    $select = " where grade = '$arr{class}' ";
} else { $select = '';} # Do 'All' students.


# Find the kids' names and studnum
$sth = $dbh->prepare("select lastname, firstname, studnum from student
  $select order by lastname, firstname");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die "$DBI::errstr\n";}    


print "$doctype\n<html><head><title>";
print $lex{'Multi-Day Attendance Entry'};
print "</title>\n<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";
print "$chartype\n</head><body>[ <a href=\"$attpage\">". $lex{Attendance};
print "</a> ] $currdate1\n<center>\n<h1>". $lex{'Multi-Day Attendance Entry'};
print "</h1>\n<b>$arr{group} $arr{class}</b>\n";


$prevjd = $mondayjd - 7; 
($prevyear, $prevmonth, $prevday) = inverse_julian_day($prevjd);
$prevdate = "$prevyear-$prevmonth-$prevday";

print "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\"><tr><td>
  <form action=\"attentrymd0.pl\" method=\"post\">
  <input type=\"hidden\" name=\"class\" value=\"$arr{class}\">
  <input type=\"hidden\" name=\"date\" value=\"$prevdate\">
  <input type=\"hidden\" name=\"group\" value=\"$arr{group}\">
  <input type=\"hidden\" name=\"mode\" value=\"$arr{mode}\">
  <input type=\"submit\" value=\"<= Previous Week\"></form>
  </td>\n\n";

$nextjd = $mondayjd + 7; 
($nextyear, $nextmonth, $nextday) = inverse_julian_day($nextjd);
$nextdate = "$nextyear-$nextmonth-$nextday";

print "<td>
  <form action=\"attentrymd0.pl\" method=\"post\">
  <input type=\"hidden\" name=\"class\" value=\"$arr{class}\">
  <input type=\"hidden\" name=\"date\" value=\"$nextdate\">
  <input type=\"hidden\" name=\"group\" value=\"$arr{group}\">
  <input type=\"hidden\" name=\"mode\" value=\"$arr{mode}\">
  <input type=\"submit\" value=\"Next Week =>\"></form>
  </td></tr></table>\n\n";


print "<form action=\"attentrymd1.pl\" method=\"post\">
<input type=\"submit\" value=\"". $lex{Continue}. "\">\n";
print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";


# Loop through table headings.
print '<tr><th>'. $lex{Name}. "</th>\n";
foreach my $day (@days){ print "<th>$datedata{$day}</th>";}
print "</tr>\n";

# Loop through kids displaying the values.
while ( @student = $sth->fetchrow){
    # print Name.
    print "<tr><td>$student[0], $student[1]</td>\n";
    # print the days for this mode:
    foreach my $day (@days){
	print "<td>";
	for (1..$maxppd){
	    print "P$_<input type=\"checkbox\" name=\"$student[2]:$day:$_\" value=\"1\">";
	}
	print "</td>";
    }
    print "</tr>\n";
}

print "</table><input type=\"submit\" value=\"". $lex{Continue}. "\"></form>
</center></body></html>\n";


#-------------
sub mkDateData {
#-------------

    my ($jd, $weeks) = (@_);
    # Build an array of ISO dates and a hash using them
    # as keys to hash values of 'Mon Jan 6' format.
    # If one week, then just 5 values, if two wk then 10, etc.
    for (1..$weeks){
	for (1..5){
	    my ($yr,$mon,$day) = inverse_julian_day($jd);
	    my $dow = day_of_week($jd);
	    push @days, "$yr-$mon-$day";
	    $datedata{"$yr-$mon-$day"} = "$week[$dow], $month[$mon] $day";
	    $jd++;
	}
	$jd = $jd + 2; # skip over weekend.
    }
}
