#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

use DBI;
use CGI;
use Number::Format qw(round);

my %lex = ('Missing Days' => 'Missing Days',
	   'or' => 'or',
	   'Start Date' => 'Start Date',
	   'End Date' => 'End Date',
	   'View' => 'View',
	   'Start' => 'Start',
	   'End' => 'End',
	   'Days' => 'Days',
	   'Ethnic Category' => 'Ethnic Category',
	   'Students' => 'Students',
	   'Student-Days' => 'Student-Days',
	   'Absent' => 'Absent',
	   'Enrolled' => 'Enrolled',
	   'Percentage' => 'Percentage',
	   'Attendance' => 'Attendance',
	   'Periods per Day' => 'Periods per Day',
	   'Error' => 'Error',

	   );


eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $periods;
# Find largest Periods per Day (PPD) and use that.
foreach my $pd (keys %g_ppd) { # %g_ppd defined in admin.conf
    if ( $g_ppd{$pd} > $periods ) { $periods = $g_ppd{$pd}; }
}


my ($sec, $min, $hour, $mday, $mon, $year, $wday, 
 $yday, $iddst) = localtime(time);
$year = $year + 1900;
$wday++;
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

print "$doctype\n<html><head><title>". $lex{View}. ' '. $lex{'Ethnic Category'}. "</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>[ <a href=\"$attpage\">Attendance</a> ]\n";

my $days = $arr{days};
my $startdate = $arr{startdate};
my $enddate = $arr{enddate};
if ( not $days or not $startdate or not $enddate ) {
    print '<h1>'. $lex{'Missing Days'}. ' '. $lex{or}. ' '. $lex{'Start Date'};
    print ' '. $lex{or}. ' '. $lex{'End Date'}. "</h1>\n";
    print "</body></html>\n";
    die;
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# First Count Ethnic Categories and stuff into ethnic hash.
my %ethnic = ();
my $sth1 = $dbh->prepare("select ethnic, count(*) from student 
  where ethnic != '' and ethnic is not null group by ethnic");
$sth1->execute;
while ( my ( $eth, $count ) = $sth1->fetchrow ) {
    $ethnic{$eth} = $count; 
}
# foreach my $key ( sort keys %ethnic ) { print "K:$key  V:$ethnic{$key}<br>\n"; }


# Now count attendance 
$sth = $dbh->prepare("select s.ethnic, count(a.attid) from student as s, attend as a 
 where a.studentid = s.studnum and 
 to_days(a.absdate) >= to_days('$startdate') and 
 to_days(a.absdate) <= to_days('$enddate') and
 a.reason = '$absentUnexcused'
 group by s.ethnic");

$sth->execute;
if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
my %attend = ();
while ( my ( $eth, $count ) = $sth->fetchrow ) {
    $attend{$eth} = $count; 
}
# foreach my $key ( sort keys %attend ) { print "K:$key  V:$attend{$key}<br>\n"; }


# Print Dates, etc.
print "<center><h1>$schoolname</h1><h3>$currdate</h3>\n";
print '<p><b>'. $lex{Start}. ":</b>$startdate &nbsp;&nbsp;<b>";
print $lex{End}. ":</b> $enddate\n";
print '&nbsp;&nbsp;<b>'. $lex{Days}. ":</b> $days &nbsp;&nbsp;<b>";
print $lex{'Periods per Day'}. ":</b> $periods</p>\n";


# Print Attendance Table head
print "<table border=\"0\" cellspacing=\"0\" cellpadding=\"8\">\n";
print '<tr><th>'. $lex{'Ethnic Category'}. '</th><th>'. $lex{Students};
print '<br>'. $lex{Enrolled}. '</th><th>'. $lex{'Student-Days'}. '<br>';
print $lex{Absent}. '</th><th>'. $lex{'Student-Days'}. '<br>'. $lex{Enrolled};
print '</th><th>'. $lex{Percentage}. '<br>'. $lex{Attendance}. "</th></tr>\n";

if ( not $periods) { $periods = 1; }
foreach my $key ( sort keys %ethnic ) {
    #print "K:$key A:$attend{$key} E:$ethnic{$key}<br>\n";
    $daysabsent = round  ( $attend{$key} / $periods), 2;
    $daysenrol =  $ethnic{$key} * $days;
    if ( not $daysenrol ) { 
	print "Days Enrol ". $lex{Error}. ": K:$key  V:$ethnic{$key}<br>\n"; 
	$daysenrol = 100;
    }
    $adjusted = (( $daysenrol - $daysabsent ) / $daysenrol ) * 100;
    $percent = round $adjusted, 2;
    print "<tr><td>$key</td><td>$ethnic{$key}</td>\n";
    print "<td>$daysabsent</td><td>$daysenrol</td><td>$percent</td></tr>\n";
}

print "</table></center></body></html>\n";
