#! /usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.


# ======= Monthly Student Attendance Report ======= 
# Get passed date for end of calendar month or school month.
# Calculate days in the month and number of days of school hols.
# Get student absences for that month, order by group,lastname,firstname.
#  Do # of present, absent, late for month.

# Absent strings for matching set in admin.conf; Comparison at line
# 305 in this file.


my %lex = ('Monthly Attendance Report' => 'Monthly Attendance Report',
	   'Missing Date' => 'Missing Date',
	   'Main' => 'Main',
	   'Attendance' => 'Attendance',
	   'Schooldays' => 'Schooldays',
	   'Weekdays' => 'Weekdays',
	   'Hols/Inserv' => 'Hols/Inserv',
	   'Red = Withdrawn' => 'Red = Withdrawn',
	   'Grand Totals' => 'Grand Totals',
	   'Periods Per Day not defined for' => 'Periods Per Day not defined for',
	   'Name' => 'Name',
	   'Enrolled' => 'Enrolled',
	   'Attended' => 'Attended',
	   'Days' => 'Days',
	   'Absent' => 'Absent',
	   'Unexec' => 'Unexec',
	   'Excuse' => 'Excuse',
	   'Other' => 'Other',
	   'Total' => 'Total',
	   'Late' => 'Late',
	   'Times' => 'Times',
	   'Grade' => 'Grade',
	   'homeroom' => 'homeroom',
	   'band' => 'band',
	   'Total School Days not calculated' => 'Total School Days not calculated',
	   'Student-Days' => 'Student-Days',
	   'Enrollment Error for student' => 'Enrollment Error for student',
	   'Records are not consistent' => 'Records are not consistent',
	   'Pg' => 'Pg',
	   'Top' => 'Top',
	   'View Log File' => 'View Log File',
	   'Error' => 'Error',
	   'View/Download' => 'View/Download',

	   );


use DBI;
use CGI;
use Date::Business;
use Number::Format qw(round);

my $maxlines = 24; # Maximum records per PDF page.

# Read config variables
eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my @tim = localtime(time);
my $year = @tim[5] + 1900;
my $month = @tim[4] + 1;
my $day = @tim[3];
if (length($month) == 1){ $month = "0".$month;}
if (length($day) == 1){ $day = "0".$day;}
my $currdate = "$year-$month-$day";

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

# Setup the TYPE of report, HTML or PDF.
my $html = $arr{html};
my $pdf = $arr{pdf};

if ( not $arr{date} ) {
    print $lex{'Missing Date'}. " (YYYY-MM-DD)\n";
    print "</body></html>\n";
    die "$lex{'Missing Date'} (YYYY-MM-DD)\n";
}

my ($year, $month, $day)  = split /-/, $arr{date};
my $enddate = "$year$month$day";
# not the best solution, yet...

my $group = $arr{group};  # Either 'grade', 'homeroom', or 'band'.

# Check to make sure start date can really be the first of month.
my ($startyear, $startmonth, $startday) = split /-/, $schoolstart;
if ( $month == $startmonth and $year == $startyear ) {
    $startdate = $schoolstart;
} else {
    $startdate = "$year-$month-01";
}

$startdate =~ s/-//g; #remove hyphens for DATE::Business fn.
my $end = new Date::Business(DATE=>$enddate);
my $start = new Date::Business(DATE=>$startdate);

my $schooldays = $end->diffb($start,'prev','next');
$schooldays++;
# Yup, this is the right way to do the date calcs...checked..

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

my $sth = $dbh->prepare("select * from dates 
  where to_days(date) >= to_days('$startdate') and 
  to_days(date) <= to_days('$enddate')");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
my $holidays = $sth->rows;

my $daysinsession = $schooldays - $holidays;

if ( $pdf ) { # Open TEX file, and print header.
    $shortname = "monthatt$$";
    $fileName = "$shortname.tex";
    open(TEX,">$fileName") || die "Can't open tex file";
    print_tex_doc_start(); # Setup the start of the file.
}

# Page Header...
print "$doctype\n<html><head><title>". $lex{'Monthly Attendance Report'}. "</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body><a name=\"top\"></a>\n";
print "[ <a href=\"$homepage\">". $lex{Main}. "</a> |\n";
print "<a href=\"$attpage\">". $lex{'Attendance'}. "</a> ]<center>\n";

if ( $html ) { # print top of page
    print '<h1>'. $lex{'Monthly Attendance Report'}. " - $month[$mon]</h1>\n";
    print '<h2>'. $lex{Schooldays}. ": $daysinsession &nbsp;&nbsp;". $lex{Weekdays};
    print ": $schooldays &nbsp;&nbsp;". $lex{'Hols/Inserv'}. ": $holidays</h2>\n";

    print "<p>[ <a href=\"#grandtotal\">". $lex{'Grand Totals'}. "</a> ] \n";
    print "<font color=\"red\">". $lex{'Red = Withdrawn'}. "</font></p>\n";
}

my $select;
$sth = $dbh->prepare("select * from studentall $select
  order by $group, lastname, firstname");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die;}
my $studrows = $sth->rows;

# Setup statement handle to find which table student is in.
my $sth5 = $dbh->prepare("select studnum from student 
  where studnum = ?");

my $curroom = -1; # Watch! Not a value entered for a reason...hopefully.
my $firstflag = 1;  # Flag to watch for first "real" person (not withdrawn)

my $studentname;

# *********** Start of Main Loop ***********

while ( my @student = $sth->fetchrow ) {
    $student[1] =~ s/\\/\//g;
    $studentname = "<b>$student[1]</b>, $student[2] $student[3]";
    # Now let's find which table student from: student or studentwd
    $sth5->execute( $student[4] );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my $active = $sth5->rows;  # returns 1 if in student, 0 if not.
    if (not $active and $html){
	$studentname = "<font color=\"red\">$studentname</font>";
    }

    # Calculate the days enrolled for this student
    # Return blocks of time when student was enrolled. 
    #(An array of hash pointers)
    @enrolblocks = &find_enrollment_blocks($student[4],$active);
    
    $schooldays = 0;

    #print "<br>$student[2] $student[1] ($student[4])<br>\n";

    foreach my $hashref (@enrolblocks){
	my $startenroldate = $hashref->{start};
	my $endenroldate = $hashref->{end};
	$startenroldate =~ s/-//g; $endenroldate =~ s/-//g; # strip hyphens

	$endenrolobj = new Date::Business(DATE=>$endenroldate);
	$startenrolobj = new Date::Business(DATE=>$startenroldate);
	# The other objects are $start and $end.
	
	# Now figure out the overlap and actual reporting period dates.
	my $startdiff = $start->diffb($startenrolobj,'prev','next');
	my $enddiff = $end->diffb($endenrolobj,'prev','next');
	
	#print "SEnrol: $startenroldate EEnrol: $endenroldate<br>\n";
	#print "SReport: ",$start->image," EReport:",$end->image,"<br>\n";
	#print "SD: $startdiff ED: $enddiff<br>\n";
       
	if ($startdiff < 0){ # Actual starting time is later.
	    $startrpobj = new Date::Business(DATE =>$startenroldate); }
	else { $startrpobj = new Date::Business(DATE=>$startdate);}
	
	if ($enddiff > 0){ # Actual ending date is earlier.
	    $endrpobj = new Date::Business(DATE=>$endenroldate); }
	else { $endrpobj = new Date::Business(DATE=>$enddate);}

	my  $rpdays = $endrpobj->diffb($startrpobj,'prev','next');
	$rpdays++; if ($rpdays < 0){ $rpdays = 0;}
       
	my $startrp = $startrpobj->image;
	my $endrp = $endrpobj->image;

	#print "StartRP: $startrp  EndRP: $endrp <br>\n";
	#print "RP Days: $rpdays <br>\n";

	if ($rpdays){
	    # Now find number of holidays during this same period.
	    $sth4 = $dbh->prepare("select * from dates 
             where to_days(date) >= to_days('$startrp') and 
             to_days(date) <= to_days('$endrp')");
	    $sth4->execute;
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	    $rphols = $sth4->rows;
	} else { $rphols = 0;}
	#print "RPHols: $rphols<br><br>\n";

	$schooldays = $schooldays + $rpdays - $rphols;
	#print "School Days: $schooldays<br>\n";

    } # End of Loop finding Student Enrollment
    
    if ($schooldays < 1){ next; }


    # Put Header printing here since we have to skip some students ====

    $oldroom = $curroom;
    if ($group eq $lex{'homeroom'}){
	$curroom = $student[6]; # homeroom field.
	# Get Teacher Name for the room.
	$sth1 = $dbh->prepare("select lastname, firstname from staff 
           where homeroom = '$curroom' ");
	$sth1->execute;
	if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr; }
	my ($fn, $ln) = $sth1->fetchrow;
	$teachername = '$fn $ln';
    } elsif ($group eq $lex{'band'}) {
	$curroom = $student[58]; # band
    } else {
	$curroom = $student[5]; # grade field.
    }
   
    if ($curroom ne $oldroom) {
        if (not $firstflag) {
	    if ($html){ 
		print_html_class_totals(); 
	    } else { 
		print_tex_class_totals(); 
		$linecount=0; 
	    } 
	    $totschooldays = 0; $totattended = 0;
	    $totlatedayun = 0; $totlatedayex = 0;
	    $totabsentdayun = 0; $totabsentdayex = 0;
	    $totother = 0; $totabsenttot = 0;
        }
	if ($html){ print "<p></p>"; print_html_header();} 
	else { print_tex_header(); }

	# Recalc the number of periods per day when class changes.
	$periods = $g_ppd{$student[5]};  # student[5] is grade.

        if (not $periods){
	    if (not $student[5]){ $student[5] = 'Undefined!';}
	    print '<h1>'. $lex{'Periods Per Day not defined for'}. ' '. $lex{Grade}. " $student[5]\n";
	    print "$student[2] $student[1]</h1>\n";
	    print "</body></html>\n";
	    die;
	}
    }

    # End of Header printing section ===========
 
    # Note: this can use startdate and $enddate since we only track absences
    # Get attendance records for this student and loop through adding up.
    $sth1 = $dbh->prepare("select * from attend 
      where studentid='$student[4]' and 
      to_days(absdate) >= to_days('$startdate') and 
      to_days(absdate) <= to_days('$enddate') 
      order by absdate,period");
    $sth1->execute;
    if ($DBI::errstr) { print $DBI::errstr; die; }

    $absentex = 0;$absentun = 0;
    $lateex = 0;$lateun = 0;
    $other = 0;

    # Track 5 groups of attendance: Absent Unexcused, Absent (any
    # other; presumed excused), Late Unexcused, Late (any other;
    # presumed excused), Other (which should be 0).

    while (@absence = $sth1->fetchrow){
	my $reason = $absence[3];
	# These string values set in the admin.conf file and match reasons
	if ($reason =~ /$absentUnexcused/) { $absentun++; }
	elsif ($reason =~ /$absentString/){ $absentex++;}
	elsif ($reason =~ /$lateUnexcused/) { $lateun++; }
	elsif ($reason =~ /$lateString/) { $lateex++; }
        else { $other++;}
    }

    if (not $periods){
	if (not $student[5]){ $student[5] = 'Undefined!';}
	print '<h1>'. $lex{'Periods Per Day not defined for'}. ' '. $lex{Grade}. "# $student[5]\n";
	print "$student[2] $student[1].</h1>\n";
	print "</body></html>\n";
	die;
    }

    $absentdayun = $absentun/$periods; # convert from periods into days
    $absentdayex = $absentex/$periods;
    $otherday = $other/$periods;

    $absentperun = ($absentdayun/$schooldays) * 100;
    $absentperex = ($absentdayex/$schooldays) * 100;
    $otherper = ($otherday/$schooldays) * 100;

    $absentdaytot = $absentdayun + $absentdayex + $otherday; 
    $absentpertot = ($absentdaytot/$schooldays) * 100;

    $latedayun = $lateun/$periods;
    $latedayex = $lateex/$periods;
    $lateperun = ($latedayun/$schooldays) * 100;
    $lateperex = ($latedayex/$schooldays) * 100;

    $attended = $schooldays - $absentdaytot;

    # For Class/Grade Summary
    $totschooldays = $schooldays + $totschooldays;
    $totattended = $attended + $totattended;

    $totabsentdayun = $absentdayun + $totabsentdayun;
    $totabsentdayex = $absentdayex + $totabsentdayex;
    $totother = $otherday + $totother;
    $totabsenttot = $absentdaytot + $totabsenttot;

    $totlatedayun = $latedayun + $totlatedayun;
    $totlatedayex = $latedayex + $totlatedayex;

    # For School Summary at Bottom
    $grtotschooldays += $schooldays; $grtotattended += $attended;
    $grtotabsentdayun += $absentdayun; $grtotabsentdayex += $absentdayex;
    $grtotother += $otherday; $grtotabsenttot += $absentdaytot;
    $grtotlatedayun += $latedayun; $grtotlatedayex += $latedayex;

    $firstflag = 0;  # Not first person to run through.

    # Print this student's record
    if ( $html ){ 
	print_html_record(); 
    } else { 
	print_tex_record($student[1], $student[2], $student[3], $student[4]);
    }
    $linecount++;
    if ($pdf and $linecount >= $maxlines){ # New page.
	print TEX "\\end{tabular}\\newpage\n\n";
	print_tex_header();
	$linecount = 0;
    }

} # End of main loop

# print out stats for the last class
if ($html){ 
    print_html_class_totals(); 
} else { 
    print_tex_class_totals();
}

# Grand totals
if ($html){ 
    print_html_grandtotals(); 
} else { 
    print_tex_grandtotals();
    print_tex_end();
}


# ================
#  Functions
# ================

# Print Table header at top of each table.
#--------------------
sub print_html_header {
#--------------------
    $heading = "\L\u$group";
    print "<table cellpadding=\"3\" cellspacing\=0\" border=\"1\">";
    print "<tr><th colspan=\"9\"><h2>$heading: $curroom</h2></th></tr>";

    print '<tr><th>'. $lex{Name}. '</th><th>'. $lex{Enrolled}. '</th><th>';
    print $lex{'Attended'}. '<br>('. $lex{Days}. ")</th>\n";

    print '<th>'. $lex{Absent}. '<br>'. $lex{Unexec}. '<br>('. $lex{Days}. ')</th><th>';
    print $lex{Absent}. '<br>'. $lex{Excuse}. '<br>('. $lex{Days}. ")</th>\n";

    print '<th>'. $lex{Absent}. '<br>'. $lex{Other}. '<br>('. $lex{Days};
    print ')</th><th>'. $lex{Absent};
    print '<br>'. $lex{Total}. '<br>('. $lex{Days}. ")</th>\n";

    print '<th>'. $lex{Late}. '<br>'. $lex{Unexec}. '<br>('. $lex{Times}. ')</th><th>';
    print $lex{Late}. '<br>'. $lex{Excuse}. '<br>('. $lex{Times}. ")</th></tr>\n";

}


#-------------------
sub print_tex_header {
#-------------------
    $heading = "\L\u$group";
    print TEX "\\begin{tabular}{|p{5cm}";
    for (1..8){ print TEX "|p{1.4cm}";}
    print TEX "|}\n\\hline\\rowcolor[gray]{0.85}{\\it $heading:} ";
    print TEX "{\\bf\\large $curroom} \n";
    print TEX "& {\\bf ". $lex{Enrolled}. ' ('. $lex{Days}. ")} & {\\bf ";
    print TEX $lex{Attended}. ' ('. $lex{Days}. ")} & ";
    print TEX "{\\bf ". $lex{Absent}. ' '. $lex{Unexec}. ' ('. $lex{Days};
    print TEX ")} &{\\bf ". $lex{Absent}. ' '. $lex{Excuse}. ' ('. $lex{Days}. ")} & ";

    print TEX "{\\bf ". $lex{Absent}. ' '. $lex{Other}. ' ('. $lex{Days}. ")} &{\\bf ";
    print TEX $lex{Absent}. ' '. $lex{Total}. ' ('. $lex{Days}. ")} & \n";
    print TEX "{\\bf ". $lex{Late}. ' '. $lex{Unexec}. ' ('. $lex{Times}. ")} &{\\bf ";
    print TEX $lex{Late}. ' '. $lex{Excuse}. ' ('. $lex{Times}. ")}\\\\ \\hline\n";
}


#--------------------------
sub print_html_class_totals {
#--------------------------

    if ( not $totschooldays ) { 
	print $lex{Error}. ": ". $lex{'Total School Days not calculated'}. "!\n"; return;
    }

    $totpercent = ($totattended/$totschooldays) * 100;

    $totabsentperun = ($totabsentdayun/$totschooldays) * 100;
    $totabsentperex = ($totabsentdayex/$totschooldays) * 100;
    $tototherper = ($totother/$totschooldays) * 100;
    $totabsentpertot = ($totabsenttot/$totschooldays) * 100;

    $totlateperun = ($totlatedayun/$totschooldays) * 100;
    $totlateperex = ($totlatedayex/$totschooldays) * 100;

    # print out stats for this class
    print "<tr class=\"gray\"><td><b>". $lex{Total};
    print ' - ('. $lex{'Student-Days'}. ")</b></td>";
    print "<td>$totschooldays</td>";
    print "<td>$totattended<br> ("; 
    print round( $totpercent, 2 );
    print "%)</td>\n";

    print "<td>$totabsentdayun<br>("; 
    print round( $totabsentperun, 2 );
    print "%)</td>\n";

    print "<td>$totabsentdayex<br>("; 
    print round( $totabsentperex, 2);
    print "%)</td>\n";

    print "<td>$totother<br>("; 
    print round( $tototherper, 2 );
    print "%)</td>\n";

    print "<td>$totabsenttot<br>("; 
    print round( $totabsentpertot, 2);
    print "%)</td>\n";

    print "<td>$totlatedayun days<br>("; 
    print round( $totlateperun, 2);
    print "%)</td>\n";

    print "<td>$totlatedayex days<br>("; 
    print round( $totlateperex, 2);
    print "%)</td>\n";

    print "</tr></table>\n";

}


#-------------------------
sub print_tex_class_totals {
#-------------------------
    if (not $totschooldays){
 
	print $lex{Error}. "! ". $lex{'Total School Days not calculated'};
	return;
    }

    $totpercent = ($totattended/$totschooldays) * 100;

    $totabsentperun = ($totabsentdayun/$totschooldays) * 100;
    $totabsentperex = ($totabsentdayex/$totschooldays) * 100;
    $tototherper = ($totother/$totschooldays) * 100;
    $totabsentpertot = ($totabsenttot/$totschooldays) * 100;

    $totlateperun = ($totlatedayun/$totschooldays) * 100;
    $totlateperex = ($totlatedayex/$totschooldays) * 100;

    # print out stats for this class
    print TEX "\\rowcolor[gray]{0.90}";
    print TEX "{\\bf Totals (Student-Days)} & $totschooldays &\n";
    print TEX "$totattended (", round( $totpercent, 2),"\\%)& \n";
    print TEX "$totabsentdayun (", round( $totabsentperun, 2),"\\%)&\n";
    print TEX "$totabsentdayex (", round( $totabsentperex, 2),"\\%)&\n";
    print TEX "$totother (", round( $tototherper, 2),"\\%)&\n";
    print TEX "$totabsenttot (", round( $totabsentpertot, 2),"\\%)&\n";
    print TEX "$totlatedayun days (", round( $totlateperun, 2),"\\%)&\n";
    print TEX "$totlatedayex days (", round( $totlateperex, 2),"\\%)\\\\";
    print TEX "\\hline\\end{tabular}\\newpage\n\n";
}


#-------------------------
sub find_enrollment_blocks {
#-------------------------

    # Return a list of hash pointers to enrollment blocks.
    # Globals: $dbh, $currdate, $schoolstart (from admin.conf)

    # Passed Student Number (studnum) and active status.
    my $studnum = shift @_;
    my $active = shift @_;

    # Read enrollment changes from the transfer table.
    # Assume the $dbh var already set...
    my $sth = $dbh->prepare("select * from transfer 
      where studnum = '$studnum'
      and to_days(date) > to_days('$schoolstart')
      order by date desc");
    # Problem if two records of same student on same day... an entry error.

    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die;}
    #my $rows = $sth->rows;
  
    my @enrollment;

    # Read the first record and set the status.
    my @transrec = $sth->fetchrow;

    if (not $transrec[0]){ # no transfer records for this student
	if ($active){ # registered but no transfer records;
	    my $hashref = {};
	    $hashref->{start} = $schoolstart;
	    $hashref->{end} = $currdate; 
	    push @enrollment, $hashref;
	}
	return @enrollment;  # return undefined if not active; 
	# return full year if active.
    }


    my $status;
    if ($transrec[3] eq 'enrol' or $transrec[3] eq 're-enrol'){ 
	# Set $status and push first record.(hashref in array)
	$status = 'enrol';
	my $hashref = {};
	$hashref->{start} = $transrec[2];
	$hashref->{end} = $currdate; 
	push @enrollment, $hashref;
	# @enrollment is an array of hashrefs to the enrollment blocks.
    } else { 
	$status = 'withdraw'; $statusdate = $transrec[2];
    }

    while (@transrec = $sth->fetchrow){
	if ($status eq 'enrol' and $transrec[3] eq 'withdraw'){
	    # change status
	    $status = 'withdraw';
	    $statusdate = $transrec[2];
	} elsif ($status eq 'withdraw' and 
		 ($transrec[3] eq 'enrol' or $transrec[3] eq 're-enrol')){
	     $status = 'enrol';
	     my $hashref = {};
	     $hashref->{start} = $transrec[2];
	     $hashref->{end} = $statusdate; 
	     push @enrollment, $hashref;
	 } else { # Error!
	     print "<br><br>". $lex{'Enrollment Error for student'}. ": $studnum<br>\n";
	     print $lex{'Records are not consistent'}. ".<br>\n";
	     #die "Enrollment error for $studnum";
	 }

    } # End of While. 
 
    return @enrollment;

} # End of Sub

#----------------------
sub print_tex_doc_start {
#----------------------

    my $papersize;
    if ( $defaultpapersize ) {
	$papersize = $defaultpapersize;
    } else {
	$papersize = 'letterpaper';
    }

    print TEX "\\documentclass[10pt,$papersize,oneside]{article}
\\usepackage{array,colortbl,inputenc}
\\inputencoding{latin1}
%\\renewcommand{\\familydefault}{\\sfdefault}
\\pagestyle{empty}
\\setlength{\\textwidth}{7.7in}
\\setlength{\\textheight}{10.6in}
\\setlength{\\hoffset}{-1.5in}
\\setlength{\\voffset}{-0.8in}
\\setlength{\\headsep}{10pt}
\\setlength{\\headheight}{14pt}
\\setlength{\\topmargin}{0pt}
\\setlength{\\parindent}{0pt}
\\setlength{\\tabcolsep}{5pt}
\\setlength{\\extrarowheight}{3pt}
\\pagestyle{headings}
\\markright{$schoolname \\hfill ". $lex{'Monthly Attendance Report'}.
" -- $month[$month] -- ". $lex{Pg}. "  }
\n\n
\\begin{document}\n";
} # End of Print Tex Doc Start


#----------------
sub print_tex_end {
#----------------
    print TEX "\\hline\\end{tabular}\n\n\\end{document}\n";
    close TEX;

    system("$pdflatex $fileName > pdflog$$.txt");
    system("mv $shortname.pdf $downloaddir");
    system("mv pdflog$$.txt $downloaddir");
    system("rm $shortname.*");
    
    print "<h1><a href=\"$webdownloaddir/$shortname.pdf\">\n";
    print $lex{'View/Download'}. ' '. $lex{'Monthly Attendance Report'}. "</a></h1>\n";
    print "<p>[ <a href=\"$attpage\">". $lex{Attendance}. "</a> |\n";
    print "<a href=\"$webdownloaddir/pdflog$$.txt\">";
    print $lex{'View Log File'}. "</a> ] </p>\n";
    print "</body></html>\n";

} # Print Tex End


#--------------------
sub print_html_record {
#--------------------
    print "<tr><td>$studentname ($student[4]) $withdrawdate</td>";
    print "<td><b>$schooldays</b></td><td><b>$attended</b></td>\n";
    
    print "<td><b>$absentdayun</b> ("; 
    print round( $absentperun,2);
    print "%)</td>";

    print "<td><b>$absentdayex</b> ("; 
    print round( $absentperex, 2);
    print "%)</td>\n";

    print "<td><b>$otherday</b> (";
    print round( $otherper, 2); 
    print "%)</td>";

    print "<td><b>$absentdaytot</b> ("; 
    print round( $absentpertot, 2 ); 
    print "%)</td>";

    print "<td><b>$lateun</b> ("; 
    print round( $lateperun, 2 );
    print "%)</td>";
    print "<td><b>$lateex</b> ("; 

    print round( $lateperex, 2);
    print "%)</td>";
    print "</tr>\n";

} # End of Print HTML Record


#-------------------
sub print_tex_record {
#-------------------
    my ($lastname, $firstname, $middlename, $studnum) = @_;
    
    print TEX "{\\bf $lastname}, $firstname $middlename ($studnum) $withdrawdate & ";
    print TEX "$schooldays &  $attended & ";
    print TEX "$absentdayun (", round( $absentperun, 2), "\\%)& ";
    print TEX "$absentdayex (", round( $absentperex, 2), "\\%) & ";
    print TEX "$otherday (", round( $otherper, 2), "\\%) & ";
    print TEX "$absentdaytot (", round( $absentpertot, 2),"\\%) & ";
    print TEX "$lateun (", round( $lateperun, 2), "\\%) & ";
    print TEX "$lateex (", round( $lateperex, 2), "\\%) \\\\ \\hline\n\n";
}


#-------------------------
sub print_html_grandtotals {
#-------------------------

    # Print Grand Totals
    print "<p>&nbsp;</p><a name=\"grandtotal\"></a>\n";
    print"<table cellpadding=\"3\" cellspacing\=0\" border=\"1\">\n";
    print "<tr><th colspan=\"9\"><h2>". $lex{'Grand Totals'}. "</h2></th></tr>\n";
    print "<tr><th></th><th>". $lex{Enrolled}. '</th><th>'. $lex{Attended};
    print '<br>('. $lex{Days}. ")</th>\n";

    print "<th>". $lex{Absent}. '<br>'. $lex{Unexec}. '<br>('. $lex{Days}. ")</th><th>\n";
    print $lex{Absent}. '<br>'. $lex{Excuse}. '<br>('. $lex{Days}. ")</th>\n";
    print '<th>'. $lex{Absent}. '<br>'. $lex{Other}. '<br>('. $lex{Days}. ')</th><th>';
    print $lex{Absent}. '<br>'. $lex{Total}. '<br>('. $lex{Days}. ")</th>\n";
    print '<th>'. $lex{Late}. '<br>'. $lex{Unexec}. '<br>('. $lex{Times}. ')</th><th>';
    print $lex{Late}. '<br>'. $lex{Excuse}. '<br>('. $lex{Times}. ")</th></tr>\n";

    $grtotpercent = ($grtotattended/$grtotschooldays) * 100;

    $grtotabsentperun = ($grtotabsentdayun/$grtotschooldays) * 100;
    $grtotabsentperex = ($grtotabsentdayex/$grtotschooldays) * 100;
    $grtototherper = ($grtotother/$grtotschooldays) * 100;
    $grtotabsentpertot = ($grtotabsenttot/$grtotschooldays) * 100;
    
    $grtotlateperun = ($grtotlatedayun/$grtotschooldays) * 100;
    $grtotlateperex = ($grtotlatedayex/$grtotschooldays) * 100;

    # print out grand totals
    print "<tr class=\"gray\"><td><b>". $lex{'Grand Totals'};
    print " (Student-Days)</b></td>";

    print "<td>$grtotschooldays</td>";
    print "<td>$grtotattended days<br> ("; 
    print round( $grtotpercent, 2 );
    print "%)</td>\n";

    print "<td>$grtotabsentdayun days<br> ("; 
    print round( $grtotabsentperun, 2 ); 
    print "%)</td>\n";
    
    print "<td>$grtotabsentdayex days<br>("; 
    print round( $grtotabsentperex, 2 ); 
    print "%)</td>\n";

    print "<td>$grtotother days<br>("; 
    print round( $grtototherper, 2 ); 
    print "%)</td>\n";
    
    print "<td>$grtotabsenttot days<br>("; 
    print round( $grtotabsentpertot, 2 ); 
    print "%)</td>\n";
    
    print "<td>$grtotlatedayun<br>("; 
    print round( $grtotlateperun, 2 ); 
    print "%)</td>\n";
    print "<td>$grtotlatedayex<br>("; 
    print round( $grtotlateperex, 2 ); 
    print "%)</td>\n";

    print "</tr></table>\n";
    
    print "<p>[ <a href=\"$homepage\">". $lex{Main}. "</a> |\n";
    print "<a href=\"$attpage\">". $lex{Attendance}. "</a> |\n";
    print "<a href=\"#top\">". $lex{Top}. "</a> ]</p></body></html>\n";

}


#------------------------
sub print_tex_grandtotals {
#------------------------

    print TEX "\\begin{tabular}{|p{4cm}";
    for (1..8){ print TEX "|p{1.5cm}";}
    print TEX "|}\n\\hline\\rowcolor[gray]{0.85}". $lex{'Grand Totals'}. " &";
    print TEX $lex{Enrolled}. " & ". $lex{Attended}. " (". $lex{Days}. ") &";
    print TEX $lex{Absent}. ' '. $lex{Unexec}. " (". $lex{Days}. ") & ". $lex{Absent};
    print TEX ' '. $lex{Excuse}. " (". $lex{Days}. ") &";
    print TEX $lex{Absent}. ' '. $lex{Other}. " (". $lex{Days}. ") & ". $lex{Absent};
    print TEX ' '. $lex{Total}. " (". $lex{Days}. ") &";
    print TEX $lex{Late}. ' '. $lex{Unexec}. " (". $lex{Times}. ") & ". $lex{Late};
    print TEX ' '. $lex{Excuse}. " (". $lex{Times}. ")\\\\ \\hline\n";

    $grtotpercent = ($grtotattended/$grtotschooldays) * 100;

    $grtotabsentperun = ($grtotabsentdayun/$grtotschooldays) * 100;
    $grtotabsentperex = ($grtotabsentdayex/$grtotschooldays) * 100;
    $grtototherper = ($grtotother/$grtotschooldays) * 100;
    $grtotabsentpertot = ($grtotabsenttot/$grtotschooldays) * 100;
    
    $grtotlateperun = ($grtotlatedayun/$grtotschooldays) * 100;
    $grtotlateperex = ($grtotlatedayex/$grtotschooldays) * 100;

    # print out grand totals
    print TEX $lex{'Grand Totals'}. " (". $lex{'Student-Days'}. ") & $grtotschooldays &";
    print TEX "$grtotattended ". $lex{Days}. " (", round( $grtotpercent, 2 ),"\\%)&\n";
    print TEX "$grtotabsentdayun ". $lex{Days}. " (", round( $grtotabsentperun, 2 ),"\\%)&\n";
    print TEX "$grtotabsentdayex ". $lex{Days}. " (", round( $grtotabsentperex, 2 ),"\\%)&\n";
    print TEX "$grtotother ". $lex{Days}. " (", round( $grtototherper, 2),"\\%)&\n";
    print TEX "$grtotabsenttot ". $lex{Days}. " (", round( $grtotabsentpertot, 2),"\\%)&\n";
    print TEX "$grtotlatedayun (", round( $grtotlateperun, 2),"\\%)&\n";
    print TEX "$grtotlatedayex (", round( $grtotlateperex, 2),"\\%)\\\\ \n";

} # End of Print_tex_grandtotals


