#! /usr/bin/perl 
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.


use DBI;
use CGI;
use Date::Business;
use Number::Format qw(:subs);

my %lex = ('Attendance Profiles' => 'Attendance Profiles',
	   'Missing Start Date or End Date' => 'Missing Start Date or End Date',
	   'Attendance' => 'Attendance',
	   'Cannot open tex file' => 'Cannot open tex file',
	   'Enrollment Changes' => 'Enrollment Changes',
	   'Error: Missing grade for student' => 'Error: Missing grade for student',
	   'Periods Per Day not defined for Grade' => 'Periods Per Day not defined for Grade',
	   'in global configuration for student' => 'in global configuration for student',
	   'Perfect Attendance' => 'Perfect Attendance',
	   'View/Download' => 'View/Download',
	   'View Log File' => 'View Log File',
	   'Total' => 'Total',
	   'Absent' => 'Absent',
	   'Late' => 'Late',
	   'Excused' => 'Excused',
	   'Unexcused' => 'Unexcused',
	   'times' => 'times',
	   'day(s)' => 'day(s)',
	   'Main' => 'Main',
	   'Name' => 'Name',
	   'Select' => 'Select',
	   'Withdrawn Students' => 'Withdrawn Students',
	   'Date format Error' => 'Date format Error',
	   'Subject' => 'Subject',
	   'Lastname, Firstname' => 'Lastname, Firstname',
	   'Homeroom, Lastname, Firstname' => 'Homeroom, Lastname, Firstname',
	   'Grade, Lastname, Firstname' => 'Grade, Lastname, Firstname',
	   'Sort by' => 'Sort by',
	   'Select' => 'Select',
	   'Grade' => 'Grade',
	   'Homeroom' => 'Homeroom',
	   'Start Date' => 'Start Date',
	   'End Date' => 'End Date',
	   'Show' => 'Show',
	   'Continue' => 'Continue',
	   'No Student(s) Found' => 'No Student(s) Found',
	   'School Days' => 'School Days',
	   'Unknown reason' => 'Unknown reason',
	   'Period' => 'Period',
	   'Error' => 'Error',
	   'Blank=All' => 'Blank=All',
	   'Band, Lastname, Firstname' => 'Band, Lastname, Firstname',
	   'Band' => 'Band',
	   'Summary Records' => 'Summary Records',
	   'Count' => 'Count',
	   'Reason' => 'Reason',
	   'Or' => 'Or',
	   'Start Term' => 'Start Term',
	   'End Term' => 'End Term',
	   'Teacher' => 'Teacher',
	   'Homeroom' => 'Homeroom',

	);

my $grayshade = '0.80';
my $self = "rptattprof.pl";

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $maxlines = 28;
my $shortname = "attprofile$$";
my $filename = "$shortname.tex";

# Set AM/PM Hash for use converting 2 period day to AM/PM
%ampm = ( 1 => 'AM', 2 => 'PM');

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

# rounding format
my $fmt = new Number::Format(-decimal_fill => '1', -decimal_digits => '2');


my $wdselectflag;
if ( $arr{wdselectflag} ) { # We have selected withdrawn students to print.
    $wdselectflag = 1;
    delete $arr{wdselectflag};
}

my $sortorder = "homeroom, lastname, firstname";
if ( $arr{sortorder} eq $lex{'Lastname, Firstname'} ) {
    $sortorder = "lastname, firstname";
} elsif ( $arr{sortorder} eq $lex{'Grade, Lastname, Firstname'} ) {
    $sortorder = "grade, lastname, firstname";
} elsif ( $arr{sortorder} eq $lex{'Band, Lastname, Firstname'} ) {
    $sortorder = "band, lastname, firstname";
} elsif ( $arr{sortorder} eq $lex{'Homeroom, Lastname, Firstname'} ) {
    $sortorder = "homeroom, lastname, firstname";
}
if ( $arr{sortorder} ) { delete $arr{sortorder}; }


my $select;
if ( $arr{group} ) {
    if ( $arr{select} eq $lex{Grade} ) { # Find this grade;
	my $grp = $dbh->quote( $arr{group} );
	$select = "where grade = $grp";
    } elsif ( $arr{select} eq $lex{Homeroom} ) {
	my $grp = $dbh->quote( $arr{group} );
	$select = "where homeroom = $grp";
    } elsif ( $arr{select} eq $lex{Band} ) {
	my $grp = $dbh->quote( $arr{group} );
	$select = "where band = $grp";
    }


}


# print page header
print "$doctype\n<html><head><title>",$lex{'Attendance Profiles'},"\n";
print "</title><link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";

print "<link rel=\"stylesheet\" type=\"text/css\" media=\"all\" ";
print "href=\"/js/calendar-blue.css\" title=\"blue\">\n";
print "<script type=\"text/javascript\" src=\"/js/calendar.js\"></script>\n";
print "<script type=\"text/javascript\" src=\"/js/lang/calendar-en.js\"></script>\n";
print "<script type=\"text/javascript\" src=\"/js/calendar-setup.js\"></script>\n";


print "$chartype\n</head><body>[ <a href=\"$homepage\">". $lex{Main}. "</a> | \n";
print "<a href=\"$attpage\">". $lex{Attendance},"</a> ]\n";

#foreach my $key (sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

if ( not $arr{startflag} ) {
    showStartPage();
} else {
    delete $arr{startflag};
}



my ($startdate, $enddate);

if ( $arr{startterm} ) {
    $arr{startdate} = $g_termstart{ $arr{startterm} };
}
if ( $arr{endterm} ) {
    $arr{enddate} = $g_termend{ $arr{endterm} };
}


# Passed variables: start and end dates for reporting period.
if ( not $arr{startdate} or not $arr{enddate} ) {
    print '<h1>',$lex{'Missing Start Date or End Date'},"</h1>\n";
    print "</body></html>\n";
    die $lex{'Missing Start Date or End Date'};
}

# Parse startdate.
if ( $arr{startdate} =~ /-/) {
    my ($yr, $mo, $da) = split /-/, $arr{startdate};
    $startdate = "$yr$mo$da";
} elsif ( length( $arr{startdate} ) == 6 ) {
    $startdate =  '20'. $arr{startdate};
} elsif ( length( $arr{startdate} ) == 8 ) {
    $startdate =  $arr{startdate};
} else {
    print "<b>$arr{startdate} - ". $lex{'Date format Error'}. "</b><br>\n";
    print "</body></html>\n";
    die $lex{'Date format Error'};
}
#delete $arr{startdate};


# Parse enddate.
if ( $arr{enddate} =~ /-/) { # if 2007-06-06 format
    my ($yr, $mo, $da) = split /-/, $arr{enddate};
    $enddate = "$yr$mo$da";
} elsif ( length( $arr{enddate} ) == 6 ) {
    $enddate =  '20'. $arr{enddate};
} elsif ( length( $arr{enddate} ) == 8 ) {
    $enddate =  $arr{enddate};
} else {
    print "<b>$arr{enddate} - ". $lex{'Date format Error'}. "</b><br>\n";
    print "</body></html>\n";
    die $lex{'Date format Error'};
}
#delete $arr{enddate};


# If checkbox for withdrawn, allow selection of students to print.
if ( $arr{withdrawn} ) {
    selectWithdrawn( $startdate, $enddate );
}

# Perfect Attendance Option
my $perfectattendance;
if ( $arr{perfectattendance} ) {
    $perfectattendance = 1;
    delete $arr{perfectattendance};
}



# Now create a hash to store student numbers of students to print (if
# not selected from withdrawn).

if ( not $wdselectflag ) { # we are doing a normal print run of active students
    my $sth = $dbh->prepare("select studnum from student $select
     order by $sortorder");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $studnum = $sth->fetchrow) {
	push @students, $studnum;
    }

} else { # we have a selected withdrawn student group to print
    foreach my $sn (keys %arr) {
	push @students, $sn;
    }
}


if ( @students ) { # We have students to print; setup LaTeX file.
    open(TEX,">$filename") || die $lex{'Cannot open tex file'};

    my $papersize;
    if ( $defaultpapersize ) {
	$papersize = $defaultpapersize;
    } else {
	$papersize = 'letterpaper';
    }

    print TEX "\\documentclass[12pt,$papersize]{article}
\\usepackage{array,colortbl,multicol,inputenc}
\\inputencoding{latin1}
\\pagestyle{empty}
\\setlength{\\textwidth}{7in}
\\setlength{\\textheight}{10in}
\\setlength{\\hoffset}{-1in}
\\setlength{\\voffset}{-1.4in}
\\setlength{\\extrarowheight}{2pt}
\\setlength{\\parindent}{0pt}\n";

    print TEX "\\begin{document}\n";

} else {
    print "<b>". $lex{'No Student(s) Found'}. "</b>\n";
    die $lex{'No Student(s) Found'}. "\n";
}


my $sth = $dbh->prepare("select lastname, firstname, grade, homeroom, band 
  from studentall where studnum = ?");

foreach my $studnum ( @students ) {

    # get rest of student data.
    $sth->execute($studnum);
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my ( $lastname, $firstname, $grade, $homeroom, $band ) = $sth->fetchrow; 

    my $tardytot = 0;
    my $absenttot = 0;
    
    my $tardyu = 0;
    my $tardye = 0;
    my $absentu = 0;
    my $absente = 0;

    # Done once per loop to carry updated start date from later enrollment.
    my $loopstartdate = $startdate;
    my $loopenddate = $enddate;
 
    # Fetch Enrollment/Withdrawal Changes
    my $sth3 = $dbh->prepare("select * from transfer where 
     studnum = ? and 
     to_days(date) >= to_days('$loopstartdate') and 
     to_days(date) <= to_days('$loopenddate') 
     order by date desc");

    $sth3->execute( $studnum );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
 
    while ( @entryrecord = $sth3->fetchrow ) {
	foreach  $rec (@entryrecord){
	    $rec =~ s/:/-/g;
	    $rec =~ s/&/\\&/g;
	    $rec =~ s/#//g;
	    $rec =~ s/$//g;
	}
	push @entries, "$entryrecord[2]:$entryrecord[3]:$entryrecord[4]";

	if ( $entryrecord[3] eq "enrol" or $entryrecord[3] eq "re-enrol"){
	    # Change the start date, format in YYYY-MM-DD
	    ($year,$mo,$day) = split /-/,$entryrecord[2];
	    if (length($day) < 2) { $day = "0".$day;}
	    if (length($mo) < 2) { $mo = "0".$mo; }
	    $loopstartdate = "$year$mo$day";
	}
    } # End of Entry/Withdrawal Records  

   
    # Now figure out the number of schools days for this student
    my $end = new Date::Business(DATE=>$loopenddate);
    my $start = new Date::Business(DATE=>$loopstartdate);
    
    $schooldays = $end->diffb($start,'prev','next');
    $schooldays++; # Increment since we want number of days, not just diff.
    # These two settings are REQUIRED to get the elapsed day calculations right.
    # I did a lot of head scratching on this one...(used lots of paper too...)
    
    # Now find number of holidays during this same period.
    my $sth4 = $dbh->prepare("select * from dates 
     where to_days(date) >= to_days('$loopstartdate') and 
     to_days(date) <= to_days('$loopenddate')");
    $sth4->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my $holidays = $sth4->rows;

    my $schooldays = $schooldays - $holidays;

    # Now format start and end dates.
    $year = substr($loopstartdate,0,4);   
    $mo = substr($loopstartdate,4,2);
    $day = substr($loopstartdate,-2,2);
    $fmtstartdate = "$month[$mo] $day, $year";

    my $year = substr($loopenddate,0,4);   
    my $mo = substr($loopenddate,4,2);
    my $day = substr($loopenddate,-2,2);
    my $fmtenddate = "$month[$mo] $day, $year";

    # Setup to fetch Attendance Records
    my $sth1 = $dbh->prepare("select absdate, reason, period, subjsec from attend  
     where studentid = ? and 
     to_days(absdate) >= to_days('$startdate') and 
     to_days(absdate) <= to_days('$enddate') 
     order by absdate, period ");
    $sth1->execute( $studnum );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my $absrows = $sth1->rows;

    # Skip student if not showing perfect attendance
    if ( not $perfectattendance and $absrows < 1 ) { next; }

    # Get teacher and homeroom data (if any), print Enrollment changes.
    $sth2 = $dbh->prepare("select sal, firstname, lastname from staff 
       where homeroom='$homeroom' and homeroom != ''");
    $sth2->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my ($tsal, $tfirstname, $tlastname) = $sth2->fetchrow;
    my $teacher;
    if ($tlastname){ $teacher = $lex{Teacher}. ":& $tsal $tfirstname $tlastname\\\\ \n"; }
    my $hroom;
    if ( $homeroom ) { $hroom = $lex{Homeroom}. ":& $homeroom\\\\ \n"; }


    # Start of Printing for this student
    print TEX "\\center {\\sf\\huge $schoolname $lex{'Attendance Profiles'} } \\\\ \n\\bigskip";
    print TEX "{\\Large $firstname $lastname} $withdrawn \\\\ \n \\bigskip\n";
    print TEX "\\begin{tabular}{r|l} \n \\hline\n$teacher $hroom\n \\hline\n";
    if ( $band ) {
	print TEX $lex{Band}. "& {\\bf $band} \\\\ \n";
    }

    print TEX $lex{'Start Date'}. ": & $fmtstartdate \\\\ \n ". $lex{'End Date'};
    print TEX ":   & $fmtenddate \\\\ \n";
    print TEX $lex{'School Days'};
    print TEX ": & $schooldays \\\\\n\\hline\n\\end{tabular}\\\\\n\\bigskip\n";

    if ( $entryrecs > 0 ) {  # then print entry/withdrawals
	print TEX "\\hrulefill\\\\\n{\\large\\sf $lex{'Enrollment Changes'} }\\\\ \n";
	print TEX "\\begin{tabular}{lll}\n";
     
	for ($i=1; $i<=$entryrecs; $i++) {
	    $entryrec = pop @entries;
	    my ($edate,$etype,$edesc) = split(/:/,$entryrec);
	    print TEX "$edate & $etype & $edesc \\\\ \n";   
	}
     
	print TEX "\\end{tabular}\\\\ \n\\medskip\n";
    }

    # Now print the periods absent.
    # Calculate periods into days 
    my $periods = $g_ppd{$grade};
    if (not $grade) {
	print "<p><b>",$lex{'Error: Missing grade for student'};
	print " $firstname $lastname</b></p></body></html>\n";
	die;
    } elsif (not $periods){  # We don't have this grade's periods defined: fatal error
	print $lex{'Periods Per Day not defined for Grade'}. " $grade} ";
	print $lex{'in global configuration for student'};
	print "\n<br>$firstname $lastname</body></html>\n";
	die;
    }

    my (%subjectslate, %subjectsabs); # hold subject information, if any...
    if ( $absrows > 0 ) {  # then we have absences; loop through doing calcs.

	if ( $arr{summary} ) { # print Summary Values only
	    printSummaryRecords( $studnum );

	} else { # print it all...

	    print TEX "\\hrulefill\\\\ \n\\setlength{\\premulticols}{5pt}\n";
	    print TEX "\\setlength{\\postmulticols}{5pt}\n";
	    print TEX "\\begin{multicols}{2}\n\\raggedright\n";
     
	    for ($i=1; $i<=$absrows; $i++) {
		my ($absdate, $reason, $period, $subjsec) = $sth1->fetchrow;
		
		# Convert to AM/PM format if 2 periods per day
		if ($periods == 2){  # $periods defined on line 226 above
		    $period = $ampm{$period}; # ampm hash defined at top.
		}

		# Fixes for crud in reasons for attendance.
		$reason =~ s/[^A-Za-z\s]//g;

		# Do the calcs for reasons
		if ($reason eq $lateUnexcused){
		    $tardyu++;
		    if ($subjsec) { $subjectslate{$subjsec}++; } # only for unexcused.
		} elsif ($reason =~ m/$lateString/) {
		    $tardye++;
		} elsif ($reason eq $absentUnexcused){
		    $absentu++;
		    if ($subjsec) { $subjectsabs{$subjsec}++; } # only for unexcused.
		} elsif ($reason =~ m/$absentString/){
		    $absente++;
		} else { # unknown reason...
		    print "<b>". $lex{'Unknown reason'};
		    print ":</b> $reason - $absdate - ". $lex{Period}. " $period<br>\n";
		    print "$firstname $lastname ($studnum)<br>\n";
		}
		
		print TEX "$absdate-$period $reason\\\\ \n";   
	    }

	    my $daysabsentu = $fmt->format_number( $absentu/$periods, 2);
	    my $daysabsente = $fmt->format_number( $absente/$periods, 2);
	    my $daysabsenttot = $fmt->format_number( $daysabsente + $daysabsentu, 2);

	    $tardytot = $tardye + $tardyu;

	    print TEX "\\end{multicols}\n\n\\hrulefill\\\\ \n\\bigskip\n";
	    print TEX "\\begin{tabular}{r|l}\\hline\n";
	    print TEX $lex{Late}. ' '. $lex{Unexcused}," & $tardyu ",$lex{times},"\\\\\n"; 
	    print TEX $lex{Late}. ' '. $lex{Excused}, "& $tardye ", $lex{times}, " \\\\\n";
	    print TEX $lex{Total}. ' '. $lex{Late}, "& $tardytot ", $lex{times}, "\\\\\n";
	    print TEX "\\hline\n", $lex{Absent}. ' '. $lex{Unexcused}, "& $daysabsentu ";
	    print TEX $lex{'day(s)'}, "\\\\\n";
	    print TEX $lex{Absent}. ' '. $lex{Excused}, "& $daysabsente ", $lex{'day(s)'}, "\\\\\n";
	    print TEX $lex{Total}. ' ', $lex{Absent}, "& $daysabsenttot ", $lex{'day(s)'}, "\\\\\n";
	    print TEX "\\hline\n\\end{tabular}\n";


	} # end of Complete or Summary Records

	# Subject Attendance
	printSubjectAttendance($studnum, $arr{startdate}, $arr{enddate} );

	print TEX "\\newpage\n\n";

    } else {  # Perfect Attendance
	print TEX "\\bigskip {\\huge ", $lex{'Perfect Attendance'}, "! } \\newpage \n\n";
    }

} # End of Student Loop

print TEX "\\end{document}";
close TEX;

system("$pdflatex $filename >pdflog$$.txt");
system("mv $shortname.pdf $downloaddir");
system("mv pdflog$$.txt $downloaddir");
system("rm $shortname.*");

print "<center><h1><a href=\"/download/$shortname.pdf\">\n";
print $lex{'View/Download'}. ' '. $lex{'Attendance Profiles'}, "</a></h1>\n";
print "[ <a href=\"$attpage\">". $lex{Attendance}. "</a> |\n";
print "<a href=\"$webdownloaddir/pdflog$$.txt\">". $lex{'View Log File'}. "\n";
print  "</a> ]\n</center></body></html>\n";


#----------------------
sub printSummaryRecords {
#----------------------

    my $studnum = shift;  # must be passed student records.

    my $sth = $dbh->prepare("select distinct reason, period, count(period) from attend
     where studentid = ? and
     to_days(absdate) >= to_days('$startdate') and 
     to_days(absdate) <= to_days('$enddate') 
     group by reason, period  order by period, reason");
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    $sth->execute( $studnum );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    print TEX "\\begin{tabular}{r|l|l}\\hline\n";
    print TEX "\\rowcolor[gray]{$grayshade} $lex{Reason} & $lex{Period} & $lex{Count}";
    print TEX "\\\\ \\hline\n";
    while ( my ( $reason, $period, $count ) = $sth->fetchrow ) {
	print TEX "$reason & $period & $count \\\\ \\hline\n";
    }
    print TEX "\\end{tabular}\n";

}

#-----------------------
sub printCompleteRecords {
#-----------------------

    my $studnum = shift;


    # Setup to fetch Attendance Records
    my $sth1 = $dbh->prepare("select absdate, reason, period, subjsec from attend  
     where studentid = ? and 
     to_days(absdate) >= to_days('$startdate') and 
     to_days(absdate) <= to_days('$enddate') 
     order by absdate, period ");
    $sth1->execute( $studnum );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my $absrows = $sth1->rows;

    my (%subjectslate, %subjectsabs); # hold subject information, if any...
    if ( $absrows > 0 ) {  # then we have absences; loop through doing calcs.

	print TEX "\\hrulefill\\\\ \n\\setlength{\\premulticols}{5pt}\n";
	print TEX "\\setlength{\\postmulticols}{5pt}\n";
	print TEX "\\begin{multicols}{2}\n\\raggedright\n";
     
	for ($i=1; $i<=$absrows; $i++) {
	    my ($absdate, $reason, $period, $subjsec) = $sth1->fetchrow;

	    # Convert to AM/PM format if 2 periods per day
	    if ($periods == 2){  # $periods defined on line 226 above
		$period = $ampm{$period}; # ampm hash defined at top.
	    }

	    # Fixes for crud in reasons for attendance.
	    $reason =~ s/[^A-Za-z\s]//g;

	    # Do the calcs for reasons
	    if ($reason eq $lateUnexcused){
		$tardyu++;
		if ($subjsec) { $subjectslate{$subjsec}++; } # only for unexcused.
	    } elsif ($reason =~ m/$lateString/) {
		$tardye++;
	    } elsif ($reason eq $absentUnexcused){
		$absentu++;
		if ($subjsec) { $subjectsabs{$subjsec}++; } # only for unexcused.
	    } elsif ($reason =~ m/$absentString/){
		$absente++;
	    } else { # unknown reason...
		print "<b>". $lex{'Unknown reason'};
		print ":</b> $reason - $absdate - ". $lex{Period}. " $period<br>\n";
		print "$firstname $lastname ($studnum)<br>\n";
	    }

	    print TEX "$absdate-$period $reason\\\\ \n";   
	}

	my $daysabsentu = $fmt->format_number( $absentu/$periods, 2);
	my $daysabsente = $fmt->format_number( $absente/$periods, 2);
	my $daysabsenttot = $fmt->format_number( $daysabsente + $daysabsentu, 2);

	$tardytot = $tardye + $tardyu;

	print TEX "\\end{multicols}\n\n\\hrulefill\\\\ \n\\bigskip\n";
	print TEX "\\begin{tabular}{r|l}\\hline\n";
	print TEX $lex{Late}. ' '. $lex{Unexcused}," & $tardyu ",$lex{times},"\\\\\n"; 
	print TEX $lex{Late}. ' '. $lex{Excused}, "& $tardye ", $lex{times}, " \\\\\n";
	print TEX $lex{Total}. ' '. $lex{Late}, "& $tardytot ", $lex{times}, "\\\\\n";
	print TEX "\\hline\n", $lex{Absent}. ' '. $lex{Unexcused}, "& $daysabsentu ";
	print TEX $lex{'day(s)'}, "\\\\\n";
	print TEX $lex{Absent}. ' '. $lex{Excused}, "& $daysabsente ", $lex{'day(s)'}, "\\\\\n";
	print TEX $lex{Total}. ' ', $lex{Absent}, "& $daysabsenttot ", $lex{'day(s)'}, "\\\\\n";
	print TEX "\\hline\n\\end{tabular}\n";

	print TEX "\\newpage\n\n";

    } else {  # Perfect Attendance
	print TEX "\\bigskip {\\huge ", $lex{'Perfect Attendance'}, "! } \\newpage \n\n";
    }

} # end of printCompleteRecords



#-------------------------
sub printSubjectAttendance {
#-------------------------

    my ( $studnum, $startdate, $enddate ) = @_;

    # Find subjects where student is absent...
    my $sth = $dbh->prepare("select distinct subjsec from attend
     where studentid = ? and 
     to_days(absdate) >= to_days('$startdate') and 
     to_days(absdate) <= to_days('$enddate') and 
     subjsec != '' and subjsec is not null and subjsec != 'NULL'");

    $sth->execute( $studnum );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my $subjrows = $sth->rows;
    if ( $subjrows < 1 ) { return; }


    # Setup for printing subject lates/absences
    print TEX "\n\\bigskip\n\n";
    print TEX "\\begin{tabular}{|l|c|c|}\\hline\n";
    print TEX "\\rowcolor[gray]{$grayshade}",$lex{Subject};
    print TEX '&',$lex{Absent},'&',$lex{Late}," \\\\ \\hline\n";

    my $sth1 = $dbh->prepare("select description from subject where subjsec = ?");
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    my $sth2 = $dbh->prepare("select count(*)from attend
     where studentid = ? and subjsec = ? and reason = ? and
     to_days(absdate) >= to_days('$startdate') and 
     to_days(absdate) <= to_days('$enddate')");
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }


    while ( my $subjsec = $sth->fetchrow ) { # loop through all subjects

	$sth1->execute( $subjsec );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $desc = $sth1->fetchrow;
	
	# Lates
	$sth2->execute($studnum, $subjsec, $lateUnexcused );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $lates = $sth2->fetchrow;

	# Absences
	$sth2->execute($studnum, $subjsec, $absentUnexcused );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $absents = $sth2->fetchrow;

	print TEX "$desc ($subjsec) & $absents ";
	print TEX "& $lates \\\\ \\hline\n";

    }

    print TEX "\\end{tabular}\n";


} # end of printSubjectAttendance





#------------------
sub selectWithdrawn {  # select withdrawn students to print.
#------------------
    my ($startdate, $enddate) = @_;

    # Get count of withdrawn students in studentwd table
    my $sth = $dbh->prepare("select count(*) from studentwd");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my $studcount = $sth->fetchrow;
    print "<p><b>$studcount $lex{'Withdrawn Students'}</b></p>\n";


    print "<form action=\"$self\" method=\"post\">";
    print "<input type=\"hidden\" name=\"startdate\" value=\"$startdate\">\n";
    print "<input type=\"hidden\" name=\"enddate\" value=\"$enddate\">\n";
    print "<input type=\"hidden\" name=\"wdselectflag\" value=\"1\">\n";
    print "<input type=\"hidden\" name=\"startflag\" value=\"1\">\n";
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>",$lex{Name},"</th><th>",$lex{Select},"</th></tr>\n";
    print "<tr><td colspan=\"2\" align=\"center\">";
    print "<input type=\"submit\" value=\"",$lex{Select},"\"></td></tr>\n";

    my $sth = $dbh->prepare("select studid,lastname, firstname, studnum from studentwd
      order by lastname, firstname");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    for (1..$studcount) {
	my ($studid, $lastname, $firstname, $studnum) = $sth->fetchrow;
	print "<tr><td><b>$lastname</b>, $firstname ($studnum)</td><td>";
	print "<input type=\"checkbox\" name=\"$studnum\" value=\"1\"></td></tr>\n";
    }

    print "<tr><td colspan=\"2\" align=\"center\">";
    print "<input type=\"submit\" value=\"",$lex{Select},"\"></td></tr>\n";
    print "</table></form></body></html>\n";

    exit;

}


#-----------------
sub showStartPage {
#-----------------

    # print sortorder and selection input form.
    print "<center><h1>". $lex{'Attendance Profiles'}. "</h1>\n";
    print "<div><form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"startflag\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";
    print "<tr><td align=\"right\">". $lex{'Sort by'}. "\n";
    print "</td><td><select name=\"sortorder\"><option>". $lex{'Lastname, Firstname'};
    print "</option>\n";
    print "<option>". $lex{'Homeroom, Lastname, Firstname'}. "</option>\n";
    print "<option>". $lex{'Grade, Lastname, Firstname'}. "</option>\n";
    print "<option>". $lex{'Band, Lastname, Firstname'}. "</option>\n";
    print "</select></td></tr>\n";

    print "<tr><td align=\"right\">". $lex{Select}. "\n";
    print "</td><td><select name=\"select\"><option>". $lex{Grade}. "</option>\n";
    print "<option>". $lex{Homeroom}. "</option>\n";
    print "<option>". $lex{Band}. "</option>\n";
    print "</select>\n";
    print "<input type=\"text\" name=\"group\" size=\"14\"> ". $lex{'Blank=All'}. "</td></tr>\n";
    
    print "<tr><td align=\"right\">". $lex{'Start Date'}. "</td><td><input type=\"text\" ";
    print "name=\"startdate\" id=\"sdate\" size=\"10\">";
    print "<button type=\"reset\" id=\"start_trigger\">...</button>\n";
    print "</td></tr>\n";

    print "<tr><td align=\"right\">". $lex{'End Date'}. "</td><td><input type=\"text\" ";
    print "name=\"enddate\" id=\"edate\" size=\"10\">";
    print "<button type=\"reset\" id=\"end_trigger\">...</button></td></tr>\n";

    print "<tr><td></td><td><b>". $lex{Or}. "</b></td></tr>\n";

    # Now do terms
    print "<tr><td align=\"right\">". $lex{'Start Term'}. "</td><td>";
    print "<select name=\"startterm\"><option></option>\n";
    foreach my $trm ( sort keys %g_termstart ) {
	print "<option>$trm</option>";
    }
    print "</td></tr>\n";

    print "<tr><td align=\"right\">". $lex{'End Term'}. "</td><td>";
    print "<select name=\"endterm\"><option></option>\n";
    foreach my $trm ( sort keys %g_termend ) {
	print "<option>$trm</option>";
    }
    print "</td></tr>\n";


    print "<tr><td align=\"right\">". $lex{Show}. q{ }. $lex{'Summary Records'}. "</td><td>\n";
    print "<input type=\"checkbox\" name=\"summary\" value=\"1\"></td></tr>\n";

    print "<tr><td align=\"right\">". $lex{Show}. q{ }. $lex{'Withdrawn Students'}. "</td><td>\n";
    print "<input type=\"checkbox\" name=\"withdrawn\" value=\"1\"></td></tr>\n";

    print "<tr><td align=\"right\">". $lex{Show}. q{ }. $lex{'Perfect Attendance'}. "</td><td>\n";
    print "<input type=\"checkbox\" name=\"perfectattendance\" value=\"1\"></td></tr>\n";



    print "<tr><td align=\"center\" colspan=\"2\">";
    print "<input type=\"submit\" value=\"". $lex{Continue}. "\"></td></tr>\n";


    print "</table></form></div>\n";

    print "<script type=\"text/javascript\">
     Calendar.setup({
        inputField     :    \"sdate\", // id of the input field
        ifFormat       :    \"%Y-%m-%d\", // format of the input field
        button         :    \"start_trigger\", // trigger for the calendar (button ID)
        singleClick    :    false,        // double-click mode
        step           :    1             // show all years in drop-down boxes 
    });

    Calendar.setup({
        inputField     :    \"edate\", // id of the input field
        ifFormat       :    \"%Y-%m-%d\", // format of the input field
        button         :    \"end_trigger\", // trigger for the calendar (button ID)
        singleClick    :    false,        // double-click mode
        step           :    1             // show all years in drop-down boxes
    });
   </script>\n";

    print "</center></body></html>\n";

    exit;

}
