#! /usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

#  Subject/Summary Attendance Report 
# Get Start date of current term. Calculate all student's attendance
# per subject (and grand total). Display descending list by all students
#  per class or per total. Data comes first from attendance linked to other
#  tables. Attend is definitive, not eval (which may change).

# Note: $lateUnexcused and $absentUnexcused (line 206) are loaded from admin.conf
#  and MUST be set correctly for this to work...

my %lex = ('Subject Attendance Report' => 'Subject Attendance Report',
	   'Attendance' => 'Attendance',
	   'No Term Found' => 'No Term Found',
	   'Make sure that termdates correctly set in the global configuration file' =>
	     'Make sure that termdates correctly set in the global configuration file',
	   'Date' => 'Date',
	   'Student Subject Attendance - by Term' => 'Student Subject Attendance - by Term',
	   'Subject' => 'Subject',
	   'Abs Unex' => 'Abs Unex',
	   'Abs Oth' => 'Abs Oth',
	   'Lates' => 'Lates',
	   'Not Found' => 'Not Found',
	   'Periods per Day' => 'Periods per Day',
	   'Lates equal one Period Absent' => 'Lates equal one Period Absent',
	   'Term' => 'Term',
	   'Red = Withdrawn Student' => 'Red = Withdrawn Student',
	   'Days' => 'Days',
	   'Periods' => 'Periods',
	   'Error' => 'Error',
	   );


my $ppd = 2; # periods per day, used to translate into days missed.
my $latesEquiv = 3; # how many lates = 1 period missed. 

use DBI;
use CGI;
use Date::Business;
use Number::Format qw{round};

# Read config variables
eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my @tim = localtime(time);
my $year = @tim[5] + 1900;
my $month = @tim[4] + 1;
my $day = @tim[3];
if (length($month) == 1){ $month = '0'.$month;}
if (length($day) == 1){ $day = '0'.$day;}
my $currdate = "$year-$month-$day";
my $currsdate = "$year$month$day";
my $currdate1 = "@month[$month] $day, $year";

my $q = new CGI;
my %arr = $q->Vars;
print $q->header( -charset, $charset );

print "$doctype\n<html><head><title>". $lex{'Subject Attendance Report'}. "</title>\n";
print "<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";
print "<style type=\"text/css\">td { text-align:center; }</style>\n";
print "$chartype\n</head><body>[ <a href=\"$attpage\">". $lex{Attendance}. "</a> ]<center>\n";
print "<b>$latesEquiv</b> ". $lex{'Lates equal one Period Absent'}. " &amp; <b>$ppd</b> ". $lex{'Periods per Day'}. "\n"; 

my $date = $currsdate; # Short Date, no hyphens
$dateobj = new Date::Business(DATE=>$date);

# Figure out the current term, if not passed term
my $term;
if (not $arr{term}){  # No Passed Term, figure it out from currdate.
    $term = 0;
    for (1..12){ # 12 is worst case scenario.. what's reasonable?
	my $startdate = $g_termstart{$_}; $startdate =~ s/-//g;
	my $startobj = new Date::Business(DATE => $startdate);
	my $enddate = $g_termend{$_}; $enddate =~ s/-//g;
	my $endobj = new Date::Business(DATE => $enddate);
	$startoffset = $dateobj->diffb($startobj,'prev','next');
	$endoffset = $dateobj->diffb($endobj,'prev','next');
	# print "StartOffset: $startoffset  Endoffset: $endoffset<br>\n";
	
	if ($startoffset >= 0 and $endoffset <= 0){
	    $term = $_; last;
	}
    } # End  of term figure out.
} else { $term = $arr{term}; }

if (not $term){ # No term found! Configuration Error?
    print '<h1>'. $lex{'No Term Found'}. "</h1>\n";
    print '<p>'. $lex{'Make sure that termdates correctly set in the global configuration file'};
    print $lex{Date}. ": $date</p>\n";
    print "</body></html>\n";
    die;
}

print '<h1>'. $lex{'Student Subject Attendance - by Term'}. "</h1>\n";
print '<p>'. $lex{Date}. ": <b>$date</b>\n";
print $lex{Term}. ": <b>$term</b><br>\n";
print "<font color=\"red\">". $lex{'Red = Withdrawn Student'}. "</font></p>\n";

# Do All terms or just this one?
if (not $arr{all}){
    $startdate = $g_termstart{$term};
} else {
    $startdate = $g_termstart{1};
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);


# First find subjects in this term..
my (@subjects, %subjects);
my $sth = $dbh->prepare("select subjsec, description from subject where
 startrptperiod <= ? and endrptperiod >= ? order by description");
$sth->execute($term, $term);
if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr; }
while ( my ( $subjsec, $desc) = $sth->fetchrow ) {
    push @subjects, $subjsec; # @subjects gives us sort order
    $subjects{$subjsec} = $desc;
}


# Now find students in those subjects
my $sth1 = $dbh->prepare("select lastname, firstname from student where studnum = ?");
my $sth2 = $dbh->prepare("select lastname, firstname from studentwd where studnum = ?");

foreach my $subjsec ( @subjects ) {

    my $sth = $dbh->prepare("select distinct studnum from eval where
     subjcode = ? and studnum > 0");
    $sth->execute( $subjsec );
    if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr; }
    while ( my ( $studnum ) = $sth->fetchrow ) {
	# Get name
	$sth1->execute( $studnum );
	my ($lastname, $firstname ) = $sth1->fetchrow;
	$wdflag = 0;
	if (not $lastname) { 
	    $sth2->execute( $studnum );
	    ( $lastname, $firstname ) = $sth2->fetchrow;
	    $wdflag = 1;
	    if (not $lastname) {
		$lastname = $lex{'Not Found'};
	    }
	}
	push @students, "$lastname:$firstname:$studnum:$subjsec:$wdflag";
    }
}
# now have all students in @students array.


print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
print '<tr><th>'. $lex{Subject}. '</th><th>'. $lex{'Abs Unex'}. '</th><th>';
print $lex{'Abs Oth'}. '</th><th>'. $lex{Lates}. "</th><th>". $lex{Periods}. "</th><th>". $lex{Days};
print "</th></tr>\n";

my ($currstud, $prevstud, $totalDays);
$currstud = -1;

my $sth3 = $dbh->prepare("select description, smdesc from subject where subjsec = ?");

# Now loop through all students, printing classes and number missed.
foreach my $stud ( sort @students ) {

    my ($lastname, $firstname, $studnum, $subjsec, $wdflag) = split /:/, $stud;

    # Get Subject Name
    my $description;
    $sth3->execute( $subjsec );
    my ($desc, $smdesc) = $sth3->fetchrow;
    if (not $smdesc) {
	$description = substr($desc, 0, 10);
    } else {
	$description = $smdesc;
    }

    if ( $wdflag ){ # withdrawn students
	$lastname = "<span style=\"color:red;\">$lastname</span>";
	$firstname = "<span style=\"color:red;\">$firstname</span>";
    }

    $prevstud = $currstud;
    $currstud = $studnum;
    if ( $currstud != $prevstud ) { # New student...
	print "<tr class=\"gray\"><td colspan=\"6\">$firstname $lastname ($studnum)</td></tr>\n";
    }

    # Get attendance reasons and count;
    my $sth= $dbh->prepare("select distinct reason, count(reason) from attend
      where subjsec = ? and studentid = ? group by reason");
    $sth->execute( $subjsec, $studnum );
    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }

    my ($totalCount, $absentCount, $lateCount, $otherCount);

    while ( my ($reason, $count) = $sth->fetchrow ) {
	if ( $reason eq $absentUnexcused ) {
	    $absentCount = $count;
	} elsif ( $reason eq $lateUnexcused ) {
	    $lateCount = $count;
	} elsif ( $reason =~ $absentString )  { # one of the other absences.
	    $otherCount += $count;
	}
    }

    $totalCount = $otherCount + int( $lateCount / $latesEquiv ) + $absentCount;

    if ( $ppd ) {
	$dayCount = $totalCount / $ppd;
    }

    $dayCount = round( $dayCount, 2);

    # print stuff
    print "<tr><td>$description ($subjsec)</td><td>$absentCount</td><td>$otherCount</td>\n";
    print "<td>$lateCount</td><td>$totalCount</td><td><b>$dayCount</b></td></tr>\n";
    
}

# Print last of last record
print "<b>$totalDays</b></td></tr>\n";


print "</table><p>[ <a href=\"$attpage\">". $lex{Attendance}. "</a> ]</p>\n";
print "</body></html>\n";
