#! /usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# ======= Subject Attendance Report ======= 
# Get Start date of current term. Calculate all student's attendance
# per subject (and grand total). Display descending list by all students
#  per class or per total. Data comes first from attendance linked to other
#  tables. Attend is definitive, not eval (which may change).

my %lex = ('No Term Found' => 'No Term Found',
	   'Subject Attendance Report' => 'Subject Attendance Report',
	   'Attendance' => 'Attendance',
	   'Main' => 'Main',
	   'Check term start/end dates in configuration file' => 
	     'Check term start/end dates in configuration file',
	   'Current Date' => 'Current Date',
	   'Not Found' => 'Not Found',
	   'Error' => 'Error',
	   'Term' => 'Term',
	   'Current Date' => 'Current Date',

	   );

use DBI;
use CGI;
use Date::Business;


# Read config variables
eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

# Set Dates
my @tim = localtime(time);
my $year = @tim[5] + 1900;
my $month = @tim[4] + 1;
my $day = @tim[3];
if (length($month) == 1){ $month = "0".$month;}
if (length($day) == 1){ $day = "0".$day;}
my $currdate = "$year-$month-$day";
my $currsdate = "$year$month$day";
my $currdate1 = "@month[$month] $day, $year";

my $q = new CGI;
my %arr = $q->Vars;
print $q->header( -charset, $charset );

my $term = $arr{term};

print "$doctype\n<html><head><title>". $lex{'Subject Attendance Report'}. "</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n<style type=\"text/css\">
td { text-align: center;}
</style>$chartype\n</head><body>[ <a href=\"$homepage\">". $lex{Main}. "</a> | 
<a href=\"$attpage\">". $lex{Attendance}. "</a> ]
<center>\n";

my $date = $currsdate; # Short Date, no hyphens
my $dateobj = new Date::Business(DATE=>$date);

# Figure out the current term, if not passed term
if (not $term) {
    for (1..12){ # 12 is worst case scenario.. what's reasonable?
	my $startdate = $g_termstart{$_}; $startdate =~ s/-//g;
	my $startobj = new Date::Business(DATE => $startdate);
	my $enddate = $g_termend{$_}; $enddate =~ s/-//g;
	my $endobj = new Date::Business(DATE => $enddate);
	my $startoffset = $dateobj->diffb($startobj,'prev','next');
	my $endoffset = $dateobj->diffb($endobj,'prev','next');
	#print "StartOffset: $startoffset  Endoffset: $endoffset<br>\n";

	if ($startoffset >= 0 and $endoffset <= 0){
	    $term = $_; last;
	}
    }
}

# if no passed term and no term found...
if ( not $term ){ # No term found!
    print '<h1>'. $lex{'No Term Found'}. ".</h1>\n";
    print $lex{'Check term start/end dates in configuration file'};
    print $lex{'Current Date'}. ": $date</p>\n";
    print "</body></html>\n";
    die;
}

print "<p><b>". $lex{'Current Date'}. ":</b> $currdate1 <b>";
print $lex{Term}. ":</b> $term</p>\n";
my $startdate = $g_termstart{$term};
my $enddate = $g_termend{$term};

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

#print "SD: $startdate ED: $enddate<br>\n";

# First find subjects in current or selected term 
my $sth = $dbh->prepare("select distinct attend.subjsec, subject.description
  from attend
  left outer join subject on attend.subjsec = subject.subjsec
  where to_days(attend.absdate) >= to_days('$startdate') and
  to_days(attend.absdate) <= to_days('$enddate')
  order by subject.grade, subject.description");
$sth->execute;
if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr;}
my $subary = $sth->fetchall_arrayref;

print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";

# Setup "Prepare" for each Student for this subject
my $sth1 = $dbh->prepare("select distinct attend.studentid, 
  student.lastname, student.firstname from attend 
  left outer join student on student.studnum = attend.studentid
  where subjsec = ? order by student.lastname, student.firstname");

# Setup "Prepare" for Student/Subject Attendance getting reasons count.
my $sth2 = $dbh->prepare("select distinct reason, count(reason) from attend 
  where studentid = ? and subjsec = ? group by reason");

my $firstflag = 1;

# Now loop through all subjects, printing students and number missed.
foreach my $subjref (@$subary){
    my $subjsec = $$subjref[0];
    my $description = $$subjref[1];

    # Find reasons for this subject. (could do global, but may be smaller)
    my $sth3 = $dbh->prepare("select distinct reason from attend
      where subjsec = '$subjsec'");
    $sth3->execute;
    if ($DBI::errstr){ print $DBI::errstr; die;}
    my @reasons;
    while (my $reason = $sth3->fetchrow){
	push @reasons,$reason;
    }

    if (not $firstflag) {
	print "</table><p>[ <a href=\"$attpage\">Attendance Page</a> ]</p>\n";
    } else { $firstflag = 0; }

    print "<h2>$description ($subjsec)</h2>\n";
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>Student</th>\n";
    foreach my $rsn (@reasons){
	print "<th>$rsn</th>";
    }
    print "</tr>\n";

    # Get Studnum, Lastname, Firstname for this subject.
    $sth1->execute($subjsec);
    if ($DBI::errstr){ print $DBI::errstr; die;}
    my $studary = $sth1->fetchall_arrayref;

    foreach my $studref (@$studary){ # Loop through all students.
	my $studnum = $$studref[0];
	my $lastname = $$studref[1];
	my $firstname = $$studref[2];

	if (not $lastname){ # look in withdrawn students
	    my $sth3 = $dbh->prepare("select lastname, firstname from studentwd
             where studnum = '$studnum'");
	    $sth3->execute;
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	    ($lastname,$firstname) = $sth3->fetchrow;
	    if (not $lastname){ $lastname = $lex{'Not Found'};}
	    $lastname = "<font color=\"red\">$lastname</font>";
	    $firstname = "<font color=\"red\">$firstname</font>";
	}

	# Get reasons and totals for this student
	$sth2->execute($studnum,$subjsec);
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errtstr;}
	my $rsnary = $sth2->fetchall_arrayref;
	my %reason;
	foreach my $rowref (@$rsnary){ 
	    $reason{$$rowref[0]} = $$rowref[1];
	}

	# Now loop through reasons (NOT from admin.conf, but attend array)
	print "<tr><td>$lastname, $firstname ($studnum)</td>\n";
	foreach my $rsn (@reasons){
	    print "<td>$reason{$rsn}</td>\n";
	}
	print "</tr>\n";
    }

}

print"</table></body></html>\n";
