#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.


my %lex = ('Add Students' => 'Add Students',
	   'Cannot sysopen student number file.' =>  'Cannot sysopen student number file.',
	   'Please click the back button and fill all required fields' => 
 	     'Please click the back button and fill all required fields',
	   'You have missed entering the following <i>required</i> fields' =>
	     'You have missed entering the following <i>required</i> fields',
	   'Cannot rewind student number file' => 'Cannot rewind student number file',
	   'Cannot truncate student number file' => 'Cannot truncate student number file',
	   'Cannot write to student number file' => 'Cannot write to student number file',
	   'Cannot close student number file' => 'Cannot close student number file',
	   'Error! A student record with this student number' => 
	     'Error! A student record with this student number',
	   'already exists! There is a configuration mistake. Please contact your network administrator!' =>
	     'already exists! There is a configuration mistake. Please contact your network administrator!',
	   'Duplicate Student Number Error!' => 'Duplicate Student Number Error!',
	   'Your student record is now stored in the database - Student Number' =>
	     'Your student record is now stored in the database - Student Number',
	   'in the pre-registration table' => 'in the pre-registration table',
	   'There was an error storing your data' => 'There was an error storing your data',
	   'Add Another Student' => 'Add Another Student',
	   'Please contact' => 'Please contact',
	   'Main' => 'Main',
	   'Export' => 'Export',
	   'Initial Enrollment' => 'Initial Enrollment',
	   'Error' => 'Error',

	   );

my $autopwd = '/usr/local/bin/apg';
my $self = 'sentry2.pl';

use DBI;
use CGI;
use Fcntl qw(:DEFAULT :flock);

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}


my $q = new CGI;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;


if ( $arr{description} ) { # We have a description for the transfer
    $description = $arr{description};
} else {
    $description = $lex{'Initial Enrollment'};
}

my $prereg = $arr{prereg};
my $waitlist = $arr{waitlist};

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Page Head
print "$doctype\n<html><head><title>",$lex{'Add Students'},"</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>\n";

#foreach my $key (sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }


# Check for required fields
$sth = $dbh->prepare("select fieldid from meta 
    where tableid = 'student' and required = 'Y'");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
my ($requiredflag, @errorfld);
while ( my $fieldid = $sth->fetchrow ) {
    if ( not $arr{$fieldid} ){
	$requiredflag = 1;
	push @errorfld,$fieldid;
    }
}

if ( $requiredflag ){ # we have missed some required fields
    print "<p style=\"font-size: 150%;\"><b>\n";
    print $lex{'You have missed entering the following <i>required</i> fields'}. ":</b><br><br>\n";
    foreach my $fail ( @errorfld ){
	print "$fail<br>\n";
    }
    print "<br><b>". $lex{'Please click the back button and fill all required fields'};
    print "</b></p>\n</body></html>\n";
    die "$self: Missing Fields\n";
}    



# Get Student number and update file.
sysopen (SNUM, "../../etc/studentnumber", O_RDWR | O_CREAT) or 
  die $lex{'Cannot sysopen student number file.'};

flock(SNUM,LOCK_EX);

$studentnum = <SNUM> || 0;
chomp $studentnum;
seek(SNUM,0,0) or die $lex{'Cannot rewind student number file'}. ": $!\n";
truncate(SNUM,0) or die $lex{'Cannot truncate student number file'}. ": $!\n";

$newstudentnum = $studentnum + 1;
print SNUM $newstudentnum or die $lex{'Cannot write to student number file'}. ": $!\n";
close SNUM or die $lex{'Cannot close student number file'}. ": $! \n";


# Now check for duplicates! (in case of installation mistakes)
$sth = $dbh->prepare("select count(*) from student where studnum = ?");
$sth->execute( $studentnum );
if ($DBI::errstr){ print $DBI::errstr; die "$DBI::errstr \n";}
$count = $sth->fetchrow;
if ( $count ) { 
    print '<h1>'. $lex{'Error! A student record with this student number'}. ": $studentnum ";
    print $lex{'already exists! There is a configuration mistake. Please contact your network administrator!'};
    print "</h1></body></html>";
    die $lex{'Duplicate Student Number Error!'}. "\n";
}

my $table;
if ( $prereg or $waitlist ) {
    $table = "prereg";
} else {
    $table = "student";
}

# Create an entry array called "@fields" storing fieldid values.
$sth = $dbh->prepare("select fieldid from meta
    where tableid = 'student' order by arrayidx");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
while (my $fld = $sth->fetchrow){
    push @fields,$fld;
}

# Populate Hash "%values" with key (ie. fieldid) and value as value
#foreach my $key (keys %arr){
#    #my ($tb,$fld) = split /:/,$key;
#    if ($arr{$key} and $key){ $values{$key} = $arr{$key};}
#    #print "K: $key FLD: $fld VAL: $arr{$key}<br>\n";
#}

# put student number value into values hash
#$values{studnum} = $studentnum;
#$values{utag} = 'ALL'; # force full update

# Add a Password ...

if ( -x $autopwd ) { # APG is installed
    if ( not $arr{password} ) { # add a password if not populated.
	my $password = `$autopwd -n1 -m6 -x6 -M CL`;
	# Note: -M characters, letters,numbers exclude 'ell' and capital 'eye'
	$arr{password} = $password;
    }
}

$arr{studnum} = $studentnum;
$arr{utag} = 'ALL'; # force full update

# Now run through @fields array and repopulate with values from hash.
foreach my $fld (@fields){
    if ( $arr{$fld} ) { # if matching hash value exists, use it.
	$fld = $dbh->quote( $arr{$fld} ). ',';
    } else {
	$fld = "$sql{default},"; # NULL value
    }
}
chop $fields[$#fields]; # chop trailing comma.

my $sth = $dbh->prepare("insert into $table values ( @fields )");
$sth->execute;
if ($DBI::errstr){ print "Table Entry error: $DBI::errstr; $!"; die $DBI::errstr; }


# If not prereg, put in the transfer data also.

$description = $dbh->quote( $description );
$arr{enroldate} = $dbh->quote( $arr{enroldate} );

if (not $prereg and not $waitlist) {  # Don't enter record for pre-registration.

    $arr{firstname} = $dbh->quote( $arr{firstname} );
    $arr{lastname} = $dbh->quote( $arr{lastname} );
    $arr{initial} = $dbh->quote( $arr{initial} );

    $arr{entrytype} = $dbh->quote( $arr{entrytype} );
    $arr{prov} = $dbh->quote( $arr{prov} );
    $arr{country} = $dbh->quote( $arr{country} );
    $arr{provnum} = $dbh->quote( $arr{provnum} );
    $arr{birthdate} = $dbh->quote( $arr{birthdate} );

    $sth2 = $dbh->prepare("insert into transfer values (
      $sql{default}, $arr{studnum}, $arr{enroldate}, 'enrol',
      $description, $arr{entrytype}, $sql{default}, $arr{prov}, $arr{country},
      'Y', $arr{lastname}, $arr{firstname}, $arr{initial},
      $arr{birthdate}, $arr{provnum} )");
    $sth2->execute;
    if ($DBI::errstr) { print "$DBI::errstr\n"; die $DBI::errstr; }
}

# Put in waiting list values...
if ( $waitlist ) {
    my $sth = $dbh->prepare("select max(waitnumber) from prereg_waitlist");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my $maxnum = $sth->fetchrow;
    $maxnum += 10;

    $sth = $dbh->prepare("insert into prereg_waitlist values (
      $sql{default},'$studentnum', $arr{enroldate},'$maxnum', $description )");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

}


if ( not $DBI::errstr ) {
    print '<p><b>'. $lex{'Your student record is now stored in the database - Student Number'};
    print " $studentnum )</b></p>";
    if ($prereg) {
	print '<p><b>'. $lex{'in the pre-registration table'}. "</b></p>\n";
    }
} else {
    print '<p><b>'. $lex{'There was an error storing your data'}. q{ }. $lex{'Please contact'};
    print "$adminname [ <a href=\"mailto:$adminemail\">$adminemail</a> ]<br>\n";
    print "$DBI::errstr</b></p>\n";
}

print "<p>[ <a href=\"sentry0.pl\">". $lex{'Add Another Student'}. "</a> |\n";
print " <a href=\"$homepage\">". $lex{Main}. "</a> | \n";
print " <a href=\"$exppage\">". $lex{Export}. "</a> ]</p></body></html>\n";
