#!/usr/bin/perl
# Copyright Les Richardson 2001-2009

# This file is part of Open Administration for Schools. Released under GPL Licensing.

my %lex = ('Student Field Reset' => 'Student Field Reset',
	   'Main' => 'Main',
	   'Eoy' => 'Eoy',
	   'Continue' => 'Continue',
	   'Select Field to Reset' => 'Select Field to Reset',
	   'Type of Field Selection' => 'Type of Field Selection',
	   'Type in Values' => 'Type in Values',
	   'Select from List' => 'Select from List',
	   'No Field Selected' => 'No Field Selected',
	   'Mistake in Input Type' => 'Mistake in Input Type',
	   'Default Fill - Blank Fields' => 'Default Fill - Blank Fields',
	   'Grade' => 'Grade',
	   'Homeroom' => 'Homeroom',
	   'Student Group List (separated by spaces)' => 
	     'Student Group List (separated by spaces)',
	   'No field to reset' => 'No field to reset',
	   'Blank=All' => 'Blank=All',
	   'Student' => 'Student',
	   'Your update is now stored.' => 'Your update is now stored.',
	   'There was an error storing your data' => 'There was an error storing your data',
	   'Contact' => 'Contact',
	   'Record the following error' => 'Record the following error',
	   'Error' => 'Error',
	   
	   );

my $self = 'resetselect.pl';
my $maxTypeCount = 30; # don't allow more than $maxTypeCount different types in a selection list
my %disallow = qw(studid 1 grade 1 homeroom 1);  


use DBI;
use CGI;

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);


# Show top of page.
print "$doctype\n<html><head><title>". $lex{'Student Field Reset'}. "</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>\n";

print "[ <a href=\"$homepage\">". $lex{Main}. "</a> | \n";
print "<a href=\"$eoypage\">". $lex{Eoy}. "</a> ]\n";

print "<center><h1>". $lex{'Student Field Reset'}. "</h1>\n";


# Select what to do.
if ( not $arr{page} ) {
    showStartPage();
} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    selectChanges();
} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    writeChanges();
}


#----------------
sub showStartPage {
#----------------

    # Read student fields, and defaults into a @fields and %fieldname hash.
    my $sth = $dbh->prepare("select fieldid, fieldname
    from meta where tableid = 'student' order by fieldname");
    $sth->execute;
    my (@fields, %fieldnames);
    while ( ( my $fieldid, $fieldname ) = $sth->fetchrow ) {
	if ( $disallow{$fieldid} ) { next; } # skip disallowed fields to edit
	$fieldname =~ s/\(//g;
	$fieldname =~ s/\)//g; # strip parenthese. (sp?)
	push @fields, $fieldid;
	$fieldname{ $fieldid } = $fieldname;
    }

    # Start the form.
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";
    print "<input type=\"submit\" value=\"". $lex{'Continue'}. "\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\" ";
    print "style=\"background-color:#CCD;\">\n";

    # Display the fields from which to select one.
    print "<tr><td>". $lex{'Select Field to Reset'}. "</td><td>\n";
    print "<select name=\"field\"><option></option>\n";
    foreach my $fld ( @fields ) {
	print "<option>$fieldname{$fld} ($fld)</option>\n";
    }
    print "</select>\n";
    print "</td></tr>\n";

    # Now type of selection: Type in Values, (Individual Text Input) or Select from a List
    print "<tr><td>". $lex{'Type of Field Selection'}. "</td><td>\n";
    print "<input type=\"radio\" name=\"inputtype\" value=\"text\" checked>";
    print $lex{'Type in Values'}. "<br>\n";
    print "<input type=\"radio\" name=\"inputtype\" value=\"select\">";
    print $lex{'Select from List'}. "\n";
    print "</td></tr>\n";

    # Now a group selection by homeroom or grade, accept multiple values separated by spaces...
    print "<tr><td>". $lex{'Student Group List (separated by spaces)'};
    print "</td><td><select name=\"grouptype\">\n";
    print "<option>". $lex{Grade}. "</option><option>". $lex{Homeroom}. "</option></select>\n";
    #print  $lex{'Student Group List (separated by spaces)'};
    print " <input type=\"text\" name=\"groups\" size=\"15\"> ". $lex{'Blank=All'}. "</td></tr>\n";


    # Add a 'Default' fill value for any fields that are blank...
    print "<tr><td>". $lex{'Default Fill - Blank Fields'}. "</td><td>\n";
    print "<input type=\"text\" name=\"defaultfill\" size=\"20\"></td></tr>\n";


    print "</table>\n";
    print "<input type=\"submit\" value=\"". $lex{'Continue'}. "\">\n";
    print "</form></body></html>\n";

    exit; 


} # end of showStartPage


#----------------
sub selectChanges {
#----------------

    #foreach my $key ( keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }


    # Passed: inputtype : 'text' or 'select'
    #         field: fieldname (fieldid)

    my $mode = $arr{inputtype};
    delete $arr{inputtype};

    # Setup field name and fieldid.
    my ( $fieldname, $fieldid ) = split /\(/, $arr{field};
    chop $fieldid; # remove trailing ).
    if ( not $fieldid ) {
	print "<h1>". $lex{'No Field Selected'}. "</h1>\n";
	print "</body><html>\n";
	exit;
    }
    
    #print "Name: $fieldname ID: $fieldid<br>\n";

    # Set Default Fill Value (if any)
    if ( $arr{defaultfill} ) {
	$defaultfill = $arr{defaultfill};
    }

    # Now get this field's metadata
    my $sth = $dbh->prepare("select viewsize, defaultvalue from meta where fieldid = ?");
    $sth->execute( $fieldid );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my ($viewsize, $defaultvalue) = $sth->fetchrow;
    my @defaults = split / /, $defaultvalue;
    if (not @defaults) { # no defaults found; use values present if less than $maxTypeCount
	my $sth1 = $dbh->prepare("select count(distinct $fieldid) from student");
	$sth1->execute;
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $count = $sth1->fetchrow;

	if ( $count < $maxTypeCount ) {
	    my $sth1 = $dbh->prepare("select distinct $fieldid from student 
              where $fieldid is not null and $fieldid != '' order by $fieldid desc");
	    $sth1->execute;
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    while (my $val = $sth1->fetchrow) {
		push @defaults, $val;
	    }
	    @defaults = reverse(@defaults);
	}
    } else { # @defaults exists
	if ( $defaults[0] eq '~' ) { $defaults[0] = ''; } # tilde signals blank start field.
    }


    # Get Student Information
    $sth = $dbh->prepare("select studid, lastname, firstname, studnum, $fieldid from student
      order by lastname, firstname");
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }


    # Start the Form...
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"2\">\n";
    print "<input type=\"hidden\" name=\"resetfield\" value=\"$fieldid\">\n";
    print "<input type=\"submit\" value=\"". $lex{'Continue'}. "\">\n";
    print "<table cellpadding=\"4\" cellspacing=\"0\" border=\"1\" style=\"background-color:#CCD;\">\n";
    print "<tr><th>". $lex{Student}. "</th><th><b>$arr{field}</b></th></tr>\n";


    # Loop through all students
    while ( my ( $studid, $lastname, $firstname, $studnum, $resetfield ) = $sth->fetchrow){

	# If no field in current record, and we have default fill set, then put it in...
	if ( (not defined $resetfield or $resetfield eq '') and defined $defaultfill ) {
	    $resetfield = $defaultfill;
	}

	print "<tr><td>$firstname $lastname ($studnum)</td><td>\n";

	if ( $mode eq 'text' ) {
	    print "<input type=\"text\" name=\"$studid\" value=\"$resetfield\" size=\"$viewsize\">";
	} elsif ( $mode eq 'select' ) {
	    print "<select name=\"$studid\"><option>$resetfield</option>\n";
	    # Now print all values...
	    foreach my $def ( @defaults ) {
		print "<option>$def</option>";
	    }
	    print "</select>\n";

	} else {
	    # Fail very badly...
	    print "<h1>". $lex{'Mistake in Input Type'}. "</h1>\n";
	    exit;

	}

	print "</td></tr>\n";


    } # End of Student Loop


    print "</table><input type=\"submit\" value=\"". $lex{'Continue'}. "\">\n";
    print "</form></body></html>\n";

    exit;

} # end of selectChanges




#---------------
sub writeChanges {
#---------------

    #foreach my $key (keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    # first get field name to update
    if ( $arr{resetfield} ) {
	$resetfield = $arr{resetfield};
	delete $arr{resetfield};
    } else {
	print "<h1>". $lex{'No field to reset'}. "</h1>\n";
	exit;
    }


    foreach my $key (keys %arr ) { 
	$sth = $dbh->prepare("update student set $resetfield = ? where studid = ?");
	$sth->execute( $arr{$key}, $key );
	if ($DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    }


    if (not $DBI::errstr ) {
	print '<p>'. $lex{'Your update is now stored.'}. "</p>";
    } else {
	print "<h3>". $lex{'There was an error storing your data'}. "\n";
	print $lex{Contact}. " $adminname at <a href=\"mailto:$adminemail\">$adminemail</a>\n";
	print $lex{'Record the following error'}. ": $DBI::errstr \n";
    }
    print "[ <a href=\"$homepage\">". $lex{Main}. "</a> | <a href=\"$eoypage\">". $lex{Eoy};
    print "</a> ]</p></body></html>\n";

    exit;

} # end of writeChanges
