#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.


# Outline: Display stored formats (.fmt) files.
#  Also display all student fields to select. 
#  

my %lex = ('Main' => 'Main',
	   'Export' => 'Export',
	   'Export Student Table' => 'Export Student Table',
	   'Field' => 'Field',
	   'Select' => 'Select',
	   'Combine failed on input' => 'Combine failed on input',
	   'Download CSV File' => 'Download CSV File',
	   'CSV (Comma Separated Value) can be directly imported into Open Office' =>
	     'CSV (Comma Separated Value) can be directly imported into Open Office',
	   'Note' => 'Note',
	   'Save' => 'Save',
	   'No student fields selected!' => 'No student fields selected!',
	   'Cannot open Export file' => 'Cannot open Export file',
	   'Error' => 'Error',
	   
	   );

my $self = 'expstudent.pl';

use DBI;
use CGI;
use Text::CSV_XS;

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

my $csv = Text::CSV_XS->new( {binary => 1} );

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# load all the fields of the student table
my @fields;
$sth = $dbh->prepare("select fieldid,arrayidx from meta
  where tableid = 'student' and fieldid != '' order by arrayidx");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
while ( my ($field,$idx) = $sth->fetchrow){
    push @fields, "$field:$idx";
}


print "$doctype\n<html><head><title>". $lex{'Export Student Table'}. "</title>
<link rel=\"stylesheet\" type=\"text/css\" href=\"$css\">
$chartype\n</head><body>\n";

print "[ <a href=\"$homepage\">". $lex{Main}. "</a> | <a href=\"$exppage\">";
print $lex{Export}. "</a> ]\n";
print "<center><h1>". $lex{'Export Student Table'}. "</h1></center>\n";


if ($arr{writeflag}){
    delete $arr{writeflag};
    createExport();
}

print "<form action=\"$self\" method=\"post\">\n";
print "<input type=\"hidden\" name=\"writeflag\" value=\"1\">\n";

# Sort Order # 1
print "<div style=\"position:absolute; padding: 1em; margin-top: 0; ";
print "margin-left:40%; border: 1px solid gray; background-color: #BDF;\">
<b>Record Sort</b>
<p>First Sort by: <select name=\"sort1\"><option></option>\n";
foreach my $fld ( @fields ){
    my ($field,$idx) = split /:/,$fld;
    print "<option>$field</option>";
}
print "</select></p>\n";


# Sort Order # 2
print "<p>Then Sort by: <select name=\"sort2\"><option></option>\n";
foreach my $fld ( @fields ){
    my ($field,$idx) = split /:/,$fld;
    print "<option>$field</option>";
}
print "</select></p>\n";


# Sort Order # 3
print "<p>Finally Sort by: <select name=\"sort3\"><option></option>\n";
foreach my $fld ( @fields ){
    my ($field,$idx) = split /:/,$fld;
    print "<option>$field</option>";
}
print "</select></p></div>\n";


print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
print "<tr><th>". $lex{Field}. "</th><th>". $lex{Select}. "</th></tr>\n";
print "<tr><td colspan=\"2\" align=\"center\">\n";
print "<input type=\"submit\" value=\"". $lex{Export}. "\">";
#print "<input type=\"submit\" name=\"save\" value=\"". $lex{Save}. "\">";
print "</td></tr>\n";


foreach my $fld (@fields){
    my ($field,$idx) = split /:/,$fld;
    print "<tr><td>$field</td><td><input type=\"checkbox\" name=\"$fld\" ";
    print "value=\"1\"></td></tr>\n"; 
}

print "<tr><td colspan=\"2\" align=\"center\">\n";
print "<input type=\"submit\" value=\"". $lex{Export}. "\">";
print "</td></tr></table>\n";

print "</form></body></html>\n";



#-------------
sub saveFormat {  # save Export Format for re-use.
#-------------

}

#---------------
sub createExport {  # create exported data for download
#---------------

    #foreach my $key (keys %arr) { print "K:$key V:$arr{$key}<br>\n"; }

    # Sort by 'sortorder'
    my $sortorder;
    for (1..3){
	my $sortname = 'sort'.$_;
	if ($arr{$sortname}){
	    $sortorder .= "$arr{$sortname},";
	}
	delete $arr{$sortname};
    }
    chop $sortorder; # remove trailing comma.
    if ($sortorder){ # if we have any sortorder, add first part.
	$sortorder = "order by $sortorder";
    }

    # Check for missing selected fields.... duh.
    if (not %arr) { # no values passed...
	print "<b>". $lex{'No student fields selected!'}. "</b>\n";
	die $lex{'No student fields selected!'}. "\n";
    }


    # Open output file
    $fileName = "expstudent$$.csv";
    open (EX,">$fileName") || die $lex{'Cannot open Export file'}. " $fileName";


    # build select string and array for reading.
    my $select;
    my @expfields;
    # Create the array
    foreach $key (keys %arr){
	my ($fld,$idx) = split /:/,$key;
	$expfields[$idx] = $fld;
    }

    # Now create the select
    foreach $val (@expfields){
	if (not $val){ next;}
	$select .= "$val,";
    }
    chop $select; # remove trailing comma


    # Get count
    $sth = $dbh->prepare("select count(*) from student");
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
    my $rows = $sth->fetchrow;

    # Get Records
    $sth = $dbh->prepare("select $select from student $sortorder");
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}

    for ($i=1; $i <= $rows; ++$i) {
	my @arr = $sth->fetchrow;
	
	foreach $element (@arr) {  # strip CR/LF
	    $element =~ s/\n/ /g;
	    $element =~ s/\r/ /g;
	}  

	if ($csv->combine(@arr)) {
	    my $record = $csv->string;
	    print EX $record, "\r\n";
	} else {
	    my $err = $csv->error_input;
	    print $lex{'Combine failed on input'}. ": $err\n\n";
	}
    }

    close EX;
    system("mv $fileName $downloaddir");

    print "<center><p><a href=\"$webdownloaddir/$fileName\">";
    print "<font size=+2>". $lex{'Download CSV File'}. "</font></a></p>\n";

    print "<p><strong>". $lex{Note}. ":</strong>\n";
    print $lex{'CSV (Comma Separated Value) can be directly imported into Open Office'};
    print "</p></center></body></html>\n";

    exit;

} # End of sub.
