#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.


my %lex = ('Print Invoices' => 'Print Invoices',
	   'Invoice Date' => 'Invoice Date',
	   'Main' => 'Main',
	   'Fees' => 'Fees',
	   'Name' => 'Name',
	   'Description' => 'Description',
	   'Amount' => 'Amount',
	   'Date' => 'Date',
	   'Continue' => 'Continue',
	   'Id' => 'Id',
	   'Taxes' => 'Taxes',
	   'Total' => 'Total',
	   'Total Owing' => 'Total Owing',
	   'Printed on' => 'Printed on',
	   'No Outstanding Transaction(s) Found' => 'No Outstanding Transaction(s) Found',
	   'View Log File' => 'View Log File',
	   'View/Download' => 'View/Download',
	   'Invoices' => 'Invoices',
	   'Error' => 'Error',

	   );

my $self = 'invoice.pl';


use CGI;
use DBI;
use Cwd;
use Number::Format qw(round);


# calc current date
my @tim = localtime(time);
my $year = @tim[5] + 1900;
my $month = @tim[4] + 1;
my $day = @tim[3];
my $currdate = "$year-$month-$day";

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;

my $checked = $arr{checked};
delete $arr{checked};

eval require "../../etc/fees.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Get current dir so know what CSS to display;
#if (getcwd() =~ /tcgi/){ # we are in tcgi
#    $css = $tchcss;
#}

# Show page Header
print "$doctype\n<html><head><title>". $lex{'Print Invoices'}. "</title>\n";
print "<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";
print "$chartype\n</head><body>[ <a href=\"$homepage\">". $lex{Main}. "</a> |\n";
print "<a href=\"/fees.html\">". $lex{Fees}. "</a> ]\n";

print "<center><h1>". $lex{'Print Invoices'}. "</h1>\n";

if ( not $arr{page} ) {
    showStartPage();
} else {
    printInvoices();
}




#----------------
sub showStartPage {
#----------------

    # Setup the form and start of table.
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";
    print "<table cellpadding=\"3\" border=\"0\" cellspacing=\"0\">\n";
    

    print "<tr><td align=\"right\">". $lex{'Invoice Date'}. "</td><td align=\"left\">\n";
    print "<input type=\"text\" name=\"date\" size=\"12\" value=\"$currdate\">\n";
    print "</td></tr>\n";

    print "<tr><td></td><td>\n";
    print "<input type=\"submit\"  value=\"". $lex{Continue}. "\">";
    print "</td></tr>\n";

    print "</table>";
    print "</form>\n";

    print "</body></html>\n";
    
    exit;

}


#----------------
sub printInvoices {
#----------------

    my $shortname = "rptinvoice$$";
    my $filename = "$shortname.tex";


    # Find outstanding transactions / invoices
    my $sth = $dbh->prepare("select * from fees_jrl
      where paid_id is null and ( trans_type = 'chg' or trans_type = 'roa') 
      order by studnum, trans_date desc");
    $sth->execute;
    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
    $rows = $sth->rows;

    if ($rows < 1){ # No outstanding transactions found
	print '<h1>'. $lex{'No Outstanding Transaction(s) Found'}. "!</h1>\n";
	print "[ <a href=\"$homepage\">". $lex{Main}. "</a> ]\n"; 
	print "</body></html>\n";

	exit;
    }


    # Start the TeX file...
    open(TEX,">$filename") || die "Can't open tex file";
    # Letter paper size is 279mm x 213mm
    # 200mm wide - 50mm name, 20mm extra fld, leaves 130mm for columns.
    # The tabcolsep of 1mm leaves 128 mm where each col is $width + 1mm wide.

    print TEX "\\documentclass[12pt,letterpaper]{article}
\\usepackage{array,newcent,rotating,colortbl}
\\pagestyle{empty}
\\setlength{\\textwidth}{200mm}
\\setlength{\\textheight}{270mm}
\\setlength{\\hoffset}{-1.4in}
\\setlength{\\voffset}{-1.4in}
\\addtolength{\\evensidemargin}{0in}
\\addtolength{\\oddsidemargin}{0in}
\\setlength{\\tabcolsep}{1mm}
\\setlength{\\extrarowheight}{2mm}
\\newcolumntype{G}{>{\\columncolor[gray]{1.00}}p{$namewidth mm}}\n";
# Note above... a columncolor of 1.00 means white, smaller number is darker.

    print TEX "\\begin{document}\n";

    
    $currstudnum = -1;
    $first = 1;
    my $studenttotal;

    # Prepare for student name search
    my $sth1 = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");
    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }

    while ( my ($id, $studnum, $trans_date, $trans_type, $name, $description, $subtotal, 
		$tax1, $tax1_name, $tax2, $tax2_name, $tax3, $tax3_name, $tax4, $tax4_name, $total, 
		$paid_id) = $sth->fetchrow ) {

	$prevstudnum = $currstudnum;
	$currstudnum = $studnum;

	# Pad out id length; $padding_digits comes from fees.conf
	while ( length($id) < $padding_digits ) {
	    $id = '0'. $id;
	}
    
	if ( $currstudnum != $prevstudnum ) {
	    if ( not $first ) { # Do end of current invoice.
		$studenttotal = round( $studenttotal, 2);

		print TEX " & & &  \\multicolumn{3}{r|}{\\bf ". $lex{'Total Owing'};
		print TEX "\\quad $studenttotal \\quad} \\\\ \\hline\n";
		print TEX "\\end{tabular}\n\\end{center}\\newpage\n\n";

	    } else { $first = 0; }

	    # Get Student Name
	    $sth1->execute( $currstudnum );
	    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
	    my ($lastname, $firstname) = $sth1->fetchrow;

	    # Now do start of next page...
	    print TEX "\\begin{center}\n{\\Large $schoolname}\n\n\\bigskip\n";
	    print TEX "{\\bf $firstname $lastname}\n\n\\bigskip\n";

	    print TEX $lex{'Printed on'}. " $currdate\n\n\\medskip\n";

	    print TEX "\\begin{tabular}{|p{0.55in}|p{0.87in}|p{3.2in}|p{0.7in}|p{0.7in}|p{0.8in}|}\\hline\n";
	    print TEX "\\rowcolor[gray]{0.92} \\hfil ". $lex{Id}. " &\\hfil ". $lex{Date}. "\\hfil & \\hfil ";
	    print TEX $lex{Name}. " / ". $lex{Description};
	    print TEX "\\hfil & \\hfil ". $lex{Amount}. " \\hfil & \\hfil ". $lex{Taxes}. " \\hfil & \\hfil ";
	    print TEX $lex{Total}. "\\hfil\\\\ \\hline\n";
	    
	    $studenttotal = 0;

	}
	
	$studenttotal += $total;

	# print normal transaction record.
	print TEX "$id & $trans_date & {\\it $name} - $description & \\hfill $subtotal ";

	# print tax values
	#if (@tax) { # from fees.conf; taxes defined
	 #   for my $i (1..$#tax) {
		#print "I:$i ";
		#my $fld = 'tax'. $i;
		#my $fldname = $fld. '_name';
		#print TEX '&'. $$fld. ' ';
	    #}
        #}
	print TEX "& \\hfill $tax1 "; # instead

	# print end of record.
	print TEX "& \\hfill $total \\\\ \\hline\n";

    }

    # print end of last record totals.
    print TEX " & & &  \\multicolumn{3}{r|}{\\bf ". $lex{'Total Owing'};
    print TEX "\\quad $studenttotal \\quad} \\\\ \\hline\n";
    
    print TEX "\\end{tabular}\\\\ \n \\end{center}\\end{document}";
    close TEX;


    system("$pdflatex $filename >pdflog$$.txt");
    system("mv $shortname.pdf $downloaddir");
    system("mv pdflog$$.txt $downloaddir");
    system("rm -f $shortname.*");
    
    print "<h1><a href=\"$webdownloaddir/$shortname.pdf\">";
    print $lex{'View/Download'}. ' ', $lex{'Invoices'}. "</a></h1>\n";
    print "<p>[ <a href=\"$feespage\">". $lex{Fees}. "</a> |\n <a href=\"$webdownloaddir/pdflog$$.txt\">";
    print $lex{'View Log File'}. "</a>\n ]</p></center></body></html>\n";

    exit;

}


#-------------------
sub printHtmlInvoice {
#-------------------
    my $studnum = shift; # passed student number.



}

