#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Student Fees Outstanding' => 'Student Fees Outstanding',
	   'Main' => 'Main',
	   'No Outstanding Transaction(s) Found' => 'No Outstanding Transaction(s) Found',
	   'Date' => 'Date',
	   'Description' => 'Description',
	   'Amount' => 'Amount',
	   'Grand Total' => 'Grand Total',
	   'Family Total' => 'Family Total',
	   'Total' => 'Total',
	   'Fees' => 'Fees',
	   'Error' => 'Error',
	   'Phone' => 'Phone',
	   'Error' => 'Error',

	   );

use DBI;
use CGI;
use Number::Format qw(round);

# Extra fields reported for each student...
my @fields = ('parent1','address1','hphone1');

# Read config variables
eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    print $lex{Error}. " $@<br>\n";
}

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;


my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, 
    $iddst) = localtime(time);
$year = $year + 1900;
$mon++;
$wday++;
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";


my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);


print "$doctype\n<html><head><title>". $lex{'Student Fees Outstanding'}. "</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>[ <a href=\"$homepage\">". $lex{Main}. "</a> |\n";
print "<a href=\"$feespage\">". $lex{Fees}. "</a> ]\n";

print "<center><h1>". $lex{'Student Fees Outstanding'}. "</h1><p><b>$currdate</b></p>\n";

# First, find distinct student numbers in outstanding invoices.
my @outstanding;
my $sth = $dbh->prepare("select distinct studnum from fees_jrl where
 paid_id is null and ( trans_type = 'chg' or trans_type = 'roa') ");
$sth->execute;
if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
while ( my $studnum = $sth->fetchrow ) {
    push @outstanding, $studnum;
}

if ( not @outstanding ){ # No outstanding transactions found
    print '<h1>'. $lex{'No Outstanding Transaction(s) Found'}. "!</h1>\n";
    print "[ <a href=\"$homepage\">". $lex{Main}. "</a> ]\n"; 
    print "</body></html>\n";
    
    die;
}

# Second, get student info from this outstanding list, including name, hphone1 (famnum)

# Create a famlist hash (really hphone1), to identify the families, if
# any. Then go through the students by lastname, firstname and push
# values from hash into an array and remove from hash. This will give
# us a sorted list of families, roughly by name. By using the famnum
# values we can get the studnum sequence to go through for the
# transaction printing order.

$sth = $dbh->prepare("select lastname, firstname, hphone1 from studentall 
  where studnum = ?");

my %family_ref;
foreach my $studnum ( @outstanding ) {
    $sth->execute( $studnum );
    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
    my ( $lastname, $firstname, $hphone1 ) = $sth->fetchrow;

    $famlist{ $hphone1 } = 1;  # unique unsorted family list
    $namelist{"$lastname, $firstname"} = $hphone1; #
    
    if (not @{ $family_ref{$hphone1} }) {
	$family_ref{$hphone1} = [ "$lastname:$firstname:$studnum" ];
    } else {
        push @{ $family_ref{$hphone1} }, "$lastname:$firstname:$studnum";
    }
}


# Now go through name list and create family ordering array ( @sortedfamily )
my @sortedfamily;
foreach my $name (sort keys %namelist ) {
    if ($famlist{ $namelist{$name} }) {
	push @sortedfamily, $namelist{$name};
	delete $famlist{$namelist{$name}};
    }
}

# Print Table Header
print "<table border=\"1\" cellpadding =\"3\" cellspacing=\"0\">\n";
print "<tr><th>". $lex{Date}. "</th><th>". $lex{Description}. "</th><th>";
print $lex{Amount}. "</th></tr>\n";

$firstfamily = 1;
my $grandtotal;


# Now loop through all families by student, showing transactions
foreach my $famnum ( @sortedfamily ) {

    $prevfam = $currfam;
    $currfam = $famnum;
    if ( $currfam ne $prevfam and not $firstfamily ) {
	#print family totals and zero family counter;
	$familytotal = round( $familytotal, 2);
	print "<tr style=\"background-color:#888; color:white;\">";
	print "<td colspan=\"2\" align=\"right\"><b>";
	print $lex{Phone}. " $famnum&nbsp;&nbsp;&nbsp;". $lex{'Family Total'};
	print "</b></td><td>$familytotal</td></tr>\n";
	$familytotal = 0;
    } else {
	$firstfamily = 0;
    }

    foreach my $stud ( @{ $family_ref{$famnum} } ) {
	my ( $lastname, $firstname, $studnum ) = split /:/, $stud;

	# Find outstanding transactions
	$sth = $dbh->prepare("select trans_date, name, description, total from fees_jrl
          where paid_id is null and ( trans_type = 'chg' or trans_type = 'roa') and
          studnum = ? order by trans_date desc");
	$sth->execute( $studnum );
	if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }

	# Loop through all outstanding student values
	my $studenttotal;
	while ( my ( $trans_date, $name, $description, $total) = $sth->fetchrow ) {
	    $studenttotal += $total;
	    $familytotal += $total;
	    $total = round( $total, 2);
	    print "<tr><td>$trans_date</td><td align=\"right\">"; 
	    if ($name ) { print $name; }
	    if ( $name and $description ) { print ' - '; }
	    if ( $description ) { print $description; } 
	    print " ($studnum)</td><td>$total</td></tr>\n"; 
	}

	$studenttotal = round( $studenttotal, 2);
	print "<tr style=\"background-color:#DDD;\">\n";
	print "<td colspan=\"2\" align=\"right\">";
	print "<b>$lastname</b>, $firstname ($studnum) <b>". $lex{Total};
	print "</b></td><td>$studenttotal</td></tr>\n";
	$grandtotal += $studenttotal;

    } # end of Student

} # End of Family

# Print final family total;
$familytotal = round( $familytotal, 2);
print "<tr style=\"background-color:#888; color:white;\">\n";
print "<td colspan=\"2\" align=\"right\"><b>";
print $lex{'Family Total'}. "</b></td><td>$familytotal</td></tr>\n";

# Print Grand Total		
print "<tr style=\"background-color:#444; color:white;\">\n";
print "<td colspan=\"2\" align=\"right\"><b>";
print $lex{'Grand Total'}. "</b></td><td>\n";
print round( $grandtotal, 2);
print "</td></tr>\n";

print "</table></center></body></html>\n";
