#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# evaldelstud.pl - delete subject enrollments/gradebook records by student.

# NOTE: Gradebook (tests and scores) deletion is commented out! It will
# not erase these. This will not cause any problems since withdrawn
# students will not show up in gradebook...


my %lex = ('Student Subject Withdrawal' => 'Student Subject Withdrawal',
	   'Please search again' => 'Please search again',
	   'Main' => 'Main',
	   'Report Card' => 'Report Card',
	   'Student Subject Enrollment Search' => 'Student Subject Enrollment Search',
	   'Last,First/Last/Initials/Studnum' => 'Last,First/Last/Initials/Studnum',
	   'Subject' => 'Subject',
	   'Withdraw' => 'Withdraw',
	   'No Student(s) Found' => 'No Student(s) Found',
	   'Withdraw from selected subjects' => 'Withdraw from selected subjects',
	   'No subject enrollments found' => 'No subject enrollments found',
	   'Search' => 'Search',
	   'Teacher' => 'Teacher',
	   'Student' => 'Student',
	   'Error' => 'Error',
	   'Enrollment' => 'Enrollment',
	   'Deleted' => 'Deleted',

	   );

my $self = 'evaldelstud.pl';

use DBI;
use CGI;

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;
my $student = $arr{student};

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);


# Print Html Head
print "$doctype\n<html><head><title>". $lex{'Student Subject Withdrawal'}. "</title>
 <link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
 $chartype\n</head><body>[ <a href=\"$homepage\">". $lex{Main}. "</a> | 
 <a href=\"$reppage\">". $lex{'Report Card'}. "</a> ]
 <center><h1>". $lex{'Student Subject Withdrawal'}. "</h1>\n";

if ( not $arr{page} ) {
    mkSearchError();
} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    searchStudent();
} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    deleteEvalRecords(); 
}



#----------------
sub searchStudent {
#----------------

    # Setup the Search
    if ( $student =~ /\d+/ ) {  # we have a student number
	$studentnumber = $student;
	$sth = $dbh->prepare("select lastname, firstname, studnum 
          from studentall where studnum = ?");
	$sth->execute( $student );
    } else { # we have words hopefully with a comma
	($lastname,$firstname)  = split /\,/, $student;
	$firstname =~ s/^\s*//;
	$lastname =~ s/^\s*//;
	if ( $lastname and $firstname ) { # both entered.
	    $sth = $dbh->prepare("select lastname, firstname, studnum 
              from studentall where lastname = ? and firstname = ?");
	    $sth->execute( $lastname, $firstname );

	} elsif ($lastname and not $firstname){ # only lastname (no comma)
	    if (length($lastname) == 2){ # search by initials: fi, li.
		$fi = substr($lastname,0,1). '%'; $li = substr($lastname,1,1). '%';
		$sth = $dbh->prepare("select lastname, firstname, studnum 
                 from studentall where lastname $sql{like} ? and firstname $sql{like} ?");
		$sth->execute( $li, $fi );
	    } else {
		$sth = $dbh->prepare("select lastname, firstname, studnum 
                 from studentall where lastname = ? order by firstname");
		$sth->execute( $lastname );
	    }
	} else { # print an error....
	    print "<h1>". $lex{'No Student(s) Found'}. "</h1>\n";
	    mkSearchError();
	}

    } # Last Else

    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my $rows = $sth->rows;

    if ($rows < 1) { 
	print "<h1>". $lex{'No Student(s) Found'}. "</h1>\n";
	mkSearchError();
    } 

    for ( 1..$rows ) {
	my ($ln,$fn,$sn) = $sth->fetchrow;
	push @students, "$ln:$fn:$sn"; 
    }

    foreach  my $s ( sort @students ) {
	my ( $lastname, $firstname, $studnum) = split /:/, $s;
	
	print "<h1>$firstname $lastname ($studnum)</h1>\n";
	print "<form action=\"$self\" method=\"post\">\n";
	print "<input type=\"hidden\" name=\"page\" value=\"2\">\n";
	print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
	print "<tr><th>". $lex{Subject}. "</th><th>". $lex{Teacher}. "</th>\n";
	print "<th>". $lex{Withdraw}. "</th>\n";

	$sth = $dbh->prepare("select distinct e.subjcode, s.description, s.teacher
          from eval as e, subject as s  where e.studnum = ? and e.subjcode = s.subjsec
          order by s.description");

	$sth->execute( $studnum );
	if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
	my $rows = $sth->rows;

	if ( $rows < 1 ){ # No eval records found!
	    print "<tr><td colspan=\"3\"><b>". $lex{'No subject enrollments found'};
	    print ".</b></td></tr>\n";
	    print "</table></form>\n";
	    next; # student
	} 

	# print Submit button
	print "<tr><td colspan=\"3\" align=\"center\"><input type=\"submit\" ";
	print "value=\"". $lex{'Withdraw from selected subjects'}. "\"></td></tr>\n";

	for ( 1..$rows ){ # Loop through all subjects

	    my ($subjsec, $desc, $teacher) = $sth->fetchrow;
	    print "<tr><td>$desc ($subjsec)</td><td>$teacher</td>\n";
	    print "<td align=\"center\"><input type=\"checkbox\" ";
	    print "name=\"$studnum:$subjsec\" value=\"1\"></td></tr>\n";
	}

	print "<tr><td colspan=\"3\" align=\"center\"><input type=\"submit\" ";
	print "value=\"". $lex{'Withdraw from selected subjects'}. "\"></td></tr>\n";
	print "</table></form><p></p>\n";


    } # end of student loop

    mkSearchError();
    print "</body></html>\n";

    exit;

} # end of searchStudent;



#----------------
sub mkSearchError {
#----------------

    print "<div style=\"border: 1px solid gray;padding:1em;margin:1em;width:400px;\">\n";
    print "<b>". $lex{'Student Subject Enrollment Search'}. "</b><br>\n";

    print "<form action=\"$self\" method=\"post\" style=\"display:inline;\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";
    print "<input type=\"submit\" value=\"". $lex{Search}. "\">\n";
    print "<input type=\"text\" name=\"student\" size=\"30\"><br>\n";
    print $lex{Student}. " (". $lex{'Last,First/Last/Initials/Studnum'}. ")\n";
    print "</form></div>\n";

    print "</body></html>\n";

    exit;

}


#--------------------
sub deleteEvalRecords { # Delete eval records and gradebook records
#--------------------

    #foreach my $key ( keys %arr ){ print "K:$key V:$arr{$key}<br>\n"; }

    # Loop through all passed values, and delete 
    foreach my $key ( keys %arr ){

	my ($studnum, $subjsec) = split /:/,$key;
	if ( $studnum and $subjsec ) {
	    my $sth = $dbh->prepare("delete from eval 
             where studnum = ? and subjcode = ?");
	    $sth->execute($studnum, $subjsec);
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

	    print "<div>". $lex{Enrollment}. q{ }. $lex{Deleted}. "</div>\n";

	    # Find the gradebook tests and delete student from each in turn.
	    $sth = $dbh->prepare("select id from gbtest where subjsec = ?");
#	    $sth->execute( $subjsec );
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    while ( my $testid = $sth->fetchrow ) {

		#print "TID: $testid<br>\n"; # remove along with other comments

		my $sth1 = $dbh->prepare("delete from gbscore
                 where studnum = ? and  testid = ?");
#		#$sth1->execute( $studnum, $testid );
		if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    }

	}
    }

    mkSearchError();

}
