#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

#  Description: Grade based subject enrollment. List all the students
#   for all the subjects in that grade. Deselect those that you don't
#   have in those subjects.

my %lex = ('Main' => 'Main',
	   'Report Card' => 'Report Card',
	   'Move Evaluations' => 'Move Evaluations',
	   'Last,First/Last/Initials/Studnum' => 
	     'Last,First/Last/Initials/Studnum',
	   'Student' => 'Student',
	   'No Student(s) Found' => 'No Student(s) Found',
	   'Checked' => 'Checked',
	   'Continue' => 'Continue',
	   'Current' => 'Current',
	   'Move To' => 'Move To',

	   );

my $self = 'evalmove.pl';

use DBI;
use CGI;

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Print Page Header
print "$doctype\n<html><head><title>". $lex{'Move Evaluations'}. "</title>\n";
print "<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";
print "$chartype\n</head><body>\n";

print "[ <a href=\"$homepage\">". $lex{Main}. "</a> |\n";
print "<a href=\"$reppage\">". $lex{'Report Card'}. "</a> ]\n";

print "<center><h1>". $lex{'Move Evaluations'}. "</h1>\n";

if ( not $arr{page} ) {
    showStartPage();
} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    selectSubject();
} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    moveEval();
}



#----------------
sub showStartPage {
#----------------

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    #print "<table border=\"0\" cellpadding=\"3\" cellspacing=\"0\">\n";

    #print "<tr><td align=\"right\">";
    print $lex{Student}. ' (';
    print $lex{'Last,First/Last/Initials/Studnum'}. ") "; #</td>\n<td align=\"left\">";
    print "<input type=\"input\" name=\"student\" size=\"20\">\n"; #</td></tr>\n";

    #print "<tr><td align=\"right\">";
    #print $lex{Checked}. "?</td>\n<td>";
    #print "<input type=\"checkbox\" name=\"studcheck\" value=\"CHECKED\"></td></tr>\n";

    #print "</table>\n";
    print "<input type=\"submit\" value=\"". $lex{Continue}. "\">\n";
    print "</form></body></html>\n";

    exit;

}



#-----------------
sub selectSubject { # select student subjects to move.
#-----------------

    #foreach my $key ( sort keys %arr) { print "K:$key V:$arr{$key}<br>\n"; }

    # Outline: Find the student(s), find their subjects, find matching ones.
    #  Pass the changes onto moveEval;

    my $student = $arr{student};
    my $sth;

    if ( $student =~ /^\d+/) {  # we have a student number
	$sth = $dbh->prepare("select studnum from studentall where studnum = ?");
	$sth->execute( $student ) ;
	
    } else { # we have words hopefully with a comma
	($lastname,$firstname)  = split /\,/, $student;
	$firstname =~ s/^\s*//;
	$lastname =~ s/^\s*//;
	if ( $lastname and $firstname ){ # both entered.
	    $sth = $dbh->prepare("select studnum from studentall 
               where lastname = ? and firstname = ?");
	    $sth->execute( $lastname, $firstname ) ;
	    
	} elsif ($lastname and not $firstname){ # only lastname (no comma)
	    if ( length($lastname) == 2 ){ # search by initials: fi, li.
		my $fi = substr($lastname,0,1). '%';
		my $li = substr($lastname,1,1). '%';
		$sth = $dbh->prepare("select studnum from studentall 
                  where lastname $sql{like} ? and firstname $sql{like} ?");
		$sth->execute( $li, $fi ) ;

	    } else {
		$sth = $dbh->prepare("select studnum
                  from studentall where lastname = ?");
		$sth->execute( $lastname ) ;
	    }
	} 
	
    } # end of character search

    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my $rows = $sth->rows;

    if ( $rows < 1 ) { # student not found
	print '<p><b>'. $lex{'No Student(s) Found'}. ".</b></p>\n";
	showStartPage();
    }

    my $sth1 = $dbh->prepare("select distinct subjcode from eval where studnum = ?");
    my $sth2 = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");
    my $sth3 = $dbh->prepare("select description, teacher from subject where subjsec = ?");
    my $sth4 = $dbh->prepare("select subjsec from subject where subjcode = ?");

    # Start Form
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"2\">\n";

    while ( my $studnum = $sth->fetchrow ) { # Display his/her subject and matching ones.

	# Get Name
	$sth2->execute( $studnum );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my ( $lastname, $firstname ) = $sth2->fetchrow;
	print "<h2>$firstname $lastname ($studnum)</h2>\n";
	print "<table border=\"1\" cellpadding=\"3\" cellspacing=\"0\">\n";
	print "<tr><th>". $lex{Current}. "</th><th>". $lex{'Move To'}. "</th></tr>\n";

	# Get Subjects
	$sth1->execute( $studnum );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	while ( my $subjsec = $sth1->fetchrow ) {
	    
	    # Get this subject description
	    $sth3->execute( $subjsec );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    my ( $description, $teacher ) = $sth3->fetchrow;
	    $teacher =~ s/\(\w+\)//g;  # strip userid
	    print "<tr><td>$description ($teacher) $subjsec</td><td>\n";

	    # Get Matching Subjects, if any
	    my ( $subjcode, $section ) = split /-/, $subjsec;
	    $sth4->execute( $subjcode );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    print "<select name=\"$subjsec:$studnum\"><option></option>\n";
	    while ( my $match_subjsec = $sth4->fetchrow ) {
		if ( $match_subjsec eq $subjsec ) { next; } # skip
		$sth3->execute( $match_subjsec );
		if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
		my ( $description, $teacher ) = $sth3->fetchrow;
		$teacher =~ s/\(\w+\)//g;  # strip userid
		print "<option>$description - $teacher ($match_subjsec)</option>\n";
	    }
	    print "</select></td></tr>\n";

	}

	print "</table>\n";
	print "<input type=\"submit\" value=\"". $lex{Continue}. "\"><p></p>\n";

    }

    print "</form></body></html>\n";

    exit;
}



#-----------
sub moveEval {
#-----------

    #foreach my $key ( sort keys %arr) { print "K:$key V:$arr{$key}<br>\n"; }

    my $sth = $dbh->prepare("select id from eval where subjcode = ? and studnum = ?");
    my $sth1 = $dbh->prepare("update eval set subjcode = ? where id = ?");
    my $sth2 = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");

    foreach my $key ( sort keys %arr ) { 

	if ( $arr{$key} ) {
	    #print "K:$key V:$arr{$key}<br>\n"; }
	    my ( $subjsec, $studnum ) = split /:/, $key;
	    my ( $dud, $newsubjsec ) = split /\(/, $arr{$key};
	    chop $newsubjsec;
	    # print "Stud: $studnum  Subjsec: $subjsec New: $newsubjsec<br>\n";

	    $sth->execute( $subjsec, $studnum );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	    while ( my $id = $sth->fetchrow ) {
		$sth1->execute( $newsubjsec, $id ); # update record.
		if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    }

	    # Write confirmation
	    $sth2->execute( $studnum );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    my ( $lastname, $firstname ) = $sth2->fetchrow;
	    print "<div style=\"text-align:left;font-size:150%;\">\n";
	    print "$firstname $lastname&nbsp;&nbsp;$subjsec -&gt; $newsubjsec</div>\n";

	}
    }

    print "</body></html>\n";
    exit;

 } # end of moveEval
