#!/usr/bin/perl
#  Copyright 2001-2007 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

#  Description: Class based subject enrollment. List all the students
#   for all the subjects in that class. Deselect those that you don't
#   have in those subjects.

my %lex = ('Subject Enrollment by Homeroom' => 'Subject Enrollment by Homeroom',
	   'Report Card' => 'Report Card',
	   'Main' => 'Main',
	   'No class found' => 'No class found',
	   'Missing starting/ending reporting periods' => 'Missing starting/ending reporting periods',
	   'Subject' => 'Subject',
	   'Select' => 'Select',
	   'for the following subjects' => 'for the following subjects',
	   'Please select the subject-sections to enrol' => 'Please select the subject-sections to enrol',
	   'Students Checked' => 'Students Checked',
	   'Continue' => 'Continue',
	   'Please enter a Homeroom' => 'Please enter a Homeroom',

	   );


use DBI;
use CGI;

require "../../etc/admin.conf" or die "Cannot open admin.conf!";

my $q = new CGI;
print $q->header; 
my %arr = $q->Vars;

my $homeroom = $arr{homeroom};
#$checked = $arr{checked};

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Print Page Header
print "$doctype\n<html><head><title>". $lex{'Subject Enrollment by Homeroom'};
print "</title>\n<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";
print "$chartype\n</head><body>[ <a href=\"$homepage\">". $lex{Main}. "</a> |\n";
print "<a href=\"$reppage\">". $lex{'Report Card'}. "</a> ]\n";


if (not $homeroom){
    print '<h3>'. $lex{'Please enter a Homeroom'}. "!</h3>\n";
    print "</body></html>\n";
    die;
}


# Find the grade(s) of this homeroom (ie. possibly multigraded)
my $sth = $dbh->prepare("select distinct grade from student
  where homeroom = ?");
$sth->execute($homeroom);
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
my $results = $sth->rows;
if ($results < 1) { 
    print "<h1>". $lex{'No class found'}. "!</h1>\n";
    die;
}

while (my $grade = $sth->fetchrow) {
    # Put all subjects of this grade into a subject array
    my $sth = $dbh->prepare("select description, teacher, subjsec, 
     startrptperiod, endrptperiod from subject 
     where grade = ? order by description");
    $sth->execute($grade);
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my $periodflag;
    while (my ($description,$teacher, $subjsec, $startterm, $endterm) = $sth->fetchrow) {
	if (not $startterm or not $endterm){ # missing start/end term
	    $periodflag = 1;
	    push @errorarray,"<b>$description</b> ($teacher) $subjsec";
	}
	push @subjarray, "$description:$teacher:$subjsec";
    }
} 

if ($periodflag){  # missing reporting periods; crash and burn.
    print "<center><h1>". $lex{'Missing starting/ending reporting periods'}. "<br>\n";
    print $lex{'for the following subjects'}. ":</h1>\n";
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>". $lex{Subject}. "</th></tr>\n";
    foreach $badsubject (@errorarray){
	print "<tr><td>$badsubject</td></tr>\n";
    }
    print "</table></center></body></html>\n";
    die;
}


print "<center><h1>". $lex{'Subject Enrollment by Homeroom'}. "</h1>\n";

print "<p><b>". $lex{'Please select the subject-sections to enrol'}. ".</p>\n";
print "<form action=\"enrolclas1.pl\" method=\"post\">\n";
print "<input type=\"hidden\" name=\"homeroom\" value=\"$homeroom\">\n";
print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\" style=\"background-color:#EEE;\">\n";
print "<tr><th>". $lex{Subject}. '</th><th>'. $lex{Select}. "</th></tr>\n";
print "<tr><td colspan=\"2\" align=\"center\"><input type=\"submit\" value=\"";
print $lex{Continue}. "\"></td></tr>\n";
print "<tr><td colspan=\"2\" align=\"center\"><b>". $lex{'Students Checked'};
print "? <input type=\"checkbox\" name=\"checked\" value=\"checked\"></td></tr>\n";


foreach my $subject (@subjarray){ #loop through subjects;
    my ($description,$teachid,$subjsec) = split /:/,$subject;
    print"<tr><td>$description ($subjsec)</td>\n";
    print "<td><input type=\"checkbox\" name=\"$subjsec\" value=\"1\"></td></tr>\n";
}

print "<tr><td colspan=\"2\" align=\"center\">\n";
print "<input type=\"submit\" value=\"". $lex{Continue}. "\"></td></tr>
</form></center></body></html>\n";




