#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('View Subject Enrollment' => 'View Subject Enrollment',
	   'Report Card' => 'Report Card',
	   'Grade' => 'Grade',
	   'Withdrawn' => 'Withdrawn',
	   'Error' => 'Error',

	   );

my $self = 'rptenrolgrade.pl';

use DBI;
use CGI;

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $self: $@<br>\n";
    die $lex{Error}. " $self: $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;
my $grade = $arr{grade};

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Step 1: Find all of the distinct subject-sections in enrollment
#  and lookup name and grade. Sort by grade and description.

my $sth = $dbh->prepare("select distinct eval.subjcode from eval
  left outer join subject on eval.subjcode = subject.subjsec
  where subject.grade = ? order by subject.description");
$sth->execute( $grade );
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

# put all subjects into an array.
my $sth1 = $dbh->prepare("select description, grade from subject where subjsec = ?");
my @subjarray;
while ( my $subjsec = $sth->fetchrow ) {
    $sth1->execute($subjsec);
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; };
    my ($description,$grade) = $sth1->fetchrow;
    push @subjarray,"$grade:$description:$subjsec";
}
@subjarray = sort @subjarray;


# Print HTML Page Header
print "$doctype\n<html><head>
<title>$lex{'View Subject Enrollment'}</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>\n";
print "[ <a href=\"$reppage\">$lex{'Report Card'}</a> ]
<center><h1>$lex{'View Subject Enrollment'}</h1></center>\n";

foreach my $subject ( @subjarray ){

    my ( $grade, $description, $subjsec ) = split /:/,$subject;

    # print the subject heading
    print "<table cellpadding=\"3\" border=\"1\" cellspacing=\"0\">\n";
    print "<tr><th colspan=\"2\">$description ($subjsec) $lex{Grade}: $grade</th></tr>\n";
    
    # Step 2: Get all of the kids enrolled in each subject in 
    #  turn, sort them, and then load and display records. (different terms)
    my $sth=$dbh->prepare("select distinct studnum from eval 
      where subjcode = ?");
    $sth->execute($subjsec);
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    
    my $sth1=$dbh->prepare("select lastname,firstname from student where studnum = ?");
    while (my $studnum = $sth->fetchrow) {

	# Get the Name
	$sth1->execute($studnum);
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	my ($lastname,$firstname) = $sth1->fetchrow;

	# Look in withdrawn for lastname
	if (not $lastname){
	    my $sth4 = $dbh->prepare("select lastname,firstname from studentall where studnum = ?");
	    $sth4->execute($studnum);
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    ($lastname,$firstname) = $sth4->fetchrow;
	    $lastname = "(<span style=\"color:red;\">". $lex{'Withdrawn'}. "</span>) $lastname";
	}

	# Get the terms for this subject
	my $sth2 = $dbh->prepare("select term from eval 
          where studnum = ? and subjcode = ? order by term");
	$sth2->execute($studnum, $subjsec);
	if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
	my $termrows = $sth2->rows;
	my $terms;
	for (1..$termrows){ 
	    $term = $sth2->fetchrow;
	    $terms = $terms." $term";
	}
	push @studarray,"<b>$lastname</b>, $firstname\t$terms";
    }

    @studarray = sort @studarray;

    foreach $student (@studarray){
	my ($name,$terms) = split /\t/,$student;
	print "<tr><td>$name</td><td>$terms</td></tr>\n";
    }

    @studarray = ();   
    print "</table><p></p>\n";

} # End of Subject Loop

print "</table></body></html>\n";
