#! /usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# Outline: Print the report cards for a single grade or homeroom.

# Customizations: Note ANY customizations for this school here
#  along with line numbers and/or markers to find them...

my %lex = ('Report Card Generation' => 'Report Card Generation',
	   'Main' => 'Main',
	   'Report Card' => 'Report Card',
	   'teacher' => 'teacher',
	   'principal' => 'principal',
	   'combo' => 'combo',
	   'No grade found or Periods per day not set in admin.conf' => 
  	     'No grade found or Periods per day not set in admin.conf',
	   'Cannot open term file' => 'Cannot open term file',
	   'View/Download/Print Report Cards' => 'View/Download/Print Report Cards',
	   'View Log File' => 'View Log File',
	   'Report to Parents' => 'Report to Parents',
	   'Grade' => 'Grade',
	   'Homeroom Teacher(s)' => 'Homeroom Teacher(s)',
	   'Evaluation and Comment Key' => 'Evaluation and Comment Key',
	   'Class Average' => 'Class Average',
	   'Attendance' => 'Attendance',
	   'Days Open' => 'Days Open',
	   'Days Enrolled' => 'Days Enrolled',
	   'Days Absent' => 'Days Absent',
	   'Periods Late' => 'Periods Late',
	   'Additional Comments' => 'Additional Comments',
	   'Term' => 'Term',
	   'Teacher' => 'Teacher',
	   'Principal' => 'Principal',
	   'Please tear off this portion and return it to the school' =>
	     'Please tear off this portion and return it to the school',
	   'in the report envelope with any comments and your signature' =>
	     'in the report envelope with any comments and your signature',
	   'Parent/Guardian Signature' => 'Parent/Guardian Signature',
	   'Student' => 'Student',
	   'Date' => 'Date',
	   'Comments' => 'Comments',
	   'Days per Month Override' => 'Days per Month Override',
	   'Student Group' => 'Student Group',
	   'Homeroom' => 'Homeroom',
	   'Days Open' => 'Days Open',
	   'Month' => 'Month',
	   'Continue' => 'Continue',
	   'Duplicate Evaluation records for student' => 
	     'Duplicate Evaluation records for student',
	   'for' => 'for',
	   'Average' => 'Average',
	   'Final' => 'Final',
	   'No Quality Score for' => 'No Quality Score for',
	   'Error' => 'Error',
	   'Printed' => 'Printed',
	   'Override Term Days' => 'Override Term Days',
	   'Font Size' => 'Font Size',
	   'One Student per File' => 'One Student per File',
	   'GPA' => 'GPA',
	   'Current Year GPA' => 'Current Year GPA',
	   'Phone' => 'Phone',
	   'Fax' => 'Fax',
	   'Absent' => 'Absent',
	   'Late' => 'Late',
	   'Alternate Report Card System' => 'Alternate Report Card System',
	   'Student Name' => 'Student Name',
	   'Date of Birth' => 'Date of Birth',
	   'Teacher Name' => 'Teacher Name',
	   'Principal' => 'Principal',
	   'Vice-Principal' => 'Vice-Principal',
	   'Period(s)' => 'Period(s)',
	   'Evaluation Key' => 'Evaluation Key',
	   'Select Subjects' => 'Select Subjects',
	   'For Duplexing/Stapling' => 'For Duplexing/Stapling',
	   'Show Withdrawn Students' => 'Show Withdrawn Students',
	   'Please Log In' => 'Please Log In',
	   'Pre-kindergarten Class' => 'Pre-kindergarten Class',
	   'Subject' => 'Subject',

	   );

use DBI;
use CGI;
use CGI::Session;
use Date::Business;
use Cwd;
use Number::Format qw(round);


# To Do: fix the ppd override for teacher mode.

$r_Centersubjects = '1';


my $self = 'rptrepcard.pl';

# Get current dir so know what path for config files.
my $configpath;
my $teachermode;
if ( getcwd() =~ /tcgi/ ){ # we are in tcgi
    $teachermode = 1;
    $configpath = '..'; # go back one to get to etc.
} else {
    $configpath = '../..'; # go back two to get to etc.
}

# main config file
eval require "$configpath/etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

# report card config file
eval require "$configpath/etc/repcard.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

# filter latex function
eval require "$configpath/lib/liblatex.pl";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

# attendance library functions
eval require "$configpath/lib/libattend.pl";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

if ( $teachermode ) {  # Load Teacher overrides
    eval require "$configpath/etc/repcardOVR.conf";
    if ( $@ ) {
	print $lex{Error}. ": $@<br>\n";
	die $lex{Error}. ": $@\n";
    }
}    


# Get current dir so know what CSS to display;
if (getcwd() =~ /tcgi/){ # we are in tcgi
    $css = $tchcss;
}

my $q = new CGI;
my %arr = $q->Vars;


my $dsn = "DBI:$dbtype:dbname=$dbase";
$dbh = DBI->connect($dsn,$user,$password);

my $session;
if ( $teachermode ) { # we are running in teacher site.

    # Get Session
    $session = new CGI::Session("driver:$dbtype;serializer:FreezeThaw",
				undef,{Handle => $dbh}) or die CGI::Session->errstr;

    if ( not $session->param('logged_in') ){
	$userid = $session->param('userid');
	print $q->header( -charset, $charset );
	print "<h1>". $lex{'Please Log In'}. "</h1>\n";
	print "</body></html>\n";
	die;
    }
    # Ok, we have a login. Values below we have in environment.

    $userid = $session->param('userid');
    $duration = $session->param('duration');
    $session->expire('logged_in', $duration );

    print $session->header( -charset, $charset );

} else { # normal mode in cgi (admin site)
    print $q->header( -charset, $charset );
}


# Set Date
my ($sec, $min, $hour, $mday, $mon, $year, $wday, 
    $yday, $iddst) = localtime(time);
$year = $year + 1900;
$mon++;
$wday++;
my $currsdate = "$year-$mon-$mday";
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";


# Print HTML Page Header
print "$doctype\n<html><head><title>". $lex{'Report Card Generation'}. "</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>[ <a href=\"$homepage\">". $lex{Main}. "</a> | \n";
print "<a href=\"$reppage\">". $lex{'Report Card'}. "</a> ]\n";

if ( $teachermode ) {
    print "$userid \n";
}

if ( not $arr{page} ) {
    showStartPage();
} 
delete $arr{page}; # no longer needed.

# remove extra values from %arr; leave only month values.
my $group = $arr{group}; # also reset group to either grade or homeroom.
if ( $group eq $lex{Grade} ) {
    $group = 'grade';
} else {
    $group = 'homeroom';
}
my $groupid = $arr{groupid};
my $fontsize = $arr{fontsize};
my $override = $arr{override};
my $showwithdrawn = $arr{showwithdrawn};

my $oneStudentPerFile;
if ( $arr{onestudentperfile} ) { $oneStudentPerFile = 1; }
my $alternateReportCard;
if ( $arr{alternatereportcard} ) { $alternateReportCard = 1; }

delete $arr{group};
delete $arr{groupid};
delete $arr{fontsize};
delete $arr{override};
delete $arr{onestudentperfile};
delete $arr{alternatereportcard};
delete $arr{showwithdrawn};

#extract the subjec's, if any
foreach my $key (sort keys %arr ) { 
    if ( $arr{$key} == -1 ) {
	#print "K:$key V:$arr{$key}<br>\n"; 
	push @subjects, $key;
	delete $arr{$key};
    }
}

my %subjects;
foreach my $subjsec ( @subjects ) {
    my $sth = $dbh->prepare("select sequence, description from subject where subjsec = ?");
    $sth->execute( $subjsec );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my ( $sequence, $description ) = $sth->fetchrow;
    $subjects{"$sequence$description"} = $subjsec;
}


# Show withdrawn students, as well, if they have data.
if ( $showwithdrawn ) {
    $studenttable = 'studentall';
} else {
    $studenttable = 'student';
}


# Get %students, to match @subjects; none in normal mode; key will provide sort order
my $sth = $dbh->prepare("select distinct e.studnum, s.lastname, s.firstname from eval as e,
   $studenttable as s where s.studnum = e.studnum and subjcode = ?");

foreach my $subjsec ( @subjects ) { # find the students for this, if any (not normally)
    $sth->execute( $subjsec );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my ( $studnum, $lastname, $firstname ) = $sth->fetchrow ) {
	$students{"$lastname$firstname$studnum"} = $studnum;
	#print "$firstname $lastname ($studnum)<br>\n";

    }
}


#foreach my $key (sort keys %students ) { print "K:$key V:$students{$key}<br>\n"; }


# Get a grade in order to get correct PPD ( which may be different for each grade )
# IN FACT this stuff should get the LARGEST PPD 
#(in case of multigraded classroom with different PPD?)
my $grade;
if ( $group eq 'homeroom' ) { # Get the grade for this class
    $sth = $dbh->prepare("select distinct grade from student
     where homeroom = ?");
    $sth->execute( $groupid );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    $grade = $sth->fetchrow;
} else {
    $grade = $groupid;
}

# $PPD is Important Variable from admin.conf
#  periods per day set via grade; needed for attendance calculations
my $periodsperday = $g_ppd{$grade};
if ( not $periodsperday and not $teachermode ) {
    print $lex{Grade}. ": $grade  PPD: $g_ppd{$grade} \n";
    print $lex{'No grade found or Periods per day not set in admin.conf'}. "!<br>\n";
    die;
} 

# this is ugly; fix later with better ppd method for teacher mode (Nov3/08)
if ( $teachermode ) {
    $periodsperday = $r_defaultppd;
}

# Read in the term(s)
my ($term, @terms, %terms);
open(TERM, "$configpath/etc/term") || die $lex{'Cannot open term file'}. "!\n"; # slurp file
my $termdata;
{ local $/; $termdata = <TERM>; close FH;}
my @terms = split /\n/, $termdata;
$term = $terms[0];


# multiTrack variable is set in the admin.conf file.
if ( $multiTrack ) { # Are we in multiTrack mode? If so, reset term.
    for my $t (1..$#terms) {
	my ($key, $val) = split /:/, $terms[$t];
	$terms{$key} = $val;  # Form is   e:3  (elementary: term 3)
    }
    my $grp = $g_termtype{$grade};
    if ( $terms{$grp} ) {
	$term = $terms{$grp};
    }
}
# Note: Multitrack schools can have different terms for same time period. 
# This script now looks for this. It didn't previously.


# CLASS AVERAGE
# Calculate the class averages for all subjects in the current term.
#  Easier than doing this just for subjects in current grade, since they
#   may take subjects from another grade level, etc.
#  Only do this if $clasavg in repcard.conf is 1;
# $markfield = 'a1'; - now located in repcard.conf.

my %classavg;
if ( $clasavg ){ # create %classavg - class average hash.
    # First Find all subjects done this term. (actually a subjsec field)
    $sth = $dbh->prepare("select distinct subjcode from eval
     where term = '$term' order by subjcode desc");
    $sth->execute;
    if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr;}
    my $subjcount = $sth->rows;

    # Loop through each subjsec (subject-section)
    for (1..$subjcount){
	my $subjsec = $sth->fetchrow;
    
	$sth1 = $dbh->prepare("select $markfield from eval
         where term = '$term' and subjcode = '$subjsec'");
	$sth1->execute;
	if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr;}
	my $rows = $sth1->rows;
	if ($rows < 1){ next;} # skip if no records.

	my $totalmark = 0;
	while (my $mark = $sth1->fetchrow){
	    $totalmark += $mark;
	}
	my $average = round( $totalmark / $rows, 0 );
	$classavg{$subjsec} = $average;
    }
} # End of Class Average Calculations (creates %classavg)


# Calculate number of days in the terms
for my $trm ( 1..$term ) {
    $termdays{$trm} = calcTermDays($g_termstart{$trm},$g_termend{$trm});
}

# Format:  key: year-month  value: days
my %schooldays = mkSchoolDays( $schoolstart, $currsdate );
if ( $override ){ # we are overriding days per month...
    foreach my $key (sort keys %arr) { # %arr now contains all the months of interest.
	$schooldaysorig{$key} = $schooldays{$key};
	$schooldays{$key} = $arr{$key}; 
    }
}

#foreach my $key ( keys %schooldays ) { print "K: $key Val: $schooldays{$key}<br>\n"; }


# What is the order of the school months?
my @schoolmonths = mkSchoolMonths($currsdate); # uses currsdate = yyyy-mm-dd;
# Format: yyyy-mm; first record is blank (zeroth)
# print "The schooldays hash is:<br>",@schoolmonths,"<br>\n";


my ($tempdir, $shortname, $filename);
if ( not $oneStudentPerFile ) {
    $shortname = "repcard$$";
    $filename = "$shortname.tex";

    open( TEX,">$filename" ) || die "Can't open tex file";
    prTexDocStart(); # print document preamble.
} else { # we are doing one student per file; create temporary folder.
    $tempdir = "tempdir$$";
    system("mkdir $tempdir");
    open( LOG,">pdflog$$.txt" ) || die "Can't open LOG file";
} 

if ( not %students ) { # then we need to find some...
    my $sth = $dbh->prepare("select studnum, lastname, firstname, homeroom
       from $studenttable where $group = ? order by homeroom, lastname, firstname");
    $sth->execute( $groupid );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my ( $studnum, $lastname, $firstname, $homeroom ) = $sth->fetchrow ) {
	$students{"$homeroom$lastname$firstname$studnum"} = $studnum;
    }
}


my $sth = $dbh->prepare("select lastname, firstname, initial, grade, 
 homeroom, birthdate, provnum from $studenttable where studnum = ?");

# Loop through all students, one report card each.
foreach my $key ( sort keys %students ) {
    my $studnum = $students{$key};

    $sth->execute( $studnum );
    my ( $lastname, $firstname, $middlename, $grade, $homeroom, 
	 $birthdate, $provnum ) = $sth->fetchrow;

    if ( $skipBlankReportCard ) {
	# Check that she/he actually has some info in database.
	my $sth1 = $dbh->prepare("select count(*) from eval 
         where term = ? and a1 != '' and studnum = ?");
	$sth1->execute( $term, $studnum );

	if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr; }
	my $evcount = $sth1->fetchrow;
	if ( not $evcount ) { next; } # skip kids with blank values this term.
    }


    if ( $oneStudentPerFile ) {
	# Create this student's filename
	my ($lname, $fname);
	$fname = $firstname; $lname = $lastname;
	$fname =~ s/\W|\-//g; # remove 'nonword' characters
	$lname =~ s/\W|\-//g;
	if ( length($lname) > 12 ) { $lname = substr $lname, 0, 12; }
	if ( length($fname) > 8 ) { $fname = substr $fname, 0, 8; }
	#print "LN: $lname  FN: $fname<br>\n";
	$shortname = "$lname$fname-$studnum";
	$filename = "$shortname.tex";

	open( TEX,">$filename" ) || die "Can't open tex file";
	prTexDocStart(); # print document preamble.
	
    }


    if ( $alternateReportCard ) {
	studentAltRepcard( $studnum, $lastname, $firstname, $middlename, $grade, 
			  $homeroom, $birthdate );

	if ( $oneStudentPerFile ) { # we also have to do the close for this student;
	    # build pdf and put into tempdir; append to pdflog$$.txt
	    print TEX "\\end{document}\n";
	    close TEX;

	    print LOG "\n\n$firstname $lastname ($studnum)\n-----------------\n";
	    
	    system("$pdflatex $filename >>pdflog$$.txt");
	    system("mv $shortname.pdf $tempdir");
	    system("rm -f $shortname.*");
	}

	next; # skip to next student

    }



    # Print the heading section for each new report card
    prTexHead( $grade, $lastname, $firstname, $middlename, $birthdate, $provnum );

    # print a Note 1 - if set in repcard.conf
    if ( $note1 ) {
	print TEX "\\fbox{\\parbox[b]{\\textwidth}{ ";
	print TEX "\\begin{center}{\\it $note1}\\end{center} }}\n\n";
    }

    # print homeroom teachers for this student (if any).
    if ( $showHomeroomTeacher ) {  # set in repcard.conf
	my @teachname = ();
	$sth1 = $dbh->prepare("select sal, firstname, lastname from staff 
        where homeroom = ? and grade = ? and position = ? ");
	# Note: $position set in repcard.conf to the teacher position.. 'Classroom Teacher'
	$sth1->execute( $homeroom, $grade, $position );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	# Fancy Stuff in case more than one teacher for that room.
	my $teachrows = $sth1->rows;
	while (my ($sal, $firstname, $lastname) = $sth1->fetchrow ) {
	    push @teachname,  "$sal $firstname $lastname";
	}
	if ( @teachname ) {  # only print if they exist.
	    prHomeroomTeachers( @teachname );
	}
    }

    # put in rule divider
    print TEX "\\hrulefill \n\\smallskip\n\n";

    if ( $showGradeScheme ) {  # set in repcard.conf
	print_grade_scheme( $grade );
    }
    # End of head section for each new report card.


    # print a Note 2 - if set in repcard.conf
    if ( $note2 ) {
	print TEX "\\begin{center}{\\it $note2}\\end{center}\n\n";
    }


    my %mysubjects = ();
    if ( not %subjects ) { # then create %mysubjects for this student...

	# Find eval records for this student  ~~updated for Additional Comments
	$sth1 = $dbh->prepare("select distinct e.subjcode, s.sequence, 
          s.startrptperiod from eval as e, subject as s where 
           e.studnum = ? and
           e.subjcode = s.subjsec and
           s.description != '$additionalcomments'
           order by s.sequence");
	$sth1->execute( $studnum );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my ( $subjsec, $seq, $startrptperiod ) = $sth1->fetchrow ) {
	    while ( length( $seq ) < 4 ) { # pad with leading zeros
		$seq = '0'. $seq;
	    }
	    $mysubjects{"$seq$subjsec"} = $subjsec;
	}

    } else { %mysubjects = %subjects; }

    #print "SN: $studnum  SUB:", %mysubjects, "<br>\n";

    my $firstsubjectflag = 1; # mark if still at first subject

    # Initialize the GPA hash.
    my %gpa = ();
    my $biggestterm; # track largest term values in subjects

    # Loop through all subjects for this report card.
    #while ( my ( $subjsec, $seq, $startrptperiod ) = $sth1->fetchrow ) {

    if ( $r_Centersubjects ) {
	print TEX "\\begin{center}\n";
    }

    foreach  my $key ( sort keys %mysubjects ) {

	my $subjsec = $mysubjects{ $key };

	# Check for enrollment (in teacher mode)
	my $sth1 = $dbh->prepare("select count(*) from eval 
          where studnum = ? and subjcode = ?");
	$sth1->execute( $studnum, $subjsec );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $count = $sth1->fetchrow;
	if ( $count < 1 ) { next; } # next subject

	# Get this subject's info...
	$sth1 = $dbh->prepare("select s.sequence, s.startrptperiod from subject as s 
           where subjsec = ?");
	$sth1->execute( $subjsec );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my ( $sequence, $startrptperiod ) = $sth1->fetchrow;

	if ( $startrptperiod > $term ) { # this subject hasn't started yet...
	    next;
	}

	@eval = ();
	@qstarray = ();

	# Now pull in the records for this subject for this student
	$sth2 = $dbh->prepare("select * from eval where studnum = ?
          and subjcode = ? order by eval.term");
	$sth2->execute( $studnum, $subjsec ) ;
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }


	# read eval records for each term of this "subject".
	my $currterm = 0; 
	my $prevterm; # error checking only.

	$markfield =~ s/a//; # remove leading 'a'; comes from repcard.conf
	my $markfieldidx = $markfield + 6; # a1 gives 7, a2 gives 8, etc.
	#print "IDX: $markfieldidx<br>\n";

	while ( my @temp = $sth2->fetchrow){

	    # Set values into GPA Hash
	    my $term = $temp[4];
	    if ( $temp[$markfieldidx] ) {
		$gpa{$subjsec}[$term] = $temp[$markfieldidx];
	    }

	    # Check for largest term (so know how many gpa calcs to do).
	    if ( $term > $biggestterm ) { $biggestterm = $term; }


	    @temp = latex_filter( @temp );
	    if ($arFilter){ # filter for trailing AR string. See repcard.conf
		$temp[7] =~ s/\d+AR/AR/i;
	    }
	    
	    $prevterm = $currterm; $currterm = $temp[4];
	    if ($currterm == $prevterm){ 
		# Error! two eval records with the same term value.
		print $lex{'Duplicate Evaluation records for student'}. ":\n";
		print " $studnum ". $lex{for}. " ". $lex{Subject};
		print " $subjnum ". $lex{for}. " ". $lex{Term}. " $currterm.\n";
		print "Contact Les Richardson\n";
	    }

 	    $eval[$currterm] = [ @temp ];

	} # Done stuffing eval recs into eval array (with reference).

	prSubject( $subjsec, $studnum ); # print subject line(s).

	prComment(); # print the comments (all)


    } # End of this subject; loop to print the next.


    # Now do the GPA row...
    if ( $r_calcgpa ) {
	calcGPA(\%gpa, $biggestterm );
	print TEX "\\smallskip\n\n";
    }

    # End Subject Centering, if turned on.
    if ( $r_Centersubjects ) {
	print TEX "\\end{center}\n";
    }


    # Start a new page for attendance, etc.
    if ( $r_NewPageBeforeAttendance ) {
	print TEX "\\newpage\n";
    }

    # Now do the attendance
    if ( $r_ShowMonthlyAttendance ) {
	prMonthlyAttendance( $studnum );
    }

    if ( $teachermode ) {
	print TEX "\\vspace{10mm}\n\n";
    } else {
	print TEX "\n\n"; # to push over additional comments.
    }

    if ( $r_ShowAdditionalComments ) {
	prAddComments( $studnum ); # Additional comments
    }

    # Now finish off the report card; last term gets the promotion format.
    # There are 3 signature fns: Teacher, Principal, or Combo. Pick your mix.

    if ($term != $finalterm){

	# Also... if no $teachersign value, then not shown...
	if ($teachersign eq $lex{teacher} ){
	    prTeacherSig();  #Teacher Signature

	} elsif ($teachersign eq $lex{principal} ){
	    prPrincipalSig();  #Principal Signature

	} elsif ($teachersign eq $lex{combo}) {
	    prComboSig(); 
	    # Combination Signature (both teacher / principal side by side.)
	}
	
	# print if we have a showTearoff (or not updated)
	if ( $showTearoff or not defined $showTearoff ){
	    if ( $r_PrintParentSig ) { # block them together
		$tearoffblock = 1;
	    }
	    prTearoff(); 

	}

	# print a signature line for parents
	if ( $r_PrintParentSig ) {
	    prParentSig( $currdate, $lastname, $firstname, $middlename ); # Parent Signature
	}
    
    } else { # we're in the last term; print graduation box, if set...

	if ( $r_PrintYearEnd ) {
	    prYearEnd("$firstname $middlename $lastname");
	}
    }
    
    # End Section for this student.
    if ( $oneStudentPerFile ) {
	# build pdf and put into tempdir; append to pdflog$$.txt
	#print TEX "\\end{center}\n";
	print TEX "\\end{document}\n";
	close TEX;

	print LOG "\n\n$firstname $lastname ($studnum)\n-----------------\n";
	system("$pdflatex $filename >>pdflog$$.txt");
	system("mv $shortname.pdf $tempdir");
	system("rm -f $shortname.*");
    } else {
	print TEX "\\newpage\n"; # new page for next student.
    }
    
}  # End of Main Loop; Back to Next Student.


# Final section

# Done report card, now setup the web page.
print "<center><h3>". $lex{'Report Card Generation'}. "</h3>\n";


if ( $oneStudentPerFile ) {
    # create single zip and copy all zip to webdownload dir; pdflog$$.txt too.
    $shortname = "repcard$$";
    system("zip $shortname.zip $tempdir/* >> pdflog$$.txt");
    system("mv $shortname.zip $downloaddir");
    system("mv pdflog$$.txt $downloaddir");
    system("rm -r $tempdir");

    print "<h1><a href=\"$webdownloaddir/$shortname.zip\">\n";
    print $lex{'View/Download/Print Report Cards'}. "</a></h1>\n";

} else { # all in one.

    # Print ending of report
    #print TEX "\\end{center}\n";
    print TEX "\\end{document}\n";
    close TEX;

    system("$pdflatex $filename >pdflog$$.txt");

    if ( $teachermode ) {
	$downloaddir = $tchdownloaddir;
    }
    
    system("mv $shortname.pdf $downloaddir");
    system("mv pdflog$$.txt $downloaddir");
    system("rm -f $shortname.*");

    print "<h1><a href=\"$webdownloaddir/$shortname.pdf\">\n";
    print $lex{'View/Download/Print Report Cards'}. "</a></h1>\n";

}

print "[ <a href=\"$reppage\">". $lex{'Report Card'}. "</a> | \n";
print "<a href=\"$webdownloaddir/pdflog$$.txt\">". $lex{'View Log File'}. "</a> ]\n";
print "</center></body></html>\n";


# ====== End of Main Section ================


#----------------
sub showStartPage {
#----------------

    my @subjects = ();
    my %subjdesc = ();
    if ( $teachermode ) { # we are running in teacher site
	# Get their subjects
	my $sth = $dbh->prepare("select subjsec, description from subject 
          where teacher like '%($userid)' order by sequence, description ");
	$sth->execute;
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my ( $subjsec, $description ) = $sth->fetchrow ) {
	    push @subjects, $subjsec;
	    $subjdesc{ $subjsec } = $description;
	}

    }

    # Get School Months for Override
    my @schoolmonths = mkSchoolMonths( $currsdate );
    # Format: yyyy-mm; first record is blank (zeroth)
    #print "Months:",@schoolmonths,"<br>\n";

    # print group and groupid selection input form.
    print "<center><h1>". $lex{'Report Card'}. "</h1>\n";
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<div style=\"border:1px solid gray;padding:1em; margin:0.5em; width:50%;\">\n";
    print "<table cellspacing=\"0\" cellpadding=\"3\" border=\"0\">\n";
    print "<tr><td align=\"right\">". $lex{'Student Group'}. "</td>\n<td>";
    print "<select name=\"group\"><option>". $lex{Grade}. "</option>\n";
    print "<option>". $lex{Homeroom}. "</option></select>\n ";
    print "<input type=\"text\" name=\"groupid\" size=\"6\"></td></tr>\n";

    print "<tr><td align=\"right\">". $lex{'Font Size'}. "</td>\n<td>";
    print "<select name=\"fontsize\"><option>$r_fontsize</option>\n";
    print "<option>10pt</option><option>11pt</option><option>12pt</option></select></td></tr>\n ";

    if ( $teachermode ) {
	print "<tr><td><b>". $lex{'Select Subjects'}. "</b></td><td></td></tr>\n";
	foreach my $subjsec ( @subjects ) {
	    print "<tr><td align=\"right\" style=\"background-color:#DDD;\">";
	    print "$subjdesc{$subjsec} ($subjsec)</td>\n";
	    print "<td><input type=\"checkbox\" name=\"$subjsec\" value=\"-1\"></td></tr>\n";
	}
    }

    print "<tr><td align=\"right\">". $lex{'Show Withdrawn Students'}. "</td>\n<td>";
    print "<input type=\"checkbox\" name=\"showwithdrawn\" value=\"1\"></td></tr>\n";

    print "<tr><td align=\"right\">". $lex{'One Student per File'}. "</td>\n<td>";
    print "<input type=\"checkbox\" name=\"onestudentperfile\" value=\"1\"> ";
    print $lex{'For Duplexing/Stapling'}. "</td></tr>\n";

    print "<tr><td align=\"right\">". $lex{'Alternate Report Card System'}. "</td>\n<td>";
    print "<input type=\"checkbox\" name=\"alternatereportcard\" value=\"1\"></td></tr>\n";

    print "<tr><td colspan=\"2\" align=\"center\"><input type=\"submit\" value=\"";
    print $lex{Continue}. "\"></td></tr>\n";
    print "</table>\n";

    print "<div style=\"padding:1em;\">". $lex{'Override Term Days'};
    print " <input type=\"checkbox\" name=\"override\" value=\"1\"></div>\n";

    print "<table cellspacing=\"0\" cellpadding=\"3\" border=\"1\">\n";
    print "<tr><th colspan=\"2\">". $lex{'Days per Month Override'}. "</th></tr>\n";
    print "<tr><th>". $lex{Month}. "</th><th>". $lex{'Days Open'};
    print "</th></tr>\n";

    for my $i (1..$#schoolmonths) {
	my ($y,$m) = split /-/,$schoolmonths[$i];
	print "<tr><td><b>$month[$m]</b> ($schoolmonths[$i])</td><td>";
	print "<input type=\"text\" name=\"$schoolmonths[$i]\" size=\"4\">";
	print "</td></tr>\n";
    }
    print "</table>\n";

    print "<div style=\"padding:0.5em;\"><input type=\"submit\" value=\"";
    print $lex{Continue}. "\"></div>\n";

    print "</div></form></center></body></html>\n";

    exit;

} # end of showStartPage



#--------------------
sub studentAltRepcard {
#--------------------

    # print single student report card to TEX file handle (already open).

    my ($studnum, $lastname, $firstname, $middlename, $grade, $homeroom, $birthdate) = @_;

    
    # Find Homeroom Teacher(s)
    my @teachers;
    my $sth = $dbh->prepare("select distinct lastname, firstname from staff 
      where homeroom = ? order by lastname, firstname");
    $sth->execute( $homeroom );
    while ( my ($lastname, $firstname) = $sth->fetchrow ) { 
	push @teachers, "$firstname $lastname";
    }

    print TEX "\\thispagestyle{empty}\n\\setcounter{page}{1}\n";
    print TEX "\\rhead{$firstname $middlename $lastname -- \\thepage}\n\\begin{center}\n";

    print TEX "\\includegraphics[width=$r_Altdivlogowidth]{";
    print TEX "$configpath/etc/$r_Divlogofile}\n";
    print TEX "\\parbox[b]{$r_Altaddresswidth}{ ";
    print TEX "{\\Large $g_Divisionname\n\\vspace{$r_Altspacer} } }\n\n"; 
    # second brace end of parbox.

    print TEX "\\vspace{15mm}\n";

    print TEX "\\includegraphics[width=$r_Altlogowidth]{";
    print TEX "$configpath/etc/$r_Altlogofile}\\quad \n\n";
    print TEX "\\end{center}\n\n";


    print TEX "\\vspace{15mm}\n";

    # Student Identity

    print TEX "\\parbox{\\textwidth}{\\Large ";

    print TEX $lex{'Student Name'}. " {\\bf $firstname $lastname}\n\n";
    
    print TEX $lex{'Date of Birth'}. " {\\bf $birthdate}\n\n";

    print TEX $lex{'Teacher Name'};
    foreach my $teacher ( @teachers ) {
	print TEX  " {\\bf $teacher}\\quad\n";
    }
    print TEX "\n\n";

    print TEX $lex{Term}. " {\\bf $term}\n\n";
    print TEX '{\\bf '. $schoolname. q{ }. $lex{'Pre-kindergarten Class'}. "}\n";

    print TEX "}\n"; # End of Parbox above...

    print TEX "\\newpage\n\n";

    # Next Page, Start all Subjects for this student.

    # Step 1 - get the subjects this student is enrolled in in this term, in order
    my $sth = $dbh->prepare("select distinct s.subjsec, s.description from rc_subject as s, 
        rc_obj_eval as e, rc_objective as o  where s.subjsec = o.subjsec and 
        o.id = e.objid and e.studnum = ? and term = ? order by s.sequence, s.description");
    $sth->execute( $studnum, $term );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }


    while ( my ( $subjsec, $subjdesc ) = $sth->fetchrow ) {

	( $subjdesc ) = latex_filter( ($subjdesc ) );

	print TEX "{\\begin{tabular}{|p{3cm}|p{5cm}|p{9.1cm}|}\\hline\n";

        #push @subjects, $subjsec;
	# Now get all objectives in order and matching choice.

	my $sth1 = $dbh->prepare("select o.description, e.eval from rc_objective as o, 
          rc_obj_eval as e where o.id = e.objid and o.subjsec = ? and 
          e.studnum = ? and e.term = ? order by o.groupseq");
	$sth1->execute( $subjsec, $studnum, $term );
        if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	my $first = 1;
	while ( my ( $description, $eval ) = $sth1->fetchrow ) {

	    ( $description, $eval ) = latex_filter( ( $description, $eval ) );
	    if ( $first ) {
		print TEX "$subjdesc & "; # removed ($subjsec)
		$first = 0;
	    } else {
		print TEX ' &';
	    }

	    print TEX "\\raggedleft $description & $eval \\\\ \n";
	}
	print TEX "\\hline \\end{tabular}\n\n";

    }

    # Not optional setting; Just comment out if you want.
    prMonthlyAttendance( $studnum );

    print TEX "\\vspace{15mm}\n";

    # Hardwired for Combo Only...
    #prTeacherSig();  #Teacher Signature
    #prPrincipalSig();  #Principal Signature
    #} elsif ($teachersign eq $lex{combo}) {
	prComboSig(); 
    ## Combination Signature (both teacher / principal side by side.)


    print TEX "\n\\newpage\n";

    return;

}



#----------------
sub prTexDocStart {
#----------------

# Preamble and Head of Document

    if ( $defaultpapersize ) {
	$papersize = $defaultpapersize;
    } else {
	$papersize = 'letterpaper';
    }

    print TEX <<"EOS1";  
\\documentclass[ $fontsize, $papersize,oneside]{article}
\\usepackage{array,graphicx,multicol,colortbl,inputenc,fancyhdr}
\\inputencoding{latin1}
\\renewcommand{\\familydefault}{\\sfdefault}

\\setlength{\\hoffset}{ $r_hoffset }
\\setlength{\\voffset}{ $r_voffset }

\\setlength{\\headsep}{10pt}
\\setlength{\\headheight}{14pt}
\\setlength{\\topmargin}{0in}

\\setlength{\\textwidth}{ $r_textwidth }
\\setlength{\\textheight}{ $r_textheight }

\\setlength{\\parindent}{0pt}
\\setlength{\\evensidemargin}{0in}
\\setlength{\\oddsidemargin}{0in}
\\setlength{\\extrarowheight}{4pt}
\\pagestyle{headings}
\\newcolumntype{G}{>{\\columncolor[gray]{ $r_grayscale }}p{$subjspace cm}}

\\pagestyle{fancy}
\\lhead{$schoolname -- $schoolyear}
\\rfoot{ $lex{Printed}: $currdate }
\\cfoot{ }

\\begin{document}
EOS1

}



#----------
sub calcGPA {
#----------

    my $gpa_ref = shift;
    my $maxterm = shift;

    eval require "../../etc/transcript.conf";
    if ( $@ ) {
	print $lex{Error}. ": $@<br>\n";
	die $lex{Error}. ": $@\n";
    }


    # Get credit and difficulty values for each subject.
    my $sth = $dbh->prepare("select credit, difficulty from subject where subjsec = ?");

    # Start TeX head
    print TEX "\\begin{tabular}{|p{$subjspace cm}|";
    for ( 1..$maxterm ){ print TEX "p{$markspace cm}|";}

    #print TEX "p{$markspace cm} |"; # removed if calcfinal...

    print TEX "}\\hline\n";
    print TEX "\\rowcolor[gray]{ $r_grayscale }";
    print TEX "\\bf ". $lex{Term}. q{ }. $lex{GPA};

    my ($totalGPQuality, $totalGPCredits); # needed for overall at end.

    # Loop through each term calculating the GPA for each term...
    foreach my $term (1..$maxterm )  { #loop through all terms
	# previously $finalterm from admin.conf..

	my ( $termCreditTotal, $termGPTotal );
	my ( $termCredits, $termAttempts ); # not used, yet. TermCredits includes P/F courses.
	my ( $termGPCredits, $termGPQuality );

	foreach my $subjsec ( keys %{$gpa_ref} ) {

	    if ( $gpa_ref->{$subjsec}[$term] ) {  # If we have a value for this term..

		my $mark = $gpa_ref->{$subjsec}[$term]; # could be letter or number...

		# Get Subject Data
		$sth->execute( $subjsec );
		if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; };
		my ($credit, $difficulty) = $sth->fetchrow;
		$credit = 1 if not $credit;
		if ( not $difficulty ) { $difficulty = 0; }

		# Convert the mark to a letter (and then to a quality score below...
		my $letter; # letter grade.
		if ( $mark =~ /[a-zA-Z]/ ) { # We have a 'letter' mark, copy into $letter 
		    $letter = $mark;
		} else { # find the matching letter for this numeric value..
		    foreach my $threshold (reverse sort keys %markToLetter ) { # large to small
			if ($mark >= $threshold) {
			    $letter = $markToLetter{$threshold};
			    # print "Mark: $mark Letter: $letter<br>\n";
			    last;
			}
		    }
		}

		# Now convert $letter to a quality score.
		# letterToQual hash in transcript.conf; add up quality scores and credit counts.
		# Check for missing Quality value in transcript.conf
		if ( ( not defined $letterToQual{ $letter } ) and 
		     ( $letter ne $passletter ) ) { 
		    print "<br>". $lex{'No Quality Score for'}. " $letter\n";
		    next; # skip to next subject. 
		}

		my $quality = $letterToQual{ $letter } + $difficulty;
		
		$termGPQuality += $quality * $credit;
		$totalGPQuality += $quality * $credit;

		$termAttempts += $credit;

		if ( $letter ne $failletter ) {
		    $termCredits += $credit;
		    $totalCredits += $credit;
		}

		# If not a fail and also not a pass/fail course
		if ( $letter ne $passletter ) {
		    $totalGPCredits += $credit;
		    $termGPCredits += $credit;
		}

		# print "$subjsec $term ". $gpa_ref->{$subjsec}[$term]. "<br>\n";
		
	    } # End of If mark exists
	} # End of Subject Loop for this term.

	my $gpa;
	if ($termGPCredits) {
	    $gpa = round( ($termGPQuality / $termGPCredits ), 2 );
	} 
	else { $gpa = 0; }

	print TEX "&\\hfil $gpa \\hfil "; # change to GPA value...
    
    } # End of terms loop


    my $cumgpa;
    if ($totalGPCredits) {
	$cumgpa = round( ($totalGPQuality / $totalGPCredits), 2 );
    } 
    else { $cumgpa = 0; }

    print TEX "\\\\ \\hline\n";
    print TEX "\\hfil\\bf ". $lex{'Current Year GPA'}. " $cumgpa \\hfil ";
    for ( 1..$maxterm ){ print TEX "& ";}

    print TEX "\\\\ \\hline \\end{tabular}\n\n";


} # end of calcGPA


#---------------
sub calcTermDays {
#---------------
    # this calculates the number of schooldays in a date range
    #  taking school holidays into account.
    # Requires: $dbh, Date::Business

    my ($startdate, $enddate) = @_;

    # It likes it in yyyymmdd format...
    $startdate =~ s/-//g;   # strip out hyphens.
    $enddate =~ s/-//g;

    $end = new Date::Business(DATE=>$enddate);
    $start = new Date::Business(DATE=>$startdate);
    my $schooldays = $end->diffb($start,'prev','next');
    $schooldays++;
    # We now have the number of school days in current term
    # excluding holidays/pd days.

    # Now find holidays
    my $sth = $dbh->prepare("select * from dates where 
      to_days(date) >= to_days('$startdate') and 
      to_days(date) <= to_days('$enddate')");
    $sth->execute;
    if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr;}
    my $holidays = $sth->rows;
    # Number of holidays in current term.

    my $termdays = $schooldays - $holidays;
    return $termdays;
}



#-----------------
sub calcAttendance {
#-----------------
    my ( $studnum, $startdate, $enddate, $periodsperday ) = @_;
    my $sth = $dbh->prepare("select * from attend 
     where studentid = ? and 
     to_days(absdate) >= to_days('$startdate') and 
     to_days(absdate) <= to_days('$enddate') 
     order by absdate,period");
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    $sth->execute( $studnum );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    my $absent = 0; my $late = 0;
    while ( my @absence = $sth->fetchrow ) {
	if ( $absence[3] =~ /$lex{Absent}/) { $absent++; }
	if ( $absence[3] =~ /$lex{Late}/) { $late++; }
    }
    if ( $periodsperday ) {
	$absent = $absent / $periodsperday; #convert periods into days
    }
    return "$absent:$late";
}


#------------ 
sub prTexHead {  # Print the head of each new report card 
#------------

    # globals: schoolname, $schoolyear
    my ($grade, $lastname, $firstname, $middlename, $birthdate, $provnum ) = @_;

    print TEX "\\thispagestyle{empty}\n\\setcounter{page}{1}\n";
    #print TEX "\\markright{$schoolname -- $schoolyear \\hfill ";
    print TEX "\\rhead{$firstname $middlename $lastname -- \\thepage}\n\\begin{center}\n";

    if ( $r_Showlogo ) {
	print TEX "\\includegraphics[width=$r_Logowidth]{$configpath/etc/$r_Logofile}\\quad \n";
	print TEX "\\parbox[b]{$r_Schooladdresswidth}{\\begin{center}{\\Large\\bf $schoolname}\\\\";
	if ( $schooladdr1 ) {
	    print TEX "$schooladdr1\\\\";
	}
	print TEX "$schoolcity, $schoolprov $schoolpcode\\\\";

	# Phones Lines
	if ( $schoolphone ) {
	    print TEX $lex{Phone}. " $schoolphone ";
	    if ( $r_ShowAdmins ) { print TEX "\\hfill "; } else { print TEX "\\qquad "; }
	}
	if ( $schoolfax ) {
	    print TEX $lex{Fax}. " $schoolfax";
	}
	print TEX "\\\\\n\n";

	# Administrator Lines
	if ( $r_Showadmins ) {
	    print TEX "$r_Principal \\hfill  $r_Viceprincipal \\\\\n\n";
	    print TEX "{\\footnotesize ". $lex{Principal}. "\\hfill ". $lex{'Vice-Principal'};
	    print TEX "}\\\\\n\n";
	}

	print TEX "\\vspace{$r_AddressSpacer}\n\n";
	print TEX " \\end{center}}\\qquad\n";

	if ( $r_Divlogofile ) {
	    print TEX "\\includegraphics[width=$r_Divlogowidth]{$configpath/etc/$r_Divlogofile}\\\\\n";
	}


    } else {
	print TEX "\n{\\sf\\Huge $schoolname}\n\n";
    }

    print TEX "\n\\smallskip\n{\\sf\\Large ". $lex{Grade}. " $grade ". $lex{'Report to Parents'};
    print TEX "}\n\n $currdate\n";
    print TEX "\n\\medskip\n{\\sf\\Large\\it $firstname $middlename $lastname} ";
    if ( $r_Showprovnum and $provnum ) {
	print TEX "{\\sf($provnum)}";
    }
    print TEX "\n\n";


    print TEX "{\\sf ($birthdate)}\n\\smallskip\n\n";
    print TEX "\\end{center}\n";


} # End of prTexHead


#---------------------
sub prHomeroomTeachers {
#---------------------
    my @teachers = @_;
    print TEX "\\begin{center}\n";
    print TEX "\\setlength{\\extrarowheight}{0pt}\n";
    print TEX "\\begin{tabular}{ll}\n{\\small\\bf  ";
    print TEX $lex{'Homeroom Teacher(s)'}. "}";
    foreach my $teach (@teachers){
	print TEX "& {\\it $teach} \\\\ \n";
    }
    print TEX "\\end{tabular}\n\\setlength{\\extrarowheight}{4pt}\n";
    print TEX "\\end{center}\n\n";

} # End of prHomeroomTeachers



#---------------------
sub print_grade_scheme {
#---------------------
    my $grade = shift;

    # Print Grading Scheme
    my $scheme = $g_evalsch{$grade}; # is letter a,b,c, etc. from repcard.conf
    my $first = 1;
    if (not $g_eval{$scheme}->[0]) { return;}  # return if blank scheme.

    print TEX "\\begin{center}{\\sf\\Large ";
    print TEX $lex{'Evaluation and Comment Key'}. "}\n\n";
    print TEX "\\parbox[b]{$assesskeywidth}";
    print TEX "{\\begin{multicols}{$assesskeycols}\n";

    for $item (@{$g_eval{$scheme}}){
	if (not $first){ 
	    print TEX "\\\\\n";
	} else { 
	    $first = 0;
	} 
	print TEX $item;
    }
    print TEX "\\end{multicols}}\n"; # Note parbox ending.
    print TEX "\\end{center}\n";

} # end of print grade scheme



#------------
sub prSubject { # Print subject entry
#------------

    # @eval array set with current subjsec, and refs...

    my ( $subjsec, $studnum )  = @_;

    # Get subject info for this subject
    my $sth = $dbh->prepare("select * from subject where subjsec = ?");
    $sth->execute( $subjsec );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my @subject = $sth->fetchrow;

    # Set Personal Growth Flag before latex filtering/escaping.
    my $personal_growth_flag; # flag for printing keys to personal growth evals
    if ( $subject[2] eq $r_personal_growth) {
	$personal_growth_flag = 1;
    } else {
	$personal_growth_flag = 0;
    }
	

    @subject = latex_filter( @subject );
    
    # Stuff subject questions into a question array, which 
    #  holds the descriptive text for each objective in the subject.
    my @qstarray;
    for ($a = 10;$a<=29; $a++) {
	if ( $subject[$a] ) { push @qstarray, $subject[$a]; }
    }


    my ($markcount, $marktotal, $markfinal);
    if ( $calcfinal ) {
	for (1..$#eval){ # Do each term
	    if ($eval[$_][7] =~ /[0-9]/){
		$markcount++;
		$marktotal = $marktotal + $eval[$_][7];
	    }
	}
	if ( $markcount ){
	    $markfinal = $marktotal / $markcount;
	    $markfinal = round( $markfinal, 0);
	}
    }

    # Check if we should add the Personal Growth Key...
    if ( $personal_growth_flag ) {  # print row with objectives...

	print TEX "{\\sf\\large\\bf $subject[2] ". $lex{'Evaluation Key'}. " }\n\n";
	#my $width = $subjspace + ($#eval * $markspace) + ($#eval * 0.5);
	#if ($calcfinal){ $width = $width + $markspace;}
	#print "Width: $width<br>\n";
	#print TEX " \\setlength{\\textwidth}{ $width cm }";
 	  #my $cols = $#eval + 1;
	  my $key = $r_pdevalgrp{$grade};
	  #print TEX "\\multicolumn{$cols}{|c|}{\\parbox{$width cm}{\n";
	  for my $item ( @{ $r_pdeval{$key} } ){
	      print TEX "$item\n\n";
	  }
	print TEX "\\setlength{\\textwidth}{$r_textwidth}\n";

    }


    print TEX "\\begin{tabular}{|p{$subjspace cm}|";
    for (1..$#eval){ print TEX "p{$markspace cm}|";}
    if ($calcfinal){ print TEX "p{$markspace cm}|";}
    print TEX "}\\hline\n";

    # print term line when commanded...
    if ( $printTermLine ){ # then we will...
	print TEX "\\rowcolor[gray]{ $r_grayscale }";
	for (1..$#eval){ print TEX "& \\hfil $term[$_] \\hfil"; }
	if ($calcfinal){ print TEX "& \\hfil ". $lex{Final}. " \\hfil"; }
	print TEX "\\\\ \\hline\n";
    }


    # Print Subject name, Teacher and term numbers.
    my ($tch,$tid) = split /\(/, $subject[4];
    $tid =~ s/\)//;
    my $sth = $dbh->prepare("select sal, firstname, lastname from staff where userid = ? ");
    $sth->execute( $tid );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my ($sal, $firstname, $lastname) = $sth->fetchrow;
    if ( $lastname ) { 
	$subjectteacher = "($sal $lastname)"; 
    }

    # Print Term Line 
    print TEX "\\rowcolor[gray]{ $r_grayscale }";
    print TEX "{\\sf\\large\\bf $subject[2]} $subjectteacher";
    for ( 1..$#eval ){ print TEX "& \\hfil $term[$_] \\hfil"; }
    if ($calcfinal){ print TEX "& \\hfil ". $lex{Average}. " \\hfil"; }
    print TEX "\\\\ \\hline\n";


    if ( $#qstarray > -1 ){  # -1 means 0 entries, 0 means 1; multiple assessment lines.
	my $qstcount = 7;
	foreach my $question ( @qstarray ) {
	    print TEX $question;
	    for (1..$#eval){
		print TEX "& \\hfil $eval[$_][$qstcount]\\hfil";
	    }
	    if ($calcfinal and $qstcount == 7){ # only print average for first.
		print TEX "& \\hfil $markfinal \\hfil";
	    } elsif ( $calcfinal ) { 
		print TEX "& ";
	    }
	    print TEX "\\\\ \\hline\n\n";
	    $qstcount++;
	}
	print TEX "\\end{tabular}\\\\";

    } else { # no assessments lines, only a mark...

	# Print Subject name and evaluation results.
	#print TEX "{\\sf\\large\\bf $subject[2]} $subjectteacher"; 

	my $absents = calcSubjectAttendance( $subjsec, $studnum );
	if ( defined $absents ) {
	    print TEX $lex{Absent}. " $absents ". $lex{'Period(s)'};
	}

	for ( 1..$#eval ){ print TEX "& \\hfil $eval[$_][7] \\hfil"; }
	if ($calcfinal){ print TEX "& \\hfil $markfinal \\hfil";}
	print TEX "\\\\ \\hline \\end{tabular}\n\n";

    }


} # End of prSubject



#------------
sub prComment { # print all the comments.
#------------

    my $comflag = 0; # Do we have any comments for this subject?
    print TEX "\n\\smallskip\n\n"; # small space
    if ( $clasavg and $classavg{$subjsec} ){ 
	print TEX "{\\bf ". $lex{'Class Average'}. ":} $classavg{$subjsec}\n\n";
    }
    for (1..$#eval){ if ($eval[$_][6]){ $comflag = 1;}}
    my $width = $subjspace + ($#eval * $markspace) + ($#eval * 0.6);
    if ($calcfinal){ $width = $width + $markspace;}
    if ($comflag){print TEX "\n\\parbox{$width cm}{\n";}
    for (1..$#eval){
	if ($eval[$_][6]){
	    # Note: This value already LaTeX filtered.
	    my $temp = $eval[$_][6];
	    $temp =~ s/\s*[\n|\r]/ /g;
	    print TEX "\n{\\bf ". $lex{Term}. " $_:} {\\it $temp }\n";
	    print TEX "\\smallskip \n";
	}
    }
    if ($comflag){
	print TEX "\n}\\smallskip\n\n"; # Note: brace ends parbox.
    } elsif ($clasavg){
	print TEX "\n\\medskip\n\n";
    }

} # End of prComment



#----------------------
sub prMonthlyAttendance {  # Version with monthly attendance info
#----------------------

    my $studnum = shift;

    # Needs globals: @schoolmonths - ordering of school months.
    #  %schooldays - days in each month in hash.
    #  $periodsperday - number of periods in the school day.

    # calculate attendance, one for each month.
    for ( 1..$#schoolmonths ) {
	my $absrec = calcMonthlyAttendance($studnum, $schoolmonths[$_],
					   $periodsperday);
	( $absent{$_}, $tardy{$_} ) = split /:/,$absrec;
	#print "SN:$studnum MN:$schoolmonths[$_] AB:$absent{$_} 
	#LT:$tardy{$_}<br>\n";
    }


    # calculate enrollment, one for each month
    # First find starting value for $active... was student enrolled at start
    my $sth = $dbh->prepare("select type from transfer where studnum = ?
      and to_days(date) >= to_days('$schoolstart')
      order by date desc");
    $sth->execute( $studnum );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
    my $etype = $sth->fetchrow;

    my $sth1 = $dbh->prepare("select lastname, firstname from studentall 
      where studnum = ?");
    $sth1->execute( $studnum );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
    my ($lastname, $firstname) = $sth1->fetchrow;


    # If withdraw during year or no record, must have been enrolled at start
    if ($etype eq 'withdraw' or not $etype){ $active = 1;} else { $active = 0;}
    #print "<b>$lastname, $firstname ETYPE: $etype  ACT: $active</b><br>\n";

    for my $monthidx (1..$#schoolmonths){
	my $enrolrec = calcMonthlyEnrollment( $studnum, $schoolmonths[ $monthidx ], $active);
	( $enrolled{ $monthidx }, $active ) = split /:/, $enrolrec;
	#print "---SN: $studnum ACT: $active<br>\n";
	#print "SN:$studnum  Month:$schoolmonths[$monthidx] ACT:$active<br>\n";
    }

    # Print Start of Attendance Table
     print TEX "\\begin{center}\n";

    # Setup Table
    print TEX "\\medskip\n\\begin{tabular}{|p{32mm}|\n";
    for (1..$#schoolmonths){
	print TEX "p{8mm}|";
    }
    print TEX "}\n";


    # Print Heading Line
    my $colcount = $#schoolmonths + 1;
    print TEX " \\multicolumn{$colcount}{\@{}c}{\\rule{0pt}{12pt} ";
    print TEX "\\sf\\bf\\Large ". $lex{Attendance}. "}\n\n";
    print TEX "\\\\\\hline\n";


    # Print Months Line
    print TEX "\\rowcolor[gray]{ $r_grayscale }\n";
    for (1..$#schoolmonths){
	my ($yr,$mn) = split /-/, $schoolmonths[$_];
	print TEX "& \\hfil $s_month[$mn] \\hfil \n";
    }
    print TEX "\\\\\\hline\n";

    # Print Days Open
    print TEX "{\\large\\bf ". $lex{'Days Open'}. "}\n";
    my $daysopen;
    for my $idx (1..$#schoolmonths){
	if ( $idx == $#schoolmonths and not $override ) { # last month in schoolmonths array.
	    # We are at end of current period... find the last day of this term.
	    $term =~ s/\s//g; # strip any spaces in the $term value...
	    my ($myear, $mmonth, $mday) = split /-/, $g_termend{ $term };
	    #print "Y: $myear M:$mmonth D:$mday<br>\n";
	    if ( $schoolmonths[$idx] eq "$myear-$mmonth" ) { # We are in last month of term.
		$daysopen = calcDaysBetween( "$myear-$mmonth-01" , $g_termend{ $term } );
		#$daysopen += 2; # add on either end day (1st and last day of term).
		print TEX "& \\hfil $daysopen \\hfil \n";
		#print "DAYS OPEN: $daysopen<br>\n";
	    } else {
		print TEX "& \\hfil $schooldays{$schoolmonths[$idx]} \\hfil \n";
	    }
	} else {
	    print TEX "& \\hfil $schooldays{$schoolmonths[$idx]} \\hfil \n";
	}
    }
    print TEX "\\\\\\hline\n";


    # Print Days Enrolled
    print TEX "{\\large\\bf ". $lex{'Days Enrolled'}. "}\n";
    for my $idx (1..$#schoolmonths){
	if ($schooldays{$schoolmonths[$idx]} != $enrolled{$idx} and 
	  $override and $enrolled{$idx} ) {
	    my $do = $schooldaysorig{$schoolmonths[$idx]}; # Days Open
	    my $ado = $schooldays{$schoolmonths[$idx]};  # Adjusted Days Open
	    my $de = $enrolled{$idx};  # Days Enrolled.
	    #print "<br>SCHOOL DAYS:",%schooldaysorig,"<br>\n";
	    #print "$student[1] $student[2] DO: $do  ADO: $ado DE: $de<br>\n";
	    my $ade = round( ($de * $ado / $do), 2);
	    print TEX "& \\hfil $ade \\hfil \n";
	    #print "$student[1] $student[2] MON: $schoolmonths[$idx] override<br>\n";
	} else {
	    if ( $idx == $#schoolmonths and $daysopen ) { # last month...
		print TEX "& \\hfil $daysopen  \\hfil \n";
		# was: $enrolled{$idx}; $daysopen set above.
	    } else {
		print TEX "& \\hfil $enrolled{$idx} \\hfil \n"; 
	    }
	}
    }
    print TEX "\\\\\\hline\n";


    # Print Days Absent
    print TEX "{\\large\\bf ". $lex{'Days Absent'}. "}\n";
    for (1..$#schoolmonths){
	$absent{$_} = round( $absent{$_}, 2);
	print TEX "& \\hfil $absent{$_} \\hfil\n"; 
    }
    print TEX "\\\\ \\hline\n";

    # Print Periods Late
    print TEX "{\\sf\\large\\bf ". $lex{'Periods Late'}. "}\n";
    for ( 1..$#schoolmonths ){ 
	print TEX "& \\hfil $tardy{$_} \\hfil\n"; 
    }
    print TEX "\\\\ \\hline\n";

    # Finish Attendance Box
    print TEX "\\end{tabular}\n";
    print TEX "\\end{center}\n"; 
    print TEX "\\bigskip\n";

}



#-----------------
sub prAttendance {
#-----------------
    # calculate attendance

    for (1..$term){
	$absrec = calcAttendance($studnum, $g_termstart{$_},$g_termend{$_},
	  $periodsperday);
	($absent{$_},$tardy{$_}) = split /:/,$absrec;
    }


    # Print Start of Attendance Table
    print TEX "\\newpage \\begin{center}{\\sf\\bf\\Large ";
    print TEX $lex{Attendance}. "}\n\n";
    print TEX "\\medskip\n\\begin{tabular}{|p{2in}|\n";
    for (1..$term){
	print TEX "p{0.75in}|";
    }

    # Print Title Line
    print TEX "}\\hline \n \\rowcolor[gray]{ $r_grayscale }\n";
    for (1..$term){
	print TEX "& \\hfil Term $_ \\hfil \n";
    }
    print TEX "\\\\\\hline\n";

    # Print Days Open
    print TEX "{\\sf\\large\\bf ". $lex{'Days Open'}. "}\n";
    for (1..$term){
	print TEX "& \\hfil $termdays{$_} \\hfil \n";
    }
    print TEX "\\\\\\hline\n";

    # Print Days Absent
    print TEX "{\\sf\\large\\bf ". $lex{'Days Absent'}. "}\n";
    for (1..$term){ 
	print TEX "& \\hfil $absent{$_} \\hfil\n"; 
    }
    print TEX "\\\\ \\hline\n";

    # Print Periods Late
    print TEX "{\\sf\\large\\bf ". $lex{'Periods Late'}. "}\n";
    for (1..$term){ 
	print TEX "& \\hfil $tardy{$_} \\hfil\n"; 
    }
    print TEX "\\\\ \\hline\n";

    # Finish Attendance Box
    print TEX "\\end{tabular}\\end{center}\n \\bigskip\n";

} # End of Attendance


#----------------
sub prAddComments {
#----------------
    my $studnum = shift;

    # Updated to show additional comment data for each student.

    # Now get student information for this student for this term.
    my $sth = $dbh->prepare("select eval.id, eval.comment, eval.term 
      from eval,subject where 
      subject.description = '$additionalcomments' and
      subject.subjsec = eval.subjcode and
      eval.studnum = '$studnum' 
      order by eval.term, eval.subjcode");
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}

    print TEX "{\\bf ". $lex{'Additional Comments'}. ":}\n\n";

    my $currterm = -1;
    my $prevterm;

    while (my ($id,$comment,$trm) = $sth->fetchrow) {
	if (not $comment) { next; }

	$comment =~ s/\s*[\n\r]/ /g;

	#print "I:$id S:$studnum T:$trm C:$comment<br>\n";
	$prevterm = $currterm;
	$currterm = $trm;
	if ($prevterm != $currterm) {
	    print TEX "{\\bf ". $lex{Term}. " $currterm:} ";
	}
	($comment) = latex_filter(($comment)); # list context required by sub
	print TEX "$comment \n\n";
    }

    print TEX "\\vspace{0.6in}\n\n";

}

 
#---------------
sub prTeacherSig {
#---------------
    print TEX "\\underline{\\hspace{2.5in}} \n\n";
    print TEX "{\\small ". $lex{Teacher}. "}\n\n \\medskip\n";
}

#-----------------
sub prPrincipalSig {
#-----------------
    print TEX "\\underline{\\hspace{2.5in}}\n\n";
    print TEX "{\\small ". $lex{'Principal'}. "}\n\n\\medskip\n";
}

#-------------
sub prComboSig { # both teacher and principal
#-------------
    print TEX "\\underline{\\hspace{2.5in}}";
    print TEX "\\hfill \\underline{\\hspace{2.5in}}\n\n";
    print TEX "{\\small ". $lex{Teacher}. "} \\hfill {\\small ";
    print TEX $lex{'Principal'}. "}\n\n\\medskip\n";
}

#------------
sub prTearoff {  # print tearoff line.
#------------

    # block these 2 sections together over page breaks
    if ( $tearoffblock ) { print TEX "\\parbox{\\textwidth}{ "; } 

    print TEX "\\dotfill\n\n\\bigskip\n\n{\\bf Parents/Guardians:}\n";
    print TEX $lex{'Please tear off this portion and return it to the school'};
    print TEX " \n";
    print TEX $lex{'in the report envelope with any comments and your signature'}. ".\n";
}

#--------------
sub prParentSig { # print parent signature.
#--------------
    my ($currdate, $lastname, $firstname, $middlename) = @_;
    print TEX "\n\n\\vspace{0.5in}\n\n\\underline{\\hspace{2.5in}}\n\n";
    print TEX "{\\small ". $lex{'Parent/Guardian Signature'}. "}\n\n\\medskip\n";
    print TEX "{\\bf ". $lex{'Student'}. ":} {\\it $firstname $middlename $lastname}\n\n";
    #print TEX "{\\bf ". $lex{Date}. ":} {\\it $currdate}\n\n";
    print TEX "{\\bf ". $lex{Comments}. ":}\n\n";

    if ( $tearoffblock ) { print TEX "} \n "; } 

}


#------------
sub prYearEnd {
#------------
    my $studentname = shift;

    my ($yearstart,$yearend) = split /-/, $schoolyear; # global

    my $nextgrade;
    if ($grade eq 'PK'){ $nextgrade = 'K';}
    elsif ($grade eq 'K'){ $nextgrade = 1;}
    else { $nextgrade = $grade + 1;}
    
    my $tempTex = $placementTex;

    # Replace variables.
    $tempTex =~ s/%studentname%/$studentname/;
    $tempTex =~ s/%nextgrade%/$nextgrade/;
    $tempTex =~ s/%schoolyearend%/$yearend/;

    print TEX "\\begin{center}\n\\bigskip\n\n";
    print TEX "\\framebox[7in][c]{";
    print TEX $tempTex;  # Variable values found in repcard.conf.
    print TEX "}\\end{center}\n"; #End of Framebox also.
}


#------------
sub calcFinal {
#------------
    # Passed values: ref to @eval array to do the calcs.
    my @localeval = ${shift};
    print @localeval; 
}


#------------------------
sub calcSubjectAttendance {
#------------------------
    use Number::Format qw(round);

    my ( $subjsec, $studnum ) = @_; # pass subject and student.

    my $periods; # periods absent/late

    my $sth = $dbh->prepare("select distinct reason, count(*) from attend 
      where subjsec = ? and studentid = ? group by reason");
    $sth->execute( $subjsec, $studnum );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    while ( my ( $reason, $count ) = $sth->fetchrow ) {
	#print "Reason: $reason Count: $count<br>\n";

	if ( $reason eq $absentUnexcused ) {
	    $periods += $count;
	} elsif ( $reason eq $lateUnexcused ) {
	    if ( $r_LateAbsentRatio ) {
		$periods += ( $count / $r_LateAbsentRatio );
	    } else {
		$periods += $count / 3; # default to 3 Late = 1 Absent
	    }
	}
	# No other reasons affecting.

    }

    if ( not defined $periods ) {
	return undef;
    } elsif ( not $periods ) {
	$periods = '0';
    }

    return round( $periods, 2);

}
