#! /usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('No Subject-Section Entered' => 'No Subject-Section Entered',
	   'Subject Mark Sheets' => 'Subject Mark Sheets',
	   'Main' => 'Main',
	   'Report Card' => 'Report Card',
	   'All' => 'All',
	   'View/Download Subject Marks' => 'View/Download Subject Marks',
	   'View Log File' => 'View Log File',
	   'Grade' => 'Grade',
	   'Error' => 'Error',
	   'WD' => 'WD',
	   'Mark' => 'Mark',

	   );


use DBI;
use CGI;

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

eval require "../../lib/liblatex.pl";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

# set the current date
my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, 
 $iddst) = localtime(time);
$year = $year + 1900;
$wday++; $mon++;
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";


# Get the current term.
open(TERM, "../../etc/term") || die "Can't find the term number!";
$term = <TERM>;

# Print the Page Header
print "$doctype\n<html><head><title>". $lex{'Subject Mark Sheets'}. "</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>\n";
print "[ <a href=\"$homepage\">". $lex{Main}. "</a> | \n";
print "<a href=\"$reppage\">". $lex{'Report Card'}. "</a> ]\n";


# Must be passed a subject/section code.
my $subjsec = $arr{subjsec};
if ( not $subjsec and not $arr{grade} ){ # $arr{grade} is for grade rec print.
    print "<h1>". $lex{'No Subject-Section Entered'}. "!</h1></body></html>\n";
    die;
}

if ( $arr{subjsec} ) { # If we have a single subject chosen..
    $subary[0] = $subjsec;
}

# Not currently set by HTML
if ( $arr{maxstudents} ) {
    $maxlines=$arr{maxstudents};
} else {
    $maxlines = 26;  # best tradeoff for vertical space for tilted objectives
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Setup TEX Code
my $shortname = "subjmark$$";
my $filename = "$shortname.tex";

open(TEX,">$filename") || die "Can't open tex file";

my $papersize;
if ( $defaultpapersize ) {
    $papersize = $defaultpapersize;
} else {
    $papersize = 'letterpaper';
}

print TEX "\\documentclass[12pt,$papersize]{article}
\\usepackage{array,rotating,inputenc}
\\inputencoding{latin1}
\\renewcommand{\\familydefault}{\\sfdefault}
\\pagestyle{empty}
\\setlength{\\textwidth}{8in}
\\setlength{\\textheight}{10.6in}
\\setlength{\\hoffset}{-0.8in}
\\setlength{\\voffset}{-2in}
\\setlength{\\evensidemargin}{0in}
\\setlength{\\oddsidemargin}{0in}
\\setlength{\\tabcolsep}{3.5pt}
\\setlength{\\extrarowheight}{6pt}\n";

print TEX "\\begin{document}\n";

# IF by Grade or just ONE. 
if ( $arr{grade} ) { # We are doing one grade

    if ( $arr{grade} eq $lex{All} ){
	$sth = $dbh->prepare("select distinct subject.subjsec, 
         subject.grade, subject.teacher, subject.description 
         from subject,eval
         where subject.subjsec = eval.subjcode and
         eval.term = '$term' 
         order by subject.grade, subject.teacher, subject.description");
    } else {
	$sth = $dbh->prepare("select distinct subjsec, 
         subject.teacher, subject.description 
         from subject,eval
         where subject.subjsec = eval.subjcode and
         eval.term = '$term' and
         subject.grade = '$arr{grade}'
         order by subject.description, subject.teacher");
    }
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die; }
    $subjectrows = $sth->rows;
    for (1..$subjectrows){
	my ($subjsec, $gr,$tch,$desc)  = $sth->fetchrow;
	push @subary,$subjsec;
    }
}


foreach my $subjsec ( @subary ) {

    # Get Subject Information
    my $sth = $dbh->prepare("select * from subject where subjsec = ?");
    $sth->execute( $subjsec );
    if ($DBI::errstr) { print $DBI::errstr; die; }
    my @subject = $sth->fetchrow;
    @subject = latex_filter( @subject );


    my @objary = ();
    # Now get the objectives and objective count
    for ( my $i=10;$i<30;$i++ ) {
	if ( $subject[$i] ) { push @objary,$subject[$i]; } # may truncate len
    }
    my $objcount = $#objary;
    if ($objcount < 1){ $objcount = 0; $objary[0] = $lex{Mark};} # Has atleast 1
    @objary = latex_filter( @objary );
    #print "Objective Count: $objcount";

    # Get all the kids for this subject/section, get names, sort, print
    $sth = $dbh->prepare("select distinct studnum from eval where subjcode = ?");
    $sth->execute( $subjsec );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my $rows = $sth->rows;
    @student = ();
    for (1..$rows){
	$studnum = $sth->fetchrow;
	push @student,$studnum;
    }

    # Next get their names
    @studentunsort = ();
    my $sth = $dbh->prepare("select lastname,firstname from studentall where studnum = ?");
    foreach my $studnum ( @student ){
	$sth->execute( $studnum );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my ($lastname,$firstname) = $sth->fetchrow;
	push @studentunsort,"$lastname:$firstname:$studnum";
    }

    @student = ();
    @student = sort @studentunsort;
    $studcount = @student;
    # We now have the student array all sorted out.

    @student = latex_filter( @student );

    # Print the header file...
    print_header(\@objary, \@subject );  # print header to tex file.

    my $studcount = @student;
    my $linecount = 0;

    # Loop through and print each student
    my $sth2 = $dbh->prepare("select count(*) from studentwd where studnum = ?");

    foreach my $student (@student){

	my ($lastname,$firstname,$studnum) = split /:/, $student;

	# Check for withdrawn and add flag.
	$sth2->execute( $studnum );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $wdflag;
	if ( ( my $wdcount = $sth2->fetchrow ) > 0 ) {
	    $wdflag = "{\\scriptsize ". $lex{WD}. q{\} };
	}


	$linecount++;
	if ( $linecount > $maxlines ) { # We'll print a new page header here
	    $linecount = 0;
	    print TEX "\\end{tabular}\\\\ \\newpage\n";
	    print_header( \@objary, \@subject );
	}

	# Get his/her evaluation for this subject in this term
	#  We will use the global $subjsec and $term
	$sth1 = $dbh->prepare("select * from eval where 
         studnum = ? and term = ? and subjcode = ?");
	$sth1->execute( $studnum, $term, $subjsec );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my @evalrec = $sth1->fetchrow;
	@evalrec = latex_filter( @evalrec );

	print TEX "\\raggedright $lastname, $firstname $wdflag& ";
	my $count = $objcount + 7;  # 7 is the first result in eval rec.
	for my $ct ( 7..$count ) { 
	    print TEX $evalrec[$ct];
	    if ( $ct != $count ){ print TEX "& "; }
	}
	print TEX "\\\\\\hline\n";  

    }  # End of Student For 

    print TEX "\\end{tabular}\\newpage\n\n";

} # End of multiple Subjects loop

print TEX "\\end{document}\n";
close TEX;


print "<center><h1><a href=\"$webdownloaddir/$shortname.pdf\">";
print $lex{'View/Download Subject Marks'}. "</a></h1>\n";
print "[ <a href=\"$reppage\">". $lex{'Report Card'}. "</a> |\n";
print "<a href=\"$webdownloaddir/pdflog$$.txt\">". $lex{'View Log File'}. "</a> ]";

system("$pdflatex $filename >pdflog$$.txt");
system("mv $shortname.pdf $downloaddir");
system("mv pdflog$$.txt $downloaddir");
system("rm -f $shortname.*");

print "</center></body></html>\n";


#---------------
sub print_header {
#---------------

    my $objref = shift;
    my $objcount = $#{$objref};
    my $subjectref = shift;

    # find longest string in objectives.
    my $len = 0; 
    foreach my $obj ( @{$objref} ) {
	if ( length($obj) > $len ){ $len = length($obj);}
    }

    my ($teacher,$teachid) = split /\(/,$subjectref->[4];
    #chop $teachid;  # chop trailing )

    my $offset = $len * 0.03 + 0.7; # very rough formula for strut spacing
    $offset = $offset . "in";
    print TEX "\\begin{tabular}{>{\\raggedright}b{50mm}";
    for (0..$objcount){	print TEX "p{6.1mm}";}

    print TEX "}\n ";
    print TEX "\\rule{0pt}{$offset}\n";
    print TEX "{\\bf\\Large $subjectref->[2]}\\\\";
    print TEX "$teacher \\\\";
    print TEX $lex{Grade}. ": $subjectref->[3]\\\\ &";

    for (0..$objcount){
	print TEX "\\hfil\\begin{rotate}{45}{\\small $objref->[$_]}\\end{rotate}\\hfil";
	if ($_ != $objcount){ print TEX "&\n";}
    }
    print TEX "\\\\ \n\\end{tabular}\n\n";

    # Now print header section WITH dividers
    print TEX "\\begin{tabular}{|p{50mm}|";
    for ( 0..$objcount ){ print TEX "p{6mm}|"; }
    print TEX "}\\hline\n";


} # end of print_header;
