#! /usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

use DBI;
use CGI;


my %lex = ( 'View/Download' => 'View/Download',
	    'Demographic Confirmations' => 'Demographic Confirmations',
	    'View Log File' => 'View Log File',
	    'Main' => 'Main',
	    'Student Confirmation' => 'Student Confirmation',
	    'Cannot open' => 'Cannot open',
	    'Error' => 'Error',
	    'Font Size' => 'Font Size',
	    'Paper Size' => 'Paper Size',
	    'Letter' => 'Letter',
	    'Legal' => 'Legal',
	    'A4' => 'A4',
	    'Use Page Break (if present)' => 'Use Page Break (if present)',
	    'Continue' => 'Continue',
	    'Grade' => 'Grade',
	    'Homeroom' => 'Homeroom',	    
	    'Blank=All' => 'Blank=All',
	    'Group' => 'Group',
	    'Values' => 'Values',
	    'Separate with Spaces' => 'Separate with Spaces',
	    'Lastname, Firstname' => 'Lastname, Firstname',
	    'Homeroom, Lastname, Firstname' => 'Homeroom, Lastname, Firstname',
	    'Grade, Lastname, Firstname' => 'Grade, Lastname, Firstname',
	    'Sort by' => 'Sort by',

	    );

# Read config variables
eval require "../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}
# Load Latex Filter
eval require "../lib/liblatex.pl";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

my $sort = $arr{sortorder};  # Do we sort by any order but Lastname, firstname?


my ($sec, $min, $hour, $mday, $mon, $year, $wday, 
    $yday, $iddst) = localtime(time);
$year = $year + 1900;
$mon++;
$wday++;
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";


# Page heading.
print "$doctype\n<html><head><title>". $lex{'Demographic Confirmations'}. "</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>[ <a href=\"$homepage\">". $lex{Main}. "</a> ]\n";

print "<center><h1>". $lex{'Demographic Confirmations'}. "</h1>\n";

if ( not $arr{page} ) {
    showStartPage();
} else {
    delete $arr{page};
}

my $fontsize = $arr{fontsize}. 'pt';
my $papersize;
if ( $arr{papersize} eq $lex{Letter} ) {
    $papersize = 'letterpaper';
    $textwidth = '191mm';
    $textheight = '254mm';
} elsif ( $arr{papersize} eq $lex{Legal} ) {
    $papersize = 'legalpaper';
    $textwidth = '191mm';
    $textheight = '330mm';
} elsif ( $arr{papersize} eq $lex{A4} ) {
    $textwidth = '185mm';
    $textheight = '264mm';
    $papersize = 'a4paper';
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Get passed group values
my @values = split /\s/, $arr{groupValue}; # split into array based on spaces
delete $arr{groupValue};
#print "Group", @group, "<br>\n";
     
my $groupType;
if ( $arr{groupType} eq $lex{Grade} ) {
    $groupType = 'grade';
} else {
    $groupType = 'homeroom';
}
delete $arr{groupType};

# Select students, sorted by lastname, firstname,
my $select;
if ( @values ) {
    $select = 'where ';
    $first = 1;
    foreach my $value ( @values ) {
	$value = $dbh->quote( $value );
	if ( not $first ) { $select .= ' or '; }
	$select .= "$groupType = $value ";
	$first = 0;
    }
}

if ($select) { print "Select: $select<br>\n"; }

my $sortorder = "lastname, firstname";
if ( $arr{sortorder} eq $lex{'Lastname, Firstname'} ) {
    $sortorder = "lastname, firstname";
} elsif ( $arr{sortorder} eq $lex{'Grade, Lastname, Firstname'} ) {
    $sortorder = "grade,lastname, firstname";
} elsif ( $arr{sortorder} eq $lex{'Homeroom, Lastname, Firstname'} ) {
    $sortorder = "homeroom, lastname, firstname";
}




# Read in Template
unless (open (FH,"<../template/democonf.tpl")) {
    print $lex{'Cannot open'}. " template - $!\n";
    die $lex{'Cannot open'}. " template - $!\n";
}
my $text;
{ local $/; $text = <FH>; close FH;}

if ( $arr{pagebreak} ) {
    $text =~ s/%\\newpage/\\newpage/g;
}

# Get Meta values: fieldid and arrayidx
$sth = $dbh->prepare("select fieldid, arrayidx from meta
  where tableid = 'student' order by arrayidx");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
while ( my ($fieldid, $arrayidx) = $sth->fetchrow ) {
    $meta[$arrayidx] = $fieldid;
}


my $shortname = "democonf$$";
my $filename = "$shortname.tex";

open(TEX,">$filename") || die "Can't open tex file";

print TEX "\\documentclass[$fontsize,$papersize,oneside]{article}
\\usepackage{graphicx,array,newcent,colortbl,inputenc}
\\inputencoding{latin1}
\\renewcommand{\\familydefault}{\\sfdefault}
\\usepackage[pdftex,bookmarks=false,pdfstartview=FitH]{hyperref}

\\setlength{\\textwidth}{$textwidth}
\\setlength{\\textheight}{$textheight}

\\setlength{\\hoffset}{-1in}
\\setlength{\\voffset}{-1in}
\\setlength{\\topmargin}{0.2in}
\\addtolength{\\evensidemargin}{0in}
\\addtolength{\\oddsidemargin}{0in}
\\setlength{\\extrarowheight}{2pt}
\\setlength{\\parindent}{0in}
\\pagestyle{myheadings}";

print TEX "\\markright{$schoolname\\hfill ",
$lex{'Student Confirmation'},"\\hfill $currdate -- Pg }
\\begin{document}\n";

if ( not $select ) { 
    $sortorder = 'grade, lastname, firstname';
} else {
    $sortorder = 'lastname, firstname';
}

$sth = $dbh->prepare("select * from student 
  $select order by $sortorder");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
my $rows = $sth->rows;

for ( my $i=1; $i<=$rows; $i++ ) {
    my @arr = $sth->fetchrow;
    @arr = latex_filter( @arr );

    # populate rec hash with rec{fieldid} = fieldvalue
    # @meta holds the fieldid's and @arr holds current record vals.
    for (0..$#arr) {
	$rec{$meta[$_]} = $arr[$_];
    }

    my $template = $text; # make a new blank record to plug in...

    # Now put replacement text back in.
    $template =~ s{\<\#(.*?)\#\>}
              { exists($rec{$1}) 
		  ? $rec{$1} 
	          : $1
	      }gsex;
    print TEX $template,"\n";


}  # End of Student For loop

print TEX "\\end{document}";
close TEX;

system("$pdflatex $filename >pdflog$$.txt");
system("mv $shortname.pdf $downloaddir");
system("mv pdflog$$.txt $downloaddir");
system("rm -f $shortname.*");

print "<center><h1><a href=\"$webdownloaddir/$shortname.pdf\">";
print $lex{'View/Download'}. ' '. $lex{'Demographic Confirmations'};
print "</a></h1>\n<p>[ <a href=\"$webdownloaddir/pdflog$$.txt\">$lex{'View Log File'}</a> |
<a href=\"$homepage\">$lex{Main}</a> ]</p></center>
</body></html>\n";


#----------------
sub showStartPage {
#----------------

    # Setup the form and start of table.
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";
    print "<table cellpadding=\"3\" border=\"0\" cellspacing=\"0\">\n";

    print "<tr><td align=\"right\">". $lex{Group}. "</td><td align=\"left\">\n";
    print "<select name=\"groupType\"><option>". $lex{Grade}. "</option>\n";
    print "<option>". $lex{Homeroom}. "</option></select>\n";
    print "</td></tr>\n";

    print "<tr><td align=\"right\">". $lex{Values}. "</td><td align=\"left\">\n";
    print "<input type=\"text\" name=\"groupValue\" size=\"25\"><br>\n";
    print $lex{'Separate with Spaces'}. q{, }. $lex{'Blank=All'}. "</td></tr>\n";

    print "<tr><td align=\"right\">". $lex{'Sort by'}. "\n";
    print "</td><td><select name=\"sortorder\"><option>". $lex{'Lastname, Firstname'};
    print "</option>\n";
    print "<option>". $lex{'Homeroom, Lastname, Firstname'}. "</option>\n";
    print "<option>". $lex{'Grade, Lastname, Firstname'}. "</option>\n";
    print "</select></td></tr>\n";

    print "<tr><td align=\"right\">". $lex{'Font Size'}. "</td><td align=\"left\">\n";
    print "<select name=\"fontsize\"><option>12</option>\n";
    print "<option>11</option><option>10</option></select></td></tr>\n";
   
    print "<tr><td align=\"right\">". $lex{'Paper Size'}. "</td><td align=\"left\">\n";
    print "<select name=\"papersize\"><option>". $lex{Letter}. "</option>\n";
    print "<option>". $lex{Legal}. "</option><option>". $lex{A4}. "</option></select>\n";
    print "</td></tr>\n";

    print "<tr><td align=\"right\">". $lex{'Use Page Break (if present)'};
    print "</td><td align=\"left\">\n";
    print "<input type=\"checkbox\" name=\"pagebreak\" value=\"1\" CHECKED>\n";
    print "</td></tr>\n";

    print "<tr><td></td><td align=\"left\">";
    print "<input type=\"submit\" value=\"". $lex{Continue}. "\">";
    print "</td></tr>\n";

    print "</table></form>\n";
    print "</body></html>\n";

    exit;

}
