#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# File: rptenrolmon.pl
# Monthly Enrollment report for Ethnic change reports
# 
# Outline: 1) Find current enrollment 
#  2) Count back to month of interest.
#  3) Find number of recorded transfers in that month and categorize into 
#     non-native, metis/status native, etc. on the basis of came or went.

my $ignoregrade = 'PK'; # grade to ignore in calculations.
# Comment out if you want all grades counted in this report.

my %lex = ('Monthly Enrollment Change' => 'Monthly Enrollment Change',
	   'Month of' => 'Month of',
	   'Enrollment at Start of month' => 'Enrollment at Start of month',
	   'Change in Enrollment' => 'Change in Enrollment',
	   'Enrollment at End of month' => 'Enrollment at End of month',
	   'Main' => 'Main',
	   'Current Date' => 'Current Date',
	   'Aging Date' => 'Aging Date',
	   'Category' => 'Category',
	   'Enrollments' => 'Enrollments',
	   'Withdrawals' => 'Withdrawals',
	   'Enrollment Summary' => 'Enrollment Summary',
	   'Error' => 'Error',
	   'YYYY-MM-DD' => 'YYYY-MM-DD',
	   'Date format Error' => 'Date format Error',

	   );


use DBI;
use CGI;

# Read config variables
eval require "../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, 
 $iddst) = localtime(time);
$year = $year + 1900;
$mon++;
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";



print "$doctype\n<html><head><title>". $lex{'Monthly Enrollment Change'}. "</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>[ <a href=\"$homepage\">". $lex{Main}. "</a> ]\n";

foreach my $type (@ethnic){
    $ethnicount{$type} = [0,0];
}

# No date defined.
my ($yr, $mo, $da);
if ( not $arr{date} ) { # We'll go back 1 month.
    if ( $mon > 1 ){ # if not January (ie. 1)
	$mo = $mon - 1;
	$yr = $year;
    } else {  # month is 1, go to prev dec.
	$mo = 12;
	$yr = $year - 1;
    }
    $da = "01";
} else { # We have been passed a date.
    ($yr, $mo, $da) = split /-/, $arr{date};
    $da = "01";
}

my $agingdate = "$yr-$mo-$da";

if ( length($mo) == 1 ){ $mo = '0'.$mo;}
$searchdate = "$yr-$mo-__"; # search for days in that month...

if ($yr == 0 or $mo == 0 or $da == 0) {
    print '<center><h1>';
    print $lex{'Date format Error'}. " (". $lex{'YYYY-MM-DD'}. ")";
    print "</h1><form><input type=\"button\" onclick=\"javascript:history.back()\" ";
    print "value=\"Back!\">\n";
    print "</form></body></html>\n"; 
    die; 
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# populate the @ethnic hash...
my @ethnic;
my $sth = $dbh->prepare("select distinct ethnic from student where ethnic != '' order by ethnic");
$sth->execute;
while ( my $eth = $sth->fetchrow ) {
    push @ethnic, $eth;
}


my $select = '';
if ( $ignoregrade ){ # if defined, skip this grade.
    $select = "where grade != '$ignoregrade'";
}

# First count how many currently enrolled (in student table)
$sth = $dbh->prepare("select count(*) from student $select");
$sth->execute;
my $curenrol = $sth->fetchrow;
#print "Current Enrollment: $curenrol<br>\n";

# Now count backwards through entry records until we reach date of interest
$sth1 = $dbh->prepare("select * from transfer 
 where to_days(date) >= to_days('$agingdate') order by date desc");
# NOTE: Quotes are needed inside the to_days function for aging date! ??
$sth1->execute;
if ($DBI::errstr) { print $DBI::errstr; die; }

while (@arr = $sth1->fetchrow){
    #print $arr[3], "&nbsp;&nbsp;";
    if ($arr[3] eq 'enrol' or $arr[3] eq 're-enrol') {
	$curenrol--;
    } elsif ($arr[3] eq 'withdraw') {
	$curenrol++;
    } else {
	# We have a programming error!
	print "Programming Error! $arr[3] for student $arr[1] on $arr[2]";
	die;
    }
}

$startenrol = $curenrol;
#print "Start Enrol: $startenrol<br>\n";

# $startenrol Should have the enrollment on the first day of the month 
#  BEFORE any enrollment on that first day. 

# Now let's find all records in that month, and calc the ethnic results.
$sth1 = $dbh->prepare("select * from transfer
  where date like '$searchdate'
  order by date desc");
$sth1->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
$entryrows = $sth1->rows;

#print "Date: $searchdate Entryrows: $entryrows<br>\n";

my $change = 0;

# Loop through the month adding up the counts in each ethnic category.
while ( @arr = $sth1->fetchrow ){
    $sth2 = $dbh->prepare("select * from student where studnum = '$arr[1]'");
    $sth2->execute;
    if ($DBI::errstr) {print $DBI::errstr; die;}
    $studentrows = $sth2->rows;

    if ( $studentrows < 1 ) {  # Student not found in student table; look in other
	$sth2 = $dbh->prepare("select * from studentwd where studnum = ?");
	$sth2->execute( $arr[1] );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    }
    @student = $sth2->fetchrow;

    # Skip PK or other set grade.
    if ( $student[5] eq $ignoregrade ) { 
	#print "$student[1] $student[2] G:$student[5] D:$arr[2]<br>\n";
	next;
    } 
  
    foreach $type (@ethnic){  # Go through each ethnic type
	if ($student[12] eq $type) {  # We have an ethnic type match.
	    if ($arr[3] eq 'enrol' or $arr[3] eq 're-enrol') {
		$ethnicount{$type}->[0]++;  # Zero are enrollment of this type
		$change++;
	    } elsif ($arr[3] eq 'withdraw') {
		$ethnicount{$type}->[1]++;  # Ones are withdrawals of this type.
		$change--;
	    }
	}
    }
    
} # End of Loop to count all transfers in month and evaluate

$endenrol = $startenrol + $change;

print "<center><h1>$schoolname ". $lex{'Monthly Enrollment Change'}. "</h1>\n<p><b>";
print $lex{'Current Date'}. ":</b> $currdate<br><b>";
print $lex{'Aging Date'}. ":</b> $agingdate</p>\n";

print '<caption><b>'. $lex{'Month of'}. " $month[$mo], $yr</b></caption>\n";


print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
print '<tr><th>'. $lex{Category}. '</th><th>'. $lex{Enrollments};
print '</th><th>'. $lex{Withdrawals}. "</th></tr>\n";

# Now print out what was happening in each category...
foreach $type (@ethnic){  # Go through each ethnic type
    print "<tr><td>$type</td>\n";
    print "<td align=\"center\">$ethnicount{$type}->[0]</td>\n";
    print "<td align=\"center\">$ethnicount{$type}->[1]</td></tr>\n";
}

print "</table><p></p>\n";

print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">
<tr><th colspan=\"2\">". $lex{'Enrollment Summary'}. "</th></tr>
<tr><td>". $lex{'Enrollment at Start of month'}. "</td><td>$startenrol</td></tr>
<tr><td>". $lex{'Change in Enrollment'}. "</td><td>$change</td></tr>
<tr><td>". $lex{'Enrollment at End of month'}. "</td><td>$endenrol</td></tr>\n";

print "</table></center></body></html>\n";
