#! /usr/bin/perl
#  Copyright 2001-2005 Leslie Richardson

#  This file is part of Open Admin for Schools.

# ======= Query Function for Courses
# Passed: studnum - local student number
# startdate, enddate - for query period.
# type:  Student, Current, Dept - three types of query
# If Student - then pass identification (prov num and bdate)
# If Current or Dept - then pass dates and school number. Scope code 
#  identifies Current/Dept.


use DBI;
use CGI;
use XML::Writer;
use XML::Writer::String;
use Data::UUID;
use IO;
use HTTP::Request::Common qw(POST); 
use HTTP::Headers; 
use LWP::UserAgent; 
use XML::LibXML;

# Read config variables
require "../../etc/admin.conf" || die "Cannot read admin.conf: $!";
require "slxmllib.pl" || die "Cannot read slxmllib.pl: $!";

$q = new CGI;
%arr = $q->Vars;
print $q->header;

# Passed Values 
my $startdate = $arr{startdate};
my $enddate = $arr{enddate};
my $studnum = $arr{studnum};
my $querytype = $arr{type};


print "$doctype\n<html><head><title>SDS Query: Courses</title> 
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>[ <a href=\"$homepage\">Main</a> | 
<a href=\"$exppage\">Export</a> ]<h1>Query Courses</h1>\n";


@month = ("","January","February","March","April","May","June","July",
	  "August","September","October","November","December");
# $sec, $min, $hour, $mday, $mon, 
# $year, $wday, $yday, $isdst ) = localtime(time);

@tim = localtime(time);
$year = $tim[5] + 1900;
$tim[4]++;
for (0..4){if (length($tim[$_]) == 1){ $tim[$_] = '0'.$tim[$_];}}
$currdate = "$year-$tim[4]-$tim[3]";
$currtime = "$tim[2]:$tim[1]:$tim[0]";

$dsn = "DBI:$dbtype:dbname=$dbase";
$dbh = DBI->connect($dsn,$user,$password);

#print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">
#<tr><th>Student Queried</th></tr>\n";

# Create a new user agent
my $ua = LWP::UserAgent->new();
$ua->agent("OpenAdmin");

# Open Update and Error Log files for appending.
open(ERR,">>error.log") or die "Cannot open error.log: $!\n";
open(UPD,">>update.log") or die "Cannot open update.log: $!\n";


if ($querytype eq 'Student'){
    # prep for retrieving from student data
    $sth = $dbh->prepare("select lastname, firstname, provnum, birthdate
       from student where studnum = '$studnum'");
    $sth->execute; 
    if ($DBI::errstr) { print $DBI::errstr; die;}
    my ($lastname, $firstname, $provnum,$birthdate) = $sth->fetchrow;
    if (not $provnum or not $birthdate){ # not found
	print "No Student Found!</body></html>";
	die;
    }
    &mkStudentQuery($provnum,$birthdate); # Generate query file.
    
} elsif ($querytype eq 'Current' or $querytype eq 'Dept'){
    &mkCurrDeptQuery($querytype,$startdate,$enddate,$file); # Generate query.
}


# Create the https post request 
my $req = POST $url, [ XML=> $output->value ];

$req->content_type('application/xml;charset="utf-8"');
$req->authorization_basic($sds_userid, $sds_password); 

# Issue the request and receive a response
my $res = $ua->request($req);

# Check the status of the response
if ($res->is_error) {
    print "Transfer Error:", $res->status_line, "\n";
    die;
}

#open(FILE,">responsecrse$$.xml") or die "Cannot open xml output file!\n";
#print FILE $res->content;
#close FILE;

# Parse the response.
my $parser = XML::LibXML->new();

eval {$doc = $parser->parse_string($res->content)};
if ($@){
    print "<pre>Sask Learning Error:\n",$res->content,"</pre><br>\n";
    print "</body></html>\n"; die;
}

$doc->setEncoding('UTF-8');
$root = $doc->getDocumentElement;
$root->setNamespace($xmlns,'sl',1);

# print Message ID.
#$mastermsgid = $root->findvalue('//sl:SL_MsgId');

$status = $root->findvalue('//sl:SL_Status/sl:SL_StatusCode');
print "<i>Response:</i> $status</p>\n";

if ($status eq 'Errors' or $status eq 'Invalid'){ 
    # Print out error and add to errorlog.
    &prErr;
} elsif ($status eq 'Successful'){ 
    &parseStudentCourseHistory;
} else { # print warnings... ($status eq 'Warnings')
    print "There were warnings...";
}


# Now parse for folks with prov number but not on our system...

#open (FILE,">>resp$$.xml");

#foreach my $key (keys %notfound){
    #print "K:$key  V:$notfound{$key}<br>\n";
    # birthdate, provnum
#    &findMissing($key,$notfound{$key});
#}
#close FILE;

print "</body></html>\n";


# Functions =============================

#--------------
sub mkStudentQuery {
#--------------
    my $provnum = shift;
    my $birthdate = shift;
    # Date and schoolnumber are globals.

    # Create Writer Instance
    $output = new XML::Writer::String;

    $wr = new XML::Writer(OUTPUT => $output, 
			  DATA_MODE => '1', 
			  DATA_INDENT => '2');

    # Set XML Header and write Root Element
    $wr->xmlDecl("UTF-8");
    $wr->startTag('SL_Message','xmlns' =>$xmlns, 
		  'xmlns:xsi' =>$xmlnsxsi, 
		  'xsi:schemaLocation' => $xsischemaLocation);

    $wr->startTag('SL_Request');
    &mkSL_Header($currdate, $currtime, $schoolnumber,$idcount);

    $wr->startTag('SL_Query');

    $wr->startTag('SL_QueryObject',
		  'ObjectName' => 'StudentCourseHistory');
    # Note: No Scope Code should be required above....

    $wr->dataElement('DeptAssignedPersonId',$provnum); 
    $wr->dataElement('BirthDate',$birthdate); 	
    
    $wr->endTag('SL_QueryObject');

    $wr->endTag('SL_Query');
    $wr->endTag('SL_Request');
    $wr->endTag('SL_Message');
    $wr->end();

}


#--------------
sub mkCurrDeptQuery {
#--------------
    my $currdept = shift;
    my $startdate = shift;
    my $enddate = shift;
    # schoolnumber is global.

    # Create Writer Instance
    $output = new XML::Writer::String;

    $wr = new XML::Writer(OUTPUT => $output, 
			  DATA_MODE => '1', 
			  DATA_INDENT => '2');

    # Set XML Header and write Root Element
    $wr->xmlDecl("UTF-8");
    $wr->startTag('SL_Message','xmlns' =>$xmlns, 
		  'xmlns:xsi' =>$xmlnsxsi, 
		  'xsi:schemaLocation' => $xsischemaLocation);

    $wr->startTag('SL_Request');
    &mkSL_Header($currdate, $currtime, $schoolnumber,$idcount);

    $wr->startTag('SL_Query');

    $wr->startTag('SL_QueryObject',
		  'ObjectName' => 'StudentCourseHistory',
		  'ScopeCode' => "$currdept");

    $wr->dataElement('SchoolId',$schoolnumber); # Global! 
    $wr->dataElement('FromDate',$startdate); 
    $wr->dataElement('ToDate',$enddate); 
    
    $wr->endTag('SL_QueryObject');

    $wr->endTag('SL_Query');
    $wr->endTag('SL_Request');
    $wr->endTag('SL_Message');
    $wr->end();

}


#---------
sub prErr {  # Print Errors to Screen and Errorlog
#---------
    print "<br><font color=\"Red\" size=\"+1\">Errors</font><br>\n";
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>Student ID</th><th>Object Name</th>";
    print "<th>Error Message</th></tr>\n";

    my @errors = $root->findnodes('//sl:SL_Error');
    foreach my $error (@errors){
	# Get Error Elements
	my $objectname = $error->getAttribute('ObjectName');
	my $errmsg = $error->findvalue('./sl:SL_ErrorMsg');
	$errmsg =~ s/[\r|\n]/ /g;
	my $errcode = $error->findvalue('./sl:SL_ErrorCode');
	
	# Write the Error Log
	print ERR "$mastermsgid:$lastname,$firstname($studnum)";
	print ERR ":$errmsg:$errcode\n";
	
	print "<tr><td>$lastname, $firstname ($studnum)</td>";
	print "<td>$studentrefid ($objectname)</td>";
	print "<td>$errmsg</td></tr>\n";
    }
    print "</table>\n";
}


#----------------------
sub parseStudentCourseHistory { # passed document root object.
#----------------------

    my @courses = $root->findnodes('//sl:StudentCourseHistory');
    
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>Name (Stud#/Bdate)</th><th>Course</th><th>Year</th>
     <th>Drop</th><th>Mark</th><th>Credits</th></tr>\n";


    foreach my $course (@courses){
	my $provnum = $course->findvalue('sl:StudentIdentification/sl:DeptAssignedPersonId');
	my $birthdate = $course->findvalue('sl:StudentIdentification/sl:BirthDate');
	my $courseid = $course->findvalue('sl:DeptAssignedCourseId');
	my $coursetitle = $course->findvalue('sl:CourseTitle');

	my $schoolyear = $course->findvalue('sl:StudentCourseInfo/sl:SchoolYear');	
	my $dropped = $course->findvalue('sl:StudentCourseInfo/sl:Dropped/@Code');
	my $finalmark = $course->findvalue('sl:StudentCourseInfo/sl:FinalMark');
	my $credits = $course->findvalue('sl:StudentCourseInfo/sl:CreditsEarned');

	my $sth = $dbh->prepare("select lastname, firstname from studentall
         where provnum = '$provnum'");
	$sth->execute; 
	if ($DBI::errstr) { print $DBI::errstr; die;}
	my ($lastname, $firstname) = $sth->fetchrow;
	if (not $lastname){ # not found...let's ask SL.
	    $notfound{$birthdate} = $provnum;
	}
        print "<tr><td><b>$lastname</b>,$firstname ($provnum/$birthdate)</td>
         <td>$coursetitle ($courseid)</td><td>$schoolyear</td><td>$dropped</td>
         <td>$finalmark</td><td>$credits</td></tr>\n";

    }
    print "</table>\n";

} # End of Sub



#--------------
sub findMissing { # find kids missing prov num locally, pulled from course
#--------------

    my $birthdate = shift;
    my $provnum = shift;

    # Grade and Provincial Number passed
    $count++;
    &mkQueryString($birthdate,$provnum,$count); # Generate $output string

    # print QRY $output->value; # not response 

    # Create the https post request 
    my $req = POST $url, [ XML=>$output->value ];


    $req->content_type('application/xml;charset="utf-8"');
    $req->authorization_basic($sds_userid, $sds_password); 

    # Issue the request and receive a response
    my $res = $ua->request($req);

    # Check the status of the response
    if ($res->is_success) {
	#print " <i>Transfer Successful.</i>\n";
	#open(FILE,">response$$.xml") or die "Cannot open xml output file!\n";
	print FILE $res->content;
	#close FILE;

	# Parse the response.
	my $parser = XML::LibXML->new();

	eval {$doc = $parser->parse_string($res->content)};
	if ($@){
	    print "<pre>Sask Learning Error:\n",$res->content,"</pre><br>\n";
	    print "</body></html>\n"; die;
	}

	$doc->setEncoding('UTF-8');
	$root = $doc->getDocumentElement;
	$root->setNamespace($xmlns,'sl',1);

	$status = $root->findvalue('//sl:SL_Status/sl:SL_StatusCode');
	#print "<i>Response:</i> $status</p>\n";

	if ($status eq 'Errors' or $status eq 'Invalid'){ 
	    # Print out error and add to errorlog.
	    &prErr($mastermsgid,"$lastname, $firstname ($studnum)");

	} elsif ($status eq 'Successful'){ 
	    &parseStudentPersonal;
	} else { # print warnings... ($status eq 'Warnings')
	    print "There were warnings...";
	}

    }  else { # Transfer Error!
	my $err = $res->status_line;
	print "<h1>Transfer Error: $err</h1>";
    }

} # End of findMissing Sub.


#----------------
sub mkQueryString {
#----------------
    my $grade = shift;
    my $provnum = shift;
    my $idcount = shift;
    # Date and schoolnumber are globals.

    # Create Writer Instance
    $output = new XML::Writer::String;

    $wr = new XML::Writer(OUTPUT => $output, 
			  DATA_MODE => '0', 
			  DATA_INDENT => '2');

    # Set XML Header and write Root Element
    $wr->xmlDecl("UTF-8");
    $wr->startTag('SL_Message','xmlns' =>$xmlns, 
		  'xmlns:xsi' =>$xmlnsxsi, 
		  'xsi:schemaLocation' => $xsischemaLocation);

    $wr->startTag('SL_Request');
    &mkSL_Header($currdate, $currtime, $schoolnumber,$idcount);

    $wr->startTag('SL_Query');

    $wr->startTag('SL_QueryObject',
		  'ObjectName' => 'StudentPersonal');

    $wr->dataElement('DeptAssignedPersonId',$provnum); 
    $wr->dataElement('BirthDate',$grade); 	
    
    $wr->endTag('SL_QueryObject');

    $wr->endTag('SL_Query');
    $wr->endTag('SL_Request');
    $wr->endTag('SL_Message');
    $wr->end();
    
}

#---------
sub prErr {  # Print Errors to Screen and Errorlog
#---------
    my $msgid = shift;
    my $name = shift;

    #print "<p style=\"color:red; font-size:150%\">Error</p>\n";
    my @errors = $root->findnodes('//sl:SL_Error');
    foreach my $error (@errors){
	# Get Error Elements
	my $objectname = $error->getAttribute('ObjectName');
	my $errmsg = $error->findvalue('./sl:SL_ErrorMsg');
	$errmsg =~ s/[\r|\n]/ /g;
	my $errcode = $error->findvalue('./sl:SL_ErrorCode');
	
	# Write the Error Log
	print ERR "$msgid:$name:$errmsg:$errcode\n";
	
	print "<p style=\"color:red; font-size:120%\">$errmsg</p>\n";
    }
    print "</table>\n";
    
}


#----------------------
sub parseStudentPersonal { # passed document root object.
#----------------------
    my @studinfo = $root->findnodes('//sl:StudentPersonal/sl:StudentInfo');
    # skip the student identification element, only StudentInfo.
    # Student Info: Name,Demographics,StudentAddress,PhoneNumber objects
    foreach my $student (@studinfo){
	my $provnum = $student->findvalue('//sl:DeptAssignedPersonId');
	my $studnum = $student->findvalue('//sl:SchoolAssignedPersonId');
	my $firstname = $student->findvalue('//sl:Name/sl:FirstName');
	my $lastname = $student->findvalue('//sl:Name/sl:LastName');
	my $middlename = $student->findvalue('//sl:Name/sl:MiddleName');
	my $gender = $student->findvalue('//sl:Demographics/sl:Gender');
	my $language = $student->findvalue('//sl:Demographics/sl:Language');
	my $address = $student->findvalue('//sl:StudentAddress/sl:Address/sl:Street');
	my $city = $student->findvalue('//sl:StudentAddress/sl:Address/sl:City');
	my $prov = $student->findvalue('//sl:StudentAddress/sl:Address/sl:StatePr/@Code');
	my $country = $student->findvalue('//sl:StudentAddress/sl:Address/sl:Country/@Code');
	my $pcode = $student->findvalue('//sl:StudentAddress/sl:Address/sl:PostalCode');
	my $phone = $student->findvalue('//sl:PhoneNumber');
	
	print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
	print "<tr><th align=\"right\">Name</th>
	    <th>$firstname $middlename $lastname ($studnum)</th></tr>\n";
        print "<tr><td align=\"right\">Prov Num</td><td>$provnum</td></tr>\n";
        print "<tr><td align=\"right\">Gender</td><td>$gender</td></tr>\n";
	print "<tr><td align=\"right\">Lang</td><td>$language</td></tr>\n";
	print "<tr><td align=\"right\">Address</td><td>$address</td></tr>\n";
	print "<tr><td align=\"right\">City</td><td>$city</td></tr>\n";
	print "<tr><td align=\"right\">Prov</td><td>$prov</td></tr>\n";
	print "<tr><td align=\"right\">Country</td><td>$country</td></tr>\n";
	print "<tr><td align=\"right\">Postal Code</td><td>$pcode</td></tr>\n";
	print "<tr><td align=\"right\">Phone</td><td>$phone</td></tr>\n";
	print "</table>\n";

	# Now see if they have a missing prov num.
	my $sth = $dbh->prepare("select studnum, provnum from studentall 
          where lastname = '$lastname' and birthdate = '$birthdate'");
	$sth->execute;
	if ($DBI::errstr){ print $DBI::errstr;}
	my ($sn, $pn) = $sth->fetchrow;
	if ($sn and not $pn){ # we have someone with a match.
	    print "<p>Local Match: $sn</p>\n";
	} elsif (not $sn){
	    print "<p>No Match for Lastname and Birthdate in local recs</p>\n";
	}

    }
} # End of Sub
