#! /usr/bin/perl
#  Copyright 2001-2005 Leslie Richardson

#  This file is part of Open Admin for Schools.

# Query Function for DeptCourses
# Passed Values: none.

use DBI;
use CGI;
use XML::Writer;
use XML::Writer::String;
use Data::UUID;
use HTTP::Request::Common qw(POST); 
use HTTP::Headers; 
use LWP::UserAgent; 
use XML::LibXML;

# Read config variables
require "../../etc/admin.conf" or die "Cannot read admin.conf: $!";
require "slxmllib.pl" or die "Cannot read slxmllib.pl: $!";

# Override V1 with V2 values
# SDS XML Schema Declarations
$xmlns = "http://www.sasked.gov.sk.ca/xsd/sl/2.x/SLMessage_V2.xsd";
$xmlnsxsi = "http://www.w3.org/2001/XMLSchema-instance";
$xsischemaLocation="http://www.sasked.gov.sk.ca/xsd/sl/2.x/SLMessage_V2.xsd
http://www.sasked.gov.sk.ca/xsd/sl/2.x/SLMessage_V2.xsd";

$q = new CGI;
%arr = $q->Vars;
print $q->header;

if ($arr{debug}){
    $debug = 1;
    delete $arr{debug};
}


print "$doctype\n<html><head><title>SDS Query: Dept Courses</title> 
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
<style type=\"text/css\">
body {
  padding: 1em 3em;
}
</style>
$chartype\n</head><body>[ <a href=\"$homepage\">Main</a> | 
<a href=\"$exppage\">Export</a> ]
<h1>Query Sask Learning Courses</h1>\n";

@month = ("","January","February","March","April","May","June","July",
	  "August","September","October","November","December");
# $sec, $min, $hour, $mday, $mon, 
# $year, $wday, $yday, $isdst ) = localtime(time);

@tim = localtime(time);
$year = $tim[5] + 1900;
$tim[4]++;
for (0..4){if (length($tim[$_]) == 1){ $tim[$_] = '0'.$tim[$_];}}
$currdate = "$year-$tim[4]-$tim[3]";
$currtime = "$tim[2]:$tim[1]:$tim[0]";

$dsn = "DBI:$dbtype:dbname=$dbase";
$dbh = DBI->connect($dsn,$user,$password);

# Create a new user agent
my $ua = LWP::UserAgent->new();
$ua->agent("OpenAdmin");

# Grade and Provincial Number passed
$count++;
&mkGeneralQuery($count); # Generate $output string

# DEBUG Data Errors
if ($debug){
    print "<div style=\"color: blue;\">\n";
    print "<h1>DEBUG - Request sent to Sask Learning</h1>\n";
    my $temp = $output->value;
    $temp =~ s/</&lt;/g;
    $temp =~ s/>/&gt;/g;
    print "<pre>",$temp,"</pre></div><p></p>\n";
}

# Create the https post request 

my $req = POST $url, [ XML=>$output->value ];

$req->content_type('application/xml;charset="utf-8"');
$req->authorization_basic($sds_userid, $sds_password); 

# Issue the request and receive a response
my $res = $ua->request($req);

# Check the status of the response
if ($res->is_success) {

    # For Debugging Data Errors
    if ($debug){
	print "<h1>DEBUG - Sask Learning XML Response</h1>\n";
	print "<div style=\"color:green;\">\n";
	my $temp = $res->content;
	$temp =~ s/</&lt;/g;
	$temp =~ s/>/&gt;/g;
	print "<pre>",$temp,"</pre></div>\n";
    }


    # Parse the response.
    my $parser = XML::LibXML->new();
    
    eval {$doc = $parser->parse_string($res->content)};
    if ($@){
	print "<pre>Sask Learning Error:\n",$res->content,"</pre><br>\n";
	print "</body></html>\n"; die;
    }

    $doc->setEncoding('UTF-8');
    $root = $doc->getDocumentElement;
    $root->setNamespace($xmlns,'sl',1);

    # find Message ID.
    #$mastermsgid = $root->findvalue('//sl:SL_MsgId');
    
    $status = $root->findvalue('//sl:SL_Status/sl:SL_StatusCode');
    
    if ($status eq 'Errors' or $status eq 'Invalid'){ 
	# Print out errors
	&prErr($mastermsgid,"$lastname, $firstname ($studnum)");

    } elsif ($status eq 'Successful'){ 
	&parseDeptCourse;  # The MAIN thing here....
    } else { # print warnings... ($status eq 'Warnings')
	print "<h1>There were warnings...</h1>";
    }
    
}  else { # Transfer Error!
    my $err = $res->status_line;
    print "<h1>Transfer Error: $err</h1>";
}

print "</body></html>\n";



#--------------
sub mkGeneralQuery {  # General Query for DeptCourses
#--------------
    my $idcount = shift;

    # Create Writer Instance
    $output = new XML::Writer::String;
    my $datamode = 0;
    if ($debug){ $datamode = 1;} # pretty print xml output
    $wr = new XML::Writer(OUTPUT => $output, 
			  DATA_MODE => $datamode, 
			  DATA_INDENT => '2');

    # Set XML Header and write Root Element
    $wr->xmlDecl("UTF-8");
    $wr->startTag('SL_Message','xmlns' =>$xmlns, 
		  'xmlns:xsi' =>$xmlnsxsi, 
		  'xsi:schemaLocation' => $xsischemaLocation);

    $wr->startTag('SL_Request');
    &mkSL_Header($currdate, $currtime, $schoolnumber,$idcount);

    $wr->startTag('SL_Query');

    $wr->startTag('GeneralQuery',
		  'RefId' => "$schoolnumber",
		  'ObjectName' => 'DeptCourses'
		  );

    #$wr->dataElement('SchoolId',$schoolnumber); 
    #$wr->dataElement('FromDate','20051201'); 	
    #$wr->dataElement('ToDate',$schoolend); 	

    $wr->endTag('GeneralQuery');

    $wr->endTag('SL_Query');
    $wr->endTag('SL_Request');
    $wr->endTag('SL_Message');
    $wr->end();
    
} # End of mkGeneralQuery


#---------
sub prErr {  # Print Errors to Screen and Errorlog
#---------
    my $msgid = shift;
    my $name = shift;

    #print "<p style=\"color:red; font-size:150%\">Error</p>\n";
    my @errors = $root->findnodes('//sl:SL_Error');
    foreach my $error (@errors){
	# Get Error Elements
	my $objectname = $error->getAttribute('ObjectName');
	my $errmsg = $error->findvalue('./sl:SL_ErrorMsg');
	$errmsg =~ s/[\r|\n]/ /g;
	my $errcode = $error->findvalue('./sl:SL_ErrorCode');
	
	print "<p style=\"color:red; font-size:120%\">$errmsg</p>\n";
    }
    print "</table>\n";
    
}


#----------------------
sub parseDeptCourse { # passed document root object.
#----------------------

    my @courseinfo = $root->findnodes('//sl:DeptCourses/sl:DeptCourse');

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th align=\"right\">Course Id</th><th>Course Title</th>";
    print "<th>Credit</th><th>Grade</th><th>Effective Date</th></tr>\n";

    foreach my $course (@courseinfo){
	my $courseid = $course->findvalue('sl:DeptAssignedCourseId');
	my $coursetitle = $course->findvalue('sl:CourseTitle');
	my $credit = $course->findvalue('sl:CreditValue');
	my $grade = $course->findvalue('sl:Grade/@Code');
	my $effectivedate = $course->findvalue('sl:EffectiveDate');
	
	print "<tr><td>$courseid</td><td>$coursetitle</td><td>$credit</td>";
	print "<td>$grade</td><td>$effectivedate</td></tr>";
    }
    print "</table>\n";

} # End of ParseDeptCourse
