#! /usr/bin/perl
#  Copyright 2001-2005 Leslie Richardson

#  This file is part of Open Admin for Schools.

# Query Function for StudentPersonal Objects
# Passed Values: none.

use DBI;
use CGI;
use XML::Writer;
use XML::Writer::String;
use Data::UUID;
use HTTP::Request::Common qw(POST); 
use HTTP::Headers; 
use LWP::UserAgent; 
use XML::LibXML;

$self = 'qryschenrol.pl';

# Read config variables
require "../../etc/admin.conf" or die "Cannot read admin.conf: $!";
require "slxmllib.pl" or die "Cannot read slxmllib.pl: $!";

# Override V1 with V2 values
# SDS XML Schema Declarations
$xmlns = "http://www.sasked.gov.sk.ca/xsd/sl/2.x/SLMessage_V2.xsd";
$xmlnsxsi = "http://www.w3.org/2001/XMLSchema-instance";
$xsischemaLocation="http://www.sasked.gov.sk.ca/xsd/sl/2.x/SLMessage_V2.xsd
http://www.sasked.gov.sk.ca/xsd/sl/2.x/SLMessage_V2.xsd";

$q = new CGI;
%arr = $q->Vars;
print $q->header;

if ($arr{debug}){
    $debug = 1;
    delete $arr{debug};
}

$dsn = "DBI:$dbtype:dbname=$dbase";
$dbh = DBI->connect($dsn,$user,$password);

# Setup Date and Times
@month = ("","January","February","March","April","May","June","July",
	  "August","September","October","November","December");
# $sec, $min, $hour, $mday, $mon, 
# $year, $wday, $yday, $isdst ) = localtime(time);

@tim = localtime(time);
$year = $tim[5] + 1900;
$tim[4]++;
for (0..4){if (length($tim[$_]) == 1){ $tim[$_] = '0'.$tim[$_];}}
$currdate = "$year-$tim[4]-$tim[3]";
$currtime = "$tim[2]:$tim[1]:$tim[0]";
$currlongdate = "$month[$tim[4]] $tim[3], $year";

print "$doctype\n<html><head><title>SDS Query: School Enrollment</title> 
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
<style type=\"text/css\">
body {
  padding: 1em 3em;
}
</style>
$chartype\n</head><body>[ <a href=\"$homepage\">Main</a> | 
<a href=\"$exppage\">Export</a> ]
<h1>Query School Student Enrollment</h1><h3>$currlongdate</h3>\n";

if ($arr{transferflag}) { # Activate/Create Transfer records
    activateTransfer();
    die;
}

# Find all local kids and push studnum into hash, display any blank provnum
my  $sth = $dbh->prepare("select studnum, provnum, lastname, firstname 
  from student order by provnum");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
$studcount = $sth->rows;

# Note any missing provincial numbers...
my $foundmissing = 0;

while (my ($studnum,$provnum,$lastname, $firstname) = $sth->fetchrow) {
    if (not $provnum) { 
	if ($foundmissing == 0){ # print the table heading, once.
	    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
	}
	$foundmissing = 1;
	print "<tr><td>Missing Provincial Number for: ";
	print "<b>$firstname $lastname</b> ($studnum)</td></tr>\n"; 
	push @noprovnum, "$lastname:$firstname:$studnum"; # used to NOT withdraw them.
	next;
    }
    $localStudent{$provnum} = 1;
    $masterList{$provnum} = 1;
}

if ($foundmissing) {
    print "</table>\n";
    print "<p><a href=\"qryschident.pl\" class=\"button\">";
    print "Run Identity Script (qryschident.pl)</a></p>\n";
    print "<p>&nbsp;</p>\n";
}


# Create a new user agent
my $ua = LWP::UserAgent->new();
$ua->agent("OpenAdmin");

$count++;
&mkQueryString($count); # Generate $output string

# DEBUG Data Errors
if ($debug){
    print "<div style=\"color: blue;\">\n";
    print "<h1>DEBUG - Request sent to Sask Learning</h1>\n";
    my $temp = $output->value;
    $temp =~ s/</&lt;/g;
    $temp =~ s/>/&gt;/g;
    print "<pre>",$temp,"</pre></div><p></p>\n";
}

# Create the https post request 

my $req = POST $url, [ XML=>$output->value ];

$req->content_type('application/xml;charset="utf-8"');
$req->authorization_basic($sds_userid, $sds_password); 

# Issue the request and receive a response
my $res = $ua->request($req);

# Check the status of the response
if ($res->is_success) {

    # For Debugging Data Errors
    if ($debug){
	print "<h1>DEBUG - Sask Learning XML Response</h1>\n";
	print "<div style=\"color:green;\">\n";
	my $temp = $res->content;
	$temp =~ s/</&lt;/g;
	$temp =~ s/>/&gt;/g;
	print "<pre>",$temp,"</pre></div>\n";
    }

    my $response = $res->content;

    # Parse the response.
    my $parser = XML::LibXML->new();
    
    eval {$doc = $parser->parse_string($response)};
    if ($@){
	print "<pre>Sask Learning Error:\n $@\n",$response,"</pre><br>\n";
	print "</body></html>\n"; die;
    }

    $doc->setEncoding('UTF-8');
    $root = $doc->getDocumentElement;
    $root->setNamespace($xmlns,'sl',1);

    $mastermsgid = $root->findvalue('//sl:SL_MsgId');
    $status = $root->findvalue('//sl:SL_Status/sl:SL_StatusCode');

    if ($status eq 'Errors' or $status eq 'Invalid'){ 
	# Print out error and add to errorlog.
	&prErr($mastermsgid,"$lastname, $firstname ($studnum)");

    } elsif ($status eq 'Successful'){ 

	my @slprovnums = parseSPLite(); # main thing... get their list of kids.

    } else { # print warnings... ($status eq 'Warnings')
	print "There were warnings...";
    }
    
}  else { # Transfer Error!
    my $err = $res->status_line;
    print "<h1>Transfer Error: $err</h1>";
    die;
}


# Now create a master list from both lists.
my $skcount = $#slprovnums + 1;
foreach my $pn (@slprovnums) { 
    $masterList{$pn} = 1;
    $saskStudent{$pn} = 1;
}

#other hash is: $localStudent{$provnum} = 1;

foreach my $pn (keys %masterList) { # go through all students (on both lists)
    # Delete them if we have a match (they're registered in both systems)
    if ($saskStudent{$pn} == $localStudent{$pn} and $localStudent{$pn} == 1) {
	delete $saskStudent{$pn};
	delete $localStudent{$pn};
    }
}

print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
print "<tr><td><b>Local School Enrollment: (from local records)</b></td>\n";
print "<td><b>$studcount</b></td></tr>\n";
print "<tr><td><b>Sask Learning Enrollment: (from SL records)</b></td>\n";
print "<td><b>$skcount</b></td></tr></table><p>&nbsp;</p>\n";


print "<form action=\"$self\" method=\"post\">\n";
print "<input type=\"hidden\" name=\"transferflag\" value=\"1\">\n";

print "<h1>Students to be Withdrawn from Sask Learning</h1>\n";
my $studcount = scalar keys %saskStudent;
if (not $studcount) {
    print "<b>No students to withdraw.</b>\n"; 
} else {
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>Prov Num / Name</th><th>Local Num</th>";
    print "<th>Activate<br>Withdrawal</th></tr>\n";

    my $count = 1;
    foreach my $key (keys %saskStudent) { 
	my $studnum;
	print "<tr><td>$count: $key $slname{$key}</td><td>\n";
	my ($lastname,$firstname,$middlename,$birthdate) = split /:/, $slname{$key};
	($firstname, $rest) = split /\s/,$firstname;

	my $sth = $dbh->prepare("select studnum from studentall
         where lastname = '$lastname' and firstname = '$firstname'");
	$sth->execute;
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	my $rows = $sth->rows;

	my $transrows; # count of transfer records...
	if ($rows < 1){ # student not found...
	    print "<font color=red>Not Found</font></td><td>\n"; 
	    print "Transfer Rec Added.\n";
	} else { # student found
	    $studnum = $sth->fetchrow;
	    print "<b>$studnum</b></td>";

	    my $sth1 = $dbh->prepare("select id, date from transfer
             where studnum = '$studnum' and type = 'withdraw' 
             order by date desc");
	    $sth1->execute;
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	    $transrows = $sth1->rows;
	    print "<td>";
	    while (my ($id,$date) = $sth1->fetchrow) {
		print "$type - $date <input type=\"checkbox\" name=\"$id\" ";
		print "value=\"1\">"; 
	    }
	}

	if ($transrows < 1 or $rows < 1) { 
	    # generate a transfer withdrawal record...
	    if (not $studnum) { my $studnum = $count;} # not really...
	    my $provnum = $key;
	    my $date = $schoolstart;
	    my $description = 'Auto-generated: qryschenrol.pl';
	    my $entrytype = '';
	    my $exittype = '01'; # trans to other school.
	    my $prov = 'SK';
	    my $country = '';

	    my $sth = $dbh->prepare("insert into transfer values (
             $sql{default},'$studnum','$date','withdraw',
            '$description', '$entrytype', '$exittype','$prov','$country',
            '','$lastname', '$firstname', '$middlename',
            '$birthdate', '$provnum')");
	    $sth->execute;
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	    
	}

	print "</td></tr>\n";
	$count++;
    } 

    print "</table><p></p>\n";
    print "<input type=\"submit\" value=\"Activate Selected Transfer Records\">\n";

}

print "<p></p>\n";


# Now outstanding enrollments
print "<h1><b>Local Students to Enrol with SL</b></h1>\n";

my $studcount = scalar keys (%localStudent);
if (not $studcount){ 
    print "<b>No students to enrol.</b>\n"; 
} else {

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>Lastname</th><th>Firstname</th><th>WD</th>\n";
    print "<th>TRN</th><th>Enrol</th></tr>\n";

    my $count = 1;
    foreach my $key (keys %localStudent) {
	# Get studnum from provnum $key
	my $sth1 = $dbh->prepare("select studnum from studentall
        where provnum = '$key'");
	$sth1->execute;
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	my $studnum = $sth1->fetchrow;

	my $sth2 = $dbh->prepare("select id from transfer
        where studnum = '$studnum' and type = 'enrol'");
	$sth2->execute;
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	my $id = $sth2->fetchrow;

	my ($lastname, $firstname, $wd, $trn) = split /:/, findStudent($key);
	print "<tr><td>$count: $lastname ($key)</td><td>$firstname</td>\n";
	print "<td>$wd</td><td>$trn</td>\n";
	print "<td><input type=\"checkbox\" name=\"$id\" ";
	print "value=\"1\"></td></tr>\n"; 
	$count++;
    }

    print "</table><p></p>\n";
    print "<input type=\"submit\" value=\"Activate Selected Transfer Records\">\n";
    
} # end of student display/no display

print "</form></body></html>\n";



# Functions =============================

#--------------
sub mkQueryString {  # QryBySchool for StudentPersonal objects.
#--------------
    my $idcount = shift;

    # Create Writer Instance
    $output = new XML::Writer::String;
    my $datamode = 0;
    if ($debug){ $datamode = 1;} # pretty print xml output
    $wr = new XML::Writer(OUTPUT => $output, 
			  DATA_MODE => $datamode, 
			  DATA_INDENT => '2');

    # Set XML Header and write Root Element
    $wr->xmlDecl("UTF-8");
    $wr->startTag('SL_Message','xmlns' =>$xmlns, 
		  'xmlns:xsi' =>$xmlnsxsi, 
		  'xsi:schemaLocation' => $xsischemaLocation);

    $wr->startTag('SL_Request');
    &mkSL_Header($currdate, $currtime, $schoolnumber,$idcount);

    $wr->startTag('SL_Query');

    $wr->startTag('QueryBySchool',
		  'RefId' => "$schoolnumber",
		  'ObjectName' => 'StudentPersonal'
		  );

    $wr->dataElement('SchoolId',$schoolnumber); 

    my $cdate = $currdate;
    $cdate =~ s/-//g; # strip hyphens
    $wr->dataElement('FromDate',$cdate); 	
    $wr->dataElement('ToDate',$cdate); 	

    $wr->endTag('QueryBySchool');

    $wr->endTag('SL_Query');
    $wr->endTag('SL_Request');
    $wr->endTag('SL_Message');
    $wr->end();
    
}



#---------
sub prErr {  # Print Errors to Screen
#---------
    my @errors = $root->findnodes('//sl:SL_Error');
    foreach my $error (@errors){
	# Get Error Elements
	my $objectname = $error->getAttribute('ObjectName');
	my $errmsg = $error->findvalue('./sl:SL_ErrorMsg');
	$errmsg =~ s/[\r|\n]/ /g;
	my $errcode = $error->findvalue('./sl:SL_ErrorCode');
	
	print "<p style=\"color:red; font-size:120%\">$errmsg</p>\n";
    }
    print "</table>\n";
     
}

#--------------
sub findStudent {
#--------------
    # Passed prov number... return data
    my $provnum = shift;

    # Get student record
    my  $sth = $dbh->prepare("select lastname, firstname, initial, 
     studnum from studentall
     where provnum = '$provnum'");
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
    my ($lastname, $firstname,$middlename,$studnum) = 
	$sth->fetchrow;
    if (not $lastname) { $lastname = 'Not Found'; return $lastname; }
    

    # Check whether current or withdrawn.
    $wd = '';
    $sth = $dbh->prepare("select count(studid) from student
      where provnum = '$provnum'");
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
    my $idcount = $sth->fetchrow;
    if ($idcount == 1){ $wd = 0 } else { $wd = 1; }

    $sth = $dbh->prepare("select studnum from student
      where provnum = '$provnum'");
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
    my $studnum = $sth->fetchrow;

    # Check if records in transfer table (if missing...)
    $sth = $dbh->prepare("select count(id) from transfer
      where studnum = '$studnum'");
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
    my $transcount = $sth->fetchrow;
 
    return "$lastname:$firstname:$wd:$transcount";

}




#----------------------
sub parseStudentPersonal { # passed document root object.
#----------------------
    my @studinfo = $root->findnodes('//sl:StudentPersonal');
     
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>Lastname</th><th>Firstname</th><th>Middlename</th>";
    print "<th>Studnum</th><th>Sask Lrn #</th></tr>\n";
    my $count = 1;

    foreach my $student (@studinfo){
       my $provnum = 
       $student->findvalue('sl:StudentIdentification/sl:DeptAssignedPersonId');
       my $firstname = 
	   $student->findvalue('sl:StudentInfo/sl:Name/sl:FirstName');
       my $lastname = 
	   $student->findvalue('sl:StudentInfo/sl:Name/sl:LastName');
       my $middlename = 
	   $student->findvalue('sl:StudentInfo/sl:Name/sl:MiddleName');

       # Get student record
       my  $sth = $dbh->prepare("select lastname, firstname, initial, 
         studnum from studentall
         where provnum = '$provnum'");
	$sth->execute;
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	my ($localLastname, $localFirstname,$localMiddlename,$studnum) = 
	    $sth->fetchrow;
	if (not $localLastname) { 
	    $mis = "<font color=\"red\">MIS</font>"; 
	} else { $mis='';  }

	
	# Check whether current or withdrawn.
	$wd = '';
	if ($localLastname) {
	    $sth = $dbh->prepare("select count(studid) from student
             where provnum = '$provnum'");
	    $sth->execute;
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	    my $idcount = $sth->fetchrow;
	    if (not $idcount){ $wd = "<font color=\"red\">WD</font>"; }
	}


	$trn = '';
	# Check if records in transfer table (if missing...)
	if (not $localLastname) { 
	    $sth = $dbh->prepare("select count(id) from transfer
              where provnum = '$provnum'");
	    $sth->execute;
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	    my $transcount = $sth->fetchrow;
	    if (not $transcount){ $trn = "<font color=\"red\">Trn</font>"; }
	}


	print "<tr><td>$mis $wd $trn $count: $lastname [$localLastname]</td>";
	print "<td>$firstname [$localFirstname]</td>";
	print "<td>$middlename [$localMiddlename]</td>";
	print "<td>$studnum</td><td>$provnum</td></tr>\n";
	$count++;

    }
    print "</table>\n";


} # End of Sub



#----------------------
sub parseSPLite { # parse only for provnum 
#----------------------
    my @studinfo = $root->findnodes('//sl:StudentPersonal');
     
    foreach my $student (@studinfo){
      my $provnum = 
       $student->findvalue('sl:StudentIdentification/sl:DeptAssignedPersonId');
      my $birthdate =
        $student->findvalue('sl:StudentIdentification/sl:BirthDate');

      my $firstname = 
	  $student->findvalue('sl:StudentInfo/sl:Name/sl:FirstName');
      my $lastname = 
	  $student->findvalue('sl:StudentInfo/sl:Name/sl:LastName');
      my $middlename = 
	  $student->findvalue('sl:StudentInfo/sl:Name/sl:MiddleName');
      $slname{$provnum} = "$lastname:$firstname:$middlename:$birthdate";
        
      push @slprovnums, $provnum;
    }

    return @slprovnums; # return list of kids in SL enrollment list.

} # End of parseSPLite;



#----------------------
sub checkIdent { # Match local and provincial students
#----------------------
    my @studinfo = $root->findnodes('//sl:StudentPersonal');
     
    #print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    #print "<tr><th>Lastname</th><th>Firstname</th><th>Middlename</th>";
    #print "<th>Studnum</th><th>Sask Lrn #</th></tr>\n";
    #my $count = 1;

    foreach my $student (@studinfo){
	my $provnum = $student->findvalue('sl:StudentIdentification/sl:DeptAssignedPersonId');
	my $birthdate = $student->findvalue('sl:StudentIdentification/sl:BirthDate');
	my $firstname = $student->findvalue('sl:StudentInfo/sl:Name/sl:FirstName');
	my $lastname = $student->findvalue('sl:StudentInfo/sl:Name/sl:LastName');
	my $middlename = $student->findvalue('sl:StudentInfo/sl:Name/sl:MiddleName');


	# Check for student record
	my  $sth = $dbh->prepare("select studnum from student
         where provnum = '$provnum'");
	$sth->execute;
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	my ($localLastname, $localFirstname,$localMiddlename,$studnum) = $sth->fetchrow;
	if (not $localLastname) { $mis = "<font color=\"red\">MIS</font>"; } else { $mis=''; }

	
	# Check whether current or withdrawn.
	$wd = '';
	if ($localLastname) {
	    $sth = $dbh->prepare("select count(studid) from student
             where provnum = '$provnum'");
	    $sth->execute;
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	    my $idcount = $sth->fetchrow;
	    if (not $idcount){ $wd = "<font color=\"red\">WD</font>"; }
	}


	$trn = '';
	# Check if records in transfer table (if missing...)
	if (not $localLastname) { 
	    $sth = $dbh->prepare("select count(id) from transfer
              where provnum = '$provnum'");
	    $sth->execute;
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	    my $transcount = $sth->fetchrow;
	    if (not $transcount){ $trn = "<font color=\"red\">Trn</font>"; }
	}


	print "<tr><td>$mis $wd $trn $count: $lastname [$localLastname]</td>";
	print "<td>$firstname [$localFirstname]</td>";
	print "<td>$middlename [$localMiddlename]</td>";
	print "<td>$studnum</td><td>$provnum</td></tr>\n";
	$count++;

	#print "<tr><td align=\"right\">First Name</td><td>$firstname</td></tr>";
	#print "<tr><td align=\"right\">Last Name</td><td>$lastname</td></tr>";
	#print "<tr><td align=\"right\">Prov Num</td><td>$lastname</td></tr>";

    }
    print "</table>\n";


} # End of checkIdent


#---------------------
sub activateTransfer { # passed list of transfer recs to activate.
#---------------------
    print "<p>Activating Student Withdrawals/Enrollments</p>\n";
    delete $arr{transferflag};
    foreach my $key (keys %arr) {
	#print "K:$key V:$arr{$key}<br>\n";
	$result = $dbh->do("update transfer set utag = 'Y'
          where id = '$key'");
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	print "K: $key  Result: $result<br>\n";
    }

} #End of activateTransfer
