#! /usr/bin/perl
#  Copyright 2001-2006 Leslie Richardson

#  This file is part of Open Admin for Schools.

# Query School Identities
# Find blank provincial number student and push into blank hash.
# Roll through SL records and find possible matches based on name and bdate.
# remove matches from blank hash.
# Once done SL records, display remaining students w/o identity.


my $self = 'qryschident.pl';

use DBI;
use CGI;
use XML::Writer;
use XML::Writer::String;
use Data::UUID;
use HTTP::Request::Common qw(POST); 
use HTTP::Headers; 
use LWP::UserAgent; 
use XML::LibXML;

# Read config variables
require "../../etc/admin.conf" or die "Cannot read admin.conf: $!";
require "slxmllib.pl" or die "Cannot read slxmllib.pl: $!";

my $dsn = "DBI:$dbtype:dbname=$dbase";
$dbh = DBI->connect($dsn,$user,$password);

# Override V1 with V2 values
# SDS XML Schema Declarations
$xmlns = "http://www.sasked.gov.sk.ca/xsd/sl/2.x/SLMessage_V2.xsd";
$xmlnsxsi = "http://www.w3.org/2001/XMLSchema-instance";
$xsischemaLocation="http://www.sasked.gov.sk.ca/xsd/sl/2.x/SLMessage_V2.xsd
http://www.sasked.gov.sk.ca/xsd/sl/2.x/SLMessage_V2.xsd";

my $q = new CGI;
my %arr = $q->Vars;
print $q->header;

if ($arr{debug}){
    $debug = 1;
    delete $arr{debug};
}


print "$doctype\n<html><head><title>SDS Query: Identity</title> 
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
<style type=\"text/css\">
body {
  padding: 1em 3em;
}
</style>
$chartype\n</head><body>[ <a href=\"$homepage\">Main</a> | 
<a href=\"$exppage\">Export</a> ]
<h1>Query Identity</h1>\n";


if ($arr{updateflag}) {
    delete $arr{updateflag};
    activateTransfer();
    die;
}


print "<div style=\"width:60%;border:1px solid black;padding:1em;\">\n";
print "Query Sask Learning's records of student enrollment to locate ";
print "student information \n for local students <b>without</b> provincial ";
print "student numbers.</div>\n";


my @month = ('','January','February','March','April','May','June','July',
	  'August','September','October','November','December');
my @tim = localtime(time);
my $year = $tim[5] + 1900;
$tim[4]++;
for (0..4){if (length($tim[$_]) == 1){ $tim[$_] = '0'.$tim[$_];}}
my $currdate = "$year-$tim[4]-$tim[3]";
my $currtime = "$tim[2]:$tim[1]:$tim[0]";


# First run a duplicate provnum check
my $sth = $dbh->prepare("select studnum, lastname, firstname, provnum 
from student 
where provnum != '' order by provnum");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}

my $currstud = '-1';
my $currname = '';

while (my ($studnum, $lastname, $firstname, $provnum) = $sth->fetchrow ) {
    $oldstud = $currstud;
    $currstud = $provnum;
    $oldname = $currname;
    $currname = "$firstname $lastname";

    if ($currstud == $oldstud) { # Ack! We have a duplicate!
	print "<div style=\"border:1px solid black;padding:1em;\">";
	print "Duplicate for provincial student number: $currstud<br>\n";
	print "<b>$currname</b> and <b>$oldname</b> are duplicated!</div>\n";
    }
}



# Find blank kids and push studnum into %noProvnum hash.
my  $sth = $dbh->prepare("select studnum from student
  where provnum = '' or provnum is null");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
while (my $studnum = $sth->fetchrow) {
    $noProvnum{$studnum} = 1;
}
if (not %noProvnum) { # No kids found...
    print "<p><b>All students have provincial numbers.</b></p>\n";
    print "</body></html>\n";
    die;
}


# Create a new user agent
my $ua = LWP::UserAgent->new();
$ua->agent("OpenAdmin");

# Grade and Provincial Number passed
$count=1;
mkQueryString($count); # Generate $output string

# DEBUG Data Errors
if ($debug){
    print "<div style=\"color: blue;\">\n";
    print "<h1>DEBUG - Request sent to Sask Learning</h1>\n";
    my $temp = $output->value;
    $temp =~ s/</&lt;/g;
    $temp =~ s/>/&gt;/g;
    print "<pre>",$temp,"</pre></div><p></p>\n";
}

# Create the https post request 
my $req = POST $url, [ XML=>$output->value ];

$req->content_type('application/xml;charset="utf-8"');
$req->authorization_basic($sds_userid, $sds_password); 

# Issue the request and receive a response
my $res = $ua->request($req);

# Check the status of the response
if ($res->is_success) {

    # For Debugging Data Errors
    if ($debug){
	print "<h1>DEBUG - Sask Learning XML Response</h1>\n";
	print "<div style=\"color:green;\">\n";
	my $temp = $res->content;
	$temp =~ s/</&lt;/g;
	$temp =~ s/>/&gt;/g;
	print "<pre>",$temp,"</pre></div>\n";
    }

    my $response = $res->content;
    $response =~ s/\xE9/\x65/g;

    # Parse the response.
    my $parser = XML::LibXML->new();
    
    eval {$doc = $parser->parse_string($response)};
    if ($@){
	print "Error: $@<br>\n";
	$response =~ s/</&lt;/g;
	$response =~ s/>/&gt;/g;
	print "<pre>Sask Learning Error:\n",$response,"</pre><br>\n";
	print "</body></html>\n"; die;
    }

    $doc->setEncoding('UTF-8');
    $root = $doc->getDocumentElement;
    $root->setNamespace($xmlns,'sl',1);

    # find Status
    $status = $root->findvalue('//sl:SL_Status/sl:SL_StatusCode');
    if ($status eq 'Errors' or $status eq 'Invalid'){ 
	# Print out error and add to errorlog.
	prErr("$lastname, $firstname ($studnum)");
    } elsif ($status eq 'Successful'){ 
	parseStudentPersonal();
    } else { # print warnings... ($status eq 'Warnings')
	print "There were warnings...\n";
    }
    
}  else { # Transfer Error!
    my $err = $res->status_line;
    print "<h1>Transfer Error: $err</h1>\n";
}

print "</body></html>\n";


# Functions =============================

#----------------
sub mkQueryString {  # StudentPersonal
#----------------

    my ($grade, $provnum, $idcount) = @_;
    # Date and schoolnumber are globals.

    # Create Writer Instance
    $output = new XML::Writer::String;
    my $datamode = 0;
    if ($debug){ $datamode = 1;} # pretty print xml output
    $wr = new XML::Writer(OUTPUT => $output, 
			  DATA_MODE => $datamode, 
			  DATA_INDENT => '2');

    # Set XML Header and write Root Element
    $wr->xmlDecl("UTF-8");
    $wr->startTag('SL_Message','xmlns' =>$xmlns, 
		  'xmlns:xsi' =>$xmlnsxsi, 
		  'xsi:schemaLocation' => $xsischemaLocation);

    $wr->startTag('SL_Request');
    &mkSL_Header($currdate, $currtime, $schoolnumber,$idcount);

    $wr->startTag('SL_Query');

    $wr->startTag('QueryBySchool',
		  'RefId' => "$schoolnumber",
		  'ObjectName' => 'StudentPersonal'
		  );

    $wr->dataElement('SchoolId',$schoolnumber); 

    #$wr->dataElement('FromDate','20051201'); 	
    #$wr->dataElement('ToDate','20051205'); 	

    $wr->endTag('QueryBySchool');

    $wr->endTag('SL_Query');
    $wr->endTag('SL_Request');
    $wr->endTag('SL_Message');
    $wr->end();
    
}



#---------
sub prErr {  # Print Errors to Screen and Errorlog
#---------
    my $name = shift;

    my @errors = $root->findnodes('//sl:SL_Error');
    foreach my $error (@errors){
	# Get Error Elements
	my $objectname = $error->getAttribute('ObjectName');
	my $errmsg = $error->findvalue('./sl:SL_ErrorMsg');
	$errmsg =~ s/[\r|\n]/ /g;
	my $errcode = $error->findvalue('./sl:SL_ErrorCode');
	
	# Write the Error Log
	print ERR "$currdate:$currtime:$name:$errmsg:$errcode\n";
	
	print "<p style=\"color:red;\">$errmsg</p>\n";
    }
    
}



#----------------------
sub parseStudentPersonal { # passed document root object.
#----------------------
    my @studinfo = $root->findnodes('//sl:StudentPersonal');

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"updateflag\" value=\"1\">\n";

    my $first = 1;

STUDENT:
    foreach my $student (@studinfo){
	my $provnum = $student->findvalue('sl:StudentIdentification/sl:DeptAssignedPersonId');
	my $birthdate = $student->findvalue('sl:StudentIdentification/sl:BirthDate');
	my $firstname = $student->findvalue('sl:StudentInfo/sl:Name/sl:FirstName');
	my $lastname = $student->findvalue('sl:StudentInfo/sl:Name/sl:LastName');
	my $middlename = $student->findvalue('sl:StudentInfo/sl:Name/sl:MiddleName');
	my $phone = $student->findvalue('sl:StudentInfo/sl:PhoneNumber');

	#print "$lastname $firstname : $birthdate<br>\n";

	# Get student record w/o provnum
	my  $sth = $dbh->prepare("select studid, lastname, firstname, initial, 
          studnum, birthdate, hphone1 from studentall 
          where provnum = '' and
          birthdate = '$birthdate' and lastname = ? and firstname = ?");
	$sth->execute($lastname, $firstname);
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	my $matchcount = $sth->rows;
	if ( $matchcount < 1 ) { next STUDENT; }

	if ($first) {
	    print "<h1>Students with No Provincial Number, Enrolled by SL</h1>\n";
	    print "<div style=\"padding:1em;\">";
	    print "<input type=\"submit\" value=\"Update Enrollments\">\n";
	    print "</div>\n";

	    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
	    print "<caption>[ <b>White</b> - Sask Learning record |";
	    print "<b>Gray</b> - Local Record | <font color=\"red\">WD</font> ";
	    print "= Withdrawn ]</caption>\n";
	    print "<tr><th>Lastname</th><th>Firstname</th><th>Middlename</th>";
	    print "<th>Birthdate</th><th>Phone</th><th>Sask Lrn #</th></tr>\n";
	    $first = 0;
	}

	my $sth1 = $dbh->prepare("select count(*) from student where studnum = ?");
	my $sth2 = $dbh->prepare("update ? set provnum = ? where studnum = ?");
	my $sth3 = $dbh->prepare("select id from transfer where studnum = ? and 
         type = 'withdraw' order by date desc");

	# Print Sask Learning Record and then matches locally.
	print "<tr><td>$lastname</td><td>$firstname</td><td>$middlename</td>\n";
	print "<td>$birthdate</td><td>$phone</td><td>$provnum</td></tr>\n";

	while (my ($id, $localLastname, $localFirstname,$localMiddlename,
		   $studnum, $localBirthdate, $localPhone) = $sth->fetchrow) {

            # Get rid of noProvnum record
            delete $noProvnum{$studnum};


	    # Check to see if student is withdrawn (ie. present in student table?)
	    $sth1->execute($studnum);
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    my $reccount = $sth1->fetchrow;
	    my ($wd, $tb);
	    my $table = 'student';
	    if ($reccount < 1){ 
		$wd = "<font color=\"red\">WD</font>"; 
		$tb = "&tb=wd";
		$table = 'studentwd';
            }

	    # Now update the provnumber in the table (student or studentwd)
	    my $sth2 = $dbh->prepare("update $table set provnum = ? where studnum = ?");
	    $sth2->execute($provnum, $studnum);

	    if ($DBI::errstr){ print "Error line 352: $DBI::errstr"; die $DBI::errstr; }

	    # Now look for transfer records in order to get date and entry/exit code.
	    $sth3->execute($studnum);
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    my $transid = $sth3->fetchrow;

    	    print "<tr style=\"background-color:#DDD;\">";
	    print "<td>$wd <a href=\"$cgiurl/studed.pl?recnum=$id$tb\">";
            print "$localLastname</a></td><td>$localFirstname</td><td>$localMiddlename</td>";
	    print "<td>$localBirthdate</td><td>$localPhone</td>\n";

            if ($reccount < 1) { # withdrawn, so update SL.
		print "<td align=\"center\"><input type=\"checkbox\" ";
		if ($transid) { # we have a transfer record with wd info.
		    print "name=\"WD:TR:$transid\" value=\"1\"></td></tr>\n";

		} else { # no transfer rec; use PN approach... all data available.
		    print "name=\"WD:PN:$provnum:$lastname:$firstname:$middlename:$birthdate\" value=\"1\"></td></tr>\n";
		}
	    } else { # do nothing... provnum already updated.
		print "<td></td></tr>\n";
	    }


	} # Single Student Loop

    }  # Overall Student Loop

    if (not $first) {
	#print "<tr><td colspan=\"6\" align=\"center\">";
	#print "<input type=\"submit\" value=\"Update Enrollments\">\n";
	#print "</td></tr>\n";
	print "</table>\n";
    }
    
    
    # Now do students not enrolled with SL (and no provnum as a result)
    print "<h1>Students with No Provincial Number, and not Enrolled by SL</h1>\n";
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>Name</th><th>Birthdate</th><th>Enrol</th></tr>\n";

    my $count = 1;

    my $sth3 = $dbh->prepare("select id from transfer where studnum = ? and 
         type = 'enrol' order by date desc");

    foreach my $studnum (keys %noProvnum) {
	# Load values from student table; 
	my $sth = $dbh->prepare("select studid,lastname, firstname, initial, birthdate from student 
           where studnum = ?");
	$sth->execute($studnum);
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	my ($id,$lastname, $firstname, $middlename, $birthdate) = $sth->fetchrow;

	# Now look for transfer records in order to get date and entry/exit code.
	$sth3->execute($studnum);
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	my $transid = $sth3->fetchrow;

	# print start of this record
	print "<tr><td>$wd <a href=\"$cgiurl/studed.pl?recnum=$id\">";
	print "$lastname</a>, $firstname $middlename</td><td>$birthdate</td>\n";

	# now the selection checkbox for enrollment.
	print "<td align=\"center\"><input type=\"checkbox\" ";
	if ($transid) { # we have a transfer record with wd info.
	    print "name=\"EN:TR:$transid\" value=\"1\"></td></tr>\n";
	} else { # no transfer rec; use SN approach... all data available.
	    print "name=\"EN:SN:$studnum:$lastname:$firstname:$middlename:$birthdate\" value=\"1\" CHECKED></td></tr>\n";
	}

        $count++;
    }

    print "</table>\n";
    print "<div style=\"padding:1em;\">";
    print "<input type=\"submit\" value=\"Update Enrollments\">\n";
    print "</div>\n";



} # End of Sub parseStudentPersonal



#-------------------
sub activateTransfer { # passed list of transfer recs to setup date/code selects.
#-------------------

    require "$globdir/global.conf" or die "Cannot open global.conf!\n";

    print "<p>Student Withdrawals / Enrollments</p>\n";
    delete $arr{transferflag};

    # Now run external and use existing updatetransfer.pl (modified to accept correct input)
    print "<form action=\"updatexfer.pl\" method=\"post\">\n";


    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>Name</th><th>Number</th><th>Type</th><th>Date<br>yyyy-mm-dd</th><th>Reason</th></tr>\n";

    my $sth1 = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");
    my $sth2 = $dbh->prepare("select lastname, firstname from studentall where provnum = ?");


    my $reccount = 1; 
    my $enrolflag = 1;

    foreach my $key (sort keys %arr) { # key is 
	#print "K:$key V:$arr{$key}<br>\n";

	my ($EnWd, $rectype, @value) = split /:/, $key;
	my (@tr, $date, $code, $type, $firstname, $lastname, $birthdate, $provnum);

	if ($EnWd eq 'EN') { 
	    $type = 'enroll';
	} else { 
	    $type = 'withdraw'; 

	    #print "<tr style=\"background-color:#CCC;height:0.4em;\"><td colspan=\"5\"></td></tr>\n";
	}

	if ($rectype eq 'TR') { # we have a TR (transfer record), read the record...
	    my $id = $value[0];

	    my $sth = $dbh->prepare("select * from transfer where id = ?");
	    $sth->execute($id);
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    @tr = $sth->fetchrow;
	    $num = $tr[1]; # student number used after this section....
	    $date = $tr[2];
	    if ( $EnWd eq 'EN' ) { $code = $tr[5]; } else { $code = $tr[6]; }
	    if ( $tr[14] ) { 
		$provnum = $tr[14];
	    } else {
		my $sth3 = $dbh->prepare("select provnum from studentall 
                 where studnum = ?");
		$sth3->execute($tr[1]);
		if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
		$provnum = $sth3->fetchrow;
		#print "PN:$provnum SN:$tr[1]<br>\n";
		
	    }

	    # Get name; tr[1] is studnum;
	    if ($tr[10]) { # we have a name in the transfer record...
		$lastname = $tr[10];
		$firstname = $tr[11];
	    } else {
		$sth1->execute($tr[1]);
		if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
		($lastname, $firstname) = $sth1->fetchrow;
	    }

	    
	} else { # Method is SN or PN (Local Studnum or Provnum)

	    # values in $rectype is PN or SN, $values[0] is number,
	    # rest of @values are name and birthdate.
	    
	    ($num, $lastname, $firstname, $middlename, $birthdate) = @value;
	    my $fieldname;
	    if ($rectype eq 'PN'){ $fieldname = 'provnum'; } else { $fieldname = 'studnum'; }
	    if (not $lastname) { # We only have the provincial number for a local record...
		my $sth = $dbh->prepare("select lastname, firstname, initial, birthdate from studentall
                  where $fieldname = ?");
		$sth->execute($num);
		if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
		($lastname, $firstname, $middlename, $birthdate) = $sth->fetchrow;
	    }
	}

	if (not $provnum) { $provnum = "S$num"; }

	print "<tr><td>$reccount. <b>$lastname</b>, $firstname $middlename";
	if ( $birthdate ) { print " ($birthdate) "; }
	$reccount++;

	print "</td><td>$provnum</td><td>$type</td>\n<td>";
	print "<input type=\"text\" name=\"D:$provnum\" value=\"$date\" size=\"12\">";
	print "<input type=\"hidden\" name=\"B:$provnum\" value=\"$birthdate\"></td>\n";

	print "<td><select name=\"C:$provnum:$type\">";
	if ($type eq 'enroll') {
	    print "<option>$g_enrol{$code} ";
	    if ($code) { print "($code)";}
	    print "</option>\n";
	    foreach my $key (sort keys %g_enrol) {
		print "<option>$g_enrol{$key} ($key)</option>\n";
	    }
	} else {
	    print "<option>$g_wdraw{$code} ";
	    if ($code) { print "($code)"; }
	    print "</option>\n";
	    foreach my $key (sort keys %g_wdraw) {
		print "<option>$g_wdraw{$key} ($key)</option>\n";
	    }
	}
	print "</select></td></tr>\n";
	
    }

    print "</table><input type=\"submit\" value=\"Update Student Enrollments\">\n";
    print "</form></body></html>\n";

} # End of activateTransfer



# ========= OLD STUFF From here Down =======
# No longer using the transfer table utagged records.... Query SL instead.



#------------------
sub activateRecords { # turn on enrollment records for students
#------------------
    # $key is studnum.
    foreach my $key (keys %arr) {
	#print "K:$key VAL: $arr{$key}<br>\n";
	my $sth = $dbh->prepare("select id, utag, lastname from transfer
          where studnum = '$key' and type = 'enrol' order by date desc");
	$sth->execute;
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	my ($id,$utag,$lastname) = $sth->fetchrow;
	if (not $id) { 
	    print "No record for $lastname, skipping<br>\n";
	    next; # next key value
	}
	if (not $utag) { # update utag
	    my $result = $dbh->do("update transfer set utag = 'Y' where id = '$id'");
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    if ($result == 1){
		print "Enrollment Record for <b>$lastname</b> updated.<br>\n";
	    } else { 
		print "Error updating enrollment record for <b>$lastname</b><br>\n";
	    }
	}
	#print "ID:$id UTag:$utag LN:$lastname<br>\n";

    }
    
    print "<h3>[ <a href=\"updatetrans.pl\">Update Enrollment Records ";
    print "(updatetrans.pl)</a> ]</h3>\n";

}


#--------------------
sub updateWithdrawals {
#--------------------
    foreach my $key (keys %arr) {
	#print "K:$key VAL: $arr{$key}<br>\n";
	my ($studnum,$provnum) = split /:/,$key;

	my $result = $dbh->do("update studentwd set provnum = '$provnum' 
           where studnum = '$studnum'");
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	if ($result != 1){
	    print "Error updating provincial number for <b>$studnum</b><br>\n";
	}
	# Now activate the withdrawal rec.
	my $sth = $dbh->prepare("select id, utag, lastname, exittype from transfer
          where studnum = '$studnum' and type = 'withdraw' order by date desc");
	$sth->execute;
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	my ($id,$utag,$lastname,$exittype) = $sth->fetchrow;
	my $exitupdate;
	if (not $exittype) { 
	    $exitupdate = ",exittype='01'";
	}
	# Now activate this record
	if ($id) {
	    my $result = $dbh->do("update transfer set utag = 'Y' $exitupdate where id = '$id'");
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    if ($result == 1){
		print "Enrollment Record for <b>$lastname</b> updated.<br>\n";
	    } else { 
		print "Error updating enrollment record for <b>$lastname</b><br>\n";
	    }
	} else { print "No withdrawal record found for $studnum<br>\n"; }

    
    } # End of Student Loop


} # End of updateWithdrawals

