#! /usr/bin/perl
#  Copyright 2001-2005 Leslie Richardson

#  This file is part of Open Admin for Schools.

# ======= Query Function for StudentPersonal Objects
# Passed Values: a long list of students to query for.(studnum)'s

use DBI;
use CGI;
use XML::Writer;
use XML::Writer::String;
use Data::UUID;
use HTTP::Request::Common qw(POST); 
use HTTP::Headers; 
use LWP::UserAgent; 
use XML::LibXML;

# Read config variables
require "../../etc/admin.conf" || die "Cannot read admin.conf: $!";
require "slxmllib.pl" || die "Cannot read slxmllib.pl: $!";

$q = new CGI;
%arr = $q->Vars;
print $q->header;

if ($arr{debug}){
    $debug = 1;
    delete $arr{debug};
}


print "$doctype\n<html><head><title>SDS Query: Student Demographics</title> 
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
<style type=\"text/css\">
body {
  padding: 1em 3em;
}
</style>
$chartype\n</head><body>[ <a href=\"$homepage\">Main</a> | 
<a href=\"$exppage\">Export</a> ]
<h1>Query Student Demographics</h1>\n";

@month = ("","January","February","March","April","May","June","July",
	  "August","September","October","November","December");
# $sec, $min, $hour, $mday, $mon, 
# $year, $wday, $yday, $isdst ) = localtime(time);

@tim = localtime(time);
$year = $tim[5] + 1900;
$tim[4]++;
for (0..4){if (length($tim[$_]) == 1){ $tim[$_] = '0'.$tim[$_];}}
$currdate = "$year-$tim[4]-$tim[3]";
$currtime = "$tim[2]:$tim[1]:$tim[0]";

$dsn = "DBI:$dbtype:dbname=$dbase";
$dbh = DBI->connect($dsn,$user,$password);

# Create a new user agent
my $ua = LWP::UserAgent->new();
$ua->agent("OpenAdmin");

# Open Update and Error Log files for appending.
open(ERR,">>error.log") or die "Cannot open error.log: $!\n";
open(UPD,">>update.log") or die "Cannot open update.log: $!\n";

# prep for retrieving from student data
$sth = $dbh->prepare("select * from studentall
  where studnum = ?");
my $count = 0; # counter for MsgId, since some may be within same second.



# Main Loop =========================
foreach my $key (sort keys %arr){ # loop through students

    my ($lastname, $firstname,$studnum) = split /:/,$key;

    # Pull in matching student data
    $sth->execute($studnum); 
    if ($DBI::errstr) { print $DBI::errstr; die;}
    @sr = $sth->fetchrow;

    # Ignore passed values above.
    $lastname = $sr[1];
    $firstname = $sr[2];
    $initial = $sr[3];

    print "<p style=\"font-size:130%; color:black; background:#CCC; ";
    print "font-weight:bold; padding: 3px 1em;border: 1px solid black;\">";
    print "$firstname $initial $lastname</p>\n";

    if (not $sr[57]){ # no prov number.
	print "<p style=\"color:red;\">No Provincial Number!</p>\n";
	next;
    }
    if (not $sr[8]){ # no birthdate
	print "<p style=\"color:red;\">No Birthdate!</p>\n";
	next;
    }

    # Grade and Provincial Number passed
    $count++;
    &mkQueryString($sr[8],$sr[57],$count); # Generate $output string

    # DEBUG Data Errors
    if ($debug){
	print "<div style=\"color: blue;\">\n";
	print "<h1>DEBUG - Request sent to Sask Learning</h1>\n";
	my $temp = $output->value;
	$temp =~ s/</&lt;/g;
	$temp =~ s/>/&gt;/g;
	print "<pre>",$temp,"</pre></div><p></p>\n";
    }

    
    # Create the https post request 

    my $req = POST $url, [ XML=>$output->value ];

    $req->content_type('application/xml;charset="utf-8"');
    $req->authorization_basic($sds_userid, $sds_password); 

    # Issue the request and receive a response
    my $res = $ua->request($req);

    # Check the status of the response
    if ($res->is_success) {

	# For Debugging Data Errors
	if ($debug){
	    print "<h1>DEBUG - Sask Learning XML Response</h1>\n";
	    print "<div style=\"color:green;\">\n";
	    my $temp = $res->content;
	    $temp =~ s/</&lt;/g;
	    $temp =~ s/>/&gt;/g;
	    print "<pre>",$temp,"</pre></div>\n";
	 }


	# Parse the response.
	my $parser = XML::LibXML->new();

	eval {$doc = $parser->parse_string($res->content)};
	if ($@){
	    print "<pre>Sask Learning Error:\n",$res->content,"</pre><br>\n";
	    print "</body></html>\n"; die;
	}

	$doc->setEncoding('UTF-8');
	$root = $doc->getDocumentElement;
	$root->setNamespace($xmlns,'sl',1);

	# find Message ID.
	$mastermsgid = $root->findvalue('//sl:SL_MsgId');

	$status = $root->findvalue('//sl:SL_Status/sl:SL_StatusCode');


	if ($status eq 'Errors' or $status eq 'Invalid'){ 
	    # Print out error and add to errorlog.
	    &prErr($mastermsgid,"$lastname, $firstname ($studnum)");

	} elsif ($status eq 'Successful'){ 
	    &parseStudentPersonal;
	} else { # print warnings... ($status eq 'Warnings')
	    print "There were warnings...";
	}

    }  else { # Transfer Error!
	my $err = $res->status_line;
	print "<h1>Transfer Error: $err</h1>";
    }

} # End of Main Student Loop

print "</body></html>\n";


# Functions =============================

#--------------
sub mkQueryString {
#--------------
    my $grade = shift;
    my $provnum = shift;
    my $idcount = shift;
    # Date and schoolnumber are globals.

    # Create Writer Instance
    $output = new XML::Writer::String;
    my $datamode = 0;
    if ($debug){ $datamode = 1;} # pretty print xml output
    $wr = new XML::Writer(OUTPUT => $output, 
			  DATA_MODE => $datamode, 
			  DATA_INDENT => '2');

    # Set XML Header and write Root Element
    $wr->xmlDecl("UTF-8");
    $wr->startTag('SL_Message','xmlns' =>$xmlns, 
		  'xmlns:xsi' =>$xmlnsxsi, 
		  'xsi:schemaLocation' => $xsischemaLocation);

    $wr->startTag('SL_Request');
    &mkSL_Header($currdate, $currtime, $schoolnumber,$idcount);

    $wr->startTag('SL_Query');

    $wr->startTag('SL_QueryObject',
		  'ObjectName' => 'StudentPersonal');

    $wr->dataElement('DeptAssignedPersonId',$provnum); 
    $wr->dataElement('BirthDate',$grade); 	
    
    $wr->endTag('SL_QueryObject');

    $wr->endTag('SL_Query');
    $wr->endTag('SL_Request');
    $wr->endTag('SL_Message');
    $wr->end();
    
}

#---------
sub prErr {  # Print Errors to Screen and Errorlog
#---------
    my $msgid = shift;
    my $name = shift;

    #print "<p style=\"color:red; font-size:150%\">Error</p>\n";
    my @errors = $root->findnodes('//sl:SL_Error');
    foreach my $error (@errors){
	# Get Error Elements
	my $objectname = $error->getAttribute('ObjectName');
	my $errmsg = $error->findvalue('./sl:SL_ErrorMsg');
	$errmsg =~ s/[\r|\n]/ /g;
	my $errcode = $error->findvalue('./sl:SL_ErrorCode');
	
	# Write the Error Log
	print ERR "$msgid:$name:$errmsg:$errcode\n";
	
	print "<p style=\"color:red; font-size:120%\">$errmsg</p>\n";
    }
    print "</table>\n";
    
}


#----------------------
sub parseStudentPersonal { # passed document root object.
#----------------------
    my @studinfo = $root->findnodes('//sl:StudentPersonal/sl:StudentInfo');
    # skip the student identification element, only StudentInfo.
    # Student Info: Name,Demographics,StudentAddress,PhoneNumber objects
    foreach my $student (@studinfo){
	my $provnum = $student->findvalue('//sl:DeptAssignedPersonId');
	my $firstname = $student->findvalue('//sl:Name/sl:FirstName');
	my $lastname = $student->findvalue('//sl:Name/sl:LastName');
	my $middlename = $student->findvalue('//sl:Name/sl:MiddleName');
	my $gender = $student->findvalue('//sl:Demographics/sl:Gender');
	my $language = $student->findvalue('//sl:Demographics/sl:Language');
	my $address = $student->findvalue('//sl:StudentAddress/sl:Address/sl:Street');
	my $prov = $student->findvalue('//sl:StudentAddress/sl:Address/sl:StatePr/@Code');
	my $city = $student->findvalue('//sl:StudentAddress/sl:Address/sl:City');
	my $country = $student->findvalue('//sl:StudentAddress/sl:Address/sl:Country/@Code');
	my $pcode = $student->findvalue('//sl:StudentAddress/sl:Address/sl:PostalCode');
	my $phone = $student->findvalue('//sl:PhoneNumber');


	# Get student record
	my  $sth = $dbh->prepare("select * from studentall
         where provnum = '$provnum'");
	$sth->execute;
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	my @sr = $sth->fetchrow;

	$sth = $dbh->prepare("select count(studid) from student
         where provnum = '$provnum'");
	$sth->execute;
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	my $count = $sth->fetchrow;
	if (not $count){ $tb = 'wd' } else { $tb = 'stud';}

	# Now do a comparison.
	if ($lastname ne $sr[1]){
	    $lastname = "<font color=\"red\">$lastname</font>";
	}
	if ($firstname ne $sr[2]){
	    $firstname = "<font color=\"red\">$firstname</font>";
	}
	if ($middlename ne $sr[3]){
	    $middlename = "<font color=\"red\">$middlename</font>";
	}
	if ($gender ne $sr[7]){
	    $gender = "<font color=\"red\">$gender</font>";
	}
	if ($address ne $sr[34]){
	    $address = "<font color=\"red\">$address</font>";
	}
	if ($city ne $sr[35]){
	    $city = "<font color=\"red\">$city</font>";
	}
	if ($prov ne $sr[69]){
	    $prov = "<font color=\"red\">$prov</font>";
	}
	if ($pcode ne $sr[36]){
	    $pcode = "<font color=\"red\">$pcode</font>";
	}
	if ($phone ne $sr[30]){
	    $phone = "<font color=\"red\">$phone</font>";
	}

	
	print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
	print "<tr><th align=\"right\">Name</th><th>Sask Lrn Value</th>";
	print "<th>Local Value</th></tr>\n";

	print "<tr><td align=\"right\">First Name</td><td>$firstname</td>";
	print "<td>$sr[2]</td></tr>\n";
	print "<tr><td align=\"right\">Middle Name</td><td>$middlename</td>";
	print "<td>$sr[3]</td></tr>\n";
	print "<tr><td align=\"right\">Last Name</td><td>$lastname</td>";
	print "<td>$sr[1]</td></tr>\n";

        print "<tr><td align=\"right\">Gender</td><td>$gender</td>";
	print "<td>$sr[7]</td></tr>\n";

	print "<tr><td align=\"right\">Language</td><td>$language</td>";
	print "<td>Not Stored</td></tr>\n";

	print "<tr><td align=\"right\">Address</td><td>$address</td>";
	print "<td>$sr[34]</td></tr>\n";

	print "<tr><td align=\"right\">City</td><td>$city</td>";
	print "<td>$sr[35]</td></tr>\n";

	print "<tr><td align=\"right\">Prov</td><td>$prov</td>";
	print "<td>$sr[69]</td></tr>\n";

	print "<tr><td align=\"right\">Country</td><td>$country</td>";
	print "<td>Not Stored</td></tr>\n";

	print "<tr><td align=\"right\">Postal Code</td><td>$pcode</td>";
	print "<td>$sr[36]</td></tr>\n";

	print "<tr><td align=\"right\">Phone</td><td>$phone</td>";
	print "<td>$sr[30]</td></tr>\n";

	print "</table>\n";

	print "<p style=\"color:red;\">[ Red = Differences Between Sask ";
	print "Learning and Local Data ]</p>\n";

	print "<form action=\"../studed.pl\" ";
	print "method=\"post\">";
	print "<input type=\"hidden\" name=\"recnum\" value=\"$sr[0]\">\n";
	print "<input type=\"hidden\" name=\"tb\" value=\"$tb\">\n";
	print "<input type=\"submit\" value=\"Edit\"></form>\n";




    }
} # End of Sub

