#!/usr/bin/perl
#  Copyright 2001-2005 Leslie Richardson

#  This file is part of Open Admin for Schools.

# A Library of XML Routines for use with Sask Learning XML Schema

#-------------------------------
# Updates:
# Oct 28/04: $nativeschool = 1 in admin.conf is required in order to generate
#  native demographics information. Otherwise not sent.

#-------------------------------

# Assume the use of the following in the calling routines.
#use XML::Writer;
#use Data::UUID;

# Read config variables
#require "../../etc/admin.conf" || die "Cannot read admin.conf!";

#$wr = new XML::Writer(OUTPUT => $output, DATA_MODE => '1',DATA_INDENT => '2');


# ============ Subroutines Start Here.===============

#----------------------------
sub mkStudentSchoolEnrollment {
#----------------------------
    my $seiRef = shift; # Pop off reference
    my @sr = @_; # Pass Entire record; used by StudentInfo routine.

    #my $ug = new Data::UUID;
    #my $refid = $ug->create_hex();
    #$refid =~ s/^0x//;  # strip leading 0x indicating hex encoded string
    my $refid = $sr[4]; # Local Student Number is the refid.

    $wr->startTag('StudentSchoolEnrollment', RefId=>$refid);
    &mkStudentIdentification($sr[57],$sr[8]); # pass SLID and Bdate 
    &mkStudentInfo(@sr); # pass entire student record
    $wr->dataElement('SchoolId',$schoolnumber); # from admin.conf file.

    # Needs MembershipType, EntryDate, EntryCode, Grade, PrevProv,PrevCountry,
    #  ExitDate, ExitType,

    &mkSchoolEnrollmentInfo($seiRef);
    
    if (not $sr[72]){ # program field empty
	$myprog = $program; # from admin.conf
    } else {
	$myprog = $sr[72];
    }
    $wr->dataElement('DeptAssignedProgramId',$myprog); 
    &mkProgramEnrollmentInfo("$currdate");

    $wr->endTag('StudentSchoolEnrollment');
    return $refid;  # return the RefID for other objects to use.
}


#----------------------------- 
sub mkStudentProgramEnrollment {
#-----------------------------
    my ($studnum, $birthdate, $programid, $startdate,$enddate) = @_;

    #my $ug = new Data::UUID;
    #my $refid = $ug->create_hex();
    #$refid =~ s/^0x//;
    my $refid = $studnum.".1"; 
    # E for enrollment; studentinfo will just use the studnum 

    $wr->startTag('StudentProgramEnrollment', RefId=>$refid);

    &mkStudentIdentification($studnum,$birthdate);

    $wr->dataElement('DeptAssignedProgramId',$programid);

    &mkProgramEnrollmentInfo($startdate,$enddate);

    $wr->endTag('StudentProgramEnrollment');
    return $refid;
}

#--------------------------
sub mkStudentClassEnrollment {
#--------------------------
    my ($refid, $studnum, $birthdate, $classid, $startdate, $enddate, 
	$override, $source, $dropped, $exitdate, $mode) = @_; 

    $wr->startTag('StudentClassEnrollment',RefId=>$refid);

    &mkStudentIdentification($studnum,$birthdate);

    &mkClassIdentification($schoolnumber, $classid, $startdate, $enddate);
    # $schoolnumber comes in from admin.conf file.

    &mkClassEnrollmentInfo($override,$source,$dropped,$exitdate,$mode);

    if ($enddate){
	&mkExamRegistrationInfo($enddate);
    }

    $wr->endTag('StudentClassEnrollment');
    return $refid;
}


#---------------------
sub mkStudentClassMark {
#---------------------
    my ($refid, $studnum, $birthdate, $classid, $startdate, $enddate, 
	$mark, $correction) = @_; 

    $wr->startTag('StudentClassMark',RefId=>$refid);

    &mkStudentIdentification($studnum,$birthdate);

    &mkClassIdentification($schoolnumber, $classid, $startdate, $enddate);
    # $schoolnumber comes in from admin.conf file.

    &mkSchoolMarkInfo($mark, $correction);

    $wr->endTag('StudentClassMark');
    return $refid;
}


#----------------
sub mkSchoolClass {
#----------------
    my ($refid, $classid, $startdate, $enddate, $courseid,$teachid,$mode) = @_;
    # Note: $schoolnumber is global.

    # Pass in refid instead (which will be subject/section code)
    #my $ug = new Data::UUID;
    #my $refid = $ug->create_hex();
    #$refid =~ s/^0x//;

    $wr->startTag('SchoolClass',RefId=>$refid);

    &mkClassIdentification($schoolnumber, $classid, $startdate, $enddate);

    &mkClassInfo($courseid, $teachid, $mode);

    $wr->endTag('SchoolClass');
    return $refid;
}

#-------------------------
sub mkStudentCourseHistory {
#-------------------------

    my $ug = new Data::UUID;
    my $refid = $ug->create_hex();
    $refid =~ s/^0x//;

    $wr->startTag('StudentCourseHistory',RefId=>$refid);

    &mkStudentIdentification($studnum, $birthdate);
    $wr->dataElement('DeptAssignedCourseId',$courseid);
    if ($coursetitle){
	$wr->dataElement('CourseTitle',$coursetitle);
    }

    &mkStudentCourseInfo();

    $wr->endTag('StudentCourseHistory');
    return $refid;
}


#====== Lower Level Elements Start Here ==============


#-------------------------
sub mkSchoolEnrollmentInfo {
#-------------------------

    # Needs MembershipType, EntryDate, EntryType, Grade, 
    # PrevProv,PrevCountry, ExitDate, ExitType,
    my ($sref) = @_;

    # print "HashRef: $sref \n";
    #($membershipType, $entryDate, $entryType, $grade,$prevProvState,
    # $prevCountry,$exitDate, $exitType) = @$SEIref;
    if ($$sref{MembershipType}){
	$wr->startTag('SchoolEnrollmentInfo',
		      'MembershipType' => $$sref{MembershipType});
    } else {
	$wr->startTag('SchoolEnrollmentInfo');
    }
    
    if ($$sref{EntryDate}){
	$wr->dataElement('EntryDate',$$sref{EntryDate});
    }

    if ($$sref{EntryType}){
	$wr->emptyTag('EntryType','Code' => $$sref{EntryType});
    }

    # Grade: Must be PK,K,01,02, etc.
    if ($$sref{Grade}){
	$grade = uc($$sref{Grade});
	if (length($grade) == 1){ # Add a zero.
	    $grade = '0'.$grade;
	}
	$wr->emptyTag('Grade','Code' => $grade);
    }

    if ($$sref{PrevProvState}){
	$wr->emptyTag('PreviousProvState','Code'=>$$sref{PrevProvState});
    }

    if ($$sref{PrevCountry}){
	$wr->emptyTag('PreviousCountry','Code'=>$$sref{PrevCountry});
    }

    if ($$sref{ExitDate}){
	$wr->dataElement('ExitDate',$$sref{ExitDate});
    }

    if ($$sref{ExitType}){
	$wr->emptyTag('ExitType','Code' => $$sref{ExitType});
    }

    $wr->endTag('SchoolEnrollmentInfo');
}


#--------------------
sub mkStudentPersonal {
#--------------------
    # Passed Complete Student Record
    my @sr = @_;

    $wr->startTag('StudentPersonal',RefId =>$sr[4]);
    &mkStudentIdentification($sr[57],$sr[8]); # pass SLID and Bdate 
    &mkStudentInfo(@sr); # pass entire student record
    $wr->endTag('StudentPersonal');
}


#--------------------------
sub mkStudentIdentification {
#--------------------------
    # Pass Prov # and Birthdate (57 and 8)
    my $deptAssignedPersonId = shift;
    my $birthDate = shift;
    $wr->startTag('StudentIdentification');

    if ($deptAssignedPersonId){
	$wr->dataElement('DeptAssignedPersonId',$deptAssignedPersonId);
    }

    if ($birthDate and $birthDate ne '0000-00-00'){
	$wr->dataElement('BirthDate',$birthDate);
    }

    $wr->endTag('StudentIdentification');
}


#----------------
sub mkStudentInfo { 
#----------------
    # Passed Complete Student Record
    my @sr = @_;

    $wr->startTag('StudentInfo');

    # Studnum
    if ($sr[4]){
	$wr->dataElement('SchoolAssignedPersonId',$sr[4]);
    }

    # Health Id (HSN)
    if ($sr[10]){
	$sr[10] =~ s/ //g; # strip spaces, just in case...
	$wr->dataElement('HSN',$sr[10]);
    }

    # Name
    if ($sr[1] and $sr[2]){ # At least first and lastname
	$wr->startTag('Name',Type => 'Legal'); # Type = Legal Name
	$wr->dataElement('LastName',$sr[1]);
	$wr->dataElement('FirstName',$sr[2]);
	if ($sr[3]){ # We have an initial/middle name
	    $wr->dataElement('MiddleName',$sr[3]);
	}
	$wr->endTag('Name');
    }



    # Start of Demographics - Gender / Treaty
    $wr->startTag('Demographics');

    if ($sr[7]){
	$wr->dataElement('Gender',$sr[7]);
    }
    
    #~ $nativeschool global used to disable this section.
    # Aboriginal Status
    if ($sr[12] and $sr[12] ne 'Non Native' and 
	$nativeschool){ # exists and is native
	if ($sr[12] eq 'Status Native'){
	    $ethnic = 'Treaty';
	} elsif ($sr[12] eq 'Non-status Native'){
	    $ethnic = 'NS';
	} elsif ($sr[12] eq 'Metis'){
	    $ethnic = 'Metis';
	} else { $ethnic = ''; }
	$wr->emptyTag('AboriginalStatus','Code' =>$ethnic);
    }

    # Treaty Number (of student); Unique.
    if ($sr[17] and $nativeschool){
	$wr->emptyTag('TreatyStatusNumber','Number'=>$sr[17]);
    }

    # Band Affiliation
    if ($sr[59] and $nativeschool){
	$wr->emptyTag('BandAffiliation','Code' =>$sr[59]);
    }

    # Reserve Residency 
    if ($sr[71] and $nativeschool){
	$wr->emptyTag('ReserveResidency','Code' =>$sr[71]);
    }

    $wr->endTag('Demographics');


    # Student Address Elements
    if ($sr[34] or $sr[35] or $sr[69] or $sr[36]){
	$wr->startTag('StudentAddress');
	$wr->startTag('Address');

	if ($sr[34]){
	    $wr->startTag('Street');
	    $wr->dataElement('Line1',$sr[34]);
	    $wr->endTag('Street');
	}
	if ($sr[35]){
	    $wr->dataElement('City',$sr[35]);
	}

	if ($sr[69]){
	    $wr->emptyTag('StatePr','Code' =>$sr[69]);
	}

	# Write out Country Anyway....
	$wr->emptyTag('Country','Code' => 'CA');

	if ($sr[36]){
	    $wr->dataElement('PostalCode',$sr[36]);
	}

	$wr->endTag('Address');
	$wr->endTag('StudentAddress');
    } # End of Address
    
    if ($sr[30]){ #home phone exists
	$wr->dataElement('PhoneNumber',$sr[30],Format=>'FF',Type => 'Home');
    }
    
    $wr->endTag('StudentInfo');

} # End of mkStudentInfo


#--------------------------
sub mkProgramEnrollmentInfo {
#--------------------------
    my $entryDate =  shift;

    $wr->startTag('ProgramEnrollmentInfo');
    if ($entryDate){
	$wr->dataElement('EntryDate',$entryDate);
    }

    # Dropped from spec
    #if ($exitDate){
    # $wr->dataElement('ExitDate',$exitDate);
    #}

    $wr->endTag('ProgramEnrollmentInfo');
}


#------------------------
sub mkClassIdentification {
#------------------------
    ($schoolid, $classid, $startdate, $enddate) = @_;

    $wr->startTag('ClassIdentification');

    $wr->dataElement('SchoolId',$schoolid);
    $wr->dataElement('ClassId',$classid);
    $wr->dataElement('StartDate',$startdate);
    $wr->dataElement('EndDate',$enddate);

    $wr->endTag('ClassIdentification');
}

#--------------
sub mkClassInfo {
#--------------
    my ($courseid, $teacherid, $instructionmode) = @_; 

    $wr->startTag('ClassInfo');
    
    $wr->dataElement('DeptAssignedCourseId',$courseid);
    
    $wr->dataElement('EducatorCertificateNumber',$teacherid);
    
    if ($instructionmode){
	$wr->dataElement('ModeOfInstruction',$instructionmode);
    }

    $wr->endTag('ClassInfo');
}

#-------------------------
sub mkExamRegistrationInfo {
#-------------------------
    ($enddate) = @_;

    # ~~ Comment this out until fixed...
    $wr->startTag('ExamRegistrationInfo');
    # optional and default value in StudentClassEnrollment -,'Type' => 'Class');

    $wr->dataElement('ExamPeriodEndDate',$enddate);

    $wr->endTag('ExamRegistrationInfo');
}

#------------------------
sub mkClassEnrollmentInfo {
#------------------------
    my ($override, $marksource, $dropped, $exitdate, $instructionmode) = @_;

    #  Note: Don't create the object unless something passed.
    #   '$dropped and $exitdate will both be passed as same time.
    if ($override or $marksource or $dropped){
	$wr->startTag('ClassEnrollmentInfo');
    }

    if ($override){
	$wr->startTag('ClassInfoOverride','Override' => $override);
	$wr->emptyTag('ModeOfInstruction','Code' => $instructionmode);
	$wr->endTag('ClassInfoOverride');
    }

    if ($marksource){
	$wr->emptyTag('MarkSource','Code'=>$marksource);
    }

    if ($dropped){
	$wr->emptyTag('Dropped','Code'=>$dropped);
    }

    if ($exitdate){
	$wr->dataElement('ExitDate',$exitdate);
    }

    if ($override or $marksource or $dropped){
	$wr->endTag('ClassEnrollmentInfo');
    }

}

#-------------------
sub mkSchoolMarkInfo {
#-------------------
    my ($mark, $corrcode) = @_;

    if ($corrcode){
	$wr->startTag('SchoolMarkInfo','CorrectionCode' =>$corrcode);
    } else {
	$wr->startTag('SchoolMarkInfo');
    }

    $wr->dataElement('SchoolMark',$mark);

    $wr->endTag('SchoolMarkInfo');
}


#----------------------
sub mkStudentCourseInfo {
#----------------------
    ($type, $enddate, $credits, $schoolyear, $dropped, $marks) = @_;

    $wr->startTag('StudentCourseInfo','Type' =>$type);

    if ($schoolyear){
	$wr->dataElement('SchoolYear',$schoolyear);
    }

    $wr->dataElement('CourseEndDate',$enddate);

    if ($dropped){
	$wr->dataElement('Dropped',$dropped);
    }
    
    if ($mark){
	$wr->dataElement('FinalMark',$mark);
    }

    $wr->dataElement('CreditsEarned',$credits);

    $wr->endTag('StudentCourseInfo');
}

#-------------
sub mkTermInfo {
#-------------
    my ($startdate, $enddate, $description) = @_;
    $wr->startTag('TermInfo');
    
    $wr->dataElement('StartDate',$startdate);
    $wr->dataElement('EndDate',$enddate);
    if ($description){
	$wr->dataElement('Description',$description);
    }

    $wr->endTag('TermInfo');
}


# ========== Message Elements Here ============

#--------------
sub mkSL_Header {
#--------------
    my ($date, $time, $sourceid, $count) = @_;

    my $timezone = 'UTC-06:00';

    #my $ug = new Data::UUID;
    #my $refid = $ug->create_hex();
    #$refid =~ s/^0x//;
    my $shortdate = $currdate;
    $shortdate =~ s/-//g;
    my $shorttime = $currtime;
    $shorttime =~ s/://g;
    my $refid = $schoolnumber.".".$shortdate.".".$shorttime.".$$".$count;

    $wr->startTag('SL_Header');

    $wr->dataElement('SL_MsgId',$refid);
    $wr->dataElement('SL_Date',$date);

    $wr->startTag('SL_Time','Zone' => $timezone);
    $wr->characters($time);
    $wr->endTag('SL_Time');

    $wr->dataElement('SL_SourceId',$sourceid);
    $wr->endTag('SL_Header');

    return $refid;
}


# No longer used.
#-----------------------
sub mkSL_UserCredentials {
#-----------------------
    my ($userid, $password) = @_;


    $wr->startTag('SL_UserCredentials');
    
    $wr->dataElement('SL_UserId',$userid);
    $wr->dataElement('SL_Password',$password);

    $wr->endTag('SL_UserCredentials');
}


# Acknowledgement... normally FROM Sask Learning
#-----------
sub mkSL_Ack {
#-----------
    my ($origMsgId, $origSourceId,$status) = @_;

    $wr->startTag('SL_Ack');
    
    $wr->dataElement('SL_OriginalMsgId',$origMsgId);
    $wr->dataElement('SL_OriginalSourceId',$origSourceId);

    $wr->startTag('SL_Status');
    $wr->dataElement('SL_StatusCode',$status);
    $wr->endTag('SL_Status');

    $wr->endTag('SL_Ack');
}


# 1 needed to return true...
1;

