#! /usr/bin/perl
#  Copyright 2001-2006 Leslie Richardson

#  This file is part of Open Admin for Schools.

# Synchronize School Courses
# Display courses by start and endterm, select those to synchronize.
# Get course enrollment data from SL.
# Find the differences between the two for each course in turn.
# Do the course enrollments to get synchronized. 
# Rerun Query to check synchronization.

# Passed Values: none.

# Notes: the dates in doEnrollments are not passed from previous page,
# but rather are the values from config files. Might be better to use
# SL date values from query... later.


$self = 'synccourses.pl';
$additionalcomments = 'Additional Comments';


use DBI;
use CGI;
use XML::Writer;
use XML::Writer::String;
use Data::UUID;
use HTTP::Request::Common qw(POST); 
use HTTP::Headers; 
use LWP::UserAgent; 
use XML::LibXML;
use Number::Format qw(round);


# Read config variables
require "../../etc/admin.conf" or die "Cannot read admin.conf: $!";
require "slxmllib.pl" or die "Cannot read slxmllib.pl: $!";

# Override V1 with V2 values
# SDS XML Schema Declarations
$xmlns2 = "http://www.sasked.gov.sk.ca/xsd/sl/2.x/SLMessage_V2.xsd";
$xmlnsxsi2 = "http://www.w3.org/2001/XMLSchema-instance";
$xsischemaLocation2="http://www.sasked.gov.sk.ca/xsd/sl/2.x/SLMessage_V2.xsd
http://www.sasked.gov.sk.ca/xsd/sl/2.x/SLMessage_V2.xsd";

$dsn = "DBI:$dbtype:dbname=$dbase";
$dbh = DBI->connect($dsn,$user,$password);

$q = new CGI;
%arr = $q->Vars;
print $q->header;

# Date Stuff setup; required for the SDS transfers header also
@month = ('','January','February','March','April','May','June','July',
	  'August','September','October','November','December');
# $sec, $min, $hour, $mday, $mon, 
# $year, $wday, $yday, $isdst ) = localtime(time);

@tim = localtime(time);
$year = $tim[5] + 1900;
$tim[4]++;
for (0..4){if (length($tim[$_]) == 1){ $tim[$_] = '0'.$tim[$_];}}
$currdate = "$year-$tim[4]-$tim[3]";
$currtime = "$tim[2]:$tim[1]:$tim[0]";


# Print Page Header
print "$doctype\n<html><head><title>Course Enrollment Synchronization</title> 
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
<style type=\"text/css\">body { padding: 1em 3em; }</style>
$chartype\n</head><body>[ <a href=\"$homepage\">Main</a> | 
<a href=\"$exppage\">Export</a> ]
<center><h1>Course Enrollment Synchronization</h1>\n";


if ($arr{enrolflag}) { # Fourth - Send Course Enrollment Updates
    delete $arr{enrolflag};
    doEnrollments();
    die;
}

if ($arr{subjectflag}) { # Third - Select Students in Subjects
    delete $arr{subjectflag};
    syncSubjects();
    die;
}

if ($arr{termflag}) { # Second - Select Subjects
    delete $arr{termflag};
    selectSubjects();
    die;

} else {  # First Choice - Select Termstart/end
    showTerms();
    die;
}


# Functions =============================

#----------------
sub mkQueryString { # for Course Enrollments
#----------------
    my $grade = shift;
    my $provnum = shift;
    my $idcount = shift;
    # Date and schoolnumber are globals.

    # Create Writer Instance
    $output = new XML::Writer::String;
    my $datamode = 0;
    if ($debug){ $datamode = 1;} # pretty print xml output
    $wr = new XML::Writer(OUTPUT => $output, 
			  DATA_MODE => $datamode, 
			  DATA_INDENT => '2');

    # Set XML Header and write Root Element
    $wr->xmlDecl("UTF-8");
    $wr->startTag('SL_Message','xmlns' =>$xmlns2, 
		  'xmlns:xsi' =>$xmlnsxsi2, 
		  'xsi:schemaLocation' => $xsischemaLocation2);

    $wr->startTag('SL_Request');
    &mkSL_Header($currdate, $currtime, $schoolnumber,$idcount);

    $wr->startTag('SL_Query');

    $wr->startTag('QueryBySchool',
		  'RefId' => "$schoolnumber",
		  'ObjectName' => 'StudentCourseEnrollments',
		  'ScopeCode' => 'Current'
		  );

    $wr->dataElement('SchoolId',$schoolnumber); 

    #$wr->dataElement('FromDate','20051201');
    #$wr->dataElement('ToDate','20051205');

    $wr->endTag('QueryBySchool');

    $wr->endTag('SL_Query');
    $wr->endTag('SL_Request');
    $wr->endTag('SL_Message');
    $wr->end();
    
}


#-----------------
sub querySLCourses {
#-----------------

    # Query Sask Learning for a list of current course enrollments
    #  Uses other subs: mkQueryString, parseSCEbyClass (makes object)

    # Create a new user agent
    my $ua = LWP::UserAgent->new();
    $ua->agent("OpenAdmin");

    $count=1;
    &mkQueryString($count); # Generate $output string

    # DEBUG Data Errors
    if ($debug){
	print "</center><div style=\"color: blue;\">\n";
	print "<h1>DEBUG - Request sent to Sask Learning</h1>\n";
	my $temp = $output->value;
	$temp =~ s/</&lt;/g;
	$temp =~ s/>/&gt;/g;
	print "<pre>",$temp,"</pre></div><p></p>\n";
    }

    # Create the https post request 

    my $req = POST $url, [ XML=>$output->value ];
    
    $req->content_type('application/xml;charset="utf-8"');
    $req->authorization_basic($sds_userid, $sds_password); 

    # Issue the request and receive a response
    my $res = $ua->request($req);

    # Check the status of the response
    if ($res->is_success) {
	
	# For Debugging Data Errors
	if ($debug){
	    print "<h1>DEBUG - Sask Learning XML Response</h1>\n";
	    print "<div style=\"color:green;\">\n";
	    my $temp = $res->content;
	    $temp =~ s/</&lt;/g;
	    $temp =~ s/>/&gt;/g;
	    print "<br>",$temp,"</div>\n";
	}


	# Parse the response.
	my $parser = XML::LibXML->new();
	
	eval {$doc = $parser->parse_string($res->content)};
	if ($@){
	    print "Error: $@<br>\n";
	    print "<pre>Sask Learning Error:\n",$res->content,"</pre><br>\n";
	    print "</body></html>\n"; die;
	}

	$doc->setEncoding('UTF-8');
	$root = $doc->getDocumentElement;
	$root->setNamespace($xmlns2,'sl',1);
	
	$status = $root->findvalue('//sl:SL_Status/sl:SL_StatusCode');
	# We are now setup to parse the root element. 

	
    }  else { # Transfer Error!
	my $err = $res->status_line;
	print "<h1>Transfer Error: $err</h1>";
	die;
    }
    
    my $course_ref = parseSCEbyClass();
    
    return $course_ref; 


}



#--------------------------------
sub parseStudentCourseEnrollments { # passed document root object.
#--------------------------------
    # This does a printout of the structure.... next sub creates object.

    my @studinfo = $root->findnodes('//sl:StudentCourseEnrollments');
    my $count = 1;
    my $sth = $dbh->prepare("select lastname, firstname from student
     where provnum = ? ");

    foreach my $student (@studinfo){
	my $provnum = $student->findvalue('sl:StudentIdentification/sl:DeptAssignedPersonId');
	my $birthdate = $student->findvalue('sl:StudentIdentification/sl:BirthDate');

	$sth->execute($provnum);
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	my ($lastname, $firstname) = $sth->fetchrow;

	print "<h1>$firstname $lastname ( $provnum - $birthdate )</h1>\n";
	print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
	print "<tr><th>Schoolyear</th><th>Termstart</th><th>Termend</th>";
	print "<th>ClassId</th><th>Mode</th><th>Mark Src</th></tr>\n";

	my @classes = $student->findnodes('sl:StudentSchoolClasses/sl:StudentTermClasses');
	foreach my $class (@classes) {
	    my $schoolyear = $class->findvalue('sl:TermInfo/@SchoolYear');
	    my $termstart = $class->findvalue('sl:TermInfo/sl:StartDate');
	    my $termend = $class->findvalue('sl:TermInfo/sl:EndDate');
	    my @sections = $class->findnodes('sl:StudentClass');
	    foreach my $section (@sections) {
		my $classid = $section->findvalue('sl:ClassId');
		my $mode = $section->find('sl:ModeOfInstruction/@Code');
		my $marksource = $section->findvalue('sl:MarkSource/@Code');
		print "<tr><td>$schoolyear</td><td>$termstart</td><td>$termend</td>\n";
		print "<td>$classid</td><td>$mode</td><td>$marksource</td></tr>\n";
	    }
	}
	print "</table>\n";
    }

} # End of Sub 


#---------------------- 
sub parseSCEbyClass { 
#---------------------- 

    # passed document root object.  
    # Parse SchoolCourseEnrollments, and return a reference to a courses
    # hash containing refs to a list (array) of provincial numbers.

    my @studinfo = $root->findnodes('//sl:StudentCourseEnrollments');

    my $count = 1;
    my %courses;

    foreach my $student (@studinfo){
	my $provnum = $student->findvalue('sl:StudentIdentification/sl:DeptAssignedPersonId');
	# my $birthdate = $student->findvalue('sl:StudentIdentification/sl:BirthDate');

	my @classes = $student->findnodes('sl:StudentSchoolClasses/sl:StudentTermClasses');
	foreach my $class (@classes) {
	    #my $schoolyear = $class->findvalue('sl:TermInfo/@SchoolYear');
	    my $termstart = $class->findvalue('sl:TermInfo/sl:StartDate');
	    my $termend = $class->findvalue('sl:TermInfo/sl:EndDate');
	    my @sections = $class->findnodes('sl:StudentClass');
	    foreach my $section (@sections) {
		my $classid = $section->findvalue('sl:ClassId');
		my $mode = $section->find('sl:ModeOfInstruction/@Code');
		my $marksource = $section->findvalue('sl:MarkSource/@Code');

		$coursedata{"$classid:$termstart:$termend"} = "$mode:$marksource";
		if (not defined $courses{"$classid:$termstart:$termend"}) {
		    $courses{"$classid:$termstart:$termend"} = [ $provnum ];
		} else {
		    push @{$courses{"$classid:$termstart:$termend"}},$provnum;
		}
	    }
	} # End of StudentSchoolClasses

    } # End of student loop

    return \%courses;

} # End of parseSCEbyCourse


#------------
sub showTerms {
#------------

    print "<div style=\"font-size:120%; font-weight:bold;\">\n";
    print "Select a start term / end term combination</div>\n";
    print "<table cellspacing=\"0\" cellpadding=\"3\" border=\"1\" style=\"text-align:center;\">\n";
    print "<tr><th>Start Term</th><th>End Term</th><th>Select</th></tr>\n";
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"termflag\" value=\"1\">\n";

    # Show the Start/End terms for choosing subjsec's.
    my $sth1 = $dbh->prepare("select distinct startrptperiod, endrptperiod from subject
      group by startrptperiod, endrptperiod order by startrptperiod, endrptperiod");
    $sth1->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
    while (my ($startterm, $endterm) = $sth1->fetchrow) {
	print "<tr><td>$startterm</td><td>$endterm</td>";
	print "<td><input type=\"checkbox\" name=\"$startterm:$endterm\" value=\"1\"></td></tr>\n";
    }

    print "<tr><td colspan=\"3\"><input type=\"submit\" value=\"Submit Term\">\n";
    print " Chk <input type=\"checkbox\" name=\"check\" value=\"1\">\n";
    print " Debug <input type=\"checkbox\" name=\"debug\" value=\"1\"></td></tr>\n";
    print "</form></table></body></html>\n";

}


#-----------------
sub selectSubjects {
#-----------------

    # foreach my $key (keys %arr) { print "K: $key V:$arr{$key}<br>\n"; }

    my ($check,$debug);
    if ($arr{check}) { 
	$check = 'CHECKED';
	delete $arr{check};
    }
    if ($arr{debug}) {
	$debug = 1;
	delete $arr{debug};
    }


    my ($key) = keys %arr;
    #print "K: $key  CHK: $check<br>\n";
    my ($startterm, $endterm) = split /:/, $key;

    print "<center><div style=\"font-size:120%; font-weight:bold;\">\n";
    print "Start Term: $startterm - End Term $endterm<br>\n";
    print "Select subjects to synchronize with SL</div>\n";
    print "<table cellspacing=\"0\" cellpadding=\"3\" border=\"1\">\n";
    #print "style=\"text-align:center;\">\n";
    print "<tr><th>Subject</th><th>Select</th>\n";
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"subjectflag\" value=\"1\">\n";
    print "<input type=\"hidden\" name=\"debug\" value=\"$debug\">\n";
 
    my $sth1 = $dbh->prepare("select subjsec, description, teacher from subject
      where startrptperiod = '$startterm' and endrptperiod = '$endterm'
      and description != '$additionalcomments'
      order by description");
    $sth1->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
    
    while (my ($subjsec, $description, $tch) = $sth1->fetchrow) {
	my ($dud,$teacherid) = split /\(/, $tch;
	$teacherid =~ s/\)//;
	my $sth = $dbh->prepare("select certification1 from staff where
          userid = ?");
	$sth->execute($teacherid);
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	my $cert = $sth->fetchrow;


	print "<tr><td><b>$description</b> ($subjsec)</td>";
	my $startdate = $g_termstart{$startterm};
	my $enddate = $g_termend{$endterm};
	if ($cert) {
	    print "<td><input type=\"checkbox\" ";
	    print "name=\"$subjsec:$startdate:$enddate\" value=\"1\" $check>";
	    print "</td></tr>\n";
	} else {
	    print "<td style=\"color:red;\">No Certification Number</td>";
	    print "</tr>\n";
	}
    }

    print "<tr><td colspan=\"2\">";
    print "<input type=\"submit\" name=\"subjsel\" ";
    print "value=\"Select these Subjects\">";
    print " Chk <input type=\"checkbox\" name=\"check\" value=\"1\">";
    print "</td></tr></table></form>\n";

    print "<p><b>Please wait while we query Sask Learning before the ";
    print "next page appears.</b></p></body></html>\n";


}


#---------------
sub syncSubjects {
#---------------

    my $debug = $arr{debug};
    delete $arr{debug};

    my $selflag = $arr{subjsel}; # from submit button widget.
    delete $arr{subjsel};


    print "<p>Querying Sask Learning</p>\n";
    querySLCourses();

    if ($debug) {
	parseStudentCourseEnrollments(); # show results of SL query.
    }
    my $course_ref = parseSCEbyClass();

    # checkbox select for everyone
    my $check;
    if ($arr{check}) {
	$check = 'CHECKED';
	delete $arr{check};
    }

    #foreach my $key (keys %arr) { print "K:$key V:$arr{$key}<br>\n"; }

    # Define the form
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"enrolflag\" value=\"1\">\n";
    print "<input type=\"submit\" value=\"Update Course Enrollments\">\n";

    # pass debug mode to next screen.
    if ($debug) {
	print "<input type=\"hidden\" name=\"debug\" value=\"1\">\n";
    }

    # Setup Prepare statements for common queries.
    my  $sth = $dbh->prepare("select distinct studnum from eval where subjcode = ?");
    my  $sth1 = $dbh->prepare("select provnum from student where studnum = ?");
    my  $sth2 = $dbh->prepare("select description from subject where subjsec = ?");
    my $sth3 = $dbh->prepare("select lastname, firstname from student where provnum = ?");
    my $sth4 = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");

    foreach my $cl (keys %arr) {
	# loop through each subject.
	my $startdate, $enddate;
	($subjsec, $startdate, $enddate) = split /:/, $cl;

	# get subjsec description
	$sth2->execute($subjsec);
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	my $desc = $sth2->fetchrow;

	# start table
	print "<h3>$desc ($subjsec)</h3>\n";
	print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
	print "<tr><th>Student</th><th>Select</th></tr>\n";

	my (%slstuds, %localstuds);

	# set Sask Learning students (prov nums) for this class.
	# Create a total studs hash to hold all pn's from both local and prov.
	my @slstuds = @{$course_ref->{$cl}};
	foreach my $tmp (@slstuds) { # now make a hash.
	    $slstuds{$tmp} = 1;
	    $totalstuds{$tmp} = 1;
	}

	# Now make the local hash and add these kids to totalstuds, too.
	$sth->execute($subjsec);
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	while (my $sn = $sth->fetchrow) {
	    $sth1->execute($sn);
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	    my $pn = $sth1->fetchrow;
	    if (not $pn) {
		$sth4->execute($sn);
		if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
		my ($lastname, $firstname) = $sth4->fetchrow;
		print "No Provincial Number for student "; 
		print "<b>$firstname $lastname ($sn)</b>. Skipping!<br>\n"; 
		next; 
	    }
	    $localstuds{$pn} = 1;
	    $totalstuds{$pn} = 1;
	}

	# Now compare the %localstuds and %slstuds by looking at
	# combined hash %totalstuds and removing matches in both local
	# and Sask Learning arrays.

	foreach my $pn (keys %totalstuds) { 
	    # Go through all students (on both lists)
	    # Delete them if we have a match 
	    if ($localstuds{$pn} == $slstuds{$pn} and $localstuds{$pn} == 1) {
		delete $slstuds{$pn};
		delete $localstuds{$pn};
	    }
	}


	# Any slstuds left should be withdrawn.
	foreach my $pn (sort keys %slstuds) {
	    $sth3->execute($pn);
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	    my ($lastname, $firstname) = $sth3->fetchrow;

	    print "<tr><td>$firstname $lastname ($pn)</td>\n";
	    print "<td style=\"color:red;\">Withdraw <input type=\"checkbox\" ";
	    print "name=\"$subjsec:W:$pn\" value=\"1\" $check></td></tr>\n";
	}
	

	# Any localstuds left should be enrolled.
	foreach my $pn (sort keys %localstuds) {
	    $sth3->execute($pn);
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	    my ($lastname, $firstname) = $sth3->fetchrow;

	    print "<tr><td><b>$lastname</b>, $firstname ($pn)</td>\n";
	    print "<td>Enrol <input type=\"checkbox\" name=\"$subjsec:E:$pn\" value=\"1\" $check>";
	    print "</td></tr>\n";
	}

	if (not %slstuds and not %localstuds) {
	    #nothing to be done... enrol OR withdraw.
	    print "<tr><td colspan=\"2\">Class is synchronized!</td></tr>\n";
	}
 
	print "</table><p></p>\n";

    } # End of Subject Loop


    print "<input type=\"submit\" value=\"Update Course Enrollments\">\n";
    print "</form></body></html>\n";

}


#----------------
sub doEnrollments {
#----------------

    #foreach my $key (keys %arr) { print "K:$key V:$arr{$key}<br>\n"; }

    my $debug;
    if ($arr{debug}) {
	delete $arr{debug};
	$debug = 1;
    }


    # Send records to SL to do class enrollments/withdrawals; passed a
    # list of students in subjectsec:provnum format.

    # Create a new user agent
    my $ua = LWP::UserAgent->new();
    $ua->agent("OpenAdmin");


    # Setup Prepare statements for common queries.
    my  $sth = $dbh->prepare("select description, subjcode, startrptperiod, 
      endrptperiod, teacher, exammix, instmode
      from subject where subjsec = ?");
    my $sth1 = $dbh->prepare("select lastname, firstname, studnum, birthdate 
      from student where provnum = ?");

    # start table
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>Student</th><th>Subject</th><th>Type</th></tr>\n";

    
    startXML(); # setup start of xmlwriter, etc.
    $first = 1;

    foreach my $rec (sort keys %arr) {

	if ($filesize >= $maxfilesize) { # write data, send, process, update 

	    my $req = endXML();

	    #==========================
	    # Issue the request and receive a response
	    $res = $ua->request($req);
	    #==========================

	    # Check the status of the transfer... (NOT result of xml transactions)
	    if ($res->is_success) { 
	
		# For Debugging ....
		if ($debug){
		    print "<h1>DEBUG - Sask Learning XML Response</h1>\n";
		    print "<div style=\"color:green;\">\n";
		    my $temp = $res->content;
		    $temp =~ s/</&lt;/g;
		    $temp =~ s/>/&gt;/g;
		    print "<pre>",$temp,"</pre></div>\n";
		}

		parseResponse(); # response results

	    } else {
		my $err = $res->status_line;
		print "<h1>Transfer Error: $err</h1>";
		print "<pre>",$res->content,"</pre>\n";
		#die;
	    }
	    
	} # End of writing record loop


	my ($subjsec, $type, $provnum) = split /:/,$rec;
	# $type = E for enrol or W for withdraw.


	# Get student name, studnum, birthdate
	$sth1->execute($provnum);
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	my ($lastname, $firstname, $studnum, $birthdate) = $sth1->fetchrow;

	# Get enrollment type information
	if ($type eq 'E'){ 
	    $enroltype = 'Add'; 
	    $enrolmsg = 'Enrol';
	} else { # its a withdraw
	    $enroltype = 'Change';
	    $dropped = 'Yes';
	    $enrolmsg = 'Withdraw';

	    # to get correct exit date look in evaljrl first; otherwise use current date.
	    my $sth2 = $dbh->prepare("select tdate from evaljrl where studnum = '$studnum'
              and subjsec = '$subjsec' and type = 'withdraw' order by tdate desc");
	    $sth2->execute;
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	    my $exitdate = $sth2->fetchrow;
	    if (not $exitdate) { $exitdate = $currdate; }

	}

	# Get Subject description, subjcode, startterm, endterm, teacher;
	$sth->execute($subjsec);
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	my ($desc, $courseid, $startterm, $endterm,$teacher, $exammix, $instmode) = $sth->fetchrow;
	# courseid is dept number; classid is subjsec.


	# Course dates; use previous parsed values from SL query, else use subject values.
	my ($startdate, $enddate);
	if ($coursedates{$subjsec}) {
	    ($startdate, $enddate) = split /:/,$coursedates{$subjsec};
	} else {
	    $startdate = $g_termstart{$startterm};
	    $enddate = $g_termend{$endterm};
	}

	print "<tr><td>$firstname $lastname ($provnum)</td><td>$desc ($subjsec)</td><td>$enrolmsg</td></tr>\n";

	$oldclass = $currclass;
	$currclass = $subjsec;

	# Write SchoolClass if new StudentClassEnrollment
	if (( $first or $oldclass ne $currclass ) and ( $enroltype eq 'Add' )){

	    # Find Teacher ID (teaching certificate #)
	    my ($tname,$userid) = split /\(/,$teacher;
	    $userid =~ s/\)//g; # remove trailing parenthesis

	    $sth3 = $dbh->prepare("select certification1 from staff
                where userid = '$userid'");
	    $sth3->execute;
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
	    my $teachid = $sth3->fetchrow;

	    my $mode = ''; # defaults to classroom instruction.
	    my $refid = $subjsec;
	    my $classid = $subjsec;

	    # Write SchoolClass object
	    &wrSchoolClass($refid, $classid, $startdate, $enddate, $courseid,
			   $teachid, $instmode);
	    $first = 0;
	}


	# Now write the student class enrollment record
	$wr->startTag('SL_EventObject',
		      'ObjectName' => 'StudentClassEnrollment', 
		      'Action' => $enroltype);
    
	my $refid = "$subjsec-$studnum"; # to give unique id for student enrolling in class.
	my $classid = $subjsec;

	&mkStudentClassEnrollment($refid, $provnum, $birthdate, $classid,
				  $startdate, $enddate,'', $exammix, $dropped, $exitdate);
	# no override - still blank above
	# Now filled in: source, dropped, exitdate
	
	$wr->endTag('SL_EventObject');
    
	$filesize = length($output->value); # Get the new larger size.

    }

    print "</table>\n";


    if ($debug) {
	print "<h1>DEBUG - XML Sent TO Sask Learning</h1>\n";
	print "<div style=\"color:blue;\">\n";
	my $temp = $output->value;
	$temp =~ s/</&lt;/g;
	$temp =~ s/>/&gt;/g;
	print "<pre>",$temp,"</pre></div>\n";
    }

    
    $req = endXML();

    #==========================
    # Issue the request and receive a response
    $res = $ua->request($req);
    #==========================

    # Check the status of the transfer... (NOT result of xml transactions)
    if ($res->is_success) { 
	
	# For Debugging ....
	if ($debug) {
	    print "<h1>DEBUG - Sask Learning XML Response</h1>\n";
	    print "<div style=\"color:green;\">\n";
	    my $temp = $res->content;
	    $temp =~ s/</&lt;/g;
	    $temp =~ s/>/&gt;/g;
	    print "<pre>",$temp,"</pre></div>\n";
	}

	parseResponse(); # response results

    } else {
	my $err = $res->status_line;
	print "<h1>Transfer Error: $err</h1>";
	print "<pre>",$res->content,"</pre>\n";
	#die;
    }
	    

} # End of doEnrollments



#------------
sub startXML {
#------------

    # Vars: needs $debug, $xfercount; 

    # Start of new XML record; create writer object and string, write
    # to start of ObjectData

    # Create Writer String Object / Set Data mode.
    $output = XML::Writer::String->new();
    my $datamode = 1;
    if ($arr{debug}){$datamode = 1;}

    # Create new writer instance
    $wr = new XML::Writer(OUTPUT => $output, DATA_MODE => $datamode, 
			  DATA_INDENT => '2');
    # Data Mode 1 turns on pretty output, Data Mode 0 is more condensed.


    # get the date,time for header sections.
    @tim = localtime(time);
    $year = $tim[5] + 1900;
    $tim[4]++;
    for (0..4){if (length($tim[$_]) == 1){ $tim[$_] = '0'.$tim[$_];}}
    my $currdate = "$year-$tim[4]-$tim[3]";
    my $currtime = "$tim[2]:$tim[1]:$tim[0]";


    # Set XML Header and write Root Element
    #  Version 1 ! interface...
    $wr->xmlDecl("UTF-8");
    $wr->startTag('SL_Message','xmlns' =>$xmlns, 
		  'xmlns:xsi' =>$xmlnsxsi, 
		  'xsi:schemaLocation' => $xsischemaLocation);

    $wr->startTag('SL_Event');

    mkSL_Header($currdate,$currtime, $schoolnumber,$xfercount);
    $xfercount++; # increment for next header.
    if ($xfercount > 98){ $xfercount = 1;}

    $wr->startTag('SL_ObjectData');

}


#---------
sub endXML {	
#---------

    # Write the ending data for the XML transfer and setup the request

    # Finish the XML object
    $wr->endTag('SL_ObjectData');
    $wr->endTag('SL_Event');
    $wr->endTag('SL_Message');
    $wr->end();


    $filesize = length($output->value); # Get Final Filesize.
    $filesize = $filesize/1000; 

    print "<p>The filesize is ", round( $filesize, 1), "KB  (Limit: 32KB)</p> \n";


    # Create the https post request 
    my $req = POST $url, [ XML=> $output->value ];

    $req->content_type('application/xml;charset="utf-8"');
    $req->authorization_basic($sds_userid, $sds_password); 

    # next do the actual request. return $req

    return $req;


}


#-----------------
sub wrSchoolClass {
#-----------------
    # Write the SchoolClass object.
    my ($refid, $classid, $startdate, $enddate, $courseid,
	$teachid, $mode) = @_;

    $wr->startTag('SL_EventObject',
		  'ObjectName' => 'SchoolClass', 
		  'Action' => 'Add');
    
    # Subject-Section is the refid, same as classid.


    &mkSchoolClass($refid, $classid, $startdate, $enddate, $courseid,
		   $teachid, $mode);  
	
    $wr->endTag('SL_EventObject');

}


#----------------
sub parseResponse {
#----------------
    #  parse response from SL, display all results 

    # Parse the response.
    my $parser = XML::LibXML->new();

    eval {$doc = $parser->parse_string($res->content)};
    if ($@){
	print "Sask Learning Error:\n<pre>",$res->content,"</pre><br>\n";
	print "</body></html>\n"; die;
    }	

    $doc->setEncoding('UTF-8');
    my $root = $doc->getDocumentElement;
    $root->setNamespace($xmlns,'sl',1);

    $status = $root->findvalue('//sl:SL_Status/sl:SL_StatusCode');
    $statusmsg = $root->findvalue('//sl:SL_Status/sl:SL_StatusMsg');

    print "<b>Response<br> Code:</b> $status <b>Message:</b> $statusmsg<br>\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>Student</th><th>Subject</th><th>Ref&nbsp;Object</th>";
    print "<th>Error&amp;Message</th></tr>\n";
    
    my @errors = $root->findnodes('//sl:SL_Error');
    foreach my $error (@errors){
	my $refid = $error->getAttribute('RefId');
	my $objectname = $error->getAttribute('ObjectName');
	
	my $studnum;
	if ($objectname eq 'SchoolClass'){ $subjsec = $refid;}
	if ($objectname eq 'StudentClassEnrollment'){
	    my $subjcode; my $section;
	    ($subjcode, $section, $studnum) = split /-/,$refid;
	    $subjsec = "$subjcode-$section";
	}
	
	my $errcode = $error->findvalue('sl:SL_ErrorCode');
	my $errmsg = $error->findvalue('sl:SL_ErrorMsg');
	my $errseverity = $error->findvalue('sl:SL_ErrorSeverity');
	$errmsg =~ s/[\r|\n]/ /g;

	my $lastname; my $firstname;
	# Get Student Name
	if ($studnum){
	    $sth1 = $dbh->prepare("select lastname, firstname 
             from studentall where studnum = '$studnum'");
	    $sth1->execute;
	    if ($DBI::errstr) { print $DBI::errstr; die;}
	    ($lastname, $firstname) = $sth1->fetchrow;
	}

	# Check that the studid field is for the student table not the 
	#  studentwd (withdrawn student) table.
	$sth1=$dbh->prepare("select count(*) from student 
          where studnum = '$studnum'");
	$sth1->execute;
	if ($DBI::errstr) { print $DBI::errstr; die;}
	my $studcount = $sth1->fetchrow;
	# Count should be 1 if in normal table; 0 if withdrawn.
	if ($studcount < 1 and $studnum){ 
	    $wd = "<font color=\"red\">WD:</font>";
	} else { $wd = '';}
	
	my $name;
	if ($studnum){ $name = "$lastname, $firstname ($studnum)";}
	
	print "<tr><td>$wd $name</td><td>$subjsec</td>\n";
	print "<td>$objectname</td>";
	print "<td>$errcode - $errmsg</td></tr>\n";
    }
    print "</table>\n";


} # End of parseResponse
