#! /usr/bin/perl
#  Copyright 2001-2008 Leslie Richardson

#  This file is part of Open Admin for Schools.
#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# This script will update SL with changes to student record fields.
#  which are located in the utag fields of the student table.

use DBI;
use CGI;
use XML::Writer;
use XML::Writer::String;
use Data::UUID;
use HTTP::Request::Common qw(POST); 
use HTTP::Headers; 
use LWP::UserAgent; 
use XML::LibXML;
use Number::Format qw(round);

my $self = 'updatedemog.pl';

my $q = new CGI;
print $q->header;
my %arr = $q->Vars;


# Read config variables
require "../../etc/admin.conf" || die "Cannot read admin.conf: $!\n";
require "slxmllib.pl" || die "Cannot read slxmllib.pl: $!\n";


# Fields of Interest to track for field changes; change to suit your needs. 
#  They are zero based (array indexes); also update to match studed1.pl
#  'studed1.pl' also contains these same fields and uses it to update the
#  utag field in each edited record.
%track =(1 => 'lastname', 2 => 'firstname', 3 =>'initial',  5 => 'grade', 
7 => 'sex', 8 => 'birthdate', 10 => 'healthid', 12 => 'ethnic', 
17 => 'treaty', 30 => 'hphone1', 34 => 'address1', 35 => 'city1', 
36 => 'pcode1', 69 => 'prov1', 57 => 'provnum', 58 => 'band', 
59 => 'bandnum', 71 => 'reserveres', 72 => 'program');

%studentupdated; # studnum hash of students updated. Used at end 
# to update Utag fields.

my $bulkmode = 1; # always ON.

print "$doctype\n<html><head><title>SDS Demographics Update</title> 
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>[ <a href=\"$homepage\">Main</a> | 
<a href=\"$exppage\">Export</a> ]<h1>Student Demographics Update</h1>\n";

if ($arr{debug}){
    if (not $arr{flag}) { # Show the options for debug data errors.
	showDebug();
    } else { # Set the data error options
	$maxfilesize = $arr{xmlsize}; # override values in config
    }
}


# Setup Date and Time
@month = ("","January","February","March","April","May","June","July",
	  "August","September","October","November","December");
# $sec, $min, $hour, $mday, $mon, 
# $year, $wday, $yday, $isdst ) = localtime(time);
@tim = localtime(time);
$year = $tim[5] + 1900;
$tim[4]++;
for (0..4){if (length($tim[$_]) == 1){ $tim[$_] = '0'.$tim[$_];}}
$currdate = "$year-$tim[4]-$tim[3]";
$currtime = "$tim[2]:$tim[1]:$tim[0]";

$dsn = "DBI:$dbtype:dbname=$dbase";
$dbh = DBI->connect($dsn,$user,$password);

# Find students by lastname, firstname
my $sth = $dbh->prepare("select * from student 
     order by lastname, firstname");
$sth->execute;
my $rows = $sth->rows;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

# Create a new user agent
my $ua = LWP::UserAgent->new();
$ua->agent("OpenAdmin");

# Open Access and Error Log files for appending.
#open(ERR,">>error.log") or die "Cannot open error.log: $!\n";
#open(UPD,">>update.log") or die "Cannot open update.log: $!\n";

my $donecount = 1;
my $reccount = $rows;
my $xfercount = 1;

while ( $donecount <= $reccount ) {

    # Create Writer Instance
    my $output = XML::Writer::String->new();
    my $datamode = 0;
    if ($arr{debug}){ $datamode = 1;} # pretty print xml output
    $wr = new XML::Writer(OUTPUT => $output, 
			  DATA_MODE => $datamode, 
			  DATA_INDENT => '2');

    # Set XML Header and write Root Element
    $wr->xmlDecl("UTF-8");
    $wr->startTag('SL_Message','xmlns' =>$xmlns, 
		  'xmlns:xsi' =>$xmlnsxsi, 
		  'xsi:schemaLocation' => $xsischemaLocation);
    
    $wr->startTag('SL_Event');
    $srcMsgId = &mkSL_Header($currdate,$currtime, $schoolnumber,$xfercount);
    $xfercount++; # update counter to keep header unique

    $wr->startTag('SL_ObjectData');

    print "<span style=\"color:black;font-size:150%;\">Query</span><br>\n";
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>Student Name</th></tr>\n";
    
    my $masternameflag = 0;

    while ( my @sr = $sth->fetchrow ) {
	if (not $sr[57]){ # No Prov Number
	    print "<tr><td>$donecount:";
	    print "<a href=\"$cgiurl/studed.pl?recnum=$sr[0]\">";
	    print "<b>$sr[1]</b>, $sr[2]</a> ($sr[4])</td>\n<td>";
	    print "<span style=\"color:red;\">No Provincial Number!</span>";
	    print "</td></tr>\n";
	    $donecount++;
	    next;
	}

	# loop through student records.
	my @ufields;
	@ufields = ('5','ALL');

	# check for program change, field 72 (program change), generate a program change element.
	my @tmpufields = split / /,$sr[73];
	foreach my $fld ( @tmpufields ) {
	    if ( $fld == 72 ) { # program change
		push @ufields, 72;
	    }
	}


	my @newsr;  # new blank student record to stuff updated field values.
	my $fields;

	foreach my $ufld (@ufields){

	    # Stuff updated field values into a new blank student array;
	    if ($ufld eq 'ALL'){ # process all
		@newsr = @sr;
	    }

	    elsif ($ufld == 72 ){ # create program enrollment
		$wr->startTag('SL_EventObject',
			      'ObjectName' => 'StudentProgramEnrollment',
			      'Action' => 'Change');
		mkStudentProgramEnrollment( $sr[57],$sr[8],$sr[72],$currdate ); 
		# function above puts a .1 after studnum in refid.
		$wr->endTag('SL_EventObject');

	    }
	    elsif ($ufld == 5 ) { # school change grade

		$wr->startTag('SL_EventObject',
			      'ObjectName' => 'StudentSchoolEnrollment',
			      'Action' => 'Change');

		# Build StudentEnrollmentInfo (sei) Hash
		$seiRef = {'MembershipType' => '', 
			   'EntryDate' => '', 'Grade' => "$sr[5]",
			   'PrevProv' => '', 'PrevCountry' => '',
			   'ExitType' => '', 'ExitDate' => ''};

		$refid = $sr[4].".1"; # Student with a .1 to make unique #
		$wr->startTag('StudentSchoolEnrollment', RefId=>$refid);
		mkStudentIdentification($sr[57],$sr[8]); # provnum/bdate 
		# Not needed for change of grade &mkStudentInfo(@sr); 

		$wr->dataElement('SchoolId',$schoolnumber); # from admin.conf
	
		mkSchoolEnrollmentInfo($seiRef);

		$wr->endTag('StudentSchoolEnrollment');

		$wr->endTag('SL_EventObject');

	    }

	} # end of @ufields
	
	my $studentname = "<b>$sr[1]</b>, $sr[2]";
	print "<tr><td>$donecount. ";
	print "<a href=\"$cgiurl/studed.pl?recnum=$sr[0]\">$studentname</a>";
	print " ($sr[4])</td></tr>\n";
	$donecount++;


	$wr->startTag('SL_EventObject',
		      'ObjectName' => 'StudentPersonal',
		      'Action' => 'Change');

	$refid = $sr[4]; # Student #
	$studentupdated{$refid} = 1; # Hash for updated records.

	$wr->startTag('StudentPersonal', RefId=>$refid);
	mkStudentIdentification($sr[57],$sr[8]); # parse provnum and bdate 
	#--------------Main Entry Approach --------------------------
	mkStudentInfo( @newsr );  # Pass NEW @SR(empty with just changed fields)
	#--------- Line Above crucial to entire process.---------

	$wr->endTag('StudentPersonal');
	$wr->endTag('SL_EventObject');

	$filesize = length($output->value);
	if ( $filesize >= $maxfilesize ){ last; } # break out of loop 

    } # End of Student Loop.

    $wr->endTag('SL_ObjectData');
    $wr->endTag('SL_Event');
    $wr->endTag('SL_Message');
    $wr->end();

    print "</table>\n";


    $filesize = length( $output->value );
    $filesize = $filesize/1000;
    print "The filesize is ", round( $filesize, 1 ), "KB (Limit: 32KB)\n";

    # DEBUG Data Errors
    if ( $arr{sentdata} ){
	print "<div style=\"color: blue;\">\n";
	print "<h1>DEBUG - Request sent to Sask Learning</h1>\n";
	my $temp = $output->value;
	$temp =~ s/</&lt;/g;
	$temp =~ s/>/&gt;/g;
	print "<pre>",$temp,"</pre></div><p></p>\n";
    }

    # Create the https post request 
    my $req = POST $url, [ XML=>$output->value ];

    $req->content_type('application/xml;charset="utf-8"');
    $req->authorization_basic($sds_userid, $sds_password); 

    # Issue the request and receive a response
    my $res = $ua->request($req);

    # Check the status of the response
    if ($res->is_success) {

	# For Debugging Data Errors
	if ( $arr{recddata} ) {
	    print "<h1>DEBUG - Sask Learning XML Response</h1>\n";
	    print "<div style=\"color:green;\">\n";
	    my $temp = $res->content;
	    $temp =~ s/</&lt;/g;
	    $temp =~ s/>/&gt;/g;
	    print "<pre>",$temp,"</pre></div>\n";
	}

	print "<p><b>Results of Request to Sask Learning.</b></p>\n";

	# Parse the response.
	my $parser = XML::LibXML->new();

	eval {$doc = $parser->parse_string($res->content)};
	if ( $@ ) {
	    print "Sask Learning Error: <pre>",$res->content,"</pre><br>\n";
	    print "</body></html>\n"; die;
	}	

	$doc->setEncoding('UTF-8');
    
	my $root = $doc->getDocumentElement;
	$root->setNamespace($xmlns,'sl',1);

	$mastermsgid = $root->findvalue('//sl:SL_MsgId');
	#print "<p><b>MsgId:</b> $mastermsgid</p>\n";

	my @errors = $root->findnodes('//sl:SL_Error');
	if ( @errors ) { 
	    print "<span style=\"color:black;font-size:150%;\">Results</span>\n";
	    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
	    print "<tr><th>Student</th><th>Response</th></tr>\n";

	    # prepare for getting name in the 'error' response loop.
	    $sth1 = $dbh->prepare("select studid, lastname,firstname 
              from student where studnum = ?");

	    foreach my $error ( @errors ) {
		my $studentrefid = $error->getAttribute('RefId');
		$studentrefid =~ s/\..*//; # strip the .1
		my $errcode = $error->findvalue('sl:SL_ErrorCode');

		# Get the Name
		$sth1->execute( $studentrefid );
		if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
		my ($studid,$lastname, $firstname) = $sth1->fetchrow;
		
		# No longer used ( studentupdated hash ).
		#if ( $errcode ) {
		   #$studentupdated{$studentrefid} = 0; 
		    # Don't clear Utag for this student! (due to error)
		#}

		my $objectname = $error->getAttribute('ObjectName');
		my $object;
		if ( $objectname eq 'StudentPersonal' ) {
		    $object = 'Demographics Update';
		} else {
		    $object = 'Grade Update';
		}

		my $errmsg = $error->findvalue('./sl:SL_ErrorMsg');
		if ( $errmsg ne 'Transaction Successful' ) {
		    $errmsg = "<span style=\"color:red;font-weight:bold;\">$errmsg</span>";
		}
		
		print"<tr><td><a href=\"$cgiurl/studed.pl?recnum=$studid\">";
		print "$lastname, $firstname</a> ($studentrefid)</td>";
		print "<td><b>$object:</b> $errmsg</td></tr>\n";
	    
	    } # End of For Errors

	    print "</table>\n";

	} # End of IF Errors
	
    } else { # We have a transfer error!
	my $err = $res->status_line;
	print "<h1>Transfer Error: $err</h1>";
	die;
    }
    print "<p><hr></p>\n";


} # End of Counted Loop for all students.

print "</body></html>\n";


#------------
sub showDebug {  # Show debug options...
#------------

    print "<h1>Transfer Update -- Debug Options</h1>\n";

    print "<p>In order to find problems with submitted data, the sizes of the XML \n";
    print "files (and the number of contained records) should be limited so that\n";
    print "it is easier to pick out the offending data. If size is less than 3000\n";
    print "characters, this will limit data to a couple of student records (which\n";
    print "are relatively easy to locate data errors within).</p>\n";

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"flag\" value=\"1\">\n";
    print "<input type=\"hidden\" name=\"debug\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";

    print "<tr><td></td><td><input type=\"submit\" value=\"Continue\"></td></tr>\n";

    print "<tr><td align=\"right\">XML File Size Limit:</td>\n";
    print "<td><input type=\"text\" name=\"xmlsize\" size=\"8\" value=\"3000\"></td></tr>\n";

    print "<tr><td align=\"right\">View Submitted Data</td>";
    print "<td><input type=\"checkbox\" name=\"sentdata\" value=\"1\" checked></td></tr>\n";

    print "<tr><td align=\"right\">View Received Data</td>\n";
    print "<td><input type=\"checkbox\" name=\"recddata\" value=\"1\" checked></td></tr>\n";

    print "<tr><td></td><td><input type=\"submit\" value=\"Continue\"></td></tr>\n";
    print "</table></body></html>\n";

    exit;

}
