#! /usr/bin/perl
#  Copyright 2001-2006 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# Previously called updatetrans.pl... now will accept input from other
# scripts rather than using the utag values from transfer. The passed
# values include date (D:provnum = date) and code, type
# "C:provnum:type = code"  B:provnum = birthdate


#  Record is a StudentSchoolEnrollment object. Log response and update the 
#  provincial number (if not present and returned via XML).
#  If Withdrawal, generate a StudentSchoolEnrollment object with exit reason.
#  Log response into log file.

$self='updatexfer.pl';

use DBI;
use CGI;
use XML::Writer;
use XML::Writer::String;
use Data::UUID;
use HTTP::Request::Common qw(POST); 
use HTTP::Headers; 
use LWP::UserAgent; 
use XML::LibXML;
use MIME::Base64;
use Number::Format qw(round);

# Read config variables
require "../../etc/admin.conf" or die "Cannot read admin.conf: $!";
require "slxmllib.pl" or die "Cannot read slxmllib.pl: $!";

my $q = new CGI;
print $q->header;
my %arr = $q->Vars;

#foreach my $key (keys %arr) { print "K:$key V:$arr{$key}<br>\n"; }

# cleanup any other passed values
my $debug = $arr{debug};
delete $arr{debug};


my %studrec; # holds enrollment change data.

foreach my $key (keys %arr) {
    my ($t, $provnum, $type) = split /:/, $key;
    if ($t eq 'D' or $t eq 'B') { next; } # skip the date (D) values and birthdate
    my $date = $arr{"D:$provnum"};
    my $birthdate = $arr{"B:$provnum"};

    # $code = $arr{$key}; # if you want full text and (code).
    my ($text, $code) = split /\(/, $arr{$key};
    $code =~ s/\)//;

    #print "PN:$provnum C:$code D:$date T:$type B:$birthdate<br>\n";
    $studrec{$provnum} = "$date:$code:$type:$birthdate";
    push @provnums, $provnum;
}



print "$doctype\n<html><head><title>SDS Enrollment Update</title> 
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>[ <a href=\"$homepage\">Main</a> | 
<a href=\"$exppage\">Export</a> | 
<a href=\"$cgiurl/entry/transview.pl\">View Transfers</a> ]\n";


# Process Mail for SL.
if ($arr{domail} == 1) {
    doMail();
    die;
}


if ($debug){
    #if (not $arr{flag}) { # Show the options for debug data errors.
    #showDebug();
    #die;
    #} else { # Set the data error options
	$maxfilesize = 3000; # override values in config
    #}
}

print "<h1>Student Transfers/Enrollment Update</h1>\n";

#print "Debug: $debug\n";

my @month = ("","January","February","March","April","May","June","July",
	  "August","September","October","November","December");
my @tim = localtime(time);
my $year = $tim[5] + 1900;
$tim[4]++;
for (0..4){if (length($tim[$_]) == 1){ $tim[$_] = '0'.$tim[$_];}}
my $currdate = "$year-$tim[4]-$tim[3]";
my $currtime = "$tim[2]:$tim[1]:$tim[0]";

my $dsn = "DBI:$dbtype:dbname=$dbase";
$dbh = DBI->connect($dsn,$user,$password);

my $rows = keys( %studrec ); # number of enrollment changes

if ($rows < 1){
    print "<hr><h1>Nothing to Report.</h1></body></html>\n";
    die;
}

# print SL Student Transfer Header.
print "<p><b>Sask Learning Update - $rows Transfer Records</b></p>\n";


my $donecount = 1;
my $reccount = $rows;
my $xfercount = 1; # Counter for message headers (to make unique).

while ($donecount <= $reccount){

    # Start New Table for Student Records
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">
    <tr><th>Student Transferring</th><th>Type</th><th>Date</th></tr>\n";

    # Create a new user agent
    my $ua = LWP::UserAgent->new();
    $ua->agent("OpenAdmin");

    # Create Writer Instance
    my $output = XML::Writer::String->new();
    my $datamode = 0;
    if ( $debug ){ $datamode = 1; }
    # override datamode since required for email mailbacks.
    $datamode = 1;

    $wr = new XML::Writer(OUTPUT => $output, DATA_MODE => $datamode, 
			  DATA_INDENT => '2');
    # Data Mode 1 turns on pretty output, Data Mode 0 is more condensed.

    # Open Update and Error Log files for appending.
     open(ERR,">>error.log") or die "Cannot open error.log: $!\n";
     open(UPD,">>update.log") or die "Cannot open update.log: $!\n";


    # Set XML Header and write Root Element
    $wr->xmlDecl("UTF-8");
    $wr->startTag('SL_Message','xmlns' =>$xmlns, 
		  'xmlns:xsi' =>$xmlnsxsi, 
		  'xsi:schemaLocation' => $xsischemaLocation);

    $wr->startTag('SL_Event');
    mkSL_Header($currdate,$currtime, $schoolnumber,$xfercount);
    $xfercount++; # increment for next header.

    $wr->startTag('SL_ObjectData');

    # prep for retrieving from student data
    $sth1 = $dbh->prepare("select * from studentall
     where provnum = ?");

    my $filesize = 0;

    # Loop through students while smaller than maxfilesize.
    while ( my $provnum = pop @provnums ){

	my @sr;
	# Some may be local studnums since no provnum...
	if ($provnum =~ m/^S/) {
	    $studnum = $provnum;
	    $studnum =~ s/^S//; # strip leading S
	    #print "Yes, a local number!: $studnum";
	    my $sth2 = $dbh->prepare("select * from studentall where studnum = ?");
	    # Pull in matching student data
	    $sth2->execute($studnum);
	    if ($DBI::errstr) { print $DBI::errstr; die; }
	    @sr = $sth2->fetchrow;  # @sr = student record

	} else { # we have provnum;
	    # Pull in matching student data
	    $sth1->execute($provnum); 
	    if ($DBI::errstr) { print $DBI::errstr; die; }
	    @sr = $sth1->fetchrow;  # @sr = student record
	}

	my ($date, $code, $type, $bdate) = split /:/, $studrec{$provnum};

	#print "PN:$provnum D:$date C:$code T:$type B:$bdate SR:",@sr,":Done<br>\n";

	prRecord($provnum, $date, $code, $type, $bdate, @sr); # print the record of this student into data object.

	$donecount++; # increment counter for all utagged records
	
	$filesize = length($output->value); # Get the new larger size.
	if ($filesize >= $maxfilesize){ last;} # break out of loop. 
	#print "$donecount:", $filesize,"<br>\n";

    } # End of Record Assembly Loop

    # Finish the XML object
    $wr->endTag('SL_ObjectData');
    $wr->endTag('SL_Event');
    $wr->endTag('SL_Message');
    $wr->end();

    # close table started as students read in.
    print "</table>\n";


    # set hash storing StudentSchoolEnrollment records sent to SL
    my $temp = $output->value;

    while ($temp =~ 
	   m{\<StudentSchoolEnrollment(.*?)\</StudentSchoolEnrollment\>}xmsg)
    { push(@xmlrecs,$1); };
    foreach my $rec (@xmlrecs) {
	my $refid = $1 if $rec =~ m/RefId="(.*?)"\>/;
	$xmlrecs{$refid} = '<StudentSchoolEnrollment '.$rec.
	    '</StudentSchoolEnrollment>';
    }

    # View the xmlrecs hash
    #foreach my $key (keys %xmlrecs) {
    #print "K:$key V:$xmlrecs{$key}<br>\n";
    #}


    # show output sent to Sask Learning if set via debug mode
    if ( $debug ){
	$temp =~ s/</&lt;/g;
	$temp =~ s/>/&gt;/g;
	print "<div style=\"color: blue;\">\n";
	print "<h1>DEBUG - Request sent to Sask Learning</h1>\n";
	print "<pre>",$temp,"</pre></div><p></p>\n";
    }

    $filesize = length($output->value)/1000; # Get Final Filesize.


    print "<p>The filesize is ", round( $filesize, 1), "KB 
     (Limit: 32KB)</p> \n";

    # Create the https post request 
    my $req = POST $url, [ XML=> $output->value ];

    $req->content_type('application/xml;charset="utf-8"');
    $req->authorization_basic($sds_userid, $sds_password); 

    # Issue the request and receive a response
    $res = $ua->request($req);

    # Check the status of the transfer... (NOT result of xml transactions)
    if ($res->is_success) { 

	# For Debugging ....
	if ($debug){
	    print "<h1>DEBUG - Sask Learning XML Response</h1>\n";
	    print "<div style=\"color:green;\">\n";
	    my $temp = $res->content;
	    $temp =~ s/</&lt;/g;
	    $temp =~ s/>/&gt;/g;
	    print "<pre>",$temp,"</pre></div>\n";
	 }
	
	# Parse returned XML.
	# This does most of the screen printing. 
	parseResponse(); # response results if errors, update student provnum
	

    } else {
	my $err = $res->status_line;
	print "<h1>Transfer Error: $err</h1>";
	print "<pre>",$res->content,"</pre>\n";
	die;
    }

} # End of the Counted loop for all Utag records.


print "</body></html>\n";


#------------
sub prRecord { # print student record to output
#------------

    # last array is complete student record.
    my ($provnum, $date, $code, $type, $birthdate, @sr) = @_;


    if (not $birthdate) {
	$birthdate = $sr[8];
    }

    if ($provnum =~ m/^S/) { # a local student number, actually...
	$studnum = $provnum;
	$studnum =~ s/^S//;
	$provnum = '';
    }

    #print "PN:$provnum D:$date C:$code T:$type<br>\n";


    # Always sr[4]
    if ($sr[4]) {
	$refid = $sr[4];
    } else {
	$refid = $provnum;
    }


    if ($type eq 'withdraw'){  # ==== WITHDRAW STUDENT =======

	print "<tr><td><b>$donecount:$sr[1]</b>, $sr[2] ($sr[4])</td>\n";
	print "<td>withdraw</td><td>$date</td></tr>";

	$studtrans{$sr[4]} = 1; # now has student number, not transfer table id 

	$wr->startTag('SL_EventObject',
		      'ObjectName' => 'StudentSchoolEnrollment',
		      'Action' => 'Change');

	# Build StudentEnrollmentInfo (sei) Hash
	$seiRef = {'MembershipType' => '', 
		   'EntryDate' => '', 'Grade' => '',
		   'PrevProv' => '', 'PrevCountry' => '',
		   'ExitType' => $code, 'ExitDate' => $date };


	$wr->startTag('StudentSchoolEnrollment', RefId=>$refid);

	mkStudentIdentification( $provnum, $birthdate ); # pass provnum and bdate ($sr[8], too)
	
	$wr->dataElement('SchoolId',$schoolnumber); # from admin.conf file.
	
	mkSchoolEnrollmentInfo($seiRef);
		   
	$wr->endTag('StudentSchoolEnrollment');

	$wr->endTag('SL_EventObject');

    } elsif ( $type eq 'enroll' ) { # STUDENT ENROLLMENT 

	if (not $sr[1]){ $sr[1] = '<font color=red>Not Found</font>'; }

	print "<tr><td><b>$donecount:$sr[1]</b>, $sr[2] ($sr[4])</td>\n";
	print "<td>enrol</td><td>$date</td></tr>\n";
	
	$studtrans{$refid} = 1;  # was transrec[0] - transfer table id

	$wr->startTag('SL_EventObject',
		      'ObjectName' => 'StudentSchoolEnrollment', 
		      'Action' => 'Add');

	$seiHashRef = {'MembershipType' => 'Base', 
		       'EntryType' => $code, 
		       'EntryDate' => $date,
		       'Grade' => $sr[5],
		       'PrevProv' => '',
		       'PrevCountry' => '',
	               'ExitType' => '', 'ExitDate' => ''};

	$refid = $sr[4]; # Student #
	$wr->startTag('StudentSchoolEnrollment', RefId=>$refid);

	mkStudentIdentification($provnum, $sr[8]); # pass provnum and bdate 
	
	mkStudentInfo(@sr); 

	$wr->dataElement('SchoolId',$schoolnumber); # from admin.conf file.
	
	mkSchoolEnrollmentInfo($seiHashRef);

	if (not $sr[72]){ # program field empty
	    $myprog = $program; # from admin.conf
	} else {
	    $myprog = $sr[72];
	}
	$wr->dataElement('DeptAssignedProgramId',$myprog); 
	&mkProgramEnrollmentInfo("$currdate");

	$wr->endTag('StudentSchoolEnrollment');

	$wr->endTag('SL_EventObject');

	#&mkStudentSchoolEnrollment($seiHashRef,@sr);
	#$wr->endTag('SL_EventObject');

    } else { 
	print "Error in record! Not withdraw or enroll!\n"; 
	die "Error in record! Not withdraw or enroll!\n"; 
    }

} # End of prRecord



#----------------
sub parseResponse {
#----------------
# parse response from SL, display errors (if any), update provnum in 
#  student table,  print to error logs.

    # Parse the response.
    my $parser = XML::LibXML->new();

    eval {$doc = $parser->parse_string($res->content)};
    if ($@){
	print "Sask Learning Error:\n<pre>$@<br>",$res->content,"</pre><br>\n";
	print "</body></html>\n"; die;
    }	

    #print "XMLns:",$xmlns,"<br>\n";

    $doc->setEncoding('UTF-8');

    my $root = $doc->getDocumentElement;
    $root->setNamespace($xmlns,'sl',1);

    # Parse Message ID.
    $mastermsgid = $root->findvalue('//sl:SL_MsgId');
    #print "MSG: $mastermsgid<br>\n";

    $status = $root->findvalue('//sl:SL_Status/sl:SL_StatusCode');
    # print "<b>Response Status:</b> $status <br>\n";

    if ($status eq 'Errors' or 
      $status eq 'Successful' or 
      $status eq 'Warnings'){ 
	print "<b>Click</b> on <b>Qry</b> to Query Sask Learning ";
	print "for Student Data.<br><b>Click</b> on <b>Trn</b> to Edit a ";
	print "local Transfer record.<br>\n";
	print "<b>Select</b> records to email to Student Records for ";
	print "<i>Identity Resolution</i>.\n";
	
	print "<form action=\"$self\" method=\"post\">\n";
	print "<input type=\"hidden\" name=\"domail\" value=\"1\">\n";

	print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
	print "<tr><th>Select</th><th>Student Name | Xfer</th><th>Prov Num</th>";
	print "<th>Message</th></tr>\n";

	my @errors = $root->findnodes('//sl:SL_Error');

	%studtrans = ();  # make local hash to store results.
	foreach my $error (@errors){  # loop through all student transfers

	    my $refid = $error->getAttribute('RefId');
	    # RefId is now the student number...
	    # RefId is the actual record id number of the transfer record.
    
	    my $errcode = $error->findvalue('sl:SL_ErrorCode');

	    if ($errcode == 0){ # if zero, success
		$studtrans{$refid} = 1; # is student number.
	    }


	    # Get Student Name, number
	    #$sth1 = $dbh->prepare("select studentall.lastname,
            #  studentall.firstname,studentall.studnum, studentall.studid, 
            #  studentall.provnum
            #  from studentall,transfer 
            #  where studentall.studnum = transfer.studnum and
            #  transfer.id = '$refid'");

	    my $sth1 = $dbh->prepare("select lastname, firstname,
	    studnum, studid, provnum from studentall where studnum = ?");
	    $sth1->execute($refid);
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
	    my ($lastname, $firstname, $studnum,$studid,
		$provnum) = $sth1->fetchrow;
	    
	    #print "$lastname, $firstname :$errcode:$studtrans{$refid}:<br>\n";

	    # Check that the studid field is for the student table not the 
	    #  studentwd (withdrawn student) table.
	    $sth1=$dbh->prepare("select count(*) from student 
               where studnum = '$studnum'");
	    $sth1->execute;
	    if ($DBI::errstr) { print $DBI::errstr; die;}
	    my $studcount = $sth1->fetchrow;
	    # Count should be 1 if in normal table; 0 if withdrawn.

	    # Get Error Elements
	    my $objectname = $error->getAttribute('ObjectName');
	    my $errmsg = $error->findvalue('sl:SL_ErrorMsg');
	    $errmsg =~ s/[\r|\n]/ /g;

	    my $emailmsg = "\n===========================\nStudent: ".
	     "$firstname $lastname\n\n". 
             "RECEIVED:\n Errcode: $errcode\n Error Message: $errmsg\n\n".
             "SENT:\n".$xmlrecs{$refid}."\n\n";
             

	    # start table row and print name, etc.
	    print "<tr>";
	    my $xmlrec_64 = encode_base64($emailmsg);
	    if ($errcode != 0) {
		print "<td align=\"center\"><input type=\"checkbox\" ";
		print "name=\"$refid\"\n value=\"$xmlrec_64\"></td>";
	    } else { print "<td></td>"; }

	    print "<td>";
	    if ($studcount){ # Either edit normal student table or studentwd
		print "<a href=\"$cgiurl/studed.pl?recnum=$studid\">"; 
	    } else {
		print "<font color=\"red\">WD:</font><a ";
		print "href=\"$cgiurl/studed.pl?recnum=$studid&tb=wd\">"; 
	    } 

	    print "$lastname, $firstname ($studnum)</a> | ";
	    print"<a href=\"$cgiurl/entry/transed.pl?id=$refid\">";
	    print "Trn</a> ";
	    if ($provnum){ 
		print "<a href=\"qrystudpsl1.pl?$studnum=1\">Qry</a>";
	    }
	    print "</td>\n";
	    print "<td>$provnum</td>";
	    #print "<td>$studentrefid ($objectname)</td>";
	    print "<td>$errmsg</td>";

	    print "</tr>\n";

	} # End of printing of response loop

	print "</table>\n<p><b>Note to Sask Learning:</b><br>\n";
	print "<textarea name=\"note\" rows=\"2\" cols=\"65\"></textarea><br>\n";
	print "<input type=\"submit\" value=\"Email Selected Errors to Sask Learning\">";
	print "</p></form>\n";
   

    } elsif ($status eq 'Invalid'){ # Print out errors
	print "<div style=\"font-size: 200%; font-weight: bold;\">";
	my @errors = $root->findnodes('//sl:SL_Error');
	foreach my $error (@errors){
	    my $refid = $error->getAttribute('RefId');
	    my $errcode = $error->findvalue('sl:SL_ErrorCode');
	    my $errmsg = $error->findvalue('sl:SL_ErrorMsg');
	    print "Error: $errmsg ($errcode) ";
	}
	print "</div>\n";

    } else {
	print "<p><b>Weird result</b> - Status: $status<br></p>\n";
	die;
    }

    # Update transfer records to clear based on values in studtrans
    #foreach my $key (keys %studtrans){ # Loop through all student transfers
    #	print "K: $key VAL: $studtrans{$key}<br>\n";
    #}

    foreach my $key (keys %studtrans){ # Loop through all student transfers
	if ($studtrans{$key}){ # all '1' keys (successful updates)
	    #print "K:$key  V:$studtrans{$key}<br>\n";

	    # Get Student Transfer Data
	    $sth1 = $dbh->prepare("select studentall.lastname, 
         studentall.firstname, studentall.studnum, studentall.provnum,
         transfer.date, transfer.type,studentall.birthdate from studentall,transfer
         where studentall.studnum = transfer.studnum and
         transfer.id = '$key'");
	    $sth1->execute;
	    if ($DBI::errstr) { print $DBI::errstr; die;}
	    my ($lastname, $firstname, $studnum,$provnum, $date, 
		$type,$bdate) = $sth1->fetchrow;

	    # Now clear the utag and update the provnum in the transfer record.
	     $result = $dbh->do("update transfer set utag = NULL, 
             provnum = '$provnum', birthdate = '$bdate' where id = '$key'");
	     if ($DBI::errstr) { print $DBI::errstr; die;}
	} else {
	    print "Student Transfer invalid K:$key VAL:$studtrans{$key}<br>\n";
	}

    }

    # Now look for returned student numbers. (in StudentPersonal objects)
    # print Update Log and also update prov nums in Student table.

    my @students = $root->findnodes('//sl:StudentPersonal');
    if (not @students){ 
	print "<p>No Student Numbers returned.</p>\n";
    } else { # We have students to print.
	print "<p><b>Returned Student Numbers</b></p>\n";
	print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
	print "<tr><th>Student Name</th><th>BirthDate</th><th>Prov#</th>";
    
	foreach my $student (@students){

	    my $studnum = $student->getAttribute('RefId');
	    #($studnum,$dud) = split /\./, $studnum;
	    my $provnum = $student->findvalue('sl:StudentIdentification/sl:DeptAssignedPersonId');
	    my $bdate = $student->findvalue('sl:StudentIdentification/sl:BirthDate');
	    
	    # Now read the transfer table for the name and other info.
	    # Get Student Name, number
	    $sth1 = $dbh->prepare("select lastname, firstname from studentall
             where studnum = ?");
	    $sth1->execute($studnum);
	    if ($DBI::errstr) { print $DBI::errstr; die; }
	    my ($lastname, $firstname) = $sth1->fetchrow;

	    print "<tr><td><b>$lastname</b>, $firstname ($studnum)</td>";
	    print "<td>$bdate</td><td>$provnum</td></tr>\n";

	    # Update the prov student number.
	    $result = $dbh->do("update student set provnum = '$provnum' 
              where studnum = '$studnum'");
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

	} # End of Student Loop
	print "</table><p><hr></p>\n";
    }

} # End of parseResponse


# Functions ============

#---------
sub showDebug {  # Show debug options...
#---------
    print <<"EOS";
<h1>Transfer Update -- Debug Options</h1>

<p>In order to find problems with submitted data, the sizes of the XML
files (and the number of contained records) should be limited so that
it is easier to pick out the offending data. If size is less than 3000
characters, this will limit data to a couple of student records (which
are relatively easy to locate data errors within).</p>

<form action="$self" method="post">
<input type="hidden" name="flag" value="1">
<input type="hidden" name="debug" value="1">


<table cellpadding="3" cellspacing="0" border="0">

<tr><td></td><td><input type="submit" value="Continue"></td></tr>

<tr><td align="right">XML File Size Limit:</td>
<td><input type="text" name="xmlsize" size="8" value="3000">
</td></tr>

<tr><td align="right">View Submitted Data</td>
<td><input type="checkbox" name="sentdata" value="1" checked></td></tr>

<tr><td align="right">View Received Data</td>
<td><input type="checkbox" name="recddata" value="1" checked></td></tr>

<tr><td></td><td><input type="submit" value="Continue"></td></tr>
</table>
</body></html>
EOS
    

}

#---------
sub doMail { # Send Mail to SL for resolution...
#---------

    my $studentrecords = "Student.Records\@sasked.gov.sk.ca";

    if (not $returnaddress) {
	print "Please set your return address in the admin.conf file!<br>\n";
	print "</body></html>\n";
	die;
    }

    delete $arr{domail};
    my $note = $arr{note};
    delete $arr{note};
    my $message = "Schoolname: $schoolname ($schoolnumber)\n\nNote: $note\n";
    print "<p><b>Schoolname:</b> $schoolname ($schoolnumber)</p>\n";
    print "<p><b>Note:</b> $note</p>\n";

    foreach my $key (keys %arr) {
	my $decoded = decode_base64($arr{$key});
	$message .= $decoded;

	$decoded =~ s/</&lt;/g;
	$decoded =~ s/>/&gt;/g;
	print "<pre><div style=\"color:black;background-color:#DDD;\">\n";
	print $decoded;
	print "</div></pre>\n";
    }

    # Watch which one is ACTIVE!!
    open (SENDMAIL,"| /usr/sbin/sendmail -t");
    #open (SENDMAIL,">>emailtest");   
    if (not SENDMAIL){
       print "Sendmail Error! File not opened\n";
       die "Sendmail Error! File not opened\n";
    }

    print SENDMAIL "From: $schoolname < $returnaddress > \n";
    print SENDMAIL "To: $studentrecords \n";
    print SENDMAIL "CC: $adminemail \n";
    print SENDMAIL "Subject: OA Identity Resolution: $schoolname \n";
    print SENDMAIL "X-Mailer: Open Admin Mail Notification System \n\n";

    print SENDMAIL "$message\n\n";

    close (SENDMAIL);

    print "<h1>Email Sent!</h1>\n";



} # End of doMail.
