#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Main' => 'Main',
	   'Schedule' => 'Schedule',
	   'Write Schedule Entries' => 'Write Schedule Entries',
	   'Edit Schedule Entries' => 'Edit Schedule Entries',
	   'Terms' => 'Terms',
	   'Update' => 'Update',
	   'Term' => 'Term',
	   'Day' => 'Day',
	   'Per' => 'Per',
	   'Subject-Section' => 'Subject-Section',
	   'Period' => 'Period',
	   'No Term' => 'No Term',
	   'Updating Record for' => 'Updating Record for',
	   'Adding Record for' => 'Adding Record for',
	   'Deleting Record for' => 'Deleting Record for',
	   'Add Backings' => 'Add Backings',
	   'Additional' => 'Additional',
	   'Adding' => 'Adding',
	   'Missing' => 'Missing',
	   'Continue' => 'Continue',
	   'Writing Entries for' => 'Writing Entries for',
	   'for Term(s)' => 'for Term(s)',
	   'Please record the following error' =>
	     'Please record the following error',
	   'Your schedule items now stored' => 
	     'Your schedule items now stored',
	   'There was an error storing your data' =>
	     'There was an error storing your data',
	   'Please contact network adminstrator' =>
	     'Please contact network adminstrator',
	   'Teacher User Id' => 'Teacher User Id',
	   'Separate multiple terms with spaces' => 'Separate multiple terms with spaces',
	   'Error' => 'Error',
	   'No Terms' => 'No Terms',

	   );

# max number of backed classes per cycle. 
#(control display and entry)
my $maxbackings = 8; 

use DBI;
use CGI;
use CGI::Session;
use Cwd;

my $self = "schedit.pl";


my @time = localtime(time);
my $year = $time[5] + 1900;
my $month = $time[4] + 1;
my $currdate = "$year-$month-$time[3]";

# Set prepath for config file: (/tcgi or cgi/schedule)
my $prepath = '../..';
if (getcwd() =~ /tcgi/){ # we are in tcgi
    $prepath = '..';
}

eval require "$prepath/etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}


my $q = new CGI;
my %arr = $q->Vars;


my $userid;
if ( $arr{userid} ) {
    $userid = $arr{userid};
    delete $arr{userid};
}


# Get current dir so know what CSS to display;
if (getcwd() =~ /tcgi/){ # we are in tcgi
    $css = $tchcss;
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);


# Session Management
my $session;
if (not $userid ) {
    $session = new CGI::Session("driver:mysql;serializer:FreezeThaw",
     undef,{Handle => $dbh}) or die CGI::Session->errstr;

    $userid = $session->param('userid');

}
print $q->header( -charset, $charset );


# print page header.
print "$doctype\n<html><head><title>". $lex{'Edit Schedule Entries'}. "</title>\n";
print "<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>\n";

print "[ <a href=\"$homepage\">". $lex{Main}. "</a> \n"; 
unless (getcwd() =~ /tcgi/){ # unless we are in tcgi
    print "| <a href=\"$schpage\">". $lex{Schedule}. "</a> \n";
}
print "]\n";


# Write Records / Do Backings
if ( $arr{writeflag} ){
    #print "<h1>". $lex{'Write Schedule Entries'}. "</h1>\n";
    updateRecords();
    exit 0;
} elsif ($arr{backingsflag}) { # Show Backings Form
    printBackingsForm( $arr{userid}, $arr{terms} );
    exit 0;
} elsif ($arr{addbackings}) { # Add backings records
    delete $arr{addbackings};
    addBackings();
    exit 0;
}

# Load terms from eval
my @terms;
$sth = $dbh->prepare("select distinct term from eval 
 where term != '' order by term");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
while (my $trm = $sth->fetchrow){
    push @terms, $trm;
}

# Get Teacher
my $teachername;
if ( not $arr{term} ) {
    print "<h1>". $lex{'Edit Schedule Entries'}. "</h1>\n";
    getTeacher();
    print "</body></html>\n";
    die;
} else { # We have teacher, get name
    $sth = $dbh->prepare("select lastname, firstname from staff where userid = ?");
    $sth->execute($userid);
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my ($lastname, $firstname) = $sth->fetchrow;
    $teachername = "$firstname $lastname";
}

# If here, we have passed term and teacher values ($arr{term} and $arr{userid})

my $select;
$select = "where teacher like '%$userid%'";


# Load Subject-Sections for a teacher.
$sth = $dbh->prepare("select description, smdesc, subjsec, grade from subject
   $select order by description");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

# Find subjects and put into hash for selection. 
# Grades needed for maxPPD calc.  
my (@subjects, @grades);

while ( my ( $description,$smdesc, $subjsec, $grade ) = $sth->fetchrow ){
    # use truncated longer description if no small description. 
    if (not $smdesc){ $smdesc = substr($description,0,8);} 

    $subjects{$subjsec} = "$smdesc ($subjsec)";
}

# Find maximum number of periods per day for table.
$sth = $dbh->prepare("select max(period) from schedat");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
my $maxPPD = $sth->fetchrow;



# print top section above table: Name and Term(s)
print "<b>$teachername &mdash; ". $lex{Term}. " $arr{term}</b>&nbsp;&nbsp;\n";
print $lex{Additional}. q{ }. $lex{Terms}. " \n";

# Print Main Layout Table
print "<form action=\"$self\" method=\"post\" style=\"display:inline;\">\n";
print "<input type=\"hidden\" name=\"writeflag\" value=\"1\">\n";
print "<input type=\"hidden\" name=\"teachername\" value=\"$teachername\">\n";
print "<input type=\"hidden\" name=\"userid\" value=\"$userid\">\n";
print "<input type=\"hidden\" name=\"currterm\" value=\"$arr{term}\">\n";
print "<input type=\"text\" name=\"terms\" size=\"6\">\n";
print $lex{'Separate multiple terms with spaces'};

print "<div style=\"text-align:center; padding:0.5em;\">
<input type=\"submit\" value=\"". $lex{Update}. "\"></div>\n";
print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";

my $days = $daysPerCycle; # One for each Day.

# Print Header
print "<tr><th></th>";
for (1..$daysPerCycle){	print "<th>". $lex{Day}. " $_</th>"; }
print "</tr>\n";

for my $ppd ( 1..$maxPPD ){
    print "<tr>";
    for (0..$days){
	if ($_ == 0){ 
	    print "<td><b>". $lex{Per}. " $ppd</b></td>";
	} else {
	    print "<td>";
	    my $id = "$ppd:$_";
	    prSelections($id, $arr{term});
	    print "</td>";
	}
    }
    print "</tr>\n";
}

my $span = $days + 1;
print "</table><div style=\"text-align:center; padding:0.5em;\">
<input type=\"submit\" value=\"". $lex{Update}. "\"></div>\n";
print "</form></body></html>\n";



#-------------
sub getTeacher {
#-------------

    print "<form action=\"$self\" method=\"post\">\n";

    print "<div>". $lex{'Teacher User Id'}. q{ };
    print "<input type=\"text\" name=\"userid\" value=\"$userid\" size=\"6\">\n";

    print $lex{'Edit Schedule Entries'}. " \n";
    print $lex{Term}. " <select name=\"term\"><option></option>\n";
    foreach my $trm (@terms){
	print "<option>$trm</option>\n";
    }
    print "</select></div>\n";

    print "</p>\n<p>A <b>term</b> is a period of time with tests at
    the end.</p><p style=\"margin-right:5em;\">The terms are the
    period of time these entries in a timetable are valid. In a K-6
    school with 3 terms for the year, the answer is all 3 terms. In a
    high school with 2 semesters and 2 terms in each semester, the
    first semester timetable entries would be for term 1 and 2 while
    the second semester would be term 3 and 4.</p>\n";

    print "<p><input type=\"submit\" value=\"". $lex{Continue}. "\"></p>";
    print "</form>\n";

}


#---------------
sub prSelections {  # print selection list for subject array
#---------------

    my $pid = shift; # format is period:day
    my $term = shift;
    my ($period,$day) = split /:/, $pid;
    my $found = 0;

    # find existing values for this period...
    $sth = $dbh->prepare("select id, subjsec from schedat 
       where day = ? and period = ? and term = ?");
    $sth->execute($day,$period,$term);
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my ($id, $subjsec) = $sth->fetchrow ) {
	if ( $subjects{$subjsec} ) { # this is a subject that teacher teaches..
	    $found = 1;
	    print "<select name=\"$pid:$id\"><option>$subjects{$subjsec}</option>";
	    print "<option></option>\n";
	    foreach my $subj ( sort values %subjects ){
		print "<option>$subj</option>";
	    }
	    print "\n</select>\n";
	    $counter++;
	}
    }

    if ( not $found ) { # no subjects found... (no teacher subjects in this period)
	print "<select name=\"$pid:-1\"><option></option>\n";
	foreach my $subj ( sort values %subjects ){
	    print "<option>$subj</option>";
	}
	print "\n</select>\n";
    }

}


#----------------
sub updateRecords { # update the records in schedat table.
#----------------

    # passed: currterm, terms, userid, teachername
    #foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }
    #die;

    delete $arr{writeflag};

    print "<p>". $lex{'Writing Entries for'};
    print " $arr{teachername} ". $lex{'for Term(s)'}. ": $arr{currterm} $arr{terms}</p>\n";
    delete $arr{teachername};

    my $userid = $arr{userid};
    delete $arr{userid};

    if ( not $arr{currterm}) { print $lex{'No Term'}; return; }
    my $currterm = $arr{currterm};
    delete $arr{currterm};

    my @terms = split /\s/, $arr{terms}; # split additional terms into array; currterm still def'd.
    my $terms = $arr{terms};
    # Now check that terms exist...
    foreach my $trm (@terms) {
	# check 'em here...

    }
    delete $arr{terms};

    # Check passed values
    #foreach my $key (sort keys %arr) { print "K:$key V:$arr{$key}<br>\n"; }


    # Setup for finding grade(s) of each subject (if any).
    my $sth = $dbh->prepare("select grade from subject where subjsec = ?");
    my $sth1 = $dbh->prepare("select subjsec from schedat where id = ?");

    foreach my $key ( sort keys %arr ) {

	my ( $period, $day, $id ) = split /:/, $key;

	my ( $desc, $subjsec ) = split /\(/, $arr{$key};
	chop $subjsec; # remove trailing bracket.

	my $grade;
	if ($subjsec) { # set grade
	    $grade = $sth->execute($subjsec);
	} else {
	    $grade = $sql{default};
	}

	my $origsubjsec;
	if ( $id > 0 ) {

	    $sth1->execute($id);
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    $origsubjsec = $sth1->fetchrow;
	}

	print " Subj: $subjsec Orig: $origsubjsec Day: $day Period: $period ";
	print "Id: $id Currtrm: $currterm<br>\n";

	if ( $id == -1 ) { # no record found;

	    if ( $arr{$key} ) { # no current record, but new subjsec exists... add 

		print '<b>'. $lex{'Adding Record for'}. " $subjsec</b> ";
		print $lex{Term}. ": $currterm \n";
		print $lex{Day}. ": $day \n";
		print $lex{Period}. ": $period<br>\n";

		my $sth2 = $dbh->prepare("insert into schedat 
                  values ($sql{default}, ? , ?, ?, ?, ?)");
		$sth2->execute($day, $period, $subjsec, $grade, $currterm);
		if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
	    }
	    
	} else { # we have an existing record for current term.

	    # if subjsec is blank, then delete the record (in all terms for this Day:Per)
	    if ( not $subjsec ) {

		print '<b>'. $lex{'Deleting Record for'}. " $origsubjsec</b> ";
		print $lex{Term}. ": $currterm \n";
		print $lex{Day}. ": $day \n";
		print $lex{Period}. ": $period<br>\n";

		my $sth2 = $dbh->prepare("delete from schedat where id = ?");
		$sth2->execute( $id );
		if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

	    } else { # we must update (if changed)

		if ( $origsubjsec ne $subjsec ) { # changed record, update

		    print '<b>'. $lex{'Updating Record for'}. " $subjsec</b> ";
		    print $lex{Term}. ": $currterm \n";
		    print $lex{Day}. ": $day \n";
		    print $lex{Period}. ": $period<br>\n";

		    my $sth2 = $dbh->prepare("update schedat set
                      subjsec = ?, grade = ? where id = ? ");
		    $sth2->execute( $subjsec, $grade, $id );
		    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

		}

	    } # End of Update for changed record. 

	} # End of Else (Existing record)
	
	# End of Current Term; Next: Other Terms.
	    
	my $sth3 = $dbh->prepare("select id from schedat
          where subjsec = ? and day = ? and period = ? and term = ?");

	# now do rest of terms 
	foreach my $trm ( @terms ) {
	    
	    # Look for original subjsec and new subjsec in current term, day, period.
	    $sth3->execute($origsubjsec, $day, $period, $trm);
	    if ( $DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    my $oldid = $sth3->fetchrow;

	    $sth3->execute($subjsec, $day, $period, $trm);
	    if ( $DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    my $newid = $sth3->fetchrow;

	    print "Subj: $subjsec Orig: $origsubjsec Day: $day Period: $period ";
	    print "Trm: $trm NewId: $newid OldID: $oldid<br>\n";

	    if ( not $subjsec and $oldid ) { # if orig exists, but new is null, delete.

		print '<b>'. $lex{'Deleting Record for'}. " $origsubjsec</b> ";
		print $lex{Term}. ": $trm \n";
		print $lex{Day}. ": $day \n";
		print $lex{Period}. ": $period<br>\n";

		my $sth2 = $dbh->prepare("delete from schedat where id = ?");
		$sth2->execute($oldid);
		if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

	    } elsif ( $subjsec and $oldid and $subjsec ne $origsubjsec ) { # we have both. update.

		print '<b>'. $lex{'Updating Record for'}. " $subjsec</b> ";
		print $lex{Term}. ": $trm \n";
		print $lex{Day}. ": $day \n";
		print $lex{Period}. ": $period<br>\n";

		my $sth2 = $dbh->prepare("update schedat set
                 subjsec = ?, grade = ? where id = ? ");
		$sth2->execute($subjsec, $grade, $oldid);
		if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

	    } elsif ( not $newid and $subjsec ) { # exist check and then add.

		print '<b>'. $lex{'Adding Record for'}. " $subjsec</b> ";
		print $lex{Term}. ": $trm \n";
		print $lex{Day}. ": $day \n";
		print $lex{Period}. ": $period<br>\n";

		$sth1 = $dbh->prepare("insert into schedat values ($sql{default},?,?,?,?,?)");
		$sth1->execute($day, $period, $subjsec, $grade, $trm);
		if ( $DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

	    }

	} # End of Term Loop


    } # End of keys loop.


    if (not $DBI::errstr) {
	print "<h3>". $lex{'Your schedule items now stored'}. ".</h3></p>";
    } else {
	print "<h3>". $lex{'There was an error storing your data'}. ".\n";
	print $lex{'Please contact network adminstrator'};
	print " <a href=\"mailto:$adminemail\">$adminemail</a>\n";
	print $lex{'Please record the following error'};
	print ": $DBI::errstr</h3></p>\n";
    }

    print "[ <form action=\"$self\" method=\"post\" style=\"display:inline;\">\n";
    print "<input type=\"hidden\" name=\"backingsflag\" value=\"1\">\n";
    print "<input type=\"hidden\" name=\"terms\" value=\"$currterm $terms\">\n";
    print "<input type=\"hidden\" name=\"userid\" value=\"$userid\">\n";
    print "<input type=\"submit\" value=\"". $lex{'Add Backings'}. "\">\n";
    print "</form>\n";

    print " | <a href=\"$schpage\">". $lex{Schedule}. "</a> ] \n";
    print "</p></center></body></html>\n";

} # end of updateRecords


#--------------------
sub printBackingsForm {
#--------------------

    my $userid = shift;
    my $terms = shift;

    my $sth = $dbh->prepare("select lastname, firstname from staff where userid = ?");
    $sth->execute($userid);
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my ($lastname, $firstname) = $sth->fetchrow;

    # Load Subject-Sections for teacher.
    my $sth = $dbh->prepare("select description, smdesc, subjsec, grade from subject
     where teacher like '%$userid%'");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    my @subjects;
    while (my ($description,$smdesc, $subjsec, $grade) = $sth->fetchrow){
	# use truncated longer description if no small description. 
	if (not $smdesc){ $smdesc = substr($description,0,8);} 

	$subjects{$subjsec} = "$smdesc ($subjsec)";
    }

    print "<b>$firstname $lastname</b> ($userid) ". $lex{Terms}. " $terms\n";

    print "<form action=\"$self\" method=\"post\" style=\"padding:1em;\">\n";
    print "<input type=\"hidden\" name=\"addbackings\" value=\"1\">\n";
    print "<input type=\"hidden\" name=\"terms\" value=\"$terms\">\n";

    #print "<p><b>". $lex{'Add Backings'}. "</b></p>\n";
    print "<input type=\"submit\" value=\"". $lex{'Add Backings'}. "\">\n";
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>". $lex{'Subject-Section'}. "</th><th>". $lex{Day};
    print "</th><th>". $lex{Period}. "</th></tr>\n";

    for my $rec (1..$maxbackings){
	print "<tr><td>";
	print "<select name=\"$rec\"><option></option>\n";
	foreach my $sub (sort values %subjects) {
	    print "<option>$sub</option>";
	}
	print "</td>\n<td><input type=\"text\" name=\"d$rec\" size=\"6\"></td>\n";
	print "</td><td><input type=\"text\" name=\"p$rec\" size=\"6\"></td>\n";
	print "</tr>\n";
    }
    print "</table>";
    print "<input type=\"submit\" value=\"". $lex{'Add Backings'}. "\">\n";
    print "</form></body></html>\n";

}


#--------------
sub addBackings {
#--------------


    my $terms = $arr{terms};
    delete $arr{terms};
    my @terms = split / /, $terms;

    print '<h1>'. $lex{'Add Backings'}. "</h1>\n";

    my $sth = $dbh->prepare("select description, grade from subject where subjsec = ?");
    if (not $terms) { 
	print $lex{'No Terms'}; 
	die $lex{'No Terms'};
    }



KEY:
    foreach my $key (sort keys %arr ) {

	if ( $key =~ m/^(\d)/ and $arr{$key} ) {

	    #print "K:$key V:$arr{$key}<br>\n";

	    my $di = "d$1"; 
	    my $pi = "p$1";

	    if ( not $arr{$di} or not $arr{$pi} ) {
		print $lex{Missing}. q{ }. $lex{Period}. "/". $lex{Day}. "<br>\n";
		die;
	    }

	    my ( $subj, $subjsec ) = split /\(/,$arr{$key};
	    chop $subjsec; # remove trailing bracket.


	    $sth->execute($subjsec);
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    my ($description, $grade) = $sth->fetchrow;

	    print $lex{'Adding'}. " $description ($subjsec) &ndash; ". $lex{Day};
	    print " $arr{$di} &ndash; ". $lex{Period}. " $arr{$pi}<br>\n";

	    $subjsec = $dbh->quote($subjsec);

	    foreach my $trm ( @terms ) {
		my $sth1 = $dbh->prepare("insert into schedat values (
	        $sql{default},'$arr{$di}','$arr{$pi}', $subjsec, '$grade','$trm')");
		$sth1->execute;
		if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    }

	} # End of IF 

    } # End of foreach key loop

    print "<p>[ <a href=\"$schpage\">". $lex{Schedule}. "</a> ]</p>\n";
    print "</body></html>\n";

} # End of addBackings
