#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# Subjschview.pl - view scheduling values in subject table both by grade and by teacher.

my %lex = ( 'View Subject Schedules' => 'View Subject Schedules',
	    'Schedule' => 'Schedule',
	    'Main' => 'Main',
	    'Select' => 'Select',
	    'Totals' => 'Totals',
	    'Grade' => 'Grade',
	    'backed with' => 'backed with',
	    'Sem' => 'Sem',
	    'Backing' => 'Backing',
	    'Description' => 'Description',
	    'Subjsec' => 'Subjsec',
	    'Teacher' => 'Teacher',
	    'Select' => 'Select',
	    'Error' => 'Error',

	   );


use DBI;
use CGI;

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

$q = new CGI;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;

# show teachers if set...
if ($arr{select} eq $lex{Teacher}) {
    $teacherflag = 1;
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# page header
print "$doctype\n<html><head><title>". $lex{'View Subject Schedules'}. "</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>\n";

# select option and nav items.
print "<form action=\"subjschview.pl\" method=\"post\">";
print "[ <a href=\"$homepage\">".$lex{Main}. "</a> | <a href=\"$schpage\">". $lex{'Schedule'}. "</a> | ";
print "<select name=\"select\">";
print '<option>'. $lex{Teacher}. '</option><option>'.$lex{Grade}. "</option>\n";
print "</select><input type=\"submit\" value=\"". $lex{Select}. "\"></form>\n";


if ($teacherflag) {
    showTeachers();
    die;
}


# print table header
print "<center><h1>Subjects</h1>\n";

# NOTE: grade field may hold multiple space separated real grades ('11 12 13');
$sth = $dbh->prepare("select distinct grade from subject");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
while (my $grade = $sth->fetchrow) {
    my @gr = split / /, $grade;
    foreach my $gr (@gr) {
	$gr =~ s/\s//g; # remove any spaces.
	$grades{$gr} = 1;
    }
}
# grades hash now populated; create @grades from hash.
my @grades = sort { $a <=> $b } keys %grades;

my $sth1 = $dbh->prepare("select description, subjsec, 
  schedppc1, schedppc2, schedppc3, schedppc4, backwith from subject 
  where grade like ? order by description");

my $sth2 = $dbh->prepare("select description from subject where subjsec = ?");
my $first = 1;


# loop through each grade and print values
foreach my $grade (@grades) {
    my ($total1, $total2, $total3, $total4);

    if (not $first) { print "</table>\n"; } else { $first = 0; }
    print "<p><b>". $lex{Grade}. " $grade</b></p>\n";

    print "<table cellpadding=\"3\" border=\"1\" cellspacing=\"0\">\n";
    print '<tr><th>'. $lex{Description}. ' ('. $lex{Subjsec}. ')</th><th>'. $lex{Sem}. "<br>1</th>\n";
    print "<th>". $lex{Sem}. "<br>2</th><th>". $lex{Sem}. "<br>3</th><th>". $lex{Sem};
    print '<br>4</th><th>'. $lex{Backing}. "</th></tr>\n";

    $sth1->execute('%'.$grade.'%');
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    while (my ($description, $subjsec, $schedppc1, $schedppc2, 
	       $schedppc3, $schedppc4, $backwith) = $sth1->fetchrow) {

	print "<tr><td>$description ($subjsec)";

	if ($backwith) { 
	    $sth2->execute($backwith);
	    my $desc = $sth2->fetchrow;
	    print ' '. $lex{'backed with'}. "<br>&nbsp;&nbsp;<i>$backwith - $desc</i></td>\n";
	    print "<td>($schedppc1)</td><td>($schedppc2)</td>\n";
	    print "<td>($schedppc3)</td><td>($schedppc4)</td><td>$backwith</td></tr>\n";
	    next;
	}

	print "\n</td><td>$schedppc1</td><td>$schedppc2</td>\n";
	print "<td>$schedppc3</td><td>$schedppc4</td><td>$backwith</td></tr>\n";

	$total1 += $schedppc1;
	$total2 += $schedppc2;
	$total3 += $schedppc3;
	$total4 += $schedppc4;
    }

    print "<tr style=\"background-color:#DDD\"><td><b>". $lex{Totals}. "</b></td><td>$total1</td>\n";
    print "<td>$total2</td><td>$total3</td><td>$total4</td><td></td></tr>\n";

}

print "</table></body></html>\n";



#---------------
sub showTeachers {
#---------------

    # print table header
    print "<center><h1>Teachers</h1>\n";

    $sth = $dbh->prepare("select teacher, description, subjsec, 
      schedppc1, schedppc2, schedppc3, schedppc4, backwith 
      from subject order by teacher, description");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    my $first = 1;
    my $currteacher = -1;
    my $oldteacher;
    my ($total1, $total2, $total3, $total4);



    # loop through each teacher and print values
    while (my ($teacher, $description, $subjsec, $schedppc1, $schedppc2, 
	       $schedppc3, $schedppc4, $backwith) = $sth->fetchrow) {

	$oldteacher = $currteacher;
	$currteacher = $teacher;

	if ($currteacher ne $oldteacher) { # start a new table.

	    if (not $first) {  # print totals, and close table; reset totals
		print "<tr style=\"background-color:#DDD\"><td><b>". $lex{Totals}. "</b></td><td>$total1</td>\n";
		print "<td>$total2</td><td>$total3</td><td>$total4</td><td></td></tr>\n";
		print "</table><p></p>\n\n"; 
		$total1 = 0; $total2 = 0; $total3 = 0; $total4 = 0;
	    } else { $first = 0; }

	    print "<table cellpadding=\"3\" border=\"1\" cellspacing=\"0\">\n";
	    print "<caption style=\"font-size:140%; padding:0.4em;\"><b>$teacher</b></caption>\n";

	    print '<tr><th>'. $lex{Description}. " (". $lex{Subjsec}. ")</th><th>". $lex{Sem}. "<br>1</th>\n";
	    print "<th>". $lex{Sem}. "<br>2</th><th>". $lex{Sem}. "<br>3</th><th>". $lex{Sem};
	    print '<br>4</th><th>'. $lex{Backing}. "</th></tr>\n";

	}

	print "<tr><td>$description ($subjsec)";

	#if ($backwith) { 
	#    $sth2->execute($backwith);
	#    my $desc = $sth2->fetchrow;
	#    print ' '. $lex{'backed with'}. "<br>&nbsp;&nbsp;<i>$backwith - $desc</i></td>\n";
	#    print "<td>($schedppc1)</td><td>($schedppc2)</td>\n";
	#    print "<td>($schedppc3)</td><td>($schedppc4)</td><td>$backwith</td></tr>\n";
	#    next;
	#}

	print "\n</td><td>$schedppc1</td><td>$schedppc2</td>\n";
	print "<td>$schedppc3</td><td>$schedppc4</td><td>$backwith</td></tr>\n";

	$total1 += $schedppc1;
	$total2 += $schedppc2;
	$total3 += $schedppc3;
	$total4 += $schedppc4;
    }

    print "<tr style=\"background-color:#DDD\"><td><b>". $lex{Totals}. "</b></td><td>$total1</td>\n";
    print "<td>$total2</td><td>$total3</td><td>$total4</td><td></td></tr>\n";
    print "</table></body></html>\n";


} # end of showTeachers;




	      
