#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# transdaily - daily report for children on bus/transport.

my %lex = ('Main' => 'Main',
	   'Schedule' => 'Sched/Trans',
	   'Daily Transportation Report' => 'Daily Transportation Report',
	   'View Daily Transportation Report' => 'View Daily Transportation Report',
	   'Student' => 'Student',
	   'Back to Top' => 'Back to Top',
	   'All' => 'All',
	   'Gr' => 'Gr',
	   'HRm' => 'Hr',
	   'Group' => 'Group',
	   'Date' => 'Date',
	   'PDF?' => 'PDF?',
	   'Attendance' => 'Attendance',
	   'Total' => 'Total',
	   'View Log File' => 'View Log File',
	   'AS Prog' => 'AS',
	   'Route' => 'Route',
	   'After School Program' => 'After School Program',
	   'Error' => 'Error',

	   );


my $defaultTransType = 'Bus'; # change for other langs, etc.
my $transtype = 'trans_type';
my $transname = 'trans_name';
my $translocation = 'trans_bus_stop_location';

my $self = 'transdaily.pl'; 

my $other_fields =  'group_after_school_prog';
my @other_fields = qw(group_after_school_prog ); # add more for space separated list.

my @transfields = @other_fields;
push (@transfields, $transtype);
push (@transfields, $transname);
push (@transfields, $translocation);

use DBI;
use CGI;


eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

my ($sec, $min, $hour, $mday, $mon, $year, $wday, 
    $yday, $iddst) = localtime(time);
$year = $year + 1900;
my $month = $mon + 1;
$wday++;
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";
my $currsdate = "$year-$month-$mday";



if ($arr{date}) {
    $currsdate = $arr{date};
    my ($y,$m,$d) = split /-/, $arr{date};
    $m--;
    $currdate = "$month[$m] $d, $y";
}


my $select;
if (not $arr{transtype}) { $arr{transtype} = $defaultTransType; }
if ($arr{transtype} ne $lex{All} ) { # no selection for All.
    $select = "where $transtype = ?";
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# get distinct transtype values
my $sth = $dbh->prepare("select distinct $transtype from student 
 where $transtype != '' group by $transtype order by $transtype");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
while (my $trans = $sth->fetchrow) {
    push @transtype, $trans;
}


# get descriptions for each field
my %fielddesc;
my $sth = $dbh->prepare("select fieldname from meta where 
  tableid = 'student' and fieldid = ?");

foreach my $fld (@transfields) {
    $sth->execute($fld);
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my $desc = $sth->fetchrow;
    #print "F:$fld D:$desc<br>\n";
    $fielddesc{$fld} = $desc;
}


# page heading
print "$doctype\n<html><head><title>". $lex{'Daily Transportation Report'};
print "</title><link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body><a name=\"top\"></a>\n";
print "[ <a href=\"$homepage\">". $lex{Main}. "</a> | ";
print "<a href=\"$schpage\">". $lex{Schedule}. "</a> ]\n";

# Selection Form.
print "<form action=\"$self\" method=\"post\" style=\"display:inline;\">\n";
print "<input type=\"submit\" value=\"Select by\">\n";
print "<select name=\"transtype\">";
if ($arr{transtype}) { print "<option>$arr{transtype}</option>\n"; }
print "<option>". $lex{All}. "</option>\n";
foreach my $tr (@transtype) {
    print "<option>$tr</option>";
}
print "</select>";
print "&nbsp;". $lex{Date}. " <input type=\"text\" name=\"date\" size=\"11\" value=\"$currsdate\">\n";
print "&nbsp;". $lex{'PDF?'}. " <input type=\"checkbox\" name=\"pdf\" value=\"1\"></form>\n";


# setup SQL query for main table - selection and sorting,
my $sth = $dbh->prepare("select studid, lastname, firstname, studnum, grade, homeroom, 
  $transname, $translocation, group_after_school_prog from student $select 
  order by $transtype, group_after_school_prog, $transname, $translocation, lastname, firstname");
if ($select) { 
    $sth->execute($arr{transtype}); 
} else {
    $sth->execute;
}
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

print "<center><h1>". $lex{'Daily Transportation Report'}. " &ndash; $schoolname</h1>\n";

if ($arr{pdf}) { # print PDF version instead.
    prTexVersion();
    die;
}

print "<h3>$currdate<br>". $lex{Group}. " &ndash; $arr{transtype}</h3>\n";

# Just print table heading
print "<table border=\"1\" cellpadding=\"3\" cellspacing=\"0\">\n";
print "<tr><th>". $lex{Student}. "</th><th>". $lex{Gr}. "</th><th>". $lex{HRm}. "</th>\n";
print "<th>$fielddesc{$transname}</th>";
print "<th>$fielddesc{$translocation}</th>";
foreach my $fld (@other_fields) {
    print "<th>$fielddesc{$fld}</th>";
}
print "<th>". $lex{Attendance}. "</th></tr>\n";

my $count = 0;
my $first = 1;
my $currgroup = '-1';
my $oldgroup;

my $sth1 = $dbh->prepare("select distinct reason, max(period) from attend 
  where studentid = ? and absdate = '$currsdate' and reason not like '%$lateString%' group by reason");

while (my ($studid, $lastname, $firstname, $studnum, $grade, $homeroom, @rest) = 
       $sth->fetchrow){

    $oldgroup = $currgroup;
    $currgroup = $rest[0];
    if ( $currgroup ne $oldgroup and not $first) {
	print "<tr style=\"background-color:#CCC;\"><td colspan=\"7\">Total: $count</td></tr>\n";
	$count = 0;
    }
    if ( $first ){ $first = 0; } 
    $count++;
    print "<tr><td><b>$count.$lastname</b>, $firstname ($studnum)</td>";
    print "<td>$grade</td><td>$homeroom</td>\n";
    foreach my $fld (@rest) {
	print "<td>$fld</td>";
    }

    # Check Attendance
    my %attend;
    $sth1->execute($studnum);
    while (my ($reason, $period) = $sth1->fetchrow) {
	    $attend{$reason} = $period;
    }
    print "<td>";
    foreach my $reason (sort keys %attend) {
	print "$reason:P$attend{$reason} ";
    }

    print "</td></tr>\n";

} # End of loop 

print "<tr style=\"background-color:#CCC;\"><td colspan=\"7\">". $lex{'Total'}. ": $count</td></tr>\n";
print "</table>\n";
print " [ <a href=\"#top\">",$lex{'Back to Top'},"</a> ]\n";
print "</body></html>\n";



#---------------
sub prTexVersion {
#---------------

    my $fmtString = '|p{1.8in}|p{0.2in}|p{0.3in}|p{1in}|p{1.9in}|p{1.7in}|';
    #                Name       Gr       Hrm    BusRoute BusStop   Attend
    # ASP removed...

    my $shortname = "dailybus$$";
    my $filename = "$shortname.tex";
    my $logfile = "pdflog$$.txt";

    # Setup Tex File 
    open(TEX,">$filename") || die "Cannot open tex file";

    # TeX Header
    print TEX "\\documentclass[11pt,letterpaper]{article}\n";
    print TEX "\\usepackage{array,multicol,colortbl,inputenc}\n";
    print TEX "\\inputencoding{latin1}\n";
    print TEX "\\setlength{\\textwidth}{7.5in}\n \\setlength{\\textheight}{10in}\n";
    print TEX "\\setlength{\\evensidemargin}{0in}\n\\setlength{\\evensidemargin}{0in}\n";
    print TEX "\\setlength{\\hoffset}{-1.5in}\n\\setlength{\\voffset}{-1in}\n";

    print TEX "\\setlength{\\extrarowheight}{2pt}\n\\setlength{\\parindent}{0pt}\n";

    print TEX "\\pagestyle{headings}\n";
    print TEX "\\markright{$currdate \\hfill $schoolname \\hfill Page }\n";

    print TEX "\\begin{document}\n\\begin{center}\n{\\sf\\huge $schoolname}\n\n";
    print TEX "{\\sf\\LARGE ". $lex{'Daily Transportation Report'}.  " - $currdate} \n\n\\bigskip\n";


    # ------ What to do about date and a common header or footer.
    #print "<h3>$currdate<br>". $lex{Group}. " &ndash; $arr{transtype}</h3>\n";


    # Just print table heading
    #print TEX "\\begin{tabular}{$fmtString}\\hline\n";

    my $count = 0;
    my $first = 1;
    my $currgroup = '-1';
    my $currasg;  # current after school group (Y or not)
    my ($oldgroup, $oldasg);

    my $sth1 = $dbh->prepare("select distinct reason, max(period) from attend 
     where studentid = ? and absdate = '$currsdate' and reason not like '%$lateString%' group by reason");


    while (my ($studid, $lastname, $firstname, $studnum, $grade, $homeroom, $trans_name, $trans_bs_location, $asg)
	   = $sth->fetchrow){

	$trans_bs_location =~ s/\&/\\&/g;

	# print total line if new group
	$oldgroup = $currgroup;
	$currgroup = $trans_name;
	
	$oldasg = $currasg;
	$currasg = $asg;

	if ( $currgroup ne $oldgroup or $currasg ne $oldasg ) {

	    if (not $first) {  # print count line and end the table.
		print TEX "\\rowcolor[gray]{0.92} ";
		print TEX $lex{Total}. ": $count & & & & & \\\\ \\hline\n\n";
		print TEX "\\end{tabular}\n\\newpage\n";
	    } else { $first = 0; }

	    # Set ASG (After School Program) display values.
	    if ($currasg) {
		$asgDisplay = $currasg;
	    } else { 
		$asgDisplay = 'N'; 
	    }

	    print TEX "\n\n{\\Large ". $lex{Route}. ": {\\bf $currgroup} \\quad ";
	    print TEX $lex{'After School Program'}. ": {\\bf $asgDisplay}}\n\n";

	    print TEX "\\begin{tabular}{$fmtString}\\hline\n";

	    # print heading line
	    print TEX "\\rowcolor[gray]{0.92}\\bf ";
	    print TEX $lex{Student}. '&\\bf '. $lex{Gr}. '&\\bf ' . $lex{HRm}. '&\\bf ';
	    print TEX $fielddesc{$transname}. '&\\bf '. $fielddesc{$translocation}. '&\\bf ';
	    #print TEX $lex{'AS Prog'}. '&\\bf ';
	    print TEX $lex{Attendance}. "\\\\\n\\hline\n";

	    $count = 0;
	}
	

	# put in divider to allow page breaks.
	#if ($count % 4 == 0) {
	#    print TEX "\\end{tabular}\n\n\\begin{tabular}{$fmtString}\\hline\n";
	#}


	$count++;
	print TEX "{\\bf $lastname}, $firstname &";
	print TEX "$grade & $homeroom & $trans_name & $trans_bs_location";

	# Check Attendance
	my %attend;
	$sth1->execute($studnum);
	while (my ($reason, $period) = $sth1->fetchrow) {
	    $attend{$reason} = $period;
	}
	print TEX '&';
	foreach my $reason (sort keys %attend) {
	    print TEX "$reason:P$attend{$reason} ";
	}

	print TEX "\\\\ \\hline\n";


    } # End of loop 

    print TEX "\\rowcolor[gray]{0.92} ";
    print TEX $lex{Total}. ": $count & & & & & \\\\ \\hline\n\n";
    print TEX "\\end{tabular}\n\\end{center}\\end{document}\n";

    close TEX;
 
    system("$pdflatex $filename >$logfile");
    system("mv $shortname.pdf $downloaddir");
    system("mv $logfile $downloaddir");
    system("rm -f $shortname.*");

    print "<div style=\"padding:1em;margin:2em;border:1px solid black;text-align:center;\">";
    print "<h1><a href=\"$webdownloaddir/$shortname.pdf\">";
    print $lex{'View Daily Transportation Report'};
    print "</a></h1>\n [ <a href=\"$schpage\">". $lex{Schedule}. "</a> |\n";
    print "<a href=\"$webdownloaddir/$logfile\">". $lex{'View Log File'};
    print "</a> ]</div></body></html>\n";


}

