#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# resetbigbus.pl - reset serveral transportation fields...


%lex = ('Main' => 'Main',
	'Schedule' => 'Sched/Trans',
	'Update Transportation Fields' => 'Update Transportation Fields',
	'Student' => 'Student',
	'Back to Top' => 'Back to Top',
	'Add New Selections' => 'Add New Selections',
	'Update Transportation' => 'Update Transportation',
	'Field' => 'Field',
	'Value' => 'Value',
	'Error' => 'Error',

	);

my $self = 'transedall.pl';

my $defaultMode = 'list'; # list or raw
my $transtype = 'trans_type';

my $transfields = 'trans_type, trans_bus_stop_location, trans_name,
 trans_am_route, trans_pickup_time, trans_pm_route, trans_dropoff_time, 
 group_after_school_prog';

my @transfields = qw(trans_type trans_bus_stop_location trans_name 
 trans_am_route trans_pickup_time trans_pm_route trans_dropoff_time 
 group_after_school_prog);


use DBI;
use CGI;

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

my $mode;
if ($arr{mode}) {
    $mode = $arr{mode};
    delete $arr{mode};
} else {
    $mode = $defaultMode;
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# get values for each field to put into list
my %list_ref; # hash of refs to arrays of values
foreach my $fld (@transfields) {
    my $sth = $dbh->prepare("select distinct $fld from student 
     where $fld != ''");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while (my $val = $sth->fetchrow) {
	push @{$list_ref{$fld}}, $val;
    }
}

# add any missing trans_type value in meta table
my $sth = $dbh->prepare("select defaultvalue from meta where tableid = 'student'
 and fieldid = 'trans_type'");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
my $defaults = $sth->fetchrow;
my @vals = split / /, $defaults;
foreach my $val (@vals) {
    if ($val eq '~') { next; }
    $trans_type{$val} = 1;
}
# remove values already present in array.
foreach my $val (@{$list_ref{trans_type}}) {
    delete $trans_type{$val};
}
# now put in any missing values
foreach my $val (sort keys %trans_type) {
    push @{$list_ref{trans_type}}, $val;
}


# Now check each passed value if 'addselect' passed.
if ($arr{addselect}) {
    delete $arr{addselect};
    foreach my $key (keys %arr) {
	if ($arr{$key}) { # if a value exists... add it.
	    my ($row,$field) = split /:/, $key;
	    #print "R:$row F:$field K:$key V:$arr{$key}<br>\n";
	    push @{$list_ref{$field}}, $arr{$key};
	}
    }
}



# get descriptions for each field
my %fielddesc;
foreach my $fld (@transfields) {
    my $sth = $dbh->prepare("select fieldname from meta where 
     tableid = 'student' and fieldid = ?");
    $sth->execute($fld);
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my $desc = $sth->fetchrow;
    $fielddesc{$fld} = $desc;
}



# page heading
print "$doctype\n<html><head><title>". $lex{'Update Transportation Fields'};
print "</title><link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body><a name=\"top\"></a>\n";
print "[ <a href=\"$homepage\">". $lex{Main}. "</a> | ";
print "<a href=\"$schpage\">". $lex{Schedule}. "</a> ]\n";

print "<form action=\"$self\" method=\"post\" style=\"display:inline;\">\n";
print "<input type=\"submit\" value=\"Select Mode\">\n";

my $listmodechecked = 'CHECKED';
my $rawmodechecked;
if ($mode eq 'raw') {
    $rawmodechecked = 'CHECKED';
    $listmodechecked = '';
}

print "Raw <input type=\"radio\" name=\"mode\" value=\"raw\" $rawmodechecked>\n";
print "List <input type=\"radio\" name=\"mode\" value=\"list\" $listmodechecked>\n";
print "</form>\n";

print "[ <a href=\"#addselects\">Add Selections</a> ]\n";



if ($arr{writeflag}){
    updateRecords();
    die;
}

my $sth = $dbh->prepare("select studid, lastname, firstname, studnum, 
  $transfields from student order by $transtype, lastname, firstname");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }


print "<center><h1>$schoolname ". $lex{'Update Transportation Fields'};
print "</h1>\n";
print "<form action =\"$self\" method=\"post\">\n";
print "<input type=\"hidden\" name=\"writeflag\" value=\"1\">\n";
print "<input type=\"submit\" value=\"". $lex{'Update Transportation Fields'}. "\">\n";


# Just print table heading
print "<table border=\"1\" cellpadding=\"3\" cellspacing=\"0\">\n";
print "<tr><th>". $lex{Student}. "</th>\n";
foreach my $fld (@transfields) {
    print "<th>$fielddesc{$fld}</th>";
}
print "</tr>\n";


while (my ($studid, $lastname, $firstname, $studnum, @trans) = 
       $sth->fetchrow){
    print "<tr><td><b>$lastname</b>, $firstname ($studnum)</td>";

    if ($mode eq 'list') {
	foreach my $fld (@transfields) {
	    my $val = shift @trans;
	    print "<td><select name=\"$studid:$fld\"><option>$val</option>\n";

	    foreach my $val (sort @{$list_ref{$fld}} ) { 
		print "<option>$val</option>"; 
	    }
	    print "</select></td>\n";
	}
    } else {
	foreach my $fld (@transfields) {
	    my $val = shift @trans;
	    print "<td><input name=\"$studid:$fld\" value=\"$val\" ";
	    print "size=\"10\"></td>";
	}
    }

    print "</tr>\n";

} # End of loop 


print "</table>";
print "<input type=\"submit\" value=\"",$lex{'Update Transportation Fields'};
print "\"></form>\n";
print " [ <a href=\"#top\">",$lex{'Back to Top'},"</a> ]\n";
print "<a name=\"addselects\"></a><hr>\n";

newSelectFields();


print "</body></html>\n";



#----------------
sub updateRecords {
#----------------

    delete $arr{writeflag};

=pod    
    print "<br>\n";
    foreach my $key (keys %arr) { 
	my ($id, $fld) = split /:/, $key;
	if ($arr{$key}) {
	    print "ID:$id FLD:$fld V:$arr{$key}<br>\n";
	}
    }
    die;

=cut


    #my $sth = $dbh->prepare("select lastname, firstname, ?
    #  from student where studid = ?");


    print "<h1>",$lex{'Update Transportation'},"</h1>\n";
    print "<table border=\"1\" cellpadding=\"3\" cellspacing=\"0\">\n";
    print "<tr><th>". $lex{Student}. "</th><th>",$lex{Field},"</th><th>";
    print $lex{Value}. "</tr>\n";

    foreach my $key (keys %arr){
	# Get Student Records currently...
	my ($id, $fld) = split /:/, $key;
	my $sth = $dbh->prepare("select lastname, firstname, $fld
          from student where studid = ?");
	$sth->execute($id);
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my ($lastname, $firstname, $fieldval) = $sth->fetchrow;

	if ($fieldval ne $arr{$key}){ # then update
	    my $sth1 = $dbh->prepare("update student set $fld = ?
             where studid = ?");
	    $sth1->execute($arr{$key}, $id); # value, id
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
	    print "<tr><td>$lastname, $firstname</td>";
	    print "<td>$fielddesc{$fld}</td>";
	    print "<td>New: $arr{$key} Old:$fieldval</td>";
	    print "</tr>\n";
	}
    }
    print "</table></body></html>\n";

}



#------------------
sub newSelectFields { # show new select fields...
#------------------
    
    print "<form action =\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"mode\" value=\"list\">\n";
    print "<input type=\"hidden\" name=\"addselect\" value=\"1\">\n";
    # Note: we have to force list mode since we are adding to sel lists.
    print "<input type=\"submit\" value=\"". $lex{'Add New Selections'};
    print "\">\n";


    # Just print table heading
    print "<table border=\"1\" cellpadding=\"3\" cellspacing=\"0\">\n";
    print "<tr><th>". $lex{Student}. "</th>\n";
    foreach my $fld (@transfields) {
	print "<th>$fielddesc{$fld}</th>";
    }
    print "</tr>\n";


    for my $row (1..3) {
	print "<tr><td></td>";
	foreach my $fld (@transfields) {
	    print "<td><input name=\"$row:$fld\" size=\"10\"></td>";
	}
	print "</tr>\n";
    }
    
    print "</table></form>\n";

}
