# Library function for attendance scripts.

#  Date::Business needed for several functions.

# - mkSchoolDays($startdate, $stopdate) - Return %schooldays hash of
#   days in month school is open.  key is 'yyyy-mm' format and value
#   is schooldays in the month.

# - mkSchoolMonths() - return an array for yyyy-mm values for each month
#   of school year. It is 1 based so first (zeroth) entry in index is
#   blank. 

# - calcMonthlyEnrollment($studnum, $yearmonth, $active) calculate
#   student attendance by month. Returns: $daysenrolled, $active

# - calcMonthlyAttendance($studnum, $yearmonth, $periodsperday) - calculate
#   the attendance for the student for the month. Return: daysabsent:timeslate

# - calcDaysBetween($startdate,$enddate) - calculate the number of
#   school days between 2 dates (inclusive). Return: $daysbetween

# - findEnrollmentBlocks(studnum, active, currdate) - return time blocks student is
#   enrolled.

# - calcDateAttendance - calculate student attendance for a date range.(OLD)


#---------------
sub mkSchoolDays { # Make a %schooldays hash containing yyyy-mm:schoolday vals 
#---------------

    # This will find school days for school by month. This is needed
    # in any case even though other functions may also troll through
    # school year looking for similar things. It should be called at
    # the top of any script to setup the schooldays hash to report
    # 'Days Open in Month' for all students. 'Days enrolled in Month',
    # etc. would be done by other functions, etc.


    # Loop through all months and create an hash %schooldays
    #  with yyyy-mm key and value containing schooldays in that month.
    #  Stop at month containing $stopdate.

    # Passed: Startdate, Enddate in yyyymmdd format. Startdate would normally
    #  be the start of the school year.
    
    my ($startd,$stopd) = @_; # Format: yyyymmdd
    $startd =~ s/-//g; # just in case
    $stopd =~ s/-//g; # just in case... strip hyphens

    my $curryear = substr($startd,0,4);
    my $currmonth = substr($startd,4,2);
    my $monthstartdate = $startd; 

    $stopdateobj =  new Date::Business(DATE=>$stopdate);

    my $doAnotherMonth = 1;

    while ($doAnotherMonth){

	# Set Next Month
	if ($currmonth == 12){ 
	    $nextmonth = 1;
	    $nextyear = $curryear + 1;
	} else { 
	    $nextmonth = $currmonth + 1;
	    $nextyear = $curryear;  
	    # Note: not really 'next' year: just year of next month's dates
	}
	#print "Current Month: $currmonth  Current Year: $curryear<br>\n";

	# Format End of Month Date using Next Month value.
	if (length($nextmonth) == 1){ $nextmonth = '0'.$nextmonth;}
	$nextmonthstart = $nextyear.$nextmonth."01";
	$nextmonthstartobj = new Date::Business(DATE=>$nextmonthstart);
	$nextmonthstartobj->prev(); # backup to end of prev month.
	$monthenddate = $nextmonthstartobj->image();

	$end = new Date::Business(DATE=>$monthenddate);
	$start = new Date::Business(DATE=>$monthstartdate);
	#print "Start: $monthstartdate End: $monthenddate<br>\n";

	$schooldays = $end->diffb($start,'prev','next');
	$schooldays++;
	# Yup, this is the right way to do the date calcs...checked..
 
	my $sth = $dbh->prepare("select * from dates 
         where to_days(date) >= to_days('$monthstartdate') and 
         to_days(date) <= to_days('$monthenddate')");
	$sth->execute;
	if ($DBI::errstr) { print $DBI::errstr; die;}
	$holidays = $sth->rows;

	my $days = $schooldays - $holidays;
	
	if ($days > 0){ # shouldn't be needed....
	    $schooldays{"$curryear-$currmonth"} = $days;
	}

	# Are we there yet?
	# Compare current month end with enddate. If <=, we are done.
	my $stopvalue = $stopdateobj->diff($end);
	if ($stopvalue <= 0){ $doAnotherMonth = 0;}

	#increment the month (and possibly the year)
	if ($currmonth == 12){ 
	    $currmonth = 1; $curryear++;
	} else {
	    $currmonth++;
	}
	if (length($currmonth) == 1){ $currmonth = '0'.$currmonth;}

	# update the month start date.
	$monthstartdate = $nextmonthstart;
     }

    return %schooldays;

} # End of mkSchoolDays


#-----------------
sub mkSchoolMonths {  # return array of current school months
#-----------------

    my $enddate = shift;

    # Needs globals: $schoolstart $currsdate (short date) yyyy-mm-dd
    # (or else passed values)

    my $startdate = $schoolstart;

    if ( not $enddate and $currsdate ) { # fall back to global.
	$enddate = $currsdate;
    }

    if (not $enddate) { $enddate = $altStartDate; }

    if (not $enddate) { 
	print "<br>No End Date for mkSchoolMonths() in libattend.pl<br>\n";
	return ();
    } # return if no end value;

    # First find the months to end of current month from year start.
    my ($styear, $stmonth,$stday) = split /-/,$startdate;
    my ($endyear, $endmonth,$endday) = split /-/,$enddate;
    if (length($endmonth) == 1){ $endmonth = '0'.$endmonth;}
    if (length($startmonth) == 1){ $startmonth = '0'.$startmonth;}

    my $tmpyear = $styear;
    my $tmpmonth = $stmonth;
    my @schoolmonths; # local array only.

    #loop to put in year-month format in array
    push @schoolmonths, ''; # Blank first zeroth record [0]

    until ($tmpmonth eq $endmonth and $tmpyear eq $endyear){
	if (length($tmpmonth) == 1){ $tmpmonth = "0".$tmpmonth; }
	push @schoolmonths, "$tmpyear-$tmpmonth";
	$tmpmonth++;
	if ($tmpmonth > 12){ $tmpmonth = '01'; $tmpyear++;}
	elsif (length($tmpmonth) == 1){ $tmpmonth = "0".$tmpmonth; }
    }
    push @schoolmonths, "$tmpyear-$tmpmonth";  # push in current month.

    return @schoolmonths;

}


#------------------------
sub calcMonthlyEnrollment { # calculate enrollment for 1 month for 1 child
#------------------------
    # Needs: $schoolstart
    # Return:  daysenrolled:active state (enrolled(1) or withdrawn(0))

    my ($studnum, $yearmonth, $active) = @_;

    #print "SN: $studnum  YM: $yearmonth  ACT: $active<br>\n";

    # yearmonth will have yyyy-mm format
    my $tmpyearmonth = $yearmonth;
    $tmpyearmonth =~ s/-//; # remove single dash for SQL query
    my ($yr,$mo) = split /-/, $yearmonth;
    if (length($mo) == 1){ $mo = '0'.$mo;}

    my $daysenrolled = 0; # number of school days student enrolled this month


    my $sth = $dbh->prepare("select date, type from transfer 
     where studnum = '$studnum' and 
     date $sql{like} '$yr-$mo-__' and 
     date >= '$schoolstart'
     order by date");

    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
    my $rows = $sth->rows;

    if ($rows < 1){ # no records
	if ($active){ 
	    return $schooldays{$yearmonth}.':'.$active;
	} else {
	    return '0:'.$active;
	}
    } else { # we have records, loop through each.
	my $ecday = 1; # EC = EnrollmentChange day

	while (my ($transday,$transtype) = $sth->fetchrow){

	    if ($transtype eq 'withdraw'){
		# Find schooldays between $transday and $ecday
		if (length($ecday) == 1){ $ecday = '0'.$ecday;}

		my $daysbetween = &calcDaysBetween($yearmonth.$ecday, $transday);
		$daysenrolled = $dayenrolled + $daysbetween;

		$active = 0; # since this was a withdrawal.

	    } else { # must have been an enrollment.
		$active = 1; # they're now in school;
	    }
	    ($dud1,$dud2,$ecday) = split /-/, $transday; # update enrollment change day.

	} # End of Loop reading enrollment change (ie.transfer) records

	if ($active){ 
	    # figure out days from here to end of month and add on to
	    # $daysenrolled
	    my ($yr,$mo) = split /-/,$yearmonth;
	    my ($nextyr, $nextmo);

	    # Find next month values
	    if ($mo == 12){
		$nextmo = 1; $nextyr = $yr + 1;
	    } else {
		$nextmo = $mo + 1; $nextyr = $yr;
	    }
	    if (length($nextmo) == 1){ $nextmo = '0'.$nextmo;}
	    my $nextmonthstart = $nextyr.$nextmo.'01';

	    my $nextmonthstartobj = new Date::Business(DATE=>$nextmonthstart);
	    $nextmonthstartobj->prev(); # backup to end of prev month.
	    my $monthenddate = $nextmonthstartobj->image();

	    if (length($ecday) == 1){ $ecday = '0'.$ecday;}
	    my $ecdate = $tmpyearmonth.$ecday;
	    
	    my $daysbetween = &calcDaysBetween($ecdate,$monthenddate);
	    #print "ST: $ecdate  END: $monthenddate BET: $daysbetween<br>\n";

	    $daysenrolled = $dayenrolled + $daysbetween;

	}
    }

    return "$daysenrolled:$active";

} # End of calcMonthlyEnrollment 



#------------------------
sub calcMonthlyAttendance { # calculate absences,lates for 1 month for 1 child
#------------------------
    my ($studnum, $yearmonth, $periodsperday) = @_;

    # yearmonth will have yyyy-mm format
    $yearmonth =~ s/-//; # remove single dash

    my $sth = $dbh->prepare("select * from attend 
     where studentid='$studnum' and 
     extract(year_month from absdate) = '$yearmonth'
     order by absdate,period");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
    my $rows = $sth->rows;
    #print "CalcMonthlyAttendance: SN: $studnum YM: $yearmonth Rows: $rows<br>\n";

    my $absent = 0; my $late = 0;
    while (@absence = $sth->fetchrow){
	if ($absence[3] =~ /Absent/) { $absent++; }
	if ($absence[3] =~ /Late/) { $late++; }
    }
    $absent = $absent/$periodsperday; #convert periods into days
    return "$absent:$late";
}


#---------------
sub calcDaysBetween {
#---------------
    # this calculates the number of schooldays in a date range
    #  taking school holidays into account.
    # Requires: $dbh, Date::Business

    my ($startdate, $enddate) = @_;

    # It likes it in yyyymmdd format...
    $startdate =~ s/-//g;   # strip out hyphens.
    $enddate =~ s/-//g;

    my $end = new Date::Business(DATE=>$enddate);
    my $start = new Date::Business(DATE=>$startdate);
    my $schooldays = $end->diffb($start,'prev','next');
    $schooldays++;
    # We now have the number of school days in current term
    # excluding holidays/pd days.

    # Now find holidays
    my $sth = $dbh->prepare("select * from dates where 
      to_days(date) >= to_days('$startdate') and 
      to_days(date) <= to_days('$enddate')");
    $sth->execute;
    if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr;}
    my $holidays = $sth->rows;
    # Number of holidays in current term.

    my $tmpdays = $schooldays - $holidays;
    return $tmpdays;
}

#-----------------
sub calcDateAttendance { # calc attendance between two dates, inclusive.
#-----------------
    my ($studnum, $startdate, $enddate, $periodsperday) = @_;
    my $sth = $dbh->prepare("select * from attend 
     where studentid='$studnum' and 
     to_days(absdate) >= to_days('$startdate') and 
     to_days(absdate) <= to_days('$enddate') 
     order by absdate,period");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die;}

    my $absent = 0; my $late = 0;
    while (@absence = $sth->fetchrow){
	if ($absence[3] =~ /Absent/) { $absent++; }
	if ($absence[3] =~ /Late/) { $late++; }
    }
    $absent = $absent/$periodsperday; #convert periods into days
    return "$absent:$late";
}


#-------------------------
sub findEnrollmentMonths {
#-------------------------
    # Globals: $dbh, $currdate, @schooldays (from mkSchoolDays fn)

    # Passed Student Number (studnum) and active status, start and end date.
    # Both dates in normal yyyy-mm-dd format.
    # Active means enrolled as of start date.

    my ($studnum, $active, $startdate, $enddate) = @_;


    # Read enrollment changes from the transfer table.
    # Assume the $dbh var already set...
    my $sth = $dbh->prepare("select * from transfer 
      where studnum = '$studnum'
      and to_days(date) > to_days('$startdate')
      order by date desc");
    # Problem if two records of same student on same day... an entry error.

    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
    #my $rows = $sth->rows;
  
    my @enrollment;

    # Read the first record and set the status.
    my @transrec = $sth->fetchrow;

    if (not $transrec[0]){ # no transfer records for this student
	if ($active){ # registered but no transfer records;
	    my $hashref = {};
	    $hashref->{start} = $schoolstart;
	    $hashref->{end} = $currdate; 
	    push @enrollment, $hashref;
	}
	return @enrollment;  # return undefined if not active; 
	# return full year if active.
    }


    my $status = '';
    if ($transrec[3] eq 'enrol' or $transrec[3] eq 're-enrol'){ 
	# Set $status and push first record.(hashref in array)
	$status = 'enrol';
	my $hashref = {};
	$hashref->{start} = $transrec[2];
	$hashref->{end} = $currdate; 
	push @enrollment, $hashref;
	# @enrollment is an array of hashrefs to the enrollment blocks.
    } else { 
	$status = 'withdraw'; $statusdate = $transrec[2];
    }

    while (@transrec = $sth->fetchrow){
	if ($status eq 'enrol' and $transrec[3] eq 'withdraw'){
	    # change status
	    $status = 'withdraw';
	    $statusdate = $transrec[2];
	} elsif ($status eq 'withdraw' and 
		 ($transrec[3] eq 'enrol' or $transrec[3] eq 're-enrol')){
	     $status = 'enrol';
	     my $hashref = {};
	     $hashref->{start} = $transrec[2];
	     $hashref->{end} = $statusdate; 
	     push @enrollment, $hashref;
	 } else { # Error!
	     my $sth1 = $dbh->prepare("select lastname,firstname 
              from student where studnum = '$studnum'");
	     $sth1->execute;
	     if ($DBI::errstr){ print $DBI::errstr; die;}
	     my ($lastname,$firstname) = $sth1->fetchrow;
	     print "<br>Enrollment Error for student:";
	     print "<b>$firstname $lastname ($studnum)</b>.<br>\n";
	     print "Transfer records are not consistent. ";
	     print " Please repair on main page under<br>";
	     print "<i>View/Edit Transfers</i>.\n";
	     #die "Enrollment error for $studnum";
	 }

    } # End of While. 

    return @enrollment;

} # End of findEnrollmentMonths.


#-------------------------
sub findEnrollmentBlocks {
#-------------------------
    # Return a list of hash pointers to enrollment blocks.
    # Globals: $dbh, $currdate, $schoolstart (from admin.conf)

    # move this later to global config for multi-lang support
    my $enroltext = 'enrol';
    my $reenroltext = 're-enrol';
    my $withdrawtext = 'withdraw';


    # Passed Student Number (studnum) and active status.
    my $studnum = shift;
    my $active = shift;
    my $currdate = shift;


    # Read enrollment changes from the transfer table.
    # Assume the $dbh var already set...
    my $sth = $dbh->prepare("select id, date, type from transfer 
      where studnum = '$studnum'
      and to_days(date) >= to_days('$schoolstart')
      order by date, type desc");
    # Problem if two records of same student on same day... an entry error.

    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}

    my @enrollment;

    # Read the first record...
    my ($id, $date,$type) = $sth->fetchrow;

    if (not $id){ # no transfer records for this student
	if ($active){ # registered but no transfer records;
	    my $hashref = {};
	    $hashref->{start} = $schoolstart;
	    $hashref->{end} = $currdate; 
	    push @enrollment, $hashref;
	}
	return @enrollment;  # return undefined if not active; 
	# return year-to-date if active.
    }

    my $status = '';
    if ($type eq $enroltext or $type eq $reenroltext){ 
	# Set $status only
	$status = 'enrol'; 
	$statusdate = $date;
    } else { # we have a withdrawal
	# update status and push block.
	$status = 'withdraw'; $statusdate = $date;
	my $hashref = {};
	$hashref->{start} = $schoolstart;
	$hashref->{end} = $date; 
	push @enrollment, $hashref;
    }


    # now do the rest of the records.
    while (my ($id, $date,$type) = $sth->fetchrow){
	if ($type eq 'withdraw' and $status eq 'enrol'){
	    # reset status
	    $status = 'withdraw';
	    $statusdate = $date;
	    # push block dates
	    my $hashref = {};
	    $hashref->{start} = $statusdate;
	    $hashref->{end} = $date; 
	    push @enrollment, $hashref;

	} elsif ( ($type eq 'enrol' or $type eq 're-enrol') and $status eq 'withdraw'){
	     $status = 'enrol';
	     $statusdate = $date;
	 } else { # Error!
	     print "<br><br>Enrollment Error for student: $studnum.\n";
	     print "Records are not consistent. Please repair.<br>\n";
	     print "Start: $status  StartDate: $statusdate<br>\n";
	     print "End: $type  EndDate: $date<br>\n";
	     #die "Enrollment error for $studnum";
	 }

    } # End of While.

    if ($status eq 'enrol'){ # must still be enrolled...
	# push block dates
	my $hashref = {};
	$hashref->{start} = $statusdate;
	$hashref->{end} = $currdate; 
	push @enrollment, $hashref;
    }


    # Show Result:
    #foreach my $bl (@enrollment){
    #print "SN: $studnum ST: $bl->{start}";
    #	print "END: $bl->{end}<br>\n";
    #}
 
    return @enrollment;

} # End of Sub



# Return true.
1;
