#!/usr/bin/perl
#  Copyright 2001-2008 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# schedteacher - display teacher timetables. If passed teacher userid,
# display only that teacher's timetable.

my $blank = 'Unscheduled';
my $colorCode = '#049';

my %lex = ('View Teacher Timetable' => 'View Teacher Timetable',
	   'Term' => 'Term',
	   'Period' => 'Period',
	   'Error' => 'Error',
	   'Day' => 'Day',

	   );

my $self = 'tchschedview.pl';

use DBI;
use CGI;
use Cwd;

my $q = new CGI;

# Set mode
my $mode;
my $currdir = getcwd();
my @dirs = split /\//,$currdir;
if ($dirs[-1] eq 'dbook') { $mode = 'popup';}
#print "Mode: $mode\n";


# Http Header to return
print $q->header( -charset, $charset );
my %arr = $q->Vars;
my $select; 

if ( $arr{userid} ){
    $select = " where subject.teacher like '%($arr{userid}%' ";
}

if ($arr{term}){
    if ( $arr{userid} ) {
	$select .= " and schedat.term = '$arr{term}' ";
    } else {
	$select = " where schedat.term = '$arr{term}' ";
    }
}

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

$dsn = "DBI:$dbtype:dbname=$dbase";
$dbh = DBI->connect($dsn,$user,$password);

if ($mode eq 'popup') { $stylesheet = $tchcss; } else { $stylesheet = $css;}

# print page header.
print "$doctype\n<html><head><title>". $lex{'View Teacher Timetable'}. "</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n";
if ( $mode eq 'popup' ) {
    print "<style type=\"text/css\"> th { background-color:$colorCode; color:white; } \n";
    #print "span.c8211-1 { background-color:green; }\n";
    print "</style>\n";
}

print "<script type=\"text/javascript\">\n";
print "function setbackground(classval) {
  classval = 'c' + classval; // add c prefix
  var spans = document.getElementsByTagName('span');
  for(var i=0;i<spans.length;i++) {
     //alert('The value is' + spans[i].class);
     if ( spans[i].className == classval ) {
        spans[i].style.backgroundColor = '$colorCode';
        spans[i].style.color = 'white';
        spans[i].style.display = 'block';
     } else {
        spans[i].style.color = 'black';
        spans[i].style.backgroundColor = 'white';
     }
  }
}
</script>\n";


print "</head><body>\n";
# onload=\"setbackground('8213-1')\">\n";

if ($mode ne 'popup') { 
    print "[ <a href=\"$schpage\">$schedulepage</a> ]\n"; 
}

#print "Select is: $select<br>\n";

# Find Numbers of Rows (periods) and Columns(Days) in Cycle
$sth = $dbh->prepare("select max(period) from schedat");
$sth->execute;
if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr;}
$tblrows = $sth->fetchrow;

$sth = $dbh->prepare("select max(day) from schedat");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
$tblcols = $sth->fetchrow;


$sth = $dbh->prepare("select distinct schedat.subjsec, subject.description, 
  subject.teacher, schedat.term from schedat
  left outer join subject on schedat.subjsec = subject.subjsec
   $select order by subject.teacher, schedat.term");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}

my $oldterm = -1; my $oldtch = -1;
my $firstflag = 1;

while (my ($subjsec,$desc,$teacher,$term) = $sth->fetchrow){
    #print "Subj: $subjsec Tch: $teacher $term<br>\n";
    $oldterm = $currterm;  $currterm = $term;
    $oldtch = $currtch;    $currtch = $teacher;

    if (($currterm ne $oldterm or $currtch ne $oldtch) and not $firstflag){ 
          # change in teacher or term

	# run printTimetable with $teacher, $term, and @subjects
	printTimetable($oldtch, $oldterm, \@subjects);

	@subjects = (); # empty array.
	push @subjects,$subjsec; # start new array with existing value.
    } else {
	push @subjects, $subjsec;
	$firstflag = 0;
    }

}

# print out the last table.
printTimetable($currtch, $currterm, \@subjects,$mode);

print "</body></html>\n";


#-----------------
sub printTimetable { # print timetable for one teacher for 1 term.
#-----------------

    my ($teacher, $term, $subjref,$mode) = @_; 

    my ($name,$userid) = split /\(/,$teacher;
    my ($lastname, $firstname) = split /,/,$name;
    if ($mode ne 'popup') {
	print "<p style=\"font-size: 150%;\"><b>$firstname $lastname</b>\n";
	print q{ }. $lex{Term}. " $term</p>\n";
    } else { 
	print "<b>$firstname $lastname - ". $lex{Term}. " $term<br>\n"; 
    }

    my @subject;
    foreach my $subjsec (@{$subjref}){
	my $sth = $dbh->prepare("select day, period from schedat 
         where term = '$term' and subjsec = '$subjsec' order by period, day");
	$sth->execute;
	while (my ($day, $period) = $sth->fetchrow){
	    if ($subject[$period][$day] and $subject[$period][$day] ne $subjsec){ 
             # if a value exists and is not identical to subjsec then add to it.
		$subject[$period][$day] .= " $subjsec";
	    } else { # just slap it in...
		$subject[$period][$day] = $subjsec;
	    }
	}
    }
    # Done filling 2D array.


    my $sth = $dbh->prepare("select description from subject where subjsec = ?");

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    # heading
    print "<tr><th></th>";
    for ( 1..$tblcols ) { print "<th>". $lex{Day}. " $_</th>"; }
    print "</tr>\n";
    
    # Main body of table.
    for my $i (1..$tblrows){
	print "<tr><td>". $lex{Period}. " $i</td>\n";
	for (1..$tblcols){
	    if ($subject[$i][$_]){ # if we have a subjsec value;
		my $firstflag = 1;
		my @subjval = split /\s/,$subject[$i][$_];
		print "<td>";
		foreach my $sv ( @subjval ) {
		    if (not $firstflag){ 
			print "<br>";
		    } else {
			$firstflag = 0;
		    }
		    $sth->execute($sv);
		    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}	    
		    my $desc = $sth->fetchrow;
		    my $cls = 'c'. $sv;

		    # We are using span elements, since there may be
		    # more than 1 subject inside each period cell; not as pretty, though.

		    print "<span class=\"$cls\" onMouseOver=\"setbackground('$sv')\">$desc ($sv)</span>";
		}
		print "</td>\n";
	    } else {
		print "<td style=\"color:green;\">$blank</td>";
	    }
	}
	print "</tr>\n";
    }
    print "</table>\n";
}
