#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# export.pl - export values to a CSV file format for download by teacher.
#   for a single subject only.
#  New Feature: compress all subjects into a zip for the teacher as a
#   backup.

my $tempdir = '/tmp'; 
# temporary working area when creating zip for complete backup.

my %lex = ('Export Grades' => 'Export Grades',
	   'Main' => 'Main',
	   'Please select a subject' => 'Please select a subject',
	   'Export' => 'Export',
	   'Combine failed on input' => 'Combine failed on input',
	   'Test Weight' => 'Test Weight',
	   'Export all subjects of this teacher' => 'Export all subjects of this teacher',
	   'and compress into a single zip for backup' => 
	     'and compress into a single zip for backup',
	   'Select Subject' => 'Select Subject',
	   'Download Grades for' => 'Download Grades for',
	   'Download Zip of all Marks' => 'Download Zip of all Marks',
	   'Error' => 'Error',
	   'Please Log In' => 'Please Log In',

	   );

use DBI;
use CGI;
use CGI::Session;
use Text::CSV_XS;

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}

my $q = CGI->new;

my $csv = Text::CSV_XS->new();

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Get Session Information...
my $session = new CGI::Session("driver:mysql;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

my $logged_in = $session->param(logged_in);
if (not $logged_in){
    print $q->header( -charset, $charset );
    print $lex{'Please Log In'}. "!<br>\n";
    die;
}

my $subjsec = $session->param('subjsec');

my ($sec, $min, $hour, $mday, $mon, $year, $wday, 
 $yday, $iddst) = localtime(time);
$mon++;
$wday++;
$year = $year + 1900;
my $currsdate = "$year-$mon-$mday";
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";

print $q->header( -charset, $charset );
%arr = $q->Vars; # Get passed values


# Get the Subject Information
$sth = $dbh->prepare("select * from subject where subjsec = ?");
$sth->execute( $subjsec );
if ($DBI::errstr) { print "$DBI::errstr"; die $DBI::errstr; }
my @subject = $sth->fetchrow;


# Get the other subject info; stuff into 'subjects' array.
$sth = $dbh->prepare("select description,smdesc,subjsec from subject 
  where teacher like '%$subject[4]%' order by description");
$sth->execute;
if ($DBI::errstr) { print "$DBI::errstr"; die $DBI::errstr; }
while (my ( $description,$smdesc,$subjsec ) = $sth->fetchrow){
    push @subjects,"$smdesc ($subjsec)";
}


# Print Page Header
print "$doctype\n<html><head><title>". $lex{'Export Grades'};
print "</title>\n<link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\">
$chartype\n</head><body>\n";

print "<form action=\"export.pl\" method=\"post\">\n";

print "<table><tr><td>[ <a href=\"$tchpage\">". $lex{Main}. "</a> |</td>
<td><a href=\"gbmain.pl\">GB&nbsp;Main</a> ]</td>\n";
print "<td style=\"letter-spacing:0.1em;font-size:150%;padding:0 1em;";
print "color:#063;\">". $lex{Export}. " - $subject[2]</td></tr>\n";

print "<tr><td colspan=\"3\">
<input type=\"submit\" value=\"". $lex{'Select Subject'}. "\"><select name=\"subjsec\">\n";
foreach my $subj ( @subjects ){
    print "<option>$subj</option>\n";
}
print "</select>\n";

print "<a href=\"export.pl?zip=1\" class=\"button\" title=\"";
print $lex{'Export all subjects of this teacher'}; 
print " ". $lex{'and compress into a single zip for backup'}. ".\">Zip All</a></td></tr>\n";
print "</table></form>\n";


if ($arr{zip}){ # We want a complete zip.
    exportZip(@subjects);
    die;
}

# In case of no subject selected, halt here.
if (not $arr{subjsec}){
    print '<p><b>'. $lex{'Please select a subject'}. ".</b></p></body></html>\n";
    die;
}

# Open output file
my $filename = "grades$$.csv";

open (EX,">$filename") || die "Can't open Export file $fileName";
exportSubjsec($subjsec); # write to the exported file.
close EX;

system("mv $filename $tchdownloaddir");

print "<p><a href=\"$tchwebdownloaddir/$filename\">
 <span style=\"font-size:160%;\">". $lex{'Download Grades for'};
print " $subject[2]</span></a></p>\n";
print "</center></body></html>\n";


# ==== Functions =========================

#----------------
sub exportSubjsec { # build export file for single subject
#----------------
    my $subjsec = shift;  # passed subjsec
    my @test;
    my @studname;

    # Get the Subject Information
    my $sth = $dbh->prepare("select * from subject where subjsec = ?");
    $sth->execute( $subjsec );;
    if ($DBI::errstr) { print "$DBI::errstr"; die $DBI::errstr; }
    my @subject = $sth->fetchrow;

    # Find the enrollments for this class and read them into @studnum array
    $sth = $dbh->prepare("select distinct eval.studnum, studentall.lastname, 
     studentall.firstname from eval
     left outer join studentall on studentall.studnum = eval.studnum 
     where eval.subjcode = ? order by studentall.lastname, studentall.firstname");

    $sth->execute( $subjsec );
    if ($DBI::errstr) { print "$DBI::errstr"; die;}
    while (my ($studnum, $lastname,$firstname) = $sth->fetchrow){
	push @studname,"$lastname, $firstname ($studnum)";
    }
    # End of adding student names.


    # lookup tests for this subjsec
    $sth = $dbh->prepare("select name, id, score, weight from gbtest 
     where subjsec = ? order by tdate");
    $sth->execute( $subjsec );
    if ($DBI::errstr) { print "$DBI::errstr"; die;}
    while (my ($name,$id,$score,$weight) = $sth->fetchrow){
	push @test, "$name:$id:$score";
	$weight{$id} = "$weight"; # used for calculating averages.
    }
    # Now done putting in the tests.


    # print the test names 
    push @labels,'Tests';
    foreach my $test (@test){
	my ($name,$id,$score) = split /:/,$test;
	push @labels,$name;
    }

    if ($csv->combine(@labels)) {
	my $record = $csv->string;
	print EX $record, "\r\n";
    } else {
	my $err = $csv->error_input;
	print "Combine failed on input: ",$err,":\n\n";
    }
    # End of Test Names

    @labels = (); # zero array


    # Print the test max scores
    push @labels,'Max Scores';
    foreach my $test (@test){
	my ($name,$id,$score) = split /:/,$test;
	push @labels,$score;
    }

    if ($csv->combine(@labels)) {
	my $record = $csv->string;
	print EX $record, "\r\n";
    } else {
	my $err = $csv->error_input;
	print "Combine failed on input: ",$err,":\n\n";
    }


    # Print the students and their marks;
    foreach my $studname (@studname) {
	my @labels;
	push @labels,$studname;
	
	foreach my $test (@test){
	    ($dudname,$studnum) = split /\(/,$studname;
	    chop $studnum; # remove trailing parens
	    my ($name,$id,$maxscore) = split /:/,$test;

	    $sth = $dbh->prepare("select score from gbscore 
              where testid = ? and studnum = ?");
	    $sth->execute( $id, $studnum );
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

	    # Testing Code for errors; there shouldn't be more than 1 record
	    $row = $sth->rows;
	    if ($row > 1){ 
		print "</table><b>Error: More than 1 score for this test for:";
		print "$studname for test $test!</b></body></html>"; die;
	    }

	    my $score = $sth->fetchrow;
	    push @labels, $score;
	} # End of test loop
   
	if ($csv->combine(@labels)) {
	    my $record = $csv->string;
	    print EX $record, "\r\n";
	} else {
	    my $err = $csv->error_input;
	    print $lex{'Combine failed on input'}. ": ",$err,":\n\n";
	}
    }
    # End of printing all Student Marks

    @labels = ();

    # Print the Test Weighting Row
    push @labels, $lex{'Test Weight'};
    foreach my $test (@test){
	my ($name,$id,$score) = split /:/,$test;
	push @labels,$weight{$id};
    }

    if ($csv->combine(@labels)) {
	my $record = $csv->string;
	print EX $record, "\r\n";
    } else {
	my $err = $csv->error_input;
	print $lex{'Combine failed on input'}. ": ",$err,":\n\n";
    }


}


#------------
sub exportZip { # Export ALL subjects of this teacher compressed into a zip.
#------------
    @subjects = @_;

    # Create a folder to do the zip in.
    my $tempdir = "tempzip$$";
    my $zipfile = "grades$currsdate-$$.zip";
    mkdir $tempdir;

    foreach my $subj (@subjects){
	my ($desc,$sub) = split /\(/,$subj;
	chop $sub;
	$desc =~ s/\s//g; # strip space.

	# Open output file
	$filename = "$tempdir/$desc.csv";
	open (EX,">$filename") || die "Can't open Export file $fileName";
	exportSubjsec($sub); # write to the exported file.
	close EX;
    }

    system("zip $zipfile $tempdir/* >/dev/null");
    system("mv $zipfile $tchdownloaddir");
    system("rm -f $tempdir/*");
    rmdir $tempdir;

    print "<center><p><a href=\"$tchwebdownloaddir/$zipfile\">";
    print "<span style=\"font-size:160%;\">". $lex{'Download Zip of all Marks'}. "</span>\n";
    print "</a></p>\n</center></body></html>\n";

    return;

}
