#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my $self = 'gbframe.pl';

# Passed Values:
#  subjsec - Subject/Section value 
#  dstart,dend - DateStart, DateEnd - for item selection
#  cellvw - Cellview: Percent, Score, Both (p,s,b)
#  grp - Group, Subgroup - for item selection
#  selgr - Groupview (Yes/No=1/0) - view group averages for all students. (also
#   item count for that group.)


# change right side values for alternate languages, let me know of any mistakes
my %lex = ('All' => 'All',
	   'Error: More than 1 score for this test for:' => 
	     'Error: More than 1 score for this test for:',
	   'Edit' => 'Edit',
	   'NA' => 'NA',
	   'No Tests Found!' => 'No Tests Found!',
	   'Subject' => 'Subject',
	   'Date Start' => 'Date Start',
	   'Date End' => 'Date End',
	   'Group Select' => 'Group Select',
	   'Gradebook Assessment View' => 'Gradebook Assessment View',
	   'Error' => 'Error',

	);


my $version = '2009-01-02';
my $specchar = '*';  # causes this item to be skipped in calculating avg.
my $altcolor = '#3B7'; #1B6 Complement to the 063 green of teacher site.
my $maincolor = '#063'; 

use DBI;
use CGI::Session;
use CGI qw/escape unescape/;
use Number::Format qw(round);

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}

# Get defaultItemWeight for a group.
eval require "../../etc/gbook.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}


my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

my $q = CGI->new;
print $q->header( -charset, $charset );
my %arr = $q->Vars; # Get passed values

#foreach my $key (keys %arr) { print "K: $key V: $arr{$key}<br>\n"; }

# Passed values
my $subjsec = $arr{subjsec};
delete $arr{subjsec};

my $datestart = $arr{dstart};
delete $arr{dstart};

my $dateend = $arr{dend};
delete $arr{dend};

my $grp = unescape($arr{grp});
delete $arr{grp};

my $cellview = $arr{cellvw};
# Val: b=both, p=percent, s=score;
delete $arr{cellvw};


# Creates array controlling student sort order.
my @studnum = mkStudNum($subjsec);

if ( $datestart ) {
    $dateStartSelect = "and to_days(tdate) > to_days('$datestart')";
}
if ( $dateend ) {
    $dateEndSelect = "and to_days(tdate) < to_days('$dateend')";
}

my $groupSelect;
if ( $grp ) {
    $groupSelect = "and grp = '$grp'";
}

# Now lookup tests for this class.
$sth = $dbh->prepare("select name, id, score, description, grp, tdate 
 from gbtest where subjsec = ? $dateStartSelect $dateEndSelect $groupSelect 
 order by tdate desc");
$sth->execute( $subjsec );
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

# Check if any tests...
{ my $rows = $sth->rows; 
  if ($rows < 1) { 
      print "<b>". $lex{'No Tests Found!'}. "</b><br>\n";
      print $lex{Subject}. ": $subjsec<br>\n";
      print $lex{'Date Start'}. ": $datestart<br>\n";
      print $lex{'Date End'}. ": $dateend<br>\n";
      print $lex{'Group Select'}. ": $groupSelect<br>\n";
      print "</body></html>\n";
      die;
  }
}

while ( my ( $name,$id,$score, $desc,$grp,$tdate) = $sth->fetchrow ) {
    # skip the 'sortorder' test
    if ($name eq 'sortorder'){ next; }
    my $sg;
    ($tgrp,$sg) = split /:/,$grp;
    $name =~ s/://g; # strip colons, if any.
    push @test, "$name:$id:$score:$desc:$grp:$sg:$tdate";
}
# Now done putting in the tests.

# Now get distinct groups for colors; populate %grpcolor;
my $colornumber = 0;
$sth = $dbh->prepare("select distinct grp from gbtest 
 where subjsec = ? and name != 'sortorder' order by grp"); 
$sth->execute( $subjsec );
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
while ( my $grp = $sth->fetchrow ) {
    $grpcolor{$grp} = $colornumber;
    $colornumber++;
}


# Now lookup ALL tests to find $totalweight and %weight.
$sth = $dbh->prepare("select id,name, weight from gbtest 
  where subjsec = ?"); # do ALL tests (assess items)
$sth->execute( $subjsec );
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
my $totalweight; # used further down for average / weight...
while ( my ( $id,$name,$wt ) = $sth->fetchrow ) {
    # skip the 'sortorder' test
    if ( $name eq 'sortorder' ){ next; }

    $weight{$id} = $wt; # used for calculating averages.
    $totalweight += $wt;
}
# Now done putting in the weights (totalweight and weight hash)


# Find the groups for this subject, and push into a group array
#  Also create a $groupref to point to subgroups of a top group.
#  ie. $groupref{term1} -> test,homework,labs as $groupref{term1}[0] = test
$sth = $dbh->prepare("select distinct grp 
  from gbtest where subjsec = ? order by grp");
$sth->execute( $subjsec );
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}

my $currgroup = '-1';
my $prevgroup;
my @group;

while (my $grp = $sth->fetchrow){
    if ( not $grp ){ next; }
    my ( $topgroup,$subgroup ) = split /:/,$grp;

    $prevgroup = $currgroup;
    $currgroup = $topgroup;

    if ($subgroup){ # push into hash of array elements
	push @{$groupref{$topgroup}},$subgroup;
    }
    if ($prevgroup ne $currgroup){
	push @group, $topgroup; 
    }
}


# Load groups or subgroup and associate colors with them;
my %groupcolor;
if ($view eq $lex{All} or 
    ($view ne $lex{All} and not $groupref{$subview}) ){ 
    # all groups; or no subs 
    for (0..$#group){ $groupcolor{$group[$_]} = $g_color[$_]; }
} else {
    for (0..$#{$groupref{$view}}){ 
	$groupcolor{${$groupref{$view}}[$_]} = $g_color[$_];
    }
}


# Heading of Document
print "$doctype\n<html><head><title>". $lex{'Gradebook Assessment View'}. "</title>\n";
print "<link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\">\n";
print "<style type=\"text/css\">body { padding:0; margin:0;}
A { color: white; }
</style>
</head><body>\n";

# Print the table header
print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n"; 


# Print Test Header Section
print "<tr style=\"text-align:center;background-color:$maincolor;\">";
print "<td>&nbsp;</td>\n";

foreach my $test ( @test ) {
    my ( $name, $id, $score, $desc, $grp, $sg, $tdate ) = split /:/, $test;
    $name =~ s/^\s+//g;
    if ( length($name) > 6 ){ 
        $sname = substr($name,0,6);
    } else { 
        $sname = $name; 
    }

    print "<td><a href=\"tested.pl?id=$id\" ";
    print "title=\"$name: $desc ($grp) $tdate\" target=\"_top\">$sname</a>";
    print "</td>\n";

}

print "</tr>\n";
# End of Test Header


# Print the Edit Rows for the tests.
print "<tr valign=\"middle\" style=\"background-color:$altcolor;\"><td>&nbsp;</td>\n";
foreach my $test (@test) {
    my ($name, $id, $score, $desc, $grp, $sg, $tdate) = split /:/, $test;
    print "<td style=\"text-align:center;";
    if ($view eq $lex{All} or ( $subview eq $lex{All} and not $groupref{$view}) ) { 
	print "background-color: $g_color[$grpcolor{$grp}];\"> \n";
    } else {
	print "background-color: $g_color[$grpcolor{$grp}]; $subview\">\n";
    }
print "<a href=\"scoreadd.pl?id=$id\" target=\"_top\" ";
print "title=\"$desc ($grp) $tdate (x/$score)\">$lex{Edit}</a></td>\n";

} # End of Test Loop

print "</tr>\n";
# End of Edit Rows


# Prep for name retrieval
my $sth1 = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");

# Print the students and their marks;
foreach my $studnum ( @studnum ) {

    # Get Student Name
    $sth1->execute( $studnum );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my ($lastname, $firstname) = $sth1->fetchrow;

    my $initials = substr($lastname,0,3).substr($firstname,0,1);
    print "<tr><td>$initials";
    #if ($cellview eq 'b' or not $cellview){ 
    #	print "<br>"; 
    #   } else { 
    #	print "&nbsp;"; 
    #   }
    #print "$studnum</td>\n";
    print "</td>\n";

    my ($totalscore,$testcount); # totalscore for these tests, count of tests.
    foreach my $test (@test){
	my ($name,$id,$maxscore,$desc,$grp,$sg,$tdate) = split /:/,$test;
	$sth = $dbh->prepare("select id,score from gbscore 
             where testid = ? and studnum = ?");
	$sth->execute($id, $studnum);
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

	# Testing Code for errors; there shouldn't be more than 1 record
	$row = $sth->rows;
	if ($row > 1){
	    print "</table><b>";
	    print $lex{'Error: More than 1 score for this test for:'};
	    print "$firstname $lastname  $test!</b></body></html>\n"; 
	    die "More than 1 record for test $test\n";
	}

	my ($dud,$score) = $sth->fetchrow;
	my $testaverage;

	if ($score =~ /\d/){ #if score is a digit...
	    $totalscore += ($weight{$id} * $score / $maxscore);
	    $testaverage = round( $score / $maxscore * 100, 1);

	    print "<td align=\"center\" ";
	    if ($testaverage > 100){ print "style=\"color:red;\" "; }
	    print '>';
	    if ($cellview eq 's'){
		print $score;
	    } elsif ($cellview eq 'p') {
		print "$testaverage%";
	    } else { # print them both...
		print "$score<br>($testaverage%)";
	    }
	    print "</td>\n";
	} elsif ($score eq $specchar) { #skip calcs for specialchar 
	    $testaverage = $lex{NA};
	    print "<td align=\"center\">$score&nbsp;(",$lex{NA};
	    print ")</td>\n";
	} elsif (not defined $score){
	    print "<td align=\"center\" ";
	    print "style=\"background-color:#E5E5E5;\"></td>\n";
	} elsif ($score) { # there IS a value, display it in red.
	    print "<td align=\"center\" style=\"color:red;\">";
	    print "<i>$score</i></td>\n";
	} else { # just print ...
	    print "<td>$score</td>\n";
	}

    } # End of test loop
    print "</tr>\n";

}
# End of printing Student Marks


# Print the Test Average Row;
print "<tr style=\"background-color: #CCC;\"><td>&nbsp;</td>\n";

foreach my $test ( @test ) {
    my ($name,$id,$score,$desc,$grp,$sg,$tdate) = split /:/,$test;
    # Get the student's scores
        my $totscore;
        my $scorecount;
        $sth = $dbh->prepare("select id, score from gbscore
 	 where testid = ? and score != '$specchar'");
        $sth->execute( $id );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
        while (my ($id,$addscore) = $sth->fetchrow){ # just throw away id
            if (not defined $addscore) { next; }
	    $totscore += $addscore;
	    $scorecount++;
	}
        if ( not $scorecount) { 
	    print "<td>&nbsp;</td>\n";
	    next; 
	}
	my $avgscore = $totscore / $scorecount;
        my $average = round( ($avgscore / $score * 100), 1);

	if ($avgscore){ # print average only if we have a score...
	    print "<td align=\"center\">$average%</td>\n";
	} else {
	    print "<td>&nbsp;</td>\n";
	}
} # End of Tests Loop
print "</tr>\n";
# End of Test Averages Row


# Print the Test Weighting Row

print "<tr style=\"background-color: $altcolor;\"><td>&nbsp;</td>\n";

# Note: $totalweight defined at start of script... it is ALL tests.
#  We are displaying the weight of the test against the total.
foreach my $test (@test){
    my ($name,$id,$score,$desc,$grp,$sg,$tdate) = split /:/, $test;
    my $wtmult;
    if ($defaultItemWeight) {
	$wtmult = round( ($weight{$id} / $defaultItemWeight), 1);
    }
    print "<td align=\"center\">$wtmult\X</td>\n";
}

#print "</tr>";
# End of Test Weight Row

print "</table>\n";
print "</body></html>\n";



#------------
sub mkStudNum {
#------------

    my $subjsec = shift;
    my (@eval, @studnum, %remove);

    # Create the studnum array that controls the order that students are
    # displayed in. Load the eval records first into an array, sorted by
    # lastname and firstname. (@eval) Load the sort order records in order
    # into an array and also into a hash (%remove)
    # Any remaining array elements in eval are added to the end of
    # the studnum array once done. If there _are_ no sortorder recs, then
    # the final array becomes the eval records only. This has the desired
    # behavior.


    # Find the enrollments for this class and read them into @eval array
    $sth = $dbh->prepare("select distinct eval.studnum from eval
     left outer join studentall on studentall.studnum = eval.studnum 
     where eval.subjcode = ? 
     order by studentall.lastname, studentall.firstname");

    $sth->execute( $subjsec );
    if ($DBI::errstr) { print "$DBI::errstr"; die $DBI::errstr;}
    while (my $studnum = $sth->fetchrow){
	push @eval,$studnum;
    }
    
    # Load 'SortOrder' test if it exists...
    $sth = $dbh->prepare("select id from gbtest where
     subjsec = ? and name = 'sortorder'");
    $sth->execute( $subjsec );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
    my $sortid = $sth->fetchrow;

    if ($sortid){ # We have a sortorder
	$sth = $dbh->prepare("select studnum from gbscore where
          testid = ? order by score");
	$sth->execute( $sortid );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
	while (my $sn = $sth->fetchrow){
	    push @studnum, $sn;
	    $remove{$sn} = 1;
	}
	
	# Now add on @eval elements who are not in @studnum
	foreach my $en (@eval){
	    if (not $remove{$en}){ push @studnum, $en;}
	}
	return @studnum;

    } else { # no sortorder
	return @eval;
    }
}


#----------------
sub checkPassword {
#----------------
    my ($userid, $password) = @_;
    
    if (not $userid){ return -1;}
    if (not $password){ return -2;}

    #check for presence of userid
    my $sth = $dbh->prepare("select count(userid) from staff where userid = ?");
    $sth->execute( $userid );
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
    my $count = $sth->fetchrow;
    if ($count < 1){ return -1;} # no userid

    #check for presence of correct password and userid
    my $sth = $dbh->prepare("select count(userid) from staff where userid = ? and passwd = ?");
    $sth->execute( $userid, $password );
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
    my $count = $sth->fetchrow;
    if ($count < 1){ return -2;} # not correct password

    return 0; # if all ok...

}


#------------------
sub checkCookieTime {
#------------------

    my ($duration) = @_;

    if ($duration) { 
	$cookietime = $duration;
    } else { 
	$cookietime = $defaulttime;
    }  

    $cookietime = $minimumtime if $cookietime < $minimumtime; 
    $cookietime = $maximumtime if $cookietime > $maximumtime; 
    $cookietime = "+".$cookietime."m"; # set format
    
    return $cookietime;

}
