#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# Passed Values: Subject/Section value (passed as subjsec). 

# change right side values for alternate languages, let me know of any mistakes
%lex = ('All' => 'All',
	'All Groups' => 'All Groups',
	'All Terms' => 'All Trm',
	'Please Log In' => 'Please Log In',
	'No User Id' => 'No User Id',
	'No Password' => 'No Password',
	'Please select a subject' => 'Please select a subject',
	'Add more assessment items' => 'Add more assessment items',
	'Add Item' => 'Add Item',
	'Delete/Edit existing assessment items' => 
	    'Delete/Edit existing assessment items',
	'Del/Ed Item' => 'Del/Ed Item',
	'Gradebook Reports Page' => 'Gradebook Reports Page',
	'Rpt' => 'Rpt',
	'Adjust the weights of individual assessment items' => 
 	  'Adjust the weights of individual assessment items',
	'Weight Item' => 'Weight Item',
	'Adjust the weightings of groups of assessment items' =>
	  'Adjust the weightings of groups of assessment items',
	'Weight Grp' => 'Weight Grp',
	'Post (copy) student averages to the report card system' =>
	  'Post (copy) student averages to the report card system',
	'Post' => 'Post',
	'Export Grades' => 'Export Grades',
	'Export these grades to a CSV file' => 
	  'Export these grades to a CSV file',
	'Exp' => 'Exp',
	'Maintenance' => 'Maintenance',
	'Maintenance: Sortorder, Withdraw, etc' => 
	'Maintenance: Sortorder, Withdraw, etc',
	'Mtce' => 'Mtce',
	'Edit multiple scores at the same time' =>
	  'Edit multiple scores at the same time',
	'MultiEdit' => 'MultiEdit&nbsp;',
	'No Maximum Score for Test' => 'No Maximum Score for Test',
	'Enter Gradebook' => 'Enter Gradebook',
	'Main' => 'Main',
	'Search' => 'Search',
	'Error' => 'Error',
	'Search: Scores, Averages' => 'Search: Scores, Averages',
	'Cookie Duration' => 'Cookie Duration',
	'User Id' => 'User Id',
	'Password' => 'Password',
	'Log Out' => 'Log Out',
	'Remove login cookies' => 'Remove login cookies',
	'min' => 'min',


	);

my $self = 'gbmain.pl';
my $version = '2009-01-02';
my $specchar = '*';  # causes this test to be skipped in calculating avg.

my $minimumtime = 1;  # Minimum time for cookie duration.
my $maximumtime = 40;  # Max cookie time.
my $defaulttime = 20;  # Default cookie time.

my $altcolor = '#3B7'; #1B6 Complement to the 063 green of teacher site.
my $maincolor = '#063'; 

use DBI;
use CGI::Session;
use CGI qw/:escape unescape/;
use Number::Format qw(round);

my $q = CGI->new;
my %arr = $q->Vars; # Get passed values

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}

eval require "../../etc/gbook.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}

# needed to $additionalcomments subject so that it can be skipped.
eval require "../../etc/repcard.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);


my ($sec, $min, $hour, $mday, $mon, $year, $wday, 
 $yday, $iddst) = localtime(time);
$year = $year + 1900;
$mon++;
$wday++;
$currdate = "$dow[$wday], $month[$mon] $mday, $year";


# Define Userid
my $userid;
if ($arr{userid}) {
    $userid = $arr{userid};
}

# define term. This will pass through session setup and be defined in
# session if it exists, and otherwise will be read from session. If it
# doesn't exist in the session, it will be set to the $lex{'All Terms'}
# value.
my $term;  # current term
if ($arr{term}) {
    $term = $arr{term};
}


# Define View and Subview
my $view; my $subview;

if ($arr{vw} and $arr{subview}){
    $view = $arr{vw}; 
    $subview = $arr{subview};
} elsif ($arr{vw}) { # only is defined, split it.
    my ($g,$sg) = split /:/,$arr{vw};
    if ($sg){ # vw contains a subgroup...
	$subview = $sg;
	$view = $g;
    } else { # no subgroup
	$view = $arr{vw};
	$subview = $lex{All};
    }
}

# Define $subjsec
my $subjsec;
if ($arr{subjsec}){
    $subjsec = $arr{subjsec};
}
# Cleanup subjsec when enclosed in parenthesis; drop leading text
# This is done when selecting new subject inside gradebook.
if ($subjsec =~ /\(/){
    ($dud,$subjsec) = split /\(/, $subjsec;
    chop $subjsec;
}


$session = new CGI::Session("driver:mysql;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

# Check for Logout
if ($arr{logout}){
    $session->delete;
    $session->flush;
    print $q->header( -charset, $charset );
    login( $lex{'Please Log In'} );
    die;
}


# Get/Set  Session Values (a defined userid means it was passed)
if ($userid){ # we want to login, passed userid/password pair.

    # Check password/userid against database (-1 no user, -2 wrong password;
    my $error = &checkPassword($arr{userid}, $arr{password});
    if ($error == -1){ print $q->header( -charset, $charset ); &login($lex{'No User Id'}); die;}
    if ($error == -2){ print $q->header( -charset, $charset ); &login($lex{'No Password'}); die; }

    $cookietime = checkCookieTime( $arr{duration} );

    # Set values for userid and logged_in in session
    $session->param('logged_in','1');
    $session->expire('logged_in',$cookietime);
    $session->param('userid',$arr{userid});
    $session->param('duration', $cookietime );

} else { # check logged_in value in session
    my $logged_in = $session->param(logged_in);
    if (not $logged_in){
        print $q->header( -charset, $charset );
        login( $lex{'Please Log In'} ); 
        die;
    }
    # Ok, we have a login. Values below we have in environment.

    $userid = $session->param('userid');
    $duration = $session->param('duration');

    if ( not ( $duration =~ m/\+/)) { # add + and m if not present
	$duration = checkCookieTime( $duration );
    }
    $session->expire('logged_in',$duration);

} # End of check for logged_in value


# Read or Write Session for views
if ($view){ # could be reset, if so update session.
    $session->param('view',$view);
} else { $view = $session->param('view');}

if ($subview){ 
    $session->param('subview',$subview);
} else { $subview = $session->param('subview');}


# Read/Write session for term
if ($term){ # if already defined we use that to set session value
    $session->param('term',$term); 
} else { # term not	defined. try to check from session.  
    my $tempterm = $session->param('term'); 
    if (not $tempterm) { # not in session either...  
	# Set session and term to all terms.
	$session->param('term',$lex{'All Terms'}); 
	$term = $lex{'All Terms'}; 
    } else { # we have a session value... use it;
	$term = $tempterm; 
    } 
} # $term is now defined from passed value, session, or to default allterms.


# Read/Write Session for subjsec
if ($subjsec){ 
    $session->param('subjsec',$subjsec);
} else { 
    $subjsec = $session->param('subjsec');
}

$session->flush;
print $session->header( -charset, $charset );


# Set the Dates for test selection by term.
my ($dstart,$dend); # Start and End dates
if ($term ne $lex{'All Terms'}) { # set selection dates
    my $trm = $term;
    $trm =~ s/\D//g; # strip non-digits cruft.
    $dstart = $g_termstart{$trm};
    $dend = $g_termend{$trm};
}


# Get the subject description information
$sth = $dbh->prepare("select description,smdesc from subject where subjsec = ?");
$sth->execute( $subjsec );
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
my ($description,$smdesc) = $sth->fetchrow;

# Get the other subjects desc and subjsec; stuff into 'subjects' array.
$sth = $dbh->prepare("select description,smdesc,subjsec from subject 
  where teacher like '%($userid)' order by description");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
while (my ($description,$smdesc,$subjsec) = $sth->fetchrow){
    # skip Additional Comments; loaded from repcard.conf file
    if ($description eq $additionalcomments) { next; } 
    if (not $smdesc) { $smdesc = substr($description,0,15); }
    push @subjects,"$smdesc ($subjsec)";
    push @bigsubjects, "$description ($subjsec)";
}

# Creates array controlling student sort order.
my @studnum = mkStudNum($subjsec);


# Now lookup tests for this class.
$sth = $dbh->prepare("select name, id, score, weight, description, grp 
  from gbtest where subjsec = ? order by tdate");
$sth->execute( $subjsec );
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
while (my ($name,$id,$score,$weight,$desc,$grp) = $sth->fetchrow){
    # skip the 'sortorder' test
    if ($name eq 'sortorder'){ next; }
    my $sg;
    ($grp,$sg) = split /:/,$grp;
    $name =~ s/://g; # strip colons, if any.
    push @test, "$name:$id:$score:$desc:$grp:$sg";
    $weight{$id} = $weight; # used for calculating averages.
    $maxscore{$id} = $score;
}
# Now done putting in the tests.

# Load the group weights from the current subject;
$sth = $dbh->prepare("select markscheme from subject where subjsec = ?");
$sth->execute( $subjsec );
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
my $markscheme = $sth->fetchrow;

# build a %groupweight hash from values in markscheme.
my @fields = split (/[\n|\r]/, $markscheme);
my %groupweight;
foreach my $fld (@fields) { 
    if ($fld) {
	#print "F:$fld<br>\n";
	my ($grp, $percent) = split /=/, $fld;
	$groupweight{$grp} = $percent;
    }
}

# Find the groups for this subject, and push into a group array
#  Also create a $groupref to point to subgroups of a top group.
#  ie. $groupref{term1} -> test,homework,labs as $groupref{term1}[0] = test
$sth = $dbh->prepare("select distinct grp 
  from gbtest where subjsec = ? and grp != '' order by grp");
$sth->execute( $subjsec );
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}

my $currgroup = '-1';
my $prevgroup;
my @group;

while (my $grp = $sth->fetchrow){
    if (not $grp){ next;}
    my ($topgroup,$subgroup) = split /:/,$grp;

    $prevgroup = $currgroup;
    $currgroup = $topgroup;

    if ($subgroup){ # push into hash of array elements
	push @{$groupref{$topgroup}},$subgroup;
    }
    if ($prevgroup ne $currgroup){
	push @group, $topgroup; 
    }
}


# Heading of Document
print "$doctype\n<html><head><title>Gradebook Main</title>
 <link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\" 
 media=\"screen\"><style type=\"text/css\" media=\"print\">
 body,html { font-family: Arial, Helvetica, sans-serif; font-size: 10pt;}
 </style></head><body>\n";


# Get Notify file, if any, and display.
{ open (FH,"notify.html"); undef $/; $notify = <FH>;}
if ($notify){ print $notify;}

# In case of no subject selected, halt here.
if (not $subjsec){
    print "[ <a href=\"$tchpage\">". $lex{Main}. "</a> ]<br>\n";
    print "<div style=\"padding:0 4em;\">\n";
    foreach my $subj (@bigsubjects){ # array with full descriptions
	my ($desc,$ss) = split /\(/,$subj;
	$ss =~ s/\)//;
	print "<p><a href=\"$self?subjsec=$ss\">$subj</a></p>\n";
    }
    print "</div><p><b>$lex{'Please select a subject'}</b></p>";
    print "</body></html>\n";
    die;
}

#foreach my $key (keys %arr) { print "K: $key V:$arr{$key}<br>\n"; }


# Selection Form for New Subject or Group/Subgroup
print "<table cellpadding=\"2\" cellspacing=\"0\" border=\"0\">\n";

print "<tr><td valign=\"top\">";

print "<form action=\"$tchpage\" method=\"get\" style=\"margin:0;padding:0;\">";
print "<input type=\"submit\" value=\"". $lex{Main};
print "\" style=\"padding:0;margin:0;\"></form></td>\n";

print "<td valign=\"top\">";
print "<form style=\"margin:0;padding:0;\" action=\"$self\" method=\"post\">\n";

print "<input type=\"submit\" ";
print "value=\"Select&nbsp;Subject|Group\"></td><td valign=\"top\">
 <select name=\"subjsec\">\n";

if ($smdesc){  # use small description, smdesc, if it exists...
    print "<option>$smdesc ($subjsec)</option>\n";
}
foreach my $subj (@subjects){
    print "<option>$subj</option>\n";
}
print "</select></td>\n";


# Print Group Options
print "<td valign=\"top\"><select name=\"vw\">";
if (defined $view){
    print "<option>$view</option><option>$lex{'All Groups'}</option>\n";
} else { # just put up all groups view...
    print "<option>$lex{'All Groups'}</option>\n";
}
foreach my $gp (@group){ 
    if ($gp eq $view) { next; }
    print "<option>$gp</option>";
}
print "</select></td>\n";


# Print Subgroup Options
if ($groupref{$view}) {
    print "<td valign=\"top\"><select name=\"subview\">\n";
    if ($subview ne $lex{All}){ print "<option>$subview</option>\n";}
    print "<option>$lex{All}</option>\n";
    foreach my $sg ( @{$groupref{$view}} ){ 
	print "<option>$sg</option>";
    }
    print "</select></td>\n";
}


# Print Term Options
print "<td valign=\"top\"><select name=\"term\">\n";
if ($term eq $lex{'All Terms'}) {
    print "<option>$lex{'All Terms'}</option>\n";
} else {
    print "<option>$term</option><option>$lex{'All Terms'}</option>\n";
}
#  Get the terms.
$sth = $dbh->prepare("select distinct term from eval where subjcode = ? order by term");
$sth->execute( $subjsec );
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
while (my $trm = $sth->fetchrow) {
    print "<option>Term $trm</option>";
}
print "</select></td>\n";


# Display Subject Name
print "<td align=\"top\" style=\"color:$maincolor;\">";
print "&nbsp;&nbsp;$description</td></tr></table></form>\n";


##### Print Methods Line ############
print "<div id=\"methods\" style=\"padding:4px;\">\n";

print "<a href=\"testadd.pl\" class=\"button\" 
 title=\"$lex{'Add more assessment items'}\">$lex{'Add Item'}</a>\n";

print "<a href=\"testdeled.pl\" class=\"button\" 
 title=\"$lex{'Delete/Edit existing assessment items'}\">
 $lex{'Del/Ed Item'}</a>\n";

print "<a href=\"rptmenu.pl\"
 class=\"button\"  title=\"$lex{'Gradebook Reports Page'}\">$lex{Rpt}</a>\n";

print "<a href=\"weightadj.pl\" class=\"button\" 
 title=\"$lex{'Adjust the weights of individual assessment items'}\">
 $lex{'Weight Item'}</a>\n";

print "<a href=\"weightgrp.pl\" class=\"button\" 
 title=\"$lex{'Adjust the weightings of groups of assessment items'}\">
 $lex{'Weight Grp'}</a>\n";

print "<a href=\"post.pl\" class=\"button\" 
 title=\"$lex{'Post (copy) student averages to the report card system'}\">
 $lex{Post}</a>\n";

print "<a href=\"export.pl\" class=\"button\" alt=\"$lex{'Export Grades'}\" 
 title=\"$lex{'Export these grades to a CSV file'}\">$lex{Exp}</a>\n";

print "<a href=\"mtcemenu.pl\" 
 class=\"button\" alt=\"$lex{Maintenance}\" 
 title=\"$lex{'Maintenance: Sortorder, Withdraw, etc'}\">$lex{Mtce}</a>\n";

print "<a href=\"search.pl\" 
 class=\"button\" alt=\"$lex{Search}\" 
 title=\"$lex{'Search: Scores, Averages'}\">$lex{Search}</a>\n";

print "</div>\n";
#### End of Methods Line ###


# Outer Table...################
print "<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\">\n";
print "<tr><td valign=\"top\">\n";


# Print the table header
print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\"  id=\"table_main\">\n";
print "<tr style=\"background-color:$maincolor;\">
<td align=\"center\"><span style=\"color:white;\">Assessments&nbsp;</span></td>
<td align=\"center\"><span style=\"color:white;\">Avg</span></td></tr>\n";


# Print the Edit Rows for the tests.
print "<tr valign=\"middle\" style=\"background-color: $altcolor;\">\n";
print "<td align=\"center\">
<a href=\"multiedit.pl\" ";
print "title=\"$lex{'Edit multiple scores at the same time'}\" \n"; 
print "style=\"color:white;\">$lex{MultiEdit}</a>";
print "</td><td>&nbsp;</td></tr>\n";
# End of Edit Rows


# Populate tests array for use by 'calcaverage' sub.
$sth = $dbh->prepare("select id, score, weight from gbtest where subjsec = ? and name != 'sortorder'");
$sth->execute( $subjsec );
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
    # Loop Through all tests...skipping sortorder.
while (my ($id,$maxscore,$weight) = $sth->fetchrow) {
    if (not $maxscore or $maxscore == 0){ # Get name, flag error and skip.
	my $sth1 = $dbh->prepare("select name from gbtest where id = ?");
	$sth1->execute( $id );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
	my $testname = $sth1->fetchrow;
	
	print $lex{'No Maximum Score for Test'}. ": $testname ($id); skipping...<br>\n"; 
	next; 
    }

    push @tests,"$id:$maxscore:$weight";
}



# Print the students and their marks;
my $studcount;
foreach my $studnum (@studnum) {
    $sth = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");
    $sth->execute( $studnum );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
    my ($lastname, $firstname) = $sth->fetchrow;
    my $avg = calcAverage($studnum,$subjsec,\@tests);

    $lastname =~ s/\s+/&nbsp;/g;  # replace spaces with non-breaking space.
    $firstname =~ s/\s+/&nbsp;/g;

    print "<tr><td><b>$lastname</b>,&nbsp;$firstname</td>\n";
    print "<td>$avg</td></tr>\n";
    $studcount++;
}
# End Student Loop


# Print the Test Average Row;
print "<tr style=\"background-color: #CCC;\">
 <td colspan=\"2\" align=\"center\">Test Average</td></tr>\n";


# Print the Test Weighting Row
print "<tr style=\"background-color: $altcolor;\">
 <td colspan=\"2\" align=\"center\">Test Weight</td></tr>\n";
print "</table>\n";
# End of Test Weight Row

# Some Javascript here for resizing...
print "<script type=\"text/javascript\">
 <!-- 
 function resize_iframe() {
  document.getElementById(\"iframe_main\").height=document.getElementById(\"table_main\").offsetHeight + 24;
  document.getElementById(\"iframe_main\").width=document.body.offsetWidth-document.getElementById(\"table_main\").offsetWidth;
 }
 -->
 </script>\n";

# Bottom Section Below tables.
print "<div style=\"padding:4px; line-height: 140%;\">";
print "[ $specchar = NA | \n";
print "<a href=\"$self?logout=1\" style=\"color:black;\"";
print " title=\"". $lex{'Log Out'}. ": ". $lex{'Remove login cookies'};
print "\">". $lex{'Log Out'}. "</a> ]<br>\n";

print "<span style=\"margin: 0.2em; border:1px solid black; font-size:80%;\n";
print "padding: 2px 6px; background-color:$altcolor;\">";
print "Version: $version</span></div>\n";

my $group;
if ($view ne $lex{'All Groups'}){
    $group = $view;
}

# Figure out the iframe height
my $frameheight = $studcount * 27 + 130; # 138 for the other 4 rows...

my $newgroup = escape($group);

# Do the IFrame
print "</td><td valign=\"top\">\n";
print "<iframe src=\"gbframe.pl?subjsec=$subjsec&cellvw=p&grp=$newgroup&dend=$dend&dstart=$dstart\"";
print " width=\"600\" \n height=\"$frameheight\" id=\"iframe_main\" ";
print "frameborder=\"0\" style=\"padding:0; margin:0;\"  onload=\"resize_iframe()\"></iframe>\n";
# Below Iframe content here.

print "</td></tr></table>\n"; 
print "</body></html>\n";



#--------
sub login { # print error, login screen and die;
#--------
    my $error = shift;
    print "$doctype\n<html><head><title>$error</title>\n";
    print "<link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\">\n";
    print "$chartype\n</head><body onload=\"document.forms[0].elements[0].focus()\">\n";
    print "<center><h1>$error</h1>\n";
    print "<p><form action=\"$self\" method=\"post\">\n";
    print $lex{'User Id'}. ": <input type=\"text\" name=\"userid\" size=\"8\">\n";
    print $lex{Password}. ": <input type=\"password\" name=\"password\" size=\"8\"><br>\n";
    print $lex{'Cookie Duration'}. ": (". $lex{min}. ") ";
    print "<input type=\"text\" name=\"duration\" size=\"4\" value=\"20\"><br>\n";
    print "<input type=\"hidden\" name=\"subjsec\" value=\"$arr{subjsec}\">\n";
    print "<input type=\"submit\" value=\"". $lex{'Enter Gradebook'}. "\"></form></p>\n";

    print "[ <b><a href=\"$tchpage\">". $lex{Main}. "</a></b> ]</center></body></html>\n";
    
    exit;

}


#------------
sub mkStudNum {
#------------

    my $subjsec = shift;
    my (@eval, %eval, @studnum, %studnum);

    # Create the studnum array that controls the order that students are
    # displayed in. Load the eval records first into an array, sorted by
    # lastname and firstname. (@eval) Load the sort order records in order
    # into an array and also into a hash (%studnum)
    # Any remaining array elements in eval are added to the end of
    # the studnum array once done. If there _are_ no sortorder recs, then
    # the final array becomes the eval records only. This has the desired
    # behavior.

    # Find the enrollments for this class and read them into @eval array
    $sth = $dbh->prepare("select distinct eval.studnum from eval
     left outer join studentall on studentall.studnum = eval.studnum 
     where eval.subjcode = ? 
     order by studentall.lastname, studentall.firstname");

    $sth->execute( $subjsec );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while (my $studnum = $sth->fetchrow){
	push @eval,$studnum;
	$eval{$studnum} = 1;
    }
    
    # Load 'sortorder' test if it exists... 
    $sth = $dbh->prepare("select id from gbtest where
     subjsec = ? and name = 'sortorder'");
    $sth->execute($subjsec);
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my $sortid = $sth->fetchrow;

    if ($sortid){ # We have a sortorder 'test'.
	my $sth = $dbh->prepare("select id, studnum from gbscore where
          testid = ? order by score");
	$sth->execute($sortid);
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

	my $sth1 = $dbh->prepare("delete from gbscore where id = ?");

	while (my ($id, $sn) = $sth->fetchrow){
	    # Check that student is still enrolled...
	    if (not $eval{$sn}) { # student is withdrawn...
		$sth1->execute($id); # remove his/her record from gbscore; # sortorder
		next; 
	    } 

	    push @studnum, $sn;
	    $studnum{$sn} = 1;
	}
	

	# Now add on @eval elements who are not in @studnum
	foreach my $sn (@eval){
	    if ( not $studnum{$sn} ){ push @studnum, $sn; }
	}

	return @studnum;

    } else { # no sortorder
	return @eval;
    }
}


#----------------
sub checkPassword {
#----------------
    my ($userid, $password) = @_;
    
    if (not $userid){ return -1;}
    if (not $password){ return -2;}

    #check for presence of userid
    my $sth = $dbh->prepare("select count(userid) from staff where userid = ?");
    $sth->execute( $userid );
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
    my $count = $sth->fetchrow;
    if ($count < 1){ return -1;} # no userid

    #check for presence of correct password and userid
    my $sth = $dbh->prepare("select count(userid) from staff where userid = ? and passwd = ?");
    $sth->execute( $userid, $password );
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
    my $count = $sth->fetchrow;
    if ($count < 1){ return -2;} # not correct password

    return 0; # if all ok...

}


#------------------
sub checkCookieTime {
#------------------
    my ($duration) = @_;

    if ($duration) { 
	$cookietime = $duration;
    } else { 
	$cookietime = $defaulttime;
    }  

    $cookietime = $minimumtime if $cookietime < $minimumtime; 
    $cookietime = $maximumtime if $cookietime > $maximumtime; 
    $cookietime = "+".$cookietime."m"; # set format
    
    return $cookietime;

}


#-----------------
sub calcAverage { # This is the new one...
#-----------------

    require strict;
    # needed: $maxscore{$id}, $weight{$id}, $groupweight{$grp}

    my ($studnum, $subjsec) = @_; # get student number and subject/section.

    my $totalweight;
    my $totalscore;

    # Get test info and test score ( of that item ) at same time.
    my $sth = $dbh->prepare("select s.id, s.testid, s.score, t.grp from gbscore as s, gbtest as t 
      where s.testid = t.id and t.name != 'sortorder' and 
      s.studnum = ? and t.subjsec = ? order by t.grp");
    $sth->execute( $studnum, $subjsec );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    my $currgrp = -1;
    my $oldgrp;
    my $first = 1;
    my $totalscore; # running total for score; zeroed for each new group.
    my $totalweight; # running total for weight; zeroed for each new group.
    my %groupscore;

    while ( my ( $id, $testid, $score, $grp) = $sth->fetchrow ) {
	$score =~ s/\s//g; # strip any spaces in score;
	if ($score eq $specchar or $score eq '' or not defined $score ) { next; } # skip

	$oldgrp = $currgrp;
	$currgrp = $grp;
	if ($currgrp ne $oldgrp and not $first) {
	    # setup group values for the current group
	    if ($totalweight) {
		$groupscore{$oldgrp} = $totalscore / $totalweight;
	    } else {
		$groupscore{$oldgrp} = 0;
	    }
	    $totalscore = 0;
	    $totalweight = 0;
	}
	if ( $first ) { $first = 0; }

	
	if ($score =~ /\d/) { # if score is a digit... do it.
	    $totalscore += $score / $maxscore{$testid} * $weight{$testid};
	    $totalweight += $weight{$testid};
	} else {
	    # some sort of text score...count as zero... update total weight.
	    $totalweight += $weight{$testid};
	}	    

	#print "TS: $totalscore TW:$totalweight GP:$grp<br>\n";

    } # End of Scores Loop


    # Get Last group, if any;
    if ($totalweight) { 
	$groupscore{$currgrp} = $totalscore / $totalweight;
    }
    $totalscore = 0; # necessary?
    $totalweight = 0; # necessary?

    #foreach my $key (keys %groupscore) { print "K: $key GS: $groupscore{$key}<br>\n"; }

    # Add up the group scores and weight them for overall average
    foreach my $grp (keys %groupscore) {
	$totalscore += $groupscore{$grp} * $groupweight{$grp};
	#print "GRP: $grp GS: $groupscore{$grp} GW:$groupweight{$grp}<br>\n";
	$totalweight += $groupweight{$grp};
    }

    #print "TS:$totalscore TW:$totalweight<br><br>\n";
    if ($totalweight) { 
	return round( 100 * $totalscore / $totalweight, 2). '%';
    } else {
	return 0;
    }


} # End of CalcAverage (new version)
