#! /usr/bin/perl
#  Copyright 2001-2007 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# rptclass.pl - A custom class based report generator


my %lex = ('Error' => 'Error',
	   'Subject Mark Sheets' => 'Subject Mark Sheets',

	   );



$format[0] = 'Letter, Portrait';
$format[1] = 'Letter, Landscape';
$format[2] = 'Legal, Portrait';
$format[3] = 'Legal, Landscape';

$view[0] = 'Scores Only';
$view[1] = 'Groups Only';
$view[2] = 'Groups + Scores';

$weightview[0] = 'Percent';
$weightview[1] = 'Raw';
$weightview[2] = 'Both';

$select[0] = 'By Date';
$select[1] = 'By Term';
$select[2] = 'By Group';
$select[3] = 'By Item';

use DBI;
use CGI;

# Configurable settings
my $maxstudents = 32; # maximum students per page.
my $width = 6;  # default width of columns, 6mm;

$entrylimit = 8; # Max entries allowed for text.
$maxentrysize = 14; # Maximum number of characters for the entry
$namewidth = 50; # Width of the first 'Name' column in mm.


my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $iddst) = localtime(time);
$year = $year + 1900;
$mon++;
$wday++;
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

my $userid = $arr{userid};

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

print "$doctype\n<html><head><title>". $lex{'Subject Mark Sheets'}. "</title>
<link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\">
$chartype\n</head><body style=\"margin-left:2em;\">
<div style=\"float:left;\"><a href=\"$tchpage\" class=\"button\">Main</a>
<a href=\"gbmain.pl\" class=\"button\">GB&nbsp;Main</a>&nbsp;&nbsp;</div>\n";

# Get values for Customization, then die.
# Values passed are width, maxstudents, repeat, entry1 to entry8
if (not $arr{flag}){
    &prEntryPage($userid);
    die;
}

if ($arr{maxstudents}) {
   $maxlines=$arr{maxstudents};
} else {
   $maxlines = $maxstudents;
}
my $wi = $arr{width} - 1; # width of each col desired (in mm)
# take away 1 for 1mm tabcolsep.

my $colcount = 128 / ($arr{width} + 1);
$colcount = int $colcount; # truncate
print "Column Count: $colcount \n";

# push entry text into array.
for (1..$entrylimit){
    my $entryname = 'entry'.$_;
    if ($arr{$entryname}){ # if it exists...push into array
	push @entry,$arr{$entryname};
    }
}
my $entrysize = $#entry + 1;
print "Entry size: $entrysize\n";

if ($arr{repeat} and $entrysize){
    $repeatcount = $colcount / $entrysize;
    $repeatcount = int $repeatcount;
} elsif ($entrysize){
    $repeatcount = 1; # only do it once if not repeat.
} else { $repeatcount = 0;}




$shortname = "cmarksheet$$";
$fileName = "$shortname.tex";

open(TEX,">$fileName") || die "Can't open tex file";
# Letter paper size is 279mm x 213mm
# 200mm wide - 50mm name, 20mm extra fld, leaves 130mm for columns.
# The tabcolsep of 1mm leaves 128 mm where each col is $width + 1mm wide.

if ( $defaultpapersize ) {
    $papersize = $defaultpapersize;
} else {
    $papersize = 'letterpaper';
}

print TEX "\\documentclass[12pt, $papersize]{article}
\\usepackage{array,newcent,rotating,colortbl, inputenc}
\\inputencoding{latin1}
\\pagestyle{empty}
\\setlength{\\textwidth}{200mm}
\\setlength{\\textheight}{270mm}
\\setlength{\\hoffset}{-1.4in}
\\setlength{\\voffset}{-1.4in}
\\addtolength{\\evensidemargin}{0in}
\\addtolength{\\oddsidemargin}{0in}
\\setlength{\\tabcolsep}{1mm}
\\setlength{\\extrarowheight}{2mm}
\\newcolumntype{G}{>{\\columncolor[gray]{0.90}}p{$namewidth mm}}\n";

print TEX "\\begin{document}\n";

#Prep for loop through each subject.
$sth = $dbh->prepare("select distinct eval.studnum, studentall.lastname, 
  studentall.firstname from eval
  left outer join studentall on eval.studnum = studentall.studnum
  where eval.subjcode = ?
  order by studentall.lastname, studentall.firstname");

# Prep for Subject Name
$sth1 = $dbh->prepare("select description from subject where subjsec = ?");

# Main Subject Loop
foreach my $key (keys %arr){ # Loop through each selected subject
    if ($arr{$key} == 1){
	#print "K: $key V:$arr{$key}<br>\n";

	# Creates array controlling student sort order.
	@studnum = mkStudNum($key); # $key is subjsec.

	# select students in $key subject.
	#$sth->execute($key);

	# Get Subject Name
	$sth1->execute($key);
	my $desc = $sth1->fetchrow;
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	
	# print New Page Header; reset counters.
	$linecount = 0; $studcount = 0;
	prHeader($key,$desc);

	# Loop through and print all students
	foreach my $studnum (@studnum){
	    $sth2 = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");
	    $sth2->execute( $studnum );
	    if ($DBI::errstr) { print "$DBI::errstr"; die;}
	    my ($lastname, $firstname) = $sth2->fetchrow;


	    #print "LN: $lastname<br>\n";
	    $linecount++;
	    $studcount++;
	    if ($linecount > $maxlines) { # print a new page header
		# New Page Header
		$linecount = 0;
		print TEX "\\end{tabular}\\\\ \\newpage\n";
		&prHeader($key,$desc);
	    }
	    # Print Normal Record.
	    &prRecord($lastname,$firstname,$studnum);
	}
	&prStudCount;
	print TEX "\\end{tabular}\\\\ \\newpage\n";
    } # End of If
}  # End of Loop for all subjects.

print TEX "\n\\end{document}";
close TEX;

system("$pdflatex $fileName >pdflog$$.txt");
system("mv $shortname.pdf $tchdownloaddir");
system("mv pdflog$$.txt $tchdownloaddir");
system("rm -f $shortname.*");

print "<h1><a href=\"$tchwebdownloaddir/$shortname.pdf\" 
 class=\"button\">View/Download Subject Mark Sheets</a></h1>
 <p><a href=\"$tchwebdownloaddir/pdflog$$.txt\" 
 class=\"button\">View Log file</a></p>
 </center></body></html>";


#======== Functions Start Here ==============

#--------------
sub prEntryPage { # Entry Values for Class Report
#--------------
    my $userid = shift; # Get passed userid.

    # Get Teacher Name
    $sth = $dbh->prepare("select lastname,firstname from staff where userid = ?");
    $sth->execute( $userid );
    if ($DBI::errstr) { print $DBI::errstr; die; }
    my ($lastname, $firstname) = $sth->fetchrow;

    # Get Subjects
    $sth = $dbh->prepare("select description, subjsec from subject
      where teacher like '%($userid%' order by description");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die; }
    while (my ($desc, $subjsec) = $sth->fetchrow){
	push @subject, "$desc:$subjsec";
    }

    print <<"EOS";
<h1>$firstname $lastname - Class Reports</h1>
<form action="rptclass.pl" method="post">
<input type="hidden" name="flag" value="2">
<input type="hidden" name="subjsec" value="$arr{subjsec}">
<input type="hidden" name="vw" value="$arr{vw}">
<table cellpadding="3" cellspacing="0" border="0">
<tr><td align="right"><b>Select Subjects</b></td><td></td></tr>\n
EOS

   foreach my $subj (@subject){
	my ($desc,$subjsec) = split /:/,$subj;
	print "<tr><td align=\"right\">$desc ($subjsec)</td>\n";
	print "<td><input type=\"checkbox\" name=\"$subjsec\" value=\"1\">"; 
	print "</td></tr>\n";
    }
    print "</table><table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";

    print "<tr><td colspan = \"2\" align=\"right\"><b>Select Options</b></td></tr>\n";
    print "<tr><td align=\"right\">Format</td><td><select name=\"format\">\n";
    foreach my $fmt (@format){
	print "<option>$fmt</option>\n";
    }
    print "</select></td></tr>\n";

    # Do Views
    print "<tr><td align=\"right\">View</td><td><select name=\"view\">\n";
    foreach my $vw (@view){
	print "<option>$vw</option>\n";
    }
    print "</select></td></tr>\n";


    # Do Scores
    print "<tr><td align=\"right\">Score View</td><td><select name=\"scoreview\">\n";
    foreach my $vw (@weightview){
	print "<option>$vw</option>\n";
    }
    print "</select></td></tr>\n";

    # Do Weightview
    print "<tr><td align=\"right\">Weight View</td><td><select name=\"weightview\">\n";
    foreach my $vw (@weightview){
	print "<option>$vw</option>\n";
    }
    print "</select></td></tr>\n";

    # Do Select
    print "<tr><td align=\"right\">Select</td><td><select name=\"select\">\n";
    foreach my $vw (@select){
	print "<option>$vw</option>\n";
    }
    print "</select></td></tr>\n";



    # Finish form and table.
    print "<tr><td></td><td><input type=\"submit\" value=\"Continue\"></td>";
    print "</tr></table></form></body></html>\n";

} # End of prEntryPage


#-----------
sub prHeader {
#-----------
    my ($subjsec,$description) = @_;
    print TEX "\\begin{tabular}{G|p{20mm}|";
    for (1..$colcount){	print TEX "p{$wi mm}|"; }
    print TEX "}\n";

    print TEX "{\\small Mark Sheet} ";
    for (1..$colcount + 1){ print TEX "& ";}
    print TEX "\\\\ \n";

    print TEX "{\\small $currdate} \\bigskip ";

    for (1..$colcount + 1){ print TEX "& ";}

    print TEX "\\\\ \n";

    print TEX "{\\bf $description $subjsec} & ";
    my $remainder = $colcount - ($repeatcount * $entrysize);

    if ($repeatcount){
	for (1..$repeatcount){ # times to loop to print rotated text.
	    foreach my $txt (@entry){
		print TEX "&\\hfil\\rule{5pt}{0pt}\\begin{rotate}{90}";
		print TEX "$txt\\end{rotate}\\hfil\n";
	    }
	}
    }
    for (1..$remainder){ print TEX "& ";}
    print TEX "\\\\ \\hline\n";

} # End of prHeader

#-----------
sub prRecord { # print normal record (ie. line)
#-----------
    my ($lastname, $firstname, $studnum) = @_;
    print TEX "$lastname, $firstname";
    print TEX " &{\\small $studnum}";
    for (1..$colcount){ print TEX "& ";}
    print TEX "\\\\\\hline\n";
}

#--------------
sub prStudCount {
#--------------
    print TEX "Students: $studcount";
    print TEX "& ";
    for (1..$colcount){ print TEX "& ";}
    print TEX "\\\\\\hline\n";
}

#------------
sub mkStudNum {
#------------

    my $subjsec = shift;
    my (@eval, @studnum, %remove);

    # Create the studnum array that controls the order that students are
    # displayed in. Load the eval records first into an array, sorted by
    # lastname and firstname. (@eval) Load the sort order records in order
    # into an array and also into a hash (%remove)
    # Any remaining array elements in eval are added to the end of
    # the studnum array once done. If there _are_ no sortorder recs, then
    # the final array becomes the eval records only. This has the desired
    # behavior.


    # Find the enrollments for this class and read them into @eval array
    $sth = $dbh->prepare("select distinct eval.studnum from eval
     left outer join studentall on studentall.studnum = eval.studnum 
     where eval.subjcode = '$subjsec' 
     order by studentall.lastname, studentall.firstname");

    $sth->execute( $subjsec );
    if ($DBI::errstr) { print "$DBI::errstr"; die $DBI::errstr;}
    while (my $studnum = $sth->fetchrow){
	push @eval,$studnum;
    }
    
    # Load 'SortOrder' test if it exists...
    $sth = $dbh->prepare("select id from gbtest where
     subjsec = ? and name = 'sortorder'");
    $sth->execute( $subjsec );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
    my $sortid = $sth->fetchrow;

    if ($sortid){ # We have a sortorder
	$sth = $dbh->prepare("select studnum from gbscore where
          testid = ? order by score");
	$sth->execute( $sortid );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
	while (my $sn = $sth->fetchrow){
	    push @studnum, $sn;
	    $remove{$sn} = 1;
	}
	
	# Now add on @eval elements who are not in @studnum
	foreach my $en (@eval){
	    if (not $remove{$en}){ push @studnum, $en;}
	}
	return @studnum;

    } else { # no sortorder
	return @eval;
    }
}
