#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson
#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# Passed Values - a subjsec code. It will then give a selection list
# of subjects to report on.

my $self = "rptsubj.pl"; # report for each subject

my %lex = ('Display Comments?' => 'Display Comments?',
	   'New Page for each?' => 'New Page for each?',
	   'Continue' => 'Continue',
	   'Groups Only?' => 'Groups Only?',
	   'Student Marks - by Subject' => 'Student Marks - by Subject',
	   'Main' => 'Main',
	   'GB&nbsp;Main' => 'GB&nbsp;Main',
	   'Subject' => 'Subject',
	   'Select' => 'Select',
	   'Enter Gradebook' => 'Enter Gradebook',
	   'Letter' => 'Letter',
	   'Legal' => 'Legal',
	   'A4' => 'A4',
	   'Font Size' => 'Font Size',
	   'Paper Size' => 'Paper Size',
	   'View/Download' => 'View/Download',
	   'Subject Report' => 'Subject Report',
	   'View Log File' => 'View Log File',
	   'Student Number' => 'Student Number',
	   'Password' => 'Password',
	   'Gradebook Marks Report' => 'Gradebook Marks Report',
	   'Absent' => 'Absent',
	   'Late' => 'Late',
	   'minutes' => 'minutes',
	   'Display Complete Attendance?' => 'Display Complete Attendance?',
	   'Display Brief Attendance?' => 'Display Brief Attendance?',
	   'Attendance' => 'Attendance',
	   'Please Select Subjects' => 'Please Select Subjects',
	   'Error' => 'Error',
	   'Please Log In' => 'Please Log In',

	);

use DBI;
use CGI;
use CGI::Session;

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}

eval require "../../etc/gbook.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}


eval require "libgbreport.pl";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}

# needed to exclude additional comments subjects; get subject.
eval require "../../etc/repcard.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}

my $q = CGI->new;

my $dsn = "DBI:$dbtype:dbname=$dbase";
$dbh = DBI->connect($dsn,$user,$password);

# Get Session Information...
my $session = new CGI::Session("driver:mysql;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

my $logged_in = $session->param(logged_in);
if ( not $logged_in ) {
    print $q->header( -charset, $charset );
    login($lex{'Please Log In'}); 
    die;
}

my $userid = $session->param('userid');
my $subjsec = $session->param('subjsec');

print $q->header( -charset, $charset );
my %arr = $q->Vars;

my ($sec, $min, $hour, $mday, $mon, $year, $wday, 
 $yday, $iddst) = localtime(time);
$year = $year + 1900;
$mon++;
$wday++;
# Now using global values for month and dow.
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";

# Control setting of newpages;
my $newpage;
if ( $arr{newpage} ){ $newpage = 1; delete $arr{newpage}; }

# Control printing of groups;
my $showgroup;
if ($arr{showgroup}){ $showgroup = 1; delete $arr{showgroup};}

my $groupsonly;
if ($arr{groupsonly}){ 
    $groupsonly = 1; 
    $showgroup = 1; # override in case groups not selected.
    delete $arr{showgroup};
}

# Control printing of comments;
my $showcomment;
if ($arr{showcomment}){ $showcomment = 1; delete $arr{showcomment};}

# Control printing of attendance;
my $showshortattend;
if ( $arr{showshortattend} ){ $showshortattend = 1; delete $arr{showshortattend};}
my $showlongattend;
if ( $arr{showlongattend} ){ $showlongattend = 1; delete $arr{showlongattend};}

# Now all passed values but subjects (subjsec) are deleted.. 


# print page header
print "$doctype\n<html><head><title>$lex{'Student Marks - by Subject'}</title>
<link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\"></head><body>\n";
print "[ <a href=\"$tchpage\">". $lex{Main}. "</a> |\n";
print "<a href=\"gbmain.pl\">". $lex{'GB&nbsp;Main'}. "</a> ]\n";


# Select subjects to print.
if ( not $arr{flag} ) { 
    selectSubject();
}
delete $arr{flag}; # don't need it anymore...
delete $arr{subjsec};

my $fontsize = $arr{fontsize};
delete $arr{fontsize};
my $papersize = $arr{papersize};
delete $arr{papersize};

startTex(); # Setup the Tex file for writing.

# loop through each subject
foreach my $subj (keys %arr){
    my ($subjsec,$desc) = split /:/,$subj;

    # Find kids in this subject.
    $sth = $dbh->prepare("select distinct eval.studnum from eval, studentall
      where eval.subjcode = ? and 
      studentall.studnum = eval.studnum
      order by studentall.lastname, studentall.firstname");
    $sth->execute( $subjsec );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    my $sth1 = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");

    my $sth2 = $dbh->prepare("select count(*) from attend where studentid = ? and subjsec = ? 
       and reason = '$absentUnexcused'");
    my $sth3 = $dbh->prepare("select count(late), sum(late) from attend 
       where studentid = ? and subjsec = ? and reason = '$lateUnexcused'");

    while ( my $studnum = $sth->fetchrow ){  # loop through kids

	# Get lastname, firstname
	$sth1->execute($studnum);
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
	my ( $lastname, $firstname ) = $sth1->fetchrow;

	my ($subjcount, $latecount, $latesum);
	if ( $showshortattend or $showlongattend ) {

	    # Get attendance
	    $sth2->execute( $studnum, $subjsec );
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
	    $subjcount = $sth2->fetchrow;

	    # Get Late
	    $sth3->execute( $studnum, $subjsec );
	    ($latecount,$latesum) = $sth3->fetchrow;
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}

	}

	print TEX "{\\LARGE $firstname $lastname}\\qquad\n";

	# Put Short Attendance Here, if selected.
	if ( $showshortattend ) {
	    print TEX $lex{Absent}. ": $subjcount\\quad ";
	    print TEX $lex{Late}. ": $latecount ";
	    if ( $latesum ) {
		print TEX "($latesum ". $lex{minutes}. ")";
	    }
	    print TEX "\n\n";

	} else {
	    print TEX "\n";
	}


	# print table of marks
        prStudentSubject($studnum, $subjsec, $groupsonly, $showcomment);

	# put full attendance here, if selected.
	if ( $showlongattend ) {

	    my $sth = $dbh->prepare("select absdate, reason, late from attend 
             where studentid = ? and subjsec = ?");
	    $sth->execute( $studnum, $subjsec );
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	    my $first = 1;
	    my $count = 1;

	    while ( my ($absdate, $reason, $late) = $sth->fetchrow ) {
		
		if ( $first ) {
		    print TEX "\\medskip\n";
		    print TEX "\\begin{tabular}{|p{6.35cm}|p{6.35cm}|} \\hline\n";
		    print TEX "\\rowcolor[gray]{0.90}\\Large ". $lex{Attendance}. " & \\\\ \\hline\n";
		    $first = 0;
		}
		
		print TEX "$absdate - $reason ";
		if ( $late ) { print TEX "- $late"; }
		#print TEX "\n\n";
		if ( $count % 2 ) { print TEX ' & '; } else { print TEX "\\\\ \\hline\n"; }
		$count++;
	    }
	    if ( not $first ) {
		if ( not ($count % 2) ) { print TEX "\\\\ \\hline\n"; }
		print TEX "\\end{tabular}\n"; # closing brace ends parbox.
	    }
	}

	if ( $newpage ){ 
	    print TEX "\\newpage\n\n";
	} else {
	    print TEX "\\bigskip\n\n";
	}
    }

    # finish subject with newpage...
    if (not $newpage){ print TEX "\\newpage\n\n";} 

} # End of this subject.

print TEX "\n\\end{document}";
close TEX;

system("$pdflatex $filename >pdflog$$.txt");
system("mv $shortname.pdf $tchdownloaddir");
system("mv pdflog$$.txt $tchdownloaddir");
system("rm -f $shortname.*");

print "<center><h1><a href=\"$tchwebdownloaddir/$shortname.pdf\">";
print $lex{'View/Download'}. ' '. $lex{'Subject Report'}. "</a></h1>";
print " <p>[ <a href=\"$tchwebdownloaddir/pdflog$$.txt\">". $lex{'View Log File'}. "</a> ]</p>
 </center></body></html>\n";


#--------
sub login { # print error, login screen and die;
#--------
    my $error = shift;
    print "$doctype\n<html><head><title>$error</title>
    <link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\">
    $chartype\n</head><body><center><h1>$error</h1>\n";

    print "<p><form action=\"gbmain.pl\" method=\"post\">\n";

    print $lex{'Student Number'}. ": <input type=\"text\" name=\"studnum\" size=\"8\">\n";
    print $lex{Password}. ": <input type=\"password\" name=\"password\" size=\"8\"><br>
    <input type=\"submit\" value=\"". $lex{'Enter Gradebook'}. "\"></form></p>\n";
    print "[ <b><a href=\"$tchpage\">". $lex{Main}. "</a></b> ]</center></body></html>\n";
    
    exit;

}


#-----------
sub startTex {
#-----------

    # Set paper size; default to letter.
    if ($papersize eq $lex{Legal}) {
	$papersize = 'legalpaper';
    } elsif ($papersize eq $lex{A4}) {
	$papersize = 'a4paper';
    } else {
	$papersize = 'letterpaper'; 
    }

    $fontsize = $fontsize . 'pt';

    $subjspace = '6cm';

    $shortname = "subjrpt$$";
    $filename = "$shortname.tex";

    open(TEX,">$filename") || die "Can't open tex file";

    # Preamble and Head of Document
    print TEX <<"EOS1";  
\\documentclass[ $fontsize, $papersize, oneside]{article}
\\usepackage{array,graphicx,multicol,colortbl,tabularx,inputenc}
\\inputencoding{latin1}
\\renewcommand{\\familydefault}{\\sfdefault}
\\setlength{\\hoffset}{-0.3in}
\\setlength{\\voffset}{-0.4in}
\\setlength{\\headsep}{10pt}
\\setlength{\\headheight}{14pt}
\\setlength{\\topmargin}{0in}
\\setlength{\\textwidth}{7.2in}
\\setlength{\\textheight}{9in}
\\setlength{\\parindent}{0pt}
\\setlength{\\evensidemargin}{0in}
\\setlength{\\oddsidemargin}{0in}
\\setlength{\\extrarowheight}{4pt}
\\pagestyle{headings}
\\newcolumntype{G}{>{\\columncolor[gray]{0.85}}p{$subjspace cm}}
\\markright{$currdate\\hfill $lex{'Gradebook Marks Report'} \\hfill Page }

\\begin{document}
EOS1

} # End of startTex


#----------------
sub selectSubject { # Select subjects to print for this teacher
#----------------

    # Note: assume that userid and subjsec is already loaded from Session.

    my @subjsec;

    # Now find the subjects and store the subjsec, desc for them 
    $sth = $dbh->prepare("select subjsec, description from subject
     where teacher $sql{like} '%$userid%' and description != '$additionalcomments'
     order by description");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr;}
    while (my ($subjsec,$desc) = $sth->fetchrow){
	$desc =~ s/://g;
	push @subjsec, "$subjsec:$desc";
    }
    
    
    print "<center><h1>". $lex{'Student Marks - by Subject'}. "</h1>\n";
    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"flag\" value=\"1\">\n";
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";

    #print "<tr><th>". $lex{Subject}. "</th><th>". $lex{Select}. "</th></tr>\n";

    print "<tr style=\"background-color:#BBB;\"><td colspan=\"2\" align=\"center\">\n";
    print "<input type=\"submit\" value=\"". $lex{Continue}. "\"></td></tr>\n";

    print "<tr style=\"background-color:#DDD;\"><td align=\"right\">";
    print $lex{'Font Size'}. "</td><td><select name=\"fontsize\"><option>11</option><option>10</option>";
    print "<option>12</option></select>pt</td></tr>\n";

    print "<tr style=\"background-color:#DDD;\"><td align=\"right\">";
    print $lex{'Paper Size'}. "</td><td><select name=\"papersize\"><option>". $lex{Letter};
    print "</option><option>". $lex{Legal}. "</option><option>". $lex{A4}. "</option></select>\n";
    print "</td></tr>\n";

    print "<tr style=\"background-color:#DDD;\"><td align=\"right\">";
    print "$lex{'Groups Only?'}</td>";
    print "<td align=\"center\"><input type=\"checkbox\" name=\"groupsonly\" value=\"1\">\n";
    print "</td></tr>\n";

    print "<tr style=\"background-color:#DDD;\"><td align=\"right\">";
    print "$lex{'New Page for each?'}</td>";
    print "<td align=\"center\"><input type=\"checkbox\" name=\"newpage\" value=\"1\">\n";
    print "</td></tr>\n";

    print "<tr style=\"background-color:#DDD;\">";
    print "<td align=\"right\">". $lex{'Display Comments?'};
    print "</td><td align=\"center\"><input type=\"checkbox\" name=\"showcomment\" value=\"1\">";
    print "</td></tr>\n";

    print "<tr style=\"background-color:#DDD;\">";
    print "<td align=\"right\">". $lex{'Display Brief Attendance?'};
    print "</td><td align=\"center\"><input type=\"checkbox\" name=\"showshortattend\" value=\"1\">";
    print "</td></tr>\n";

    print "<tr style=\"background-color:#DDD;\">";
    print "<td align=\"right\">". $lex{'Display Complete Attendance?'};
    print "</td><td align=\"center\"><input type=\"checkbox\" name=\"showlongattend\" value=\"1\">";
    print "</td></tr>\n";

    print "<tr><th colspan=\"2\">". $lex{'Please Select Subjects'}. "</th></tr>\n";
    foreach my $subj (@subjsec){
	my ($subjsec, $desc) = split /:/,$subj;
	print "<tr><td align=\"right\">$desc ($subjsec)</td>\n";
	print "<td align=\"center\"><input type=\"checkbox\" ";
	print "name=\"$subj\" value=\"1\"></td></tr>\n";
    }

    print "<tr style=\"background-color:#BBB;\"><td colspan=\"2\" align=\"center\">";
    print "<input type=\"submit\" value=\"". $lex{Continue}. "\">";
    print "</td></tr></table></form></center></body></html>\n";

    exit;

} # End of selectSubject

