#!/usr/bin/perl 
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

#  Passed subjsec of the subject to create sortorder 'test' since
#  'sortorder' is the name of the special test used to store sortorder info.

#  Design: We have an array called '@sortorder' with the studnum's in
#  the correct order for display. We have two hashes with studnum
#  keys: %studnum which holds the student names and %order which holds
#  the ordering value. When setting ordering values, they will
#  normally increase by 5 to allow room to place others 'in between'.

#  If there is no test named 'sortorder' then we will put students
#  initially in lastname, firstname order. The order values will start
#  at 5 and increment by 5.

# If there IS a sortorder test, then it will be used to populate the
# sortorder array in order. (and the matching order and studname
# hashes). Students not found in the array (and matching hashes) when
# loading the student eval records will be added to the end of the
# array. Students not found in the eval records (due to withdrawal,
# will be removed from the studname hash, and skipped when going
# through the sortorder array to display students.

my %lex = ('Adjust Student Sorting Order' => 'Adjust Student Sorting Order',
	   'Control the display order of students' => 'Control the display order of students',
	   'Re-Order' => 'Re-Order',
	   'Name' => 'Name',
	   'Order' => 'Order',
	   'GB&nbsp;Main' => 'GB&nbsp;Main',
	   'Display order of students' => 'Display order of students',
	   'Main' => 'Main',
	   'Please Log in!' => 'Please Log in!',
	   'Error' => 'Error',


	   );

use DBI;
use CGI;
use CGI::Session;

my $increment = 5; # how much each sequence number increases by.
my $currval = 100; # starting value for sequence numbers.
# 3 digits to ensure still have 3 digits while incrementing.
# Make sure ascii sort still works right...

# Set the current date
my @tim = localtime(time);
my $year = $tim[5] + 1900;
my $month = $tim[4] + 1;
my $day = $tim[3];
my $currdate = "$year-$month-$day"; 

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}

my $q = new CGI;

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Get Session Information...
my $session = new CGI::Session("driver:mysql;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

my $logged_in = $session->param(logged_in);
if (not $logged_in){
    print $q->header( -charset, $charset );
    print $lex{'Please Log in!'}. "<br>\n";
    die;
}

my $subjsec = $session->param('subjsec');

print $q->header( -charset, $charset );
my %arr = $q->Vars;


my $sth = $dbh->prepare("select description from subject where subjsec = ?");
$sth->execute($subjsec);
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
my $desc = $sth->fetchrow;


# If looping, we'll have id, subjsec, update values also. Update tables.
if ($arr{update}){ # write values into gbscore, gbtest
    delete $arr{update};
    updater();
}

# Lookup 'sortorder' test id,  if any.
$sth = $dbh->prepare("select id from gbtest 
  where subjsec = ? and name = 'sortorder'");
$sth->execute($subjsec);
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
my $id = $sth->fetchrow; # id is used to find all of the sortorder recs.
#(no $id = no test)


if ($id){ # We HAVE a sortorder test id

    # Get the sortorder score records (giving the order)
    $sth = $dbh->prepare("select id, studnum, score from gbscore
     where testid = ? order by score");
    $sth->execute($id);
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    while (my ($scoreid, $studnum, $seq) = $sth->fetchrow){
	push @sortorder, $studnum; 
	$order{$studnum} = $seq;
	$recnum{$studnum} = $scoreid; # stores the score id's.
	if ($seq > $currseq){ $currseq = $seq ;} # bump up currseq value
    }


    # Find the enrollments for class and read them into %studname hash
    $sth = $dbh->prepare("select distinct eval.studnum, studentall.lastname, 
     studentall.firstname from eval
     left outer join studentall on studentall.studnum = eval.studnum 
     where eval.subjcode = ?");
    $sth->execute( $subjsec );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    # Read in enrolled student names. (there may be more or less...)
    while (my ($studnum, $lastname,$firstname) = $sth->fetchrow){
	$studname{$studnum} = "$lastname, $firstname";
	if (not $order{$studnum}){ # student not in sortorder...
	    $currseq = $currseq + $increment;
	    $order{$studnum} = $currseq;
	    push @sortorder,$studnum;
	}
    }
    # At this point we should have fully populated @sortorder, %studname,
    # %order hashes. Next check for missing parts to check for removed
    # students from eval recs.

    # If order hash is larger, we have to remove some sortorder records.
    if (scalar keys %order > scalar keys %studname){ # larger order hash
	foreach my $studnum (keys %order){ # too many keys in sortorder hash.
	    if (not $studname{$studnum}){ 
            # not found in studname hash (ie. not in eval recs; not registered)
		
		delete $order{$studnum};
		
		#delete from sortorder array. 
		for  (0..$#sortorder){
		     if ($sortorder[$_] == $studnum) { $index = $_; }
		 }
		# now do the splice to remove the offending element.

		if ($index){ splice (@sortorder,$index,1); }
	    
		# delete from gbscore table.
		$sth = $dbh->prepare("delete from gbscore where testid = ? and studnum = ?");
		$sth->execute( $id, $studnum );
		if ($DBI::errstr) { print "$DBI::errstr"; die;}
		
	    }
	}
    }



} else {  # NO sortorder test found... do by lastname, firstname
    # Find the enrollments for this class and read into %studname hash
    #  ordered by lastname, firstname.
    $sth = $dbh->prepare("select distinct eval.studnum, studentall.lastname, 
     studentall.firstname from eval
     left outer join studentall on studentall.studnum = eval.studnum 
     where eval.subjcode = ? 
     order by studentall.lastname, studentall.firstname");
    $sth->execute( $subjsec );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    while (my ($studnum, $lastname,$firstname) = $sth->fetchrow){
        push @sortorder, $studnum;
	$studname{$studnum} = "$lastname, $firstname";
	$currval += $increment;
	$order{$studnum} = $currval;
    }

}

# At this point we should have fully populated @sortorder, %studname,
# %order hashes.

#print "SO:",@sortorder,"<br>\n";
#print "ORD:",%order,"<br>\n";
#print "Name:",%studname,"<br>\n";

# Now setup HTML header
print "$doctype\n<html><head><title>". $lex{'Adjust Student Sorting Order'};
print  "</title>\n<link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\">
$chartype\n</head><body>\n";

print "<div>[ <a href=\"$tchpage\">". $lex{Main}. "</a> | 
<a href=\"gbmain.pl\">". $lex{'GB&nbsp;Main'}. "</a> ]
</div>\n";

print "<center><h1>$desc - ". $lex{'Adjust Student Sorting Order'};
print  "</h1>\n<p><i>". $lex{'Control the display order of students'};
print "</i></p>\n";

print "<form action=\"sortorder.pl\" method=\"post\"> 
<input type=\"hidden\" name=\"update\" value=\"1\">
<input type=\"hidden\" name=\"subjsec\" value=\"$subjsec\">
<input type=\"hidden\" name=\"id\" value=\"$id\">\n";

print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
print "<tr><td colspan=\"2\" align=\"center\">
<input type=\"submit\" value=\"". $lex{'Re-Order'}. "\">
</td></tr>\n<tr><th>". $lex{Name}. "</th><th>". $lex{Order}. "</th></tr>\n";

foreach my $studnum (@sortorder){
    if ($recnum{$studnum}){ 
	$id = $recnum{$studnum};
    } else {
	$id = 0;
    }
    print "<tr><td>$studname{$studnum} ($studnum)</td>";
    print "<td><input type=\"text\" name=\"$studnum:$id\" ";
    print "size=\"4\" value=\"$order{$studnum}\"></td></tr>\n";
}

print "<tr><td colspan=\"2\" align=\"center\">
<input type=\"submit\" value=\"". $lex{'Re-Order'}. "\">
</td></tr></table></form></body></html>\n";



#----------
sub updater {
#----------

    #foreach my $key (keys %arr) { print "K:$key V:$arr{$key}<br>\n"; }

    my $id = $arr{id};
    delete $arr{id};

    my $subjsec = $arr{subjsec};
    delete $arr{subjsec};

    # These records are passed as studentnumber:number. If the number is
    # one then a record already exists and it is updated. If the number is
    # a 0 then no record exists and a new one is added to the gbscore
    # table.

    if ( not $id ){ # We don't have a test 'sortorder' for this subject.
	$sth = $dbh->prepare("insert into gbtest values(
         $sql{default}, ?, 'sortorder', '$lex{'Display order of students'}',
         $sql{default}, $sql{default}, $sql{default}, $sql{default}, $sql{default})");
	$sth->execute( $subjsec );
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

	# Now get the id for this test.
	$sth = $dbh->prepare("select id from gbtest 
          where name = 'sortorder' and subjsec = ? ");
	$sth->execute( $subjsec );
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	$id = $sth->fetchrow;
	    
    }
    # Now we should have an $id...

    foreach $key (keys %arr) { 
	my ($studnum,$flag) = split /:/,$key;
	if ( $flag ){  #if non zero, we have an existing record to update
	    $sth = $dbh->prepare("update gbscore set score = ? where studnum = ? and testid = ?");
	    $sth->execute($arr{$key}, $studnum, $id);
	} else { # zero; we must add a new record.
	    $sth = $dbh->prepare("insert into gbscore values (
              $sql{default}, ?,?,?,$sql{default} )");
	    $sth->execute( $studnum, $id, $arr{$key} );
	    # id, studnum, testid, score, comment
	}
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    }
    
} # End of updater Sub.

