#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Weight by Item' => 'Weight by Item',
	   'GB&nbsp;Main' => 'GB&nbsp;Main',
	   'Weight&nbsp;by&nbsp;Group' => 'Weight&nbsp;by&nbsp;Group',
	   'Please Log in!' => 'Please Log in!',
	   'Name' => 'Name',
	   'Date' => 'Date',
	   'Group' => 'Group',
	   'Weight' => 'Weight',
	   'Update Weights' => 'Update Weights',
	   'Group Weight' => 'Group Weight',
	   'Percent' => 'Perct',
	   'C = custom weight set by direct edit' => 'C = custom weight set by direct edit',
	   'Error' => 'Error',

	   );

my $self = 'weightadj.pl';  # script to adjust test weighting

use DBI;
use CGI;
use CGI::Session;
use Number::Format qw{ round };

my $q = new CGI;
my %arr = $q->Vars;

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}

eval require "../../etc/gbook.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Get Session Information...
my $session = new CGI::Session("driver:mysql;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

my $logged_in = $session->param(logged_in);
if (not $logged_in){
    print $q->header( -charset, $charset );
    print "[ <a href=\"gbmain.pl\">". $lex{'GB&nbsp;Main'}. "</a> ]\n";
    print $lex{'Please Log in!'}. "<br>\n";
    die;
}

my $subjsec = $session->param('subjsec');

print $q->header( -charset, $charset );


# build a hash to map a weight number -> multiplier value (ie. 500 -> 5, 750 -> 6)
foreach my $key (keys %wtmult) { # %wtmult from gbook.conf file
    my $value = round( $wtmult{$key} * $defaultItemWeight, 0);
    $weighttomult{$value} = $key;
}
#foreach my $key (keys %weighttomult) { print "K:$key V:$weighttomult{$key}<br>\n"; }


# Get Totals for Weights and setup weight hash.
my $sth = $dbh->prepare("select id, weight from gbtest where subjsec = ? ");
$sth->execute( $subjsec );
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

my $totalweight;
while ( my ($id,$wt) = $sth->fetchrow ) {
    $totalweight += $wt;
    $weight{$id} = $wt;
}

# Write out updates
if ($arr{update}) {
    delete $arr{update};
    weightupdate();
}

# Now get group weights
my $sth = $dbh->prepare("select distinct grp, sum(weight) from gbtest 
  where subjsec = ? group by grp");
$sth->execute( $subjsec );
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
while (my ($grp, $wt) = $sth->fetchrow) {
    $grpweight{$grp} = $wt;
}

# Get name of subject
$sth = $dbh->prepare("select description from subject where subjsec = ?");
$sth->execute( $subjsec );
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
my $subject = $sth->fetchrow;

# select item values
$sth = $dbh->prepare("select id, name, description, tdate, weight, grp from gbtest 
  where subjsec = ? and name != 'sortorder' order by grp,tdate");
$sth->execute( $subjsec );
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }


# Print Page head
print "$doctype\n<html><head><title>". $lex{'Weight by Item'}. "</title>
<link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\">
$chartype</head><body>\n";

print "[ <a href=\"gbmain.pl\">". $lex{'GB&nbsp;Main'}.  "</a> | \n";
print "<a href=\"weightgrp.pl\">". $lex{'Weight&nbsp;by&nbsp;Group'}. "</a> ]\n";
print "<center><h1 style=\"display:inline;\">". $lex{'Weight by Item'}. " - $subject</h1>\n";

print "<form action=\"$self\" method=\"post\">\n";
print "<input type=\"hidden\" name=\"update\" value=\"1\">\n";
print "<table cellpadding=\"3\" border=\"1\" cellspacing=\"0\">\n";

print '<tr><th>'. $lex{Name}. '</th><th>'. $lex{Date}. "</th>\n";
print '<th>'. $lex{Group}. '</th><th>'. $lex{Percent}.'</th><th>'. $lex{Weight}. "</th></tr>\n";

my $grpcolor = -1;
my $group;
my $groupweight;

while (my ($id, $name, $desc, $tdate, $weight, $grp) = $sth->fetchrow){

    if ($group ne $grp){ 
	$grpcolor++;
	if (5 == 4) { #$group){ # Avoids an empty row at top.
	    if (not $totalweight){ $grouppercent = "0.0%";} else {
		$grouppercent = round($groupweight / $totalweight * 100, 1);
	    }
	    print "<tr style=\"background-color: #EEE; color: black;\">";
	    print "<td colspan=\"4\">". $lex{'Group Weight'}. " - <b>$group</b></td>";
	    print "<td>$groupweight ($grouppercent%)</td></tr>\n";
	}
	$groupweight = 0;
	$group = $grp;
    }
    
    $groupweight += $weight;
    my $percent;
    if ($grpweight{$grp}) {
	$percent =  round( $weight / $grpweight{$grp} * 100, 1);
    } else {
	$percent = '0.0';
    }

    print "<tr style=\"background-color: $g_color[$grpcolor];\">\n";
    print "<td>$name ($desc)</td><td>$tdate</td><td>$grp</td><td>$percent%</td>\n";

    # print the select box or else an entry box.
    if ( $useMultipliers ) {

	my $key = $weighttomult{$weight};
	print "<td><select name=\"$id\">\n";
	if (not defined $key){
	    #print "K:$key W:$weight<br>\n";
	    my $val = round( $weight / $defaultItemWeight, 2);
	    print "<option>C ($val\X)</option>\n";
	}

	foreach my $mult (sort keys %wtmult) {
	    my $value = round( $wtmult{$mult}, 2);
	    print "<option ";
	    if ($mult eq $key) { print 'selected'; }
	    print ">$mult ($value\X)</option>";
	}
	print "</select></td></tr>\n";

    } else {
	print "<td><input type=\"text\" name=\"$id\" size =\"5\" ";
	print "value=\"$weight\"></td></tr>\n";
    }

} # End of test loop.

print "<tr style=\"background-color: #CCC; color: black;\">";
print "<td align=\"center\" colspan=\"4\">
 <input type=\"submit\" value=\"". $lex{'Update Weights'}. "\"></td><td></td></tr>\n";
 
print "</table>\n";
print $lex{'C = custom weight set by direct edit'};

print "</center></body></html>\n";



#---------------
sub weightupdate {
#---------------

    my $sth = $dbh->prepare("update gbtest set weight = ? where id = ?");
    #foreach my $key (keys %arr) { print "K: $key V: $arr{$key}<br>\n"; }


    foreach my $id (keys %arr) {
	$arr{$id} =~ s/\(.*\)//; # strip stuff in brackets.
	$arr{$id} =~ s/\s//g;
	if ($arr{$id} eq 'C') { next; }
	my $multiplier = $wtmult{$arr{$id}};
	my $weight = round( $defaultItemWeight * $multiplier, 0);
	# $weight is an integer value...

	if ( $weight != $weight{$id}) { # update weight if changed. 
	    #print "ID: $id  V: $arr{$id} W: $weight{$id}<br>\n";
	    $sth->execute( $weight, $id );
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    $weight{$id} = $arr{$id}; # update the hash.
	}
    }

    # reset total
    $totalweight = 0;
    foreach my $wt (values %weight) { $totalweight += $wt;}

    # reload grpweight values
    $sth = $dbh->prepare("select distinct grp, sum(weight) from gbtest 
     where subjsec = ? group by grp");
    $sth->execute($subjsec);
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    while (my ($grp, $wt) = $sth->fetchrow) {
	$grpweight{$grp} = $wt;
    }

}
