#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# weightgrp.pl - script to adjust test weighting by group percent
# vars showWeights - from gbook.conf

my %lex = ( 'Adjust Group Weights' => 'Adjust Group Weights',
	    'GB&nbsp;Main' => 'GB&nbsp;Main',
	    'Weight by Item' => 'Weight by Item',
	    'Group' => 'Group',
	    'Weight' => 'Weight',
	    'Items' => 'Items',
	    'Update Weights' => 'Update Weights',
	    'Total Weight' => 'Total Weight',
	    'Please Log In' => 'Please Log In',
	    'Put X in weight to delete' => 'Put X in weight to delete',
	    'Error' => 'Error',
	    'No Subject' => 'No Subject',

	   );

# Number of blank group entries to allow
my $maxEntries = 6;

my $self = 'weightgrp.pl';

use DBI;
use CGI;
use CGI::Session;

my $q = new CGI;
my %arr = $q->Vars;

delete $arr{subjsec}; # no longer needed; session based.

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}

eval require "../../etc/gbook.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@<br>\n";
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Get Session Information...
my $session = new CGI::Session("driver:mysql;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

print $q->header( -charset, $charset );

my $logged_in = $session->param(logged_in);
if ( not $logged_in ) {
    print $lex{'Please Log In'}. "<br>\n";
    die;
}

my $subjsec = $session->param('subjsec');

# Get Subject Description
$sth = $dbh->prepare("select description from subject 
  where subjsec = ?");
$sth->execute($subjsec);
if ($DBI::errstr){ print $DBI::errstr; die "$DBI::errstr:$!\n";}
my $subject = $sth->fetchrow;


# Print Document Head
print "$doctype\n<html><head><title>". $lex{'Adjust Group Weights'};
print "</title>\n<link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\">\n";
print "$chartype\n</head><body>\n";
print "[ <a href=\"gbmain.pl\">". $lex{'GB&nbsp;Main'}. "</a> | \n";
print "<a href=\"weightadj.pl\">". $lex{'Weight by Item'}. "</a> ]\n";
print  "<center><h1>". $lex{'Adjust Group Weights'}. " - $subject</h1>\n";
#print  "<div>". $lex{'Put X in weight to delete'}. "</div>\n";


if ( $arr{writeflag} )  {
    delete $arr{writeflag};
    writeWeights($subjsec);
}


# Load the assessment groups
my %groups = ();
$sth = $dbh->prepare("select distinct grp from gbtest where subjsec = ? order by grp desc");
$sth->execute( $subjsec );
if ( $DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
while ( my $grp = $sth->fetchrow ) {
    $groups{$grp} = '-1';
}


# Load the markscheme field; possibly updated.
$sth = $dbh->prepare("select markscheme from subject where subjsec = ?");
$sth->execute( $subjsec );
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
my $markscheme = $sth->fetchrow;

# put in %groups percent weights from values in markscheme.
my @fields = split (/[\n|\r]/, $markscheme);
foreach my $fld ( @fields ) {
    if ($fld) {
	#print "F:$fld<br>\n";
	my ($grp, $percent) = split /=/, $fld;
	if ( $groups{$grp} ) { # only assign weights to groups that exist in gbtest.
	    $groups{$grp} = $percent; 
	}
    }
}

print "<form action=\"$self\" method=\"post\">\n";
print "<input type=\"hidden\" name=\"writeflag\" value=\"1\">\n";

print "<table cellpadding=\"3\" border=\"1\" cellspacing=\"0\">\n";
print '<tr><th>'. $lex{Group}. '</th><th>'. $lex{Items};
print '</th><th>'. $lex{Weight}. "</th></tr>\n";

my $grpcolor = -1;
my $totalweight;

my $sth = $dbh->prepare("select count(*) from gbtest where grp = ? and subjsec = ?");

foreach my $grp ( sort keys %groups) { 

    $sth->execute($grp, $subjsec);
    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
    my $count = $sth->fetchrow;

    $grpcolor++;

    print "<tr style=\"background-color: $g_color[$grpcolor]; color: black;\">";
    print "<td>$grp</b></td><td align=\"center\">$count</td>\n<td align=\"center\">";
    print "<input type=\"text\" name=\"$grp\" value=\"$groups{$grp}\" size=\"4\">";
    print "<b>%</b></td></tr>\n";

    $totalweight += $groups{$grp};

}

print "<tr style=\"background-color: #BBB; color: black;\">";
print "<td align=\"center\" colspan=\"2\">". $lex{'Total Weight'}. "</td>\n";
print "<td align=\"center\">$totalweight%</td></tr>\n";

print "<tr style=\"background-color: #DDD; color: black;\">";
print "<td align=\"center\" colspan=\"3\">
<input type=\"submit\" value=\"". $lex{'Update Weights'}. "\"></td></tr>\n";
print "</table></form>\n";

print "</center></body></html>\n";


#---------------
sub writeWeights {
#---------------
    
    my $subjsec = shift; # passed subject
    if (not $subjsec) {
	print $lex{'No Subject'}. "!<br>\n";
	die $lex{'No Subject'}. "!<br>\n";
    }

    #foreach my $key (sort keys %arr) { print "K:$key V:$arr{$key}<br>\n"; }
    #die;

    my $newscheme;
    foreach my $grp (sort keys %arr){
	#print "G:$grp V:$arr{$grp}<br>\n";
	$newscheme .= "$grp=$arr{$grp}\n";
    }

    # now write it back into the subject markscheme field
    $sth = $dbh->prepare("update subject set markscheme = ? where subjsec = ?");
    $sth->execute($newscheme, $subjsec);
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

    return;

}
