#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Student Chips Report' => 'Student Chips Report',
	   'Main' => 'Main',
	   'Name' => 'Name',
	   'Prev' => 'Prev',
	   'Week' => 'Week',
	   'Curr' => 'Curr',
	   'Month' => 'Month',
	   'YTD' => 'YTD',
	   'Total' => 'Total',
	   'Date Range Points Count' => 'Date Range Points Count',
	   'Start Date' => 'Start Date',
	   'End Date' => 'End Date',
	   'Error' => 'Error',

	   );

my %modes = ('Name by Grade' => 'namebygrade',
	   'Name by Homeroom' => 'namebyroom',
	   'Name by School' => 'namebyschool',
	   'Points by Grade' => 'pointsbygrade',
	   'Points by Homeroom' => 'pointsbyroom',
	   'Points by School' => 'pointsbyschool'
	   );

my %invmodes = reverse %modes;
my $defaultMode = 'namebyroom';


use DBI;
use CGI;
use Cwd;
use Time::JulianDay;


# Read config variables
eval require "../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

my ($currsdate, $currdate, $currldate);
my ($day, $month, $year, $wday);

# Passed a date? 
if ($arr{date}) {
    # Generate a list from those dates. Assume year-mon-day
    ($year,$month,$day) = split /-/,$arr{date};

    $jd = julian_day($year,$month,$day);
    $dow = day_of_week($jd) + 1;
    $mondayjd = $jd - ($dow-1);
    $fridayjd = $mondayjd + 4;

    # Check for date before start of year; reset to year start.
    #if ($mondayjd < $startjd ){ $mondayjd = $startjd;}

    if (length($month) == 1){ $month = "0".$month;}
    if (length($day) == 1){ $day = "0".$day;}
    $currsdate = "$year$month$day"; 
    $currdate = "$year-$month-$day";
    $currdate = "$dow[ $dow ], $month[$month] $day, $year";

} else {

    @tim = localtime(time);
    $year = $tim[5] + 1900;
    $month = $tim[4] + 1;
    $wday = $time[6] + 1;
    $day = $tim[3];

    $jd = julian_day($year,$month,$day);
    $mondayjd = $jd - ($tim[6]-1); #This will now give Monday of that week.
    $fridayjd = $mondayjd + 4;

    if (length($month) == 1){ $month = "0".$month;}
    if (length($day) == 1){ $day = "0".$day;}
    #$currdate = "$year-$month-$day";
    $currsdate = "$year$month$day";
    $currdate = "$dow[ $tim[6] ], $month[$month] $day, $year";

} 

my $mode;
if ( $arr{mode} and $modes{$arr{mode}} ) { # If mode exists, and matches...
    $mode = $modes{$arr{mode}};
} else {
    $mode = $defaultMode; # set at top of file.
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Change location of CSS files used. We must be running in tcgi, not cgi.
if (getcwd() =~ /tcgi/) {
    $css = $tchcss;
    $homepage = $tchpage;
}

# Print Page header.
print "$doctype\n<html><head><title>". $lex{'Student Chips Report'}. "</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>[ <a href=\"$tchpage\">". $lex{Main}. "</a> ]\n";


# Do Date Mode Report (between selected date ranges)
if ( $arr{datemode} ) {
    doDateRange();
}


# Set Selection and Sort Order based on mode values.
my $select;
my $sortorder;
if ($mode eq 'namebyroom' or $mode eq 'pointsbyroom') {
    if ($arr{group}) {
	my $homeroom = $dbh->quote($arr{group});
	#print "G:$arr{group} HR: $homeroom<br>\n";
	$select = "where homeroom = $homeroom";
    }
    # Now do sortorder...
    if ($mode eq 'namebyroom') {
	$sortorder = "homeroom, lastname, firstname";
    } else { 
	$sortorder = "homeroom, totalpoints desc, lastname, firstname";
    }

} elsif ($mode eq 'namebygrade' or $mode eq 'pointsbygrade') {
    if ($arr{group}) {
	my $grade = $dbh->quote($arr{group});
	$select = "where grade = $grade";
    }
    # Now do sortorder...
    if ($mode eq 'namebygrade') {
	$sortorder = "grade, lastname, firstname";
    } else { # pointsbygrade mode
	$sortorder = "grade, totalpoints desc, lastname, firstname";
    }
} else { # school modes.
    $select = ''; # no select; we want everyone...
    # Now do sortorder...
    if ($mode eq 'namebyschool') {
	$sortorder = "lastname, firstname";
	$select = '';
    } else { # pointsbygrade mode
	$sortorder = "totalpoints desc, lastname, firstname";
	$select = '';
    }
}


# Convert JD to weekly dates for all 4 weekly dates.
my ($cmyear, $cmmonth,$cmday) = inverse_julian_day($mondayjd);
$currmonday = "$cmyear-$cmmonth-$cmday";
my ($cfyear, $cfmonth,$cfday) = inverse_julian_day($fridayjd);
$currfriday = "$cfyear-$cfmonth-$cfday";

my $prevfridayjd = $fridayjd - 7;
my $prevmondayjd = $mondayjd - 7;

my ($pmyear, $pmmonth,$pmday) = inverse_julian_day($prevmondayjd);
$prevmonday = "$pmyear-$pmmonth-$pmday";
my ($pfyear, $pfmonth,$pfday) = inverse_julian_day($prevfridayjd);
$prevfriday = "$pfyear-$pfmonth-$pfday";

#print "CF: $currfriday  CM:$currmonday<br>\n";
#print "PM: $prevmonday PF: $prevfriday<br>\n";


my $sth = $dbh->prepare("select student.lastname, student.firstname, student.studnum, student.grade, 
 student.homeroom, sum(choices.points) as totalpoints from student 
 left outer join choices on choices.studnum = student.studnum
 $select group by studnum order by $sortorder");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
my $rows = $sth->rows;


#print "Select: $select<br>\n";
#print "Sort Order: $sortorder<br>\n";
#print "Mode: $mode<br>\n";

print "<form action=\"$self\" method=\"post\" style=\"display:inline;\">\n";
# moved to end of form... print "<input type=\"hidden\" name=\"group\" value=\"$arr{group}\">\n";
print "<input type=\"submit\" value=\"Sort\">\n";
print "<select name=\"mode\"><option>$invmodes{$mode}</option>\n";
foreach my $key (sort keys %modes) {
    print "<option>$key</option>";
}

print "</select> Grp <input name=\"group\" size=\"4\" value=\"$group\"></form>\n";

print "<center><h1>". $lex{'Student Chips Report'}. "</h1><p><b>$currdate</b></p>\n";


# Print Date Selector Form Entry
print "<div style=\"position:absolute;top:0.5em; right:0.5em; background-color:#CCC; padding:0.3em;\">\n";
print "<form action=\"$self\" method=\"post\" style=\"display:inline;\">\n";
print "<input type=\"hidden\" name=\"datemode\" value=\"1\">\n";
print "<input type=\"submit\" value=\"Select by Date\">\n";
print "Strt <input type=\"text\" name=\"startdate\" size=\"8\">\n";
print "End <input type=\"text\" name=\"enddate\" size=\"8\">\n";
print "</div>\n";


$currgrade = -1;
$curroom = -1;
$first = 1;

# setup selections
my $sth1 = $dbh->prepare("select sum(points) from choices where studnum = ?");

my $sth2 = $dbh->prepare("select sum(points) from choices where studnum = ? and month(cdate) = ?");

my $sth3 = $dbh->prepare("select sum(points) from choices where studnum = ? and 
 to_days(cdate) >= to_days('$currmonday') and to_days(cdate) <= to_days('$currfriday')");

my $sth4 = $dbh->prepare("select sum(points) from choices where studnum = ? and 
 to_days(cdate) >= to_days('$prevmonday') and to_days(cdate) <= to_days('$prevfriday')");


while (my ($lastname, $firstname, $studnum, $grade, $homeroom ) = $sth->fetchrow) {

    $sth1->execute($studnum);
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my $totalpoints = $sth1->fetchrow;

    #print "Month: $month<br>\n";
    $sth2->execute($studnum, $month);
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my $currMonthPoints = $sth2->fetchrow;

    if ($month == 1){ $prevmonth = 12; } else { $prevmonth = $month -1; }

    #print "PrevMonth: $prevmonth<br>\n";
    $sth2->execute($studnum, $prevmonth);
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my $prevMonthPoints = $sth2->fetchrow;

    $sth3->execute($studnum);
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my $currWeekPoints = $sth3->fetchrow;

    $sth4->execute($studnum);
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my $prevWeekPoints = $sth4->fetchrow;


    $oldroom = $curroom;
    $curroom = $homeroom;

    $oldgrade = $currgrade;
    $currgrade = $grade;

    if ( ($mode eq 'namebyroom' or $mode eq 'pointsbyroom') and
	 $curroom ne $oldroom) {
	if ( not $first ){ print "</table>\n"; } else { $first = 0; }

	print "<p><b>Room: $curroom</b></p>";
	prTableHeader();

    } elsif (($mode eq 'namebygrade' or $mode eq 'pointsbygrade') and
	$currgrade ne $oldgrade) {
	if ( not $first ){ print "</table>\n"; } else { $first = 0; }

	print "<p><b>Grade: $currgrade</b></p>";
	prTableHeader();

    } else {
	if ($first) {
	    prTableHeader();
	    $first = 0;
	}
    }

    print "<tr><td><b>$lastname</b>, $firstname</td><td>$prevWeekPoints</td>";
    print "<td>$currWeekPoints</td>\n";
    print "<td>$prevMonthPoints</td><td>$currMonthPoints</td>";
    print "<td>$totalpoints</td></tr>\n";
  
}

print "</table></center></body></html>\n";


#----------------
sub prTableHeader {
#----------------
    print "<table border=\"1\" cellpadding =\"3\" cellspacing=\"0\">\n";
    print '<tr><th>'. $lex{Name}. '</th><th>'. $lex{Prev}. '<br>'. $lex{Week}. "</th>\n";
    print '<th>'. $lex{Curr}. '<br>'. $lex{Week}. '</th><th>'. $lex{Prev}. '<br>'. $lex{Month}. "</th>\n";
    print '<th>'. $lex{Curr}. '<br>'. $lex{Month}. '</th><th>'. $lex{YTD}. '<br>'. $lex{Total}. "</th></tr>\n";

}

#--------------
sub doDateRange {
#--------------

    my $sth = $dbh->prepare("select distinct studnum, sum(points) from choices where 
      to_days(cdate) >= to_days(?) and
      to_days(cdate) <= to_days(?) group by studnum");

    $sth->execute( $arr{startdate}, $arr{enddate} );
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

    # Print a date range report...
    print "<center><h1>". $lex{'Date Range Points Count'}. "</h1>\n";
    print "<p><b>". $lex{'Start Date'}. ":</b> $arr{startdate}\n";
    print "<b>". $lex{'End Date'}. ":</b> $arr{enddate}</p>\n";

    print "<table border=\"1\" cellpadding =\"3\" cellspacing=\"0\">\n";
    print "<tr><th>". $lex{Name}. "</th><th>". $lex{Total}. "</th></tr>\n";


    my $sth1 = $dbh->prepare("select lastname, firstname from student where studnum = ?");

    while ( my ($studnum, $totalpoints) = $sth->fetchrow ) {

	$sth1->execute( $studnum );
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	my ( $lastname, $firstname ) = $sth1->fetchrow;
	print "<tr><td><b>$lastname</b>, $firstname ($studnum)</td><td>$totalpoints</td></tr>\n";

    }

    print "</center></body></html>\n";

    exit;

} # end of doDateRange();
