#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my $self = 'rptchipscat.pl'; # Chip Report by Category

my %lex = ('Student Chips Category Report' => 'Student Chips Category Report',
	   'Main' => 'Main',
	   'Error' => 'Error',
	   'Name' => 'Name',

	   );

my %modes = ('Name by Grade' => 'namebygrade',
	   'Name by Homeroom' => 'namebyroom',
	   'Name by School' => 'namebyschool',
	   'Points by Grade' => 'pointsbygrade',
	   'Points by Homeroom' => 'pointsbyroom',
	   'Points by School' => 'pointsbyschool'
	   );

my %invmodes = reverse %modes;
my $defaultMode = 'namebyroom';
my $defaultCat = 'Scholarship';


use DBI;
use CGI;
use Cwd;
use Time::JulianDay;

# Read config variables
eval require "../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

eval require "../etc/choices.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;


my ($currsdate, $currdate, $currldate);
my ($day, $month, $year);

# Passed a date? 
if ($arr{date}) {
    # Generate a list from those dates. Assume year-mon-day
    ($year,$month,$day) = split /-/,$arr{date};

    $jd = julian_day($year,$month,$day);
    $dow = day_of_week($jd) + 1;
    $mondayjd = $jd - ($dow-1);
    $fridayjd = $mondayjd + 4;

    # Check for date before start of year; reset to year start.
    #if ($mondayjd < $startjd ){ $mondayjd = $startjd;}

    if (length($month) == 1){ $month = "0".$month;}
    if (length($day) == 1){ $day = "0".$day;}
    $currsdate = "$year$month$day"; 
    $currdate = "$year-$month-$day";
    $currdate = "$dow[ $dow ], $month[$month] $day, $year";

} else {

    @tim = localtime(time);
    $year = $tim[5] + 1900;
    $month = $tim[4] + 1;
    $day = $tim[3];
    $tim[6]++; # increment dow.

    $jd = julian_day($year,$month,$day);
    $mondayjd = $jd - ($tim[6]-1); #This will now give Monday of that week.
    $fridayjd = $mondayjd + 4;

    if (length($month) == 1){ $month = "0".$month;}
    if (length($day) == 1){ $day = "0".$day;}
    #$currdate = "$year-$month-$day";
    $currsdate = "$year$month$day";
    $currdate = "$dow[ $tim[6] ], $month[$month] $day, $year";

} 

my $mode;
if ($arr{mode} and $modes{$arr{mode}} ) { # If mode exists, and matches...
    $mode = $modes{$arr{mode}};
} else {
    $mode = $defaultMode; # set at top of file.
}


my %invcategory = reverse %category;
if (not $arr{cat}) {
    $arr{cat} = $defaultCat;
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);


# Set Selection and Sort Order based on mode values.
my $select;
my $sortorder;
if ($mode eq 'namebyroom' or $mode eq 'pointsbyroom') {
    if ($arr{group}) {
	my $homeroom = $dbh->quote($arr{group});
	#print "G:$arr{group} HR: $homeroom<br>\n";
	$select = "where homeroom = $homeroom";
    }
    # Add category selection to records so we can get this cat total points.
    if ($select) {
	$select = $select . " and category = '$invcategory{$arr{cat}}' ";
    } else { $select = "where category = '$invcategory{$arr{cat}}' "; }

    # Now do sortorder...
    if ($mode eq 'namebyroom') {
	$sortorder = "homeroom, lastname, firstname";
    } else { 
	$sortorder = "homeroom, totalpoints desc, lastname, firstname";
    }

} elsif ($mode eq 'namebygrade' or $mode eq 'pointsbygrade') {
    if ($arr{group}) {
	my $grade = $dbh->quote($arr{group});
	$select = "where grade = $grade";
    }
    # Add category selection to records so we can get this cat total points.
    if ($select) {
	$select = $select . " and category = '$invcategory{$arr{cat}}' ";
    } else { $select = "where category = '$invcategory{$arr{cat}}' "; }

    # Now do sortorder...
    if ($mode eq 'namebygrade') {
	$sortorder = "grade, lastname, firstname";
    } else { # pointsbygrade mode
	$sortorder = "grade, totalpoints desc, lastname, firstname";
    }
} else { # school modes.
    $select = ''; # no select; we want everyone...
    # Now do sortorder...
    if ($mode eq 'namebyschool') {
	$sortorder = "lastname, firstname";
	$select = '';
    } else { # pointsbyschool mode
	$sortorder = "totalpoints desc, lastname, firstname";
	$select = "where category = '$invcategory{$arr{cat}}' "; 
    }
}


# Change location of CSS files used. We must be running in tcgi, not cgi.
if (getcwd() =~ /tcgi/) {
    $css = $tchcss;
}


# Convert JD to weekly dates for all 4 weekly dates.
my ($cmyear, $cmmonth,$cmday) = inverse_julian_day($mondayjd);
$currmonday = "$cmyear-$cmmonth-$cmday";
my ($cfyear, $cfmonth,$cfday) = inverse_julian_day($fridayjd);
$currfriday = "$cfyear-$cfmonth-$cfday";

my $prevfridayjd = $fridayjd - 7;
my $prevmondayjd = $mondayjd - 7;

my ($pmyear, $pmmonth,$pmday) = inverse_julian_day($prevmondayjd);
$prevmonday = "$pmyear-$pmmonth-$pmday";
my ($pfyear, $pfmonth,$pfday) = inverse_julian_day($prevfridayjd);
$prevfriday = "$pfyear-$pfmonth-$pfday";

#print "CF: $currfriday  CM:$currmonday<br>\n";
#print "PM: $prevmonday PF: $prevfriday<br>\n";


my $sth = $dbh->prepare("select student.lastname, student.firstname, student.studnum, student.grade, 
 student.homeroom, sum(choices.points) as totalpoints from student 
 left outer join choices on choices.studnum = student.studnum
 $select group by studnum order by $sortorder");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
my $rows = $sth->rows;


print "$doctype\n<html><head><title>". $lex{'Student Chips Category Report'}. "</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>[ <a href=\"$tchpage\">". $lex{Main}. "</a> ]\n";

#print "Select: $select<br>\n";
#print "Sort Order: $sortorder<br>\n";
#print "Mode: $mode<br>\n";

print "<form action=\"$self\" method=\"post\" style=\"display:inline;\">\n";
print "<input type=\"submit\" value=\"Sort\">\n";
print "<select name=\"mode\"><option>$invmodes{$mode}</option>\n";
foreach my $key (sort keys %modes) {
    print "<option>$key</option>";
}

print "</select> Cat\n";
print "<select name=\"cat\"><option>$arr{cat}</option>\n";
foreach my $cat (@category) {
    print "<option>$category{$cat}</option>";
}
print "</select>\n";
print " Grp <input name=\"group\" size=\"4\" value=\"$group\"></form>\n";

print "<center><h1>". $lex{'Student Chips Category Report'}. "</h1><p><b>$currdate</b></p>\n";

$currgrade = -1;
$curroom = -1;
$first = 1;

# setup selection
my $sth1 = $dbh->prepare("select distinct category, sum(points) from choices 
 where studnum = ? and month(cdate) = ? group by category");

while (my ($lastname, $firstname, $studnum, $grade, $homeroom ) = $sth->fetchrow) {

    # get category points for this student...
    $sth1->execute($studnum, $month);
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my %catrec; 
    while (my ($cat,$tot) = $sth1->fetchrow) {
	#print "C:$cat T:$tot<br>\n";
	$catrec{$cat} = $tot;
    }

    $oldroom = $curroom;
    $curroom = $homeroom;

    $oldgrade = $currgrade;
    $currgrade = $grade;

    if ( ($mode eq 'namebyroom' or $mode eq 'pointsbyroom') and
	 $curroom ne $oldroom) {
	if ( not $first ){ print "</table>\n"; } else { $first = 0; }

	print "<p><b>Room: $curroom</b></p>";
	prCategoryTableHeader();

    } elsif (($mode eq 'namebygrade' or $mode eq 'pointsbygrade') and
	$currgrade ne $oldgrade) {
	if ( not $first ){ print "</table>\n"; } else { $first = 0; }

	print "<p><b>Grade: $currgrade</b></p>";
	prCategoryTableHeader();

    } else {
	if ($first) {
	    prCategoryTableHeader();
	    $first = 0;
	}
    }

    print "<tr><td><b>$lastname</b>, $firstname</td>";
    foreach my $cat (@category) {
	if ($cat eq $invcategory{$arr{cat}}) {
	    print "<td style=\"background-color:#DDD;\">$catrec{$cat}</td>"; # catrec populated above.
	} else {
	    print "<td>$catrec{$cat}</td>"; # catrec populated above.
	}
    }
    print "</tr>\n";
  
}

print "</table></center></body></html>\n";


#----------------
sub prCategoryTableHeader {
#----------------
    print "<table border=\"1\" cellpadding =\"3\" cellspacing=\"0\">\n";
    print "<tr><th>". $lex{Name}. "</th>";
    foreach my $cat ( @category ) { print "<th>$cat</th>"; }
    print "</tr>\n";

}
