#!/usr/bin/perl
# Copyright Leslie Richardson 2001-2009

#  This file is part of Open Admin for Schools
#  Freely licensed under the GNU GPL.

my %lex = ('View Student Attendance' => 'View Student Attendance',
	   'Main' => 'Main',
	   'Student Attendance Search Error' => 'Student Attendance Search Error',
	   'Date' => 'Date',
	   'Reason' => 'Reason',
	   'Period' => 'Period',
	   'Subject' => 'Subject',
	   'Please enter a student number or name' => 'Please enter a student number or name',
	   'Student Attendance Search' => 'Student Attendance Search',
	   'Name' => 'Name',
	   'Last Name' => 'Last Name',
	   'First Name' => 'First Name',
	   'Initials' => 'Initials',
	   'Student Number' => 'Student Number',
	   'Multiple Students found. Please select one.' => 
	     'Multiple Students found. Please select one.',
	   'Continue' => 'Continue',
	   'Error' => 'Error',

	   );

my $self = 'studattview.pl';

use DBI;
use CGI;

# Read config variables
eval require "../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;


my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Page Header.
print "$doctype\n<html><head><title>". $lex{'View Student Attendance'}. "</title>
<link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\">
$chartype\n</head><body>[ <a href=\"$tchpage\">". $lex{Main}. "</a> ]\n";

# Parse passed value and return prepared SQL query.
doSearch( $arr{student} );

# Now run the count SQL script
$sth1->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
my $count = $sth1->fetchrow;

# Execute the find student query.
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }


# Find out how many hits and decide what to do.
if ($count < 1){ # nobody found, try again.
    print "<h1>". $lex{'Student Attendance Search Error'}. "</h1>\n";
    prSearchError();
    print "</body></html>\n";
    die; 
} elsif ($count > 1) { # more than one found, display selection list.
    prSelectionList();
    die;
} # There can only be one...

# Get student info...
my ($lastname, $firstname,$studnum) = $sth->fetchrow;

# Retrieve Student records
$sth = $dbh->prepare("select attid, absdate, reason, period, subjsec 
  from attend where studentid = ?
  order by absdate desc, period");
$sth->execute( $studnum );
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}

print "<center><h1>". $lex{'View Student Attendance'}. "</h1>\n";
print "<div style=\"font-size:150%;font-weight:bold;\">";
print "$firstname $lastname ($studnum)</div>\n";
print "<table border=\"1\" cellpadding=\"3\" cellspacing=\"0\">\n";

print "<tr><th>". $lex{Date}. "</th><th>". $lex{Reason}. "</th><th>";
print $lex{Period}. "</th><th>". $lex{Subject}. "</th></tr>\n";

my $colcolor = 'blue'; # tr colors are blue or gray;
my $currdate = -1;

while( my ($attid, $absdate, $reason, $period, $subjsec) = $sth->fetchrow ) {
    $prevdate = $currdate;
    $currdate = $absdate;
    if ($prevdate ne $currdate){
	if ($colcolor eq 'blue'){ 
	    $colcolor = 'gray';
	} else { 
	    $colcolor = 'blue';
	}
    }

    print "<tr class=\"$colcolor\"><td>$absdate</td><td>$reason</td>";
    print "<td>$period</td><td>$subjsec</td></tr>\n";
}

print "</table><p></p>";
prSearchError();
print "</center></body></html>\n";


#-----------
sub doSearch {
#-----------
    my $student = shift;

    # Setup the Search
    if ($student =~ /\d+/) {  # we have a student number
	$studentnumber = $student;
	$sth = $dbh->prepare("select lastname,firstname,studnum
         from studentall where studnum = '$student'");
	$sth1 = $dbh->prepare("select count(*)
         from studentall where studnum = '$student'");
    } else { # we have words hopefully with a comma
	($lastname,$firstname)  = split /\,/, $student;
	$firstname =~ s/^\s*//;
	$lastname =~ s/^\s*//;
	if ($lastname and $firstname){ # both entered.
	    $sth = $dbh->prepare("select lastname,firstname,studnum
             from studentall
             where lastname = '$lastname' and firstname = '$firstname'");

	    $sth1 = $dbh->prepare("select count(*) from studentall
             where lastname = '$lastname' and firstname = '$firstname'");

	} elsif ($lastname and not $firstname){ # only lastname (no comma)
	    if (length($lastname) == 2){ # search by initials: fi, li.
		$fi = substr($lastname,0,1); $li = substr($lastname,1,1);
		$sth = $dbh->prepare("select lastname,firstname,studnum
                  from studentall 
                  where lastname $sql{like} '$li%' and 
                    firstname $sql{like} '$fi%'");

		$sth1 = $dbh->prepare("select count(*) from studentall 
                where lastname like '$li%' and firstname like '$fi%'");

	    } else {  # Lastname only 
		$sth = $dbh->prepare("select lastname, firstname, 
                  studnum from studentall 
                  where lastname = '$lastname' order by studentall.firstname");

		$sth1 = $dbh->prepare("select count(*) from studentall 
                  where lastname = '$lastname'");
	    }
	} else { # print an error....
	    prSearchError();
	    print "</body></html>\n";
	    exit; 
	}
    }


} # End of Sub doSearch


#----------------
sub prSearchError {
#----------------
    
    print "<p><i>". $lex{'Please enter a student number or name'}. "</i></p>\n";
    print "<p><h3>". $lex{'Student Attendance Search'}. "</h3>";
    print "<form action=\"$self\" method=\"post\">\n";
    print "<b>". $lex{Name}. "</b>(<i>". $lex{'Last Name'}. q{,};
    print $lex{'First Name'}. "</i> or <i>". $lex{'Last Name'}. "</i> or ";
    print "<i>". $lex{Initials}. "</i>) or <b>". $lex{'Student Number'}. "</b><br>\n";
    print "<input type=\"text\" name=\"student\" size=\"30\">\n";
    print "<input type=\"submit\" value=\"". $lex{'Student Attendance Search'}. "\">";
    print "</form></p>\n";
}


#------------------
sub prSelectionList {
#------------------
    # print list of students to select...
    # $sth has been set to return lastname, firstname, studnum
    print "<center>\n";
    print "<h1>". $lex{'Multiple Students found. Please select one.'}. "</h1>\n";
    print "<form action=\"$tcgiurl/studattview.pl\" method=\"post\">\n";
    print "<table cellpadding=\"3\" border=\"1\" cellspacing=\"0\">\n";
    while (my ($lastname, $firstname, $studnum) = $sth->fetchrow){
	print "<tr><td> $firstname $lastname</td><td>";
	print "<input type=\"radio\" name=\"student\" value=\"$studnum\">";
	print "</td></tr>\n";
    }
    print "<tr><td colspan=\"2\" align=\"center\">";
    print "<input type=\"submit\" value=\"". $lex{Continue}. "\"></td></tr>\n";
    print "</table></form></center>\n";
    
}
