#!/usr/bin/perl
#  Copyright 2001-2009 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# display teacher timetables. If passed teacher userid,
# display only that teacher's timetable.

my %lex = ('Day' => 'Day',
	   'Term' => 'Term',
	   'No Val' => 'No Val',
	   'View Teacher Timetable' => 'View Teacher Timetable',
	   'Schedule' => 'Schedule',
	   'Per' => 'Per',
	   'Main' => 'Main',
	   'Teacher' => 'Teacher',
	   'View' => 'View',
	   'No Records Found' => 'No Records Found',
	   'Error' => 'Error',

	   );

my $self = 'tchschedview.pl';

use DBI;
use CGI;
use Cwd;



# Set prepath for config file: (/tcgi or cgi/schedule)
my $prepath = '../..';
if (getcwd() =~ /tcgi/){ # we are in tcgi
    $prepath = '..';
}

eval require "$prepath/etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}


my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

my $select; 
if ( $arr{userid} ) {
    $select = " where subject.teacher like '%($arr{userid}%' ";
}
if ($arr{term}){
    if ($arr{userid}){
	$select .= " and schedat.term = '$arr{term}' ";
    } else {
	$select = " where schedat.term = '$arr{term}' ";
    }
}



# Get current dir so know what CSS to display;
if (getcwd() =~ /tcgi/){ # we are in tcgi
    $css = $tchcss;
    $homepage = $tchpage;
}


# print page header.
print "$doctype\n<html><head><title>". $lex{'View Teacher Timetable'}. "</title>
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
$chartype\n</head><body>\n";

print "[ <a href=\"$homepage\">". $lex{Main}. "</a> \n";
unless (getcwd() =~ /tcgi/){ # unless we are in tcgi
    print "| <a href=\"$schpage\">". $lex{Schedule}. "</a> \n";
}

print "] <form action=\"$self\" method=\"post\" style=\"display:inline;\">\n";
print $lex{Term}. " <input type=\"text\" size=\"6\" name=\"term\">&nbsp;&nbsp;";
print $lex{Teacher}. " <input type=\"text\" size=\"10\" name=\"teacher\">\n";
print "<input type=\"submit\" value=\"". $lex{View}. "\"></form>\n";



# Find Numbers of Rows (periods) and Columns(Days) in Cycle
my $sth = $dbh->prepare("select max(period) from schedat");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
my $tblrows = $sth->fetchrow;

$sth = $dbh->prepare("select max(day) from schedat");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
$tblcols = $sth->fetchrow;

if ( $tblrows < 1 ) {
    print '<p><b>'. $lex{'No Records Found'}. ".</b></p>\n";
    print "</body></html>\n";
    die;
}

$sth = $dbh->prepare("select distinct schedat.subjsec, subject.description, 
  subject.teacher, schedat.term from schedat
  left outer join subject on schedat.subjsec = subject.subjsec
  $select order by subject.teacher, schedat.term");
$sth->execute;
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}

my $oldterm = -1; 
my $oldtch = -1;
my $firstflag = 1;

while (my ($subjsec,$desc,$teacher,$term) = $sth->fetchrow){
    #print "Subj: $subjsec Tch: $teacher $term<br>\n";
    $oldterm = $currterm;
    $currterm = $term;
    $oldtch = $currtch;
    $currtch = $teacher;

    if (($currterm ne $oldterm or $currtch ne $oldtch) and not $firstflag){ 
	# change in teacher or term
	# ship to printTimetable with $teacher, $term, and @subjects
	printTimetable($oldtch, $oldterm, \@subjects);
	@subjects = (); # empty array.
	push @subjects,$subjsec; # start new array with existing value.
    } else {
	push @subjects, $subjsec;
	$firstflag = 0;
    }

}

# print out the last table.
printTimetable($currtch, $currterm, \@subjects);

print "</body></html>\n";


#-----------------
sub printTimetable { # print timetable for one teacher for 1 term.
#-----------------

    my ($teacher, $term, $subjref) = @_; 

    my ($name,$userid) = split /\(/,$teacher;
    my ($lastname, $firstname) = split /,/,$name;
    print "<p style=\"font-size: 150%;\"><b>$firstname $lastname</b> ". $lex{Term}. ": $term</p>\n";

    my @subject;
    my ($tblcols, $tblrows); # not globals anymore
    foreach my $subjsec ( @{$subjref} ) {
	my $sth = $dbh->prepare("select day, period from schedat 
         where term = ? and subjsec = ? order by period, day");
	$sth->execute($term, $subjsec);
	while (my ($day, $period) = $sth->fetchrow){
	    # setup values for $tblrows and $tblcols
	    if ( $day > $tblcols ) { $tblcols = $day; }
	    if ( $period > $tblrows ) { $tblrows = $period; }

	    if ($subject[$period][$day] and $subject[$period][$day] ne $subjsec){ 
		# if a value exists and is not identical to subjsec then add to it.
		$subject[$period][$day] .= " $subjsec";
	    } else { # just slap it in...
		$subject[$period][$day] = $subjsec;
	    }
	}
    }
    # Done filling 2D array.

    my $sth = $dbh->prepare("select description from subject where subjsec = ?");

    # print heading
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th></th>";
    for (1..$tblcols){ print "<th>". $lex{Day}. " $_</th>"; }
    print "</tr>\n";
    
    # Main body of table.
    for $i (1..$tblrows){
	print "<tr><td>". $lex{Per}. " $i</td>";
	for (1..$tblcols){
	    if ($subject[$i][$_]){ # if we have a subjsec value;
		my $firstflag = 1;
		@subjval = split /\s/,$subject[$i][$_];
		print "<td>";
		foreach my $sv (@subjval){
		    if (not $firstflag){ print "<br>";} else { $firstflag = 0;}
		    $sth->execute($sv);
		    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
		    my $desc = $sth->fetchrow;
		    print "$desc ($sv)";
		}
		print "</td>";
	    } else {
		print "<td style=\"color:green;\">". $lex{'No Val'}. "</td>";
	    }
	}
	print "</tr>\n";
    }
    print "</table>\n";
}
