#! /usr/bin/perl
#  Copyright 2001-2008 Leslie Richardson

#  This file is part of Open Admin for Schools.

# Update school populations from SL Values at Sask Learning

use DBI;
use CGI;
use XML::Writer;
use XML::Writer::String;
use Data::UUID;
use HTTP::Request::Common qw(POST); 
use HTTP::Headers; 
use LWP::UserAgent; 
use XML::LibXML;
use Fcntl qw(:DEFAULT :flock);

$self = 'getSLstudents.pl';

# Read config variables
eval require "../../etc/admin.conf";
if ( $@ ) {
    print "Error: getslstudent: $@<br>\n";
    die "Error: getslstudent: $@\n";
}

eval require "slxmllib.pl";
if ( $@ ) {
    print "Error: getslstudent: $@<br>\n";
    die "Error: getslstudent: $@\n";
}


# Override V1 with V2 values
# SDS XML Schema Declarations
$xmlns = "http://www.sasked.gov.sk.ca/xsd/sl/2.x/SLMessage_V2.xsd";
$xmlnsxsi = "http://www.w3.org/2001/XMLSchema-instance";
$xsischemaLocation="http://www.sasked.gov.sk.ca/xsd/sl/2.x/SLMessage_V2.xsd
http://www.sasked.gov.sk.ca/xsd/sl/2.x/SLMessage_V2.xsd";

my $q = new CGI;
my %arr = $q->Vars;
print $q->header;

if ($arr{debug}){
    $debug = 1;
    delete $arr{debug};
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Setup Date and Times
@month = ("","January","February","March","April","May","June","July",
	  "August","September","October","November","December");
# $sec, $min, $hour, $mday, $mon, 
# $year, $wday, $yday, $isdst ) = localtime(time);

@tim = localtime(time);
$year = $tim[5] + 1900;
$tim[4]++;
for (0..4){if (length($tim[$_]) == 1){ $tim[$_] = '0'.$tim[$_];}}
$currdate = "$year-$tim[4]-$tim[3]";
$currtime = "$tim[2]:$tim[1]:$tim[0]";
$currlongdate = "$month[$tim[4]] $tim[3], $year";

print "$doctype\n<html><head><title>Get School Enrollment</title> 
<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">
<style type=\"text/css\">
body {
  padding: 1em 3em;
}
</style>
$chartype\n</head><body>[ <a href=\"$homepage\">Main</a> | 
<a href=\"$exppage\">Export</a> ]
<h1>Get School Enrollment</h1><h3>$currlongdate</h3>\n";


# Create a new user agent
my $ua = LWP::UserAgent->new();
$ua->agent("OpenAdmin");

# Create the SchoolEnrollment Query
my $count++;
mkQueryString($count,'StudentPersonal'); # Generate $output string

# DEBUG Data Errors
if ($debug){
    print "<div style=\"color: blue;\">\n";
    print "<h1>DEBUG - Request sent to Sask Learning</h1>\n";
    my $temp = $output->value;
    $temp =~ s/</&lt;/g;
    $temp =~ s/>/&gt;/g;
    print "<pre>",$temp,"</pre></div><p></p>\n";
}

# Create the https post request 

my $req = POST $url, [ XML=>$output->value ];

$req->content_type('application/xml;charset="utf-8"');
$req->authorization_basic($sds_userid, $sds_password); 

# Issue the request and receive a response
my $res = $ua->request($req);

# Check the status of the response
if ($res->is_success) {

    # For Debugging Data Errors
    if ($debug){
	print "<h1>DEBUG - Sask Learning XML Response</h1>\n";
	print "<div style=\"color:green;\">\n";
	my $temp = $res->content;
	$temp =~ s/</&lt;/g;
	$temp =~ s/>/&gt;/g;
	print "<pre>",$temp,"</pre></div>\n";
    }

    my $response = $res->content;

    # Parse the response.
    my $parser = XML::LibXML->new();
    
    eval {$doc = $parser->parse_string($response)};
    if ( $@ ){
	print "<pre>Sask Learning Error:\n $@\n",$response,"</pre><br>\n";
	print "</body></html>\n"; die;
    }

    $doc->setEncoding('UTF-8');
    $root = $doc->getDocumentElement;
    $root->setNamespace($xmlns,'sl',1);

    $mastermsgid = $root->findvalue('//sl:SL_MsgId');
    $status = $root->findvalue('//sl:SL_Status/sl:SL_StatusCode');

    if ($status eq 'Errors' or $status eq 'Invalid'){ 
	# Print out error
	prErr($mastermsgid,"$lastname, $firstname ($studnum)");

    } elsif ($status eq 'Successful'){ 

	# Main Stuff here!
	parseStudentPersonal();

    } else { # print warnings... ($status eq 'Warnings')
	print "There were warnings...";
    }
    
}  else { # Transfer Error!
    my $err = $res->status_line;
    print "<h1>Transfer Error: $err</h1>";
    die;
}


# Now do the grade updates
$count++;
mkQueryString($count, 'StudentEnrollment'); # Generate $output string

# DEBUG Data Errors
if ($debug){
    print "<div style=\"color: blue;\">\n";
    print "<h1>DEBUG - Request sent to Sask Learning</h1>\n";
    my $temp = $output->value;
    $temp =~ s/</&lt;/g;
    $temp =~ s/>/&gt;/g;
    print "<pre>",$temp,"</pre></div><p></p>\n";
}


$req = POST $url, [ XML=>$output->value ];

$req->content_type('application/xml;charset="utf-8"');
$req->authorization_basic($sds_userid, $sds_password); 

# Issue the request and receive a response
my $res = $ua->request($req);

# Check the status of the response
if ($res->is_success) {
    $debug = 1;
    # For Debugging Data Errors
    if ($debug){
	print "<h1>DEBUG - Sask Learning XML Response</h1>\n";
	print "<div style=\"color:green;\">\n";
	my $temp = $res->content;
	$temp =~ s/</&lt;/g;
	$temp =~ s/>/&gt;/g;
	print "<pre>",$temp,"</pre></div>\n";
    }

    my $response = $res->content;

    # Parse the response.
    my $parser = XML::LibXML->new();
    
    eval {$doc = $parser->parse_string($response)};
    if ( $@ ){
	print "<pre>Sask Learning Error:\n $@\n",$response,"</pre><br>\n";
	print "</body></html>\n"; die;
    }

    $doc->setEncoding('UTF-8');
    $root = $doc->getDocumentElement;
    $root->setNamespace($xmlns,'sl',1);

    $mastermsgid = $root->findvalue('//sl:SL_MsgId');
    $status = $root->findvalue('//sl:SL_Status/sl:SL_StatusCode');

    if ($status eq 'Errors' or $status eq 'Invalid'){ 
	# Print out error
	prErr($mastermsgid,"$lastname, $firstname ($studnum)");

    } elsif ($status eq 'Successful'){ 

	# Main Stuff here!
	parseStudentEnrollment();

    } else { # print warnings... ($status eq 'Warnings')
	print "There were warnings...";
    }
    
}  else { # Transfer Error!
    my $err = $res->status_line;
    print "<h1>Transfer Error: $err</h1>";
    die;
}


#-----------------
print "</form></body></html>\n";



# Functions =============================

#--------------
sub mkQueryString {  # QryBySchool for StudentPersonal objects.
#--------------
    my $idcount = shift;
    my $object = shift;

    # Create Writer Instance
    $output = new XML::Writer::String;
    my $datamode = 1;
    if ($debug){ $datamode = 1;} # pretty print xml output
    $wr = new XML::Writer(OUTPUT => $output,
			  DATA_MODE => $datamode, 
			  DATA_INDENT => '2');

    # Set XML Header and write Root Element
    $wr->xmlDecl("UTF-8");
    $wr->startTag('SL_Message','xmlns' =>$xmlns, 
		  'xmlns:xsi' =>$xmlnsxsi, 
		  'xsi:schemaLocation' => $xsischemaLocation);

    $wr->startTag('SL_Request');
    mkSL_Header($currdate, $currtime, $schoolnumber,$idcount);

    $wr->startTag('SL_Query');

    $wr->startTag('QueryBySchool',
		  'RefId' => "$schoolnumber",
		  'ObjectName' => $object
		  );

    $wr->dataElement('SchoolId',$schoolnumber); 

    #if ( $object eq 'StudentPersonal' ) {
	my $cdate = $currdate;
        #$cdate =~ s/-//g; # strip hyphens
	$wr->dataElement('FromDate',$cdate); 	
	$wr->dataElement('ToDate',$cdate);
    #} 	

    $wr->endTag('QueryBySchool');

    $wr->endTag('SL_Query');
    $wr->endTag('SL_Request');
    $wr->endTag('SL_Message');
    $wr->end();
    
}


#---------
sub prErr {  # Print Errors to Screen
#---------
    my @errors = $root->findnodes('//sl:SL_Error');
    foreach my $error (@errors){
	# Get Error Elements
	my $objectname = $error->getAttribute('ObjectName');
	my $errmsg = $error->findvalue('./sl:SL_ErrorMsg');
	$errmsg =~ s/[\r|\n]/ /g;
	my $errcode = $error->findvalue('./sl:SL_ErrorCode');
	
	print "<p style=\"color:red; font-size:120%\">$errmsg</p>\n";
    }
    print "</table>\n";
     
}


#----------------------
sub parseStudentPersonal { # passed document root object.
#----------------------
    

    my @studinfo = $root->findnodes('//sl:StudentPersonal');
     
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>Name</th></tr>\n";
    my $count = 1;


    # Check for existing student
    my  $sth1 = $dbh->prepare("select count(*) from student where
        provnum = ? or ( lastname = ? and firstname = ? )");


    # Create an entry array called "@fields" storing fieldid values.
    $sth = $dbh->prepare("select fieldid from meta
      where tableid = 'student' order by arrayidx");
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
    while ( my $fld = $sth->fetchrow){
	push @fields,$fld;
    }

    foreach my $student ( @studinfo ){

	my %arr = ();

	#my $temp = $student->textContent; # works but only text...
	#my $temp = $student->toString();
	#$temp =~ s/</&lt;/g;
	#$temp =~ s/>/&gt;/g;
	#print "<pre>",$temp,"</pre>\n";

	$arr{provnum} = $student->findvalue('sl:StudentIdentification/sl:DeptAssignedPersonId');
	$arr{birthdate} = $student->findvalue('sl:StudentIdentification/sl:BirthDate');

	$arr{firstname} = $student->findvalue('sl:StudentInfo/sl:Name/sl:FirstName');
	$arr{lastname} = $student->findvalue('sl:StudentInfo/sl:Name/sl:LastName');
	$arr{initial} = $student->findvalue('sl:StudentInfo/sl:Name/sl:MiddleName');


	# Check for existing student, jump if already enroled.
	$sth1->execute( $arr{provnum}, $arr{lastname}, $arr{firstname} );
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
	my $studcount = $sth1->fetchrow;
	if ( $studcount > 0 ) { next; } # skip to next student;


	# Parse rest of records.
	$arr{healthnum} = $student->findvalue('sl:StudentInfo/sl:HSN');

	$arr{sex} = $student->findvalue('sl:StudentInfo/sl:Demographics/sl:Gender');
	$arr{ethnic} = $student->findvalue('sl:StudentInfo/sl:Demographics/sl:AboriginalStatus/@Code');
	$arr{treatynumber} = $student->findvalue('sl:StudentInfo/sl:Demographics/sl:TreatyStatusNumber/@Number');
	$arr{band} = $student->findvalue('sl:StudentInfo/sl:Demographics/sl:BandAffiliation/@Code');
	$arr{reserveresident} = $student->findvalue('sl:StudentInfo/sl:Demographics/sl:ReserveResidency/@Code');

	$arr{address1} = $student->findvalue('sl:StudentInfo/sl:StudentAddress/sl:Address/sl:Street/sl:Line1');
	$arr{city1} = $student->findvalue('sl:StudentInfo/sl:StudentAddress/sl:Address/sl:City');
	$arr{prov1} = $student->findvalue('sl:StudentInfo/sl:StudentAddress/sl:Address/sl:StatePr/@Code');
	$arr{pcode1} = $student->findvalue('sl:StudentInfo/sl:StudentAddress/sl:Address/sl:PostalCode');

	$arr{hphone1} = $student->findvalue('sl:StudentInfo/sl:PhoneNumber');


	# Now fix case in address1 and city1 
	$arr{city1} =~ s/(\w+)/\u\L$1/g;
	$arr{address1} =~ s/(\w+)/\u\L$1/g;



	# Get Student number and update file.
	sysopen (SNUM, "../../etc/studentnumber", O_RDWR | O_CREAT) or 
	    die $lex{'Cannot sysopen student number file.'};
	flock(SNUM,LOCK_EX);
	$studentnum = <SNUM> || 0;
	chomp $studentnum;
	seek(SNUM,0,0) or die $lex{'Cannot rewind student number file'}. ": $!\n";
	truncate(SNUM,0) or die $lex{'Cannot truncate student number file'}. ": $!\n";
	$newstudentnum = $studentnum + 1;
	print SNUM $newstudentnum or die $lex{'Cannot write to student number file'}. ": $!\n";
	close SNUM or die $lex{'Cannot close student number file'}. ": $! \n";

	$arr{studnum} = $studentnum;

	my @tempfields = @fields;

	# Now run through @fields array and use to populate array with values from hash.
	foreach my $fld ( @tempfields ) {
	    if ( $arr{$fld} ) { # if matching hash value exists, use it.
		$fld = $dbh->quote( $arr{$fld} ). ',';
	    } else {
		$fld = "$sql{default},"; # NULL value
	    }
	}
	chop $tempfields[$#fields]; # chop trailing comma.

	my $sth = $dbh->prepare("insert into student values ( @tempfields )");
	$sth->execute;
	if ($DBI::errstr){ print "Table Entry error: $DBI::errstr; $!"; die $DBI::errstr; }

	print "<tr><td>$arr{firstname} $arr{lastname} Inserted!</td></tr>";
	$count++;

    }
    print "</table>\n";


} # End of Sub parseStudentPersonal



#----------------------
sub parseStudentEnrollment { # passed document root object.
#----------------------

    print "Student Enrollment - Start\n";

    my @studinfo = $root->findnodes('//sl:StudentEnrollment');
     
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>Name</th><th>Provnum</th><th>Grade</th><th>Completed</th></tr>\n";
    my $count = 1;

    my $sth = $dbh->prepare("update student set grade = ? where provnum = ?");

    my $sth1 = $dbh->prepare("select lastname, firstname from student where provnum = ?");

    my $count;

    foreach my $student ( @studinfo ){

	#my $temp = $student->textContent; # works but only text...
	#my $temp = $student->toString();
	#$temp =~ s/</&lt;/g;
	#$temp =~ s/>/&gt;/g;
	#print "<pre>",$temp,"</pre>\n";

	my $provnum = $student->findvalue('sl:StudentIdentification/sl:DeptAssignedPersonId');
	my $grade = $student->findvalue('sl:Grade/@Code');

	$sth1->execute( $provnum );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my ( $lastname, $firstname ) = $sth1->fetchrow;
	$grade =~ s/^0//; # strip leading zero in grade.

	print "<tr><td>$firstname $lastname</td><td>$provnum</td><td>$grade</td>";
	if ( $grade ) {
	    $sth->execute( $grade, $provnum );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    print "<td>DID $grade - $provnum</td>";
	}
	print "</tr>\n";
	$count++;
	#print "$count ";
	

    }
    print "</table>\n";


} # End of Sub parseStudentEnrollment
